@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.algorithm.clustering;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -41,21 +41,21 @@ import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDMIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.ids.ModifiableDBIDs;
+import de.lmu.ifi.dbs.elki.database.ids.distance.DistanceDBIDList;
+import de.lmu.ifi.dbs.elki.database.ids.distance.DistanceDBIDListIter;
 import de.lmu.ifi.dbs.elki.database.query.range.RangeQuery;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.DistanceFunction;
-import de.lmu.ifi.dbs.elki.distance.distancefunction.EuclideanDistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.IndexBasedDistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.LocallyWeightedDistanceFunction;
-import de.lmu.ifi.dbs.elki.distance.distanceresultlist.DistanceDBIDResult;
-import de.lmu.ifi.dbs.elki.distance.distanceresultlist.DistanceDBIDResultIter;
+import de.lmu.ifi.dbs.elki.distance.distancefunction.minkowski.EuclideanDistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.DoubleDistance;
 import de.lmu.ifi.dbs.elki.logging.progress.FiniteProgress;
 import de.lmu.ifi.dbs.elki.logging.progress.IndefiniteProgress;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.AbstractParameterizer;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.GreaterConstraint;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.CommonConstraints;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.ChainedParameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.ListParameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
@@ -179,7 +179,7 @@ public abstract class AbstractProjectedDBSCAN<R extends Clustering<Model>, V ext
   public Clustering<Model> run(Database database, Relation<V> relation) {
     FiniteProgress objprog = getLogger().isVerbose() ? new FiniteProgress("Processing objects", relation.size(), getLogger()) : null;
     IndefiniteProgress clusprog = getLogger().isVerbose() ? new IndefiniteProgress("Number of clusters", getLogger()) : null;
-    resultList = new ArrayList<ModifiableDBIDs>();
+    resultList = new ArrayList<>();
     noise = DBIDUtil.newHashSet();
     processedIDs = DBIDUtil.newHashSet(relation.size());
 
@@ -215,14 +215,14 @@ public abstract class AbstractProjectedDBSCAN<R extends Clustering<Model>, V ext
       clusprog.setProcessed(resultList.size(), getLogger());
     }
 
-    Clustering<Model> result = new Clustering<Model>(getLongResultName(), getShortResultName());
+    Clustering<Model> result = new Clustering<>(getLongResultName(), getShortResultName());
     for(Iterator<ModifiableDBIDs> resultListIter = resultList.iterator(); resultListIter.hasNext();) {
       Cluster<Model> c = new Cluster<Model>(resultListIter.next(), ClusterModel.CLUSTER);
-      result.addCluster(c);
+      result.addToplevelCluster(c);
     }
 
     Cluster<Model> n = new Cluster<Model>(noise, true, ClusterModel.CLUSTER);
-    result.addCluster(n);
+    result.addToplevelCluster(n);
 
     if(objprog != null && clusprog != null) {
       objprog.setProcessed(processedIDs.size(), getLogger());
@@ -279,7 +279,7 @@ public abstract class AbstractProjectedDBSCAN<R extends Clustering<Model>, V ext
     }
 
     // compute weighted epsilon neighborhood
-    DistanceDBIDResult<DoubleDistance> neighbors = rangeQuery.getRangeForDBID(startObjectID, epsilon);
+    DistanceDBIDList<DoubleDistance> neighbors = rangeQuery.getRangeForDBID(startObjectID, epsilon);
     // neighbors < minPts -> noise
     if(neighbors.size() < minpts) {
       noise.add(startObjectID);
@@ -294,7 +294,7 @@ public abstract class AbstractProjectedDBSCAN<R extends Clustering<Model>, V ext
     // try to expand the cluster
     ModifiableDBIDs currentCluster = DBIDUtil.newArray();
     ModifiableDBIDs seeds = DBIDUtil.newHashSet();
-    for (DistanceDBIDResultIter<DoubleDistance> seed = neighbors.iter(); seed.valid(); seed.advance()) {
+    for(DistanceDBIDListIter<DoubleDistance> seed = neighbors.iter(); seed.valid(); seed.advance()) {
       int nextID_corrDim = distFunc.getIndex().getLocalProjection(seed).getCorrelationDimension();
       // nextID is not reachable from start object
       if(nextID_corrDim > lambda) {
@@ -320,11 +320,11 @@ public abstract class AbstractProjectedDBSCAN<R extends Clustering<Model>, V ext
         continue;
       }
 
-      DistanceDBIDResult<DoubleDistance> reachables = rangeQuery.getRangeForDBID(iter, epsilon);
+      DistanceDBIDList<DoubleDistance> reachables = rangeQuery.getRangeForDBID(iter, epsilon);
       iter.remove();
-      
+
       if(reachables.size() > minpts) {
-        for (DistanceDBIDResultIter<DoubleDistance> r = reachables.iter(); r.valid(); r.advance()) {
+        for(DistanceDBIDListIter<DoubleDistance> r = reachables.iter(); r.valid(); r.advance()) {
           int corrDim_r = distFunc.getIndex().getLocalProjection(r).getCorrelationDimension();
           // r is not reachable from q
           if(corrDim_r > lambda) {
@@ -351,9 +351,10 @@ public abstract class AbstractProjectedDBSCAN<R extends Clustering<Model>, V ext
         }
       }
 
-      /* if(processedIDs.size() == relation.size() && noise.size() == 0) {
-        break;
-      } */
+      /*
+       * if(processedIDs.size() == relation.size() && noise.size() == 0) {
+       * break; }
+       */
     }
 
     if(currentCluster.size() >= minpts) {
@@ -375,7 +376,7 @@ public abstract class AbstractProjectedDBSCAN<R extends Clustering<Model>, V ext
   public TypeInformation[] getInputTypeRestriction() {
     return TypeUtil.array(distanceFunction.getInputTypeRestriction());
   }
-  
+
   /**
    * Parameterization class.
    * 
@@ -395,7 +396,7 @@ public abstract class AbstractProjectedDBSCAN<R extends Clustering<Model>, V ext
     protected Integer lambda;
 
     protected void configInnerDistance(Parameterization config) {
-      ObjectParameter<DistanceFunction<V, D>> innerdistP = new ObjectParameter<DistanceFunction<V, D>>(AbstractProjectedDBSCAN.INNER_DISTANCE_FUNCTION_ID, DistanceFunction.class, EuclideanDistanceFunction.class);
+      ObjectParameter<DistanceFunction<V, D>> innerdistP = new ObjectParameter<>(AbstractProjectedDBSCAN.INNER_DISTANCE_FUNCTION_ID, DistanceFunction.class, EuclideanDistanceFunction.class);
       if(config.grab(innerdistP)) {
         innerdist = innerdistP.instantiateClass(config);
       }
@@ -403,7 +404,7 @@ public abstract class AbstractProjectedDBSCAN<R extends Clustering<Model>, V ext
 
     protected void configEpsilon(Parameterization config, DistanceFunction<V, D> innerdist) {
       D distanceParser = innerdist != null ? innerdist.getDistanceFactory() : null;
-      DistanceParameter<D> epsilonP = new DistanceParameter<D>(EPSILON_ID, distanceParser);
+      DistanceParameter<D> epsilonP = new DistanceParameter<>(EPSILON_ID, distanceParser);
       if(config.grab(epsilonP)) {
         epsilon = epsilonP.getValue();
       }
@@ -411,14 +412,14 @@ public abstract class AbstractProjectedDBSCAN<R extends Clustering<Model>, V ext
 
     protected void configMinPts(Parameterization config) {
       IntParameter minptsP = new IntParameter(MINPTS_ID);
-      minptsP.addConstraint(new GreaterConstraint(0));
+      minptsP.addConstraint(CommonConstraints.GREATER_EQUAL_ONE_INT);
       if(config.grab(minptsP)) {
         minpts = minptsP.getValue();
       }
     }
 
     protected void configOuterDistance(Parameterization config, D epsilon, int minpts, Class<?> preprocessorClass, DistanceFunction<V, D> innerdist) {
-      ObjectParameter<LocallyWeightedDistanceFunction<V>> outerdistP = new ObjectParameter<LocallyWeightedDistanceFunction<V>>(OUTER_DISTANCE_FUNCTION_ID, LocallyWeightedDistanceFunction.class, LocallyWeightedDistanceFunction.class);
+      ObjectParameter<LocallyWeightedDistanceFunction<V>> outerdistP = new ObjectParameter<>(OUTER_DISTANCE_FUNCTION_ID, LocallyWeightedDistanceFunction.class, LocallyWeightedDistanceFunction.class);
       if(config.grab(outerdistP)) {
         // parameters for the distance function
         ListParameterization distanceFunctionParameters = new ListParameterization();
@@ -435,7 +436,7 @@ public abstract class AbstractProjectedDBSCAN<R extends Clustering<Model>, V ext
 
     protected void configLambda(Parameterization config) {
       IntParameter lambdaP = new IntParameter(LAMBDA_ID);
-      lambdaP.addConstraint(new GreaterConstraint(0));
+      lambdaP.addConstraint(CommonConstraints.GREATER_EQUAL_ONE_INT);
       if(config.grab(lambdaP)) {
         lambda = lambdaP.getValue();
       }
