@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.algorithm.clustering;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -46,12 +46,9 @@ import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.result.IterableResult;
 import de.lmu.ifi.dbs.elki.result.optics.ClusterOrderEntry;
 import de.lmu.ifi.dbs.elki.result.optics.ClusterOrderResult;
-import de.lmu.ifi.dbs.elki.utilities.datastructures.hierarchy.HierarchyHashmapList;
-import de.lmu.ifi.dbs.elki.utilities.datastructures.hierarchy.ModifiableHierarchy;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.AbstractParameterizer;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.GreaterEqualConstraint;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.LessConstraint;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.CommonConstraints;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.ClassParameter;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.DoubleParameter;
@@ -135,13 +132,13 @@ public class OPTICSXi<N extends NumberDistance<N, ?>> extends AbstractAlgorithm<
     double mib = 0.0;
     // TODO: make it configurable to keep this list; this is mostly useful for
     // visualization
-    List<SteepArea> salist = new ArrayList<SteepArea>();
-    List<SteepDownArea> sdaset = new ArrayList<SteepDownArea>();
-    ModifiableHierarchy<Cluster<OPTICSModel>> hier = new HierarchyHashmapList<Cluster<OPTICSModel>>();
-    HashSet<Cluster<OPTICSModel>> curclusters = new HashSet<Cluster<OPTICSModel>>();
+    List<SteepArea> salist = new ArrayList<>();
+    List<SteepDownArea> sdaset = new ArrayList<>();
+    final Clustering<OPTICSModel> clustering = new Clustering<>("OPTICS Xi-Clusters", "optics");
+    HashSet<Cluster<OPTICSModel>> curclusters = new HashSet<>();
     HashSetModifiableDBIDs unclaimedids = DBIDUtil.newHashSet(relation.getDBIDs());
 
-    SteepScanPosition<N> scan = new SteepScanPosition<N>(clusterOrder);
+    SteepScanPosition<N> scan = new SteepScanPosition<>(clusterOrder);
     while(scan.hasNext()) {
       final int curpos = scan.index;
       // Update maximum-inbetween
@@ -242,6 +239,10 @@ public class OPTICSXi<N extends NumberDistance<N, ?>> extends AbstractAlgorithm<
             // By default, clusters cover both the steep up and steep down area
             int cstart = sda.getStartIndex();
             int cend = sua.getEndIndex();
+            // Hotfix: never include infinity-reachable points at the end
+            while(cend > cstart && Double.isInfinite(clusterOrder.get(cend).getReachability().doubleValue())) {
+              --cend;
+            }
             // However, we sometimes have to adjust this (Condition 4):
             {
               // Case b)
@@ -285,7 +286,7 @@ public class OPTICSXi<N extends NumberDistance<N, ?>> extends AbstractAlgorithm<
               LOG.debugFine("Found cluster with " + dbids.size() + " new objects, length " + (cstart - cend + 1));
             }
             OPTICSModel model = new OPTICSModel(cstart, cend);
-            Cluster<OPTICSModel> cluster = new Cluster<OPTICSModel>("Cluster_" + cstart + "_" + cend, dbids, model, hier);
+            Cluster<OPTICSModel> cluster = new Cluster<>("Cluster_" + cstart + "_" + cend, dbids, model);
             // Build the hierarchy
             {
               Iterator<Cluster<OPTICSModel>> iter = curclusters.iterator();
@@ -293,7 +294,7 @@ public class OPTICSXi<N extends NumberDistance<N, ?>> extends AbstractAlgorithm<
                 Cluster<OPTICSModel> clus = iter.next();
                 OPTICSModel omodel = clus.getModel();
                 if(model.getStartIndex() <= omodel.getStartIndex() && omodel.getEndIndex() <= model.getEndIndex()) {
-                  hier.add(cluster, clus);
+                  clustering.addChildCluster(cluster, clus);
                   iter.remove();
                 }
               }
@@ -308,23 +309,22 @@ public class OPTICSXi<N extends NumberDistance<N, ?>> extends AbstractAlgorithm<
       }
     }
     if(curclusters.size() > 0 || unclaimedids.size() > 0) {
-      final Clustering<OPTICSModel> clustering = new Clustering<OPTICSModel>("OPTICS Xi-Clusters", "optics");
       if(unclaimedids.size() > 0) {
         final Cluster<OPTICSModel> allcluster;
         if(clusterOrder.get(clusterOrder.size() - 1).getReachability().isInfiniteDistance()) {
-          allcluster = new Cluster<OPTICSModel>("Noise", unclaimedids, true, new OPTICSModel(0, clusterOrder.size() - 1), hier);
+          allcluster = new Cluster<>("Noise", unclaimedids, true, new OPTICSModel(0, clusterOrder.size() - 1));
         }
         else {
-          allcluster = new Cluster<OPTICSModel>("Cluster", unclaimedids, new OPTICSModel(0, clusterOrder.size() - 1), hier);
+          allcluster = new Cluster<>("Cluster", unclaimedids, new OPTICSModel(0, clusterOrder.size() - 1));
         }
         for(Cluster<OPTICSModel> cluster : curclusters) {
-          hier.add(allcluster, cluster);
+          clustering.addChildCluster(allcluster, cluster);
         }
-        clustering.addCluster(allcluster);
+        clustering.addToplevelCluster(allcluster);
       }
       else {
         for(Cluster<OPTICSModel> cluster : curclusters) {
-          clustering.addCluster(cluster);
+          clustering.addToplevelCluster(cluster);
         }
       }
       clustering.addChildResult(clusterOrderResult);
@@ -657,13 +657,13 @@ public class OPTICSXi<N extends NumberDistance<N, ?>> extends AbstractAlgorithm<
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
       DoubleParameter xiP = new DoubleParameter(XI_ID);
-      xiP.addConstraint(new GreaterEqualConstraint(0.0));
-      xiP.addConstraint(new LessConstraint(1.0));
+      xiP.addConstraint(CommonConstraints.GREATER_EQUAL_ZERO_DOUBLE);
+      xiP.addConstraint(CommonConstraints.LESS_THAN_ONE_DOUBLE);
       if(config.grab(xiP)) {
         xi = xiP.doubleValue();
       }
 
-      ClassParameter<OPTICSTypeAlgorithm<D>> opticsP = new ClassParameter<OPTICSTypeAlgorithm<D>>(XIALG_ID, OPTICSTypeAlgorithm.class, OPTICS.class);
+      ClassParameter<OPTICSTypeAlgorithm<D>> opticsP = new ClassParameter<>(XIALG_ID, OPTICSTypeAlgorithm.class, OPTICS.class);
       if(config.grab(opticsP)) {
         optics = opticsP.instantiateClass(config);
       }
@@ -671,7 +671,7 @@ public class OPTICSXi<N extends NumberDistance<N, ?>> extends AbstractAlgorithm<
 
     @Override
     protected OPTICSXi<D> makeInstance() {
-      return new OPTICSXi<D>(optics, xi);
+      return new OPTICSXi<>(optics, xi);
     }
   }
 }
\ No newline at end of file
