@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.algorithm.clustering.correlation;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -66,14 +66,15 @@ import de.lmu.ifi.dbs.elki.math.linearalgebra.Matrix;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.pca.FirstNEigenPairFilter;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.pca.PCAFilteredRunner;
 import de.lmu.ifi.dbs.elki.utilities.ClassGenericsUtil;
-import de.lmu.ifi.dbs.elki.utilities.datastructures.heap.Heap;
+import de.lmu.ifi.dbs.elki.utilities.datastructures.heap.ComparableMinHeap;
 import de.lmu.ifi.dbs.elki.utilities.datastructures.heap.IntegerPriorityObject;
+import de.lmu.ifi.dbs.elki.utilities.datastructures.heap.ObjectHeap;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Description;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Title;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.AbstractParameterizer;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.GreaterConstraint;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.CommonConstraints;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.ListParameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.DoubleParameter;
@@ -263,8 +264,8 @@ public class CASH<V extends NumberVector<?>> extends AbstractAlgorithm<Clusterin
    */
   private Relation<ParameterizationFunction> preprocess(Database db, Relation<V> vrel) {
     DBIDs ids = vrel.getDBIDs();
-    SimpleTypeInformation<ParameterizationFunction> type = new SimpleTypeInformation<ParameterizationFunction>(ParameterizationFunction.class);
-    MaterializedRelation<ParameterizationFunction> prep = new MaterializedRelation<ParameterizationFunction>(db, type, ids);
+    SimpleTypeInformation<ParameterizationFunction> type = new SimpleTypeInformation<>(ParameterizationFunction.class);
+    MaterializedRelation<ParameterizationFunction> prep = new MaterializedRelation<>(db, type, ids);
 
     // Project
     for (DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
@@ -284,12 +285,12 @@ public class CASH<V extends NumberVector<?>> extends AbstractAlgorithm<Clusterin
    * @return a mapping of subspace dimensionalities to clusters
    */
   private Clustering<Model> doRun(Relation<ParameterizationFunction> relation, FiniteProgress progress) {
-    Clustering<Model> res = new Clustering<Model>("CASH clustering", "cash-clustering");
+    Clustering<Model> res = new Clustering<>("CASH clustering", "cash-clustering");
 
     final int dim = dimensionality(relation);
 
     // init heap
-    Heap<IntegerPriorityObject<CASHInterval>> heap = new Heap<IntegerPriorityObject<CASHInterval>>();
+    ObjectHeap<IntegerPriorityObject<CASHInterval>> heap = new ComparableMinHeap<>();
     ModifiableDBIDs noiseIDs = DBIDUtil.newHashSet(relation.getDBIDs());
     initHeap(heap, relation, dim, noiseIDs);
 
@@ -338,7 +339,7 @@ public class CASH<V extends NumberVector<?>> extends AbstractAlgorithm<Clusterin
           // add result of dim-1 to this result
           Clustering<Model> res_dim_minus_1 = doRun(db, progress);
           for (Cluster<Model> cluster : res_dim_minus_1.getAllClusters()) {
-            res.addCluster(cluster);
+            res.addToplevelCluster(cluster);
             noiseIDs.removeDBIDs(cluster.getIDs());
             clusterIDs.addDBIDs(cluster.getIDs());
             processedIDs.addDBIDs(cluster.getIDs());
@@ -349,23 +350,23 @@ public class CASH<V extends NumberVector<?>> extends AbstractAlgorithm<Clusterin
       else {
         LinearEquationSystem les = runDerivator(relation, dim - 1, interval.getIDs());
         Cluster<Model> c = new Cluster<Model>(interval.getIDs(), new LinearEquationModel(les));
-        res.addCluster(c);
+        res.addToplevelCluster(c);
         noiseIDs.removeDBIDs(interval.getIDs());
         clusterIDs.addDBIDs(interval.getIDs());
         processedIDs.addDBIDs(interval.getIDs());
       }
 
       // Rebuild heap
-      ArrayList<IntegerPriorityObject<CASHInterval>> heapVector = new ArrayList<IntegerPriorityObject<CASHInterval>>(heap.size());
-      for (IntegerPriorityObject<CASHInterval> obj : heap) {
-        heapVector.add(obj);
+      ArrayList<IntegerPriorityObject<CASHInterval>> heapVector = new ArrayList<>(heap.size());
+      for (ObjectHeap.UnsortedIter<IntegerPriorityObject<CASHInterval>> iter = heap.unsortedIter(); iter.valid(); iter.advance()) {
+        heapVector.add(iter.get());
       }
       heap.clear();
       for (IntegerPriorityObject<CASHInterval> pair : heapVector) {
         CASHInterval currentInterval = pair.getObject();
         currentInterval.removeIDs(clusterIDs);
         if (currentInterval.getIDs().size() >= minPts) {
-          heap.add(new IntegerPriorityObject<CASHInterval>(currentInterval.priority(), currentInterval));
+          heap.add(new IntegerPriorityObject<>(currentInterval.priority(), currentInterval));
         }
       }
 
@@ -378,12 +379,12 @@ public class CASH<V extends NumberVector<?>> extends AbstractAlgorithm<Clusterin
     if (!noiseIDs.isEmpty()) {
       if (dim == noiseDim) {
         Cluster<Model> c = new Cluster<Model>(noiseIDs, true, ClusterModel.CLUSTER);
-        res.addCluster(c);
+        res.addToplevelCluster(c);
         processedIDs.addDBIDs(noiseIDs);
       } else if (noiseIDs.size() >= minPts) {
         LinearEquationSystem les = runDerivator(fulldatabase, dim - 1, noiseIDs);
         Cluster<Model> c = new Cluster<Model>(noiseIDs, true, new LinearEquationModel(les));
-        res.addCluster(c);
+        res.addToplevelCluster(c);
         processedIDs.addDBIDs(noiseIDs);
       }
     }
@@ -427,7 +428,7 @@ public class CASH<V extends NumberVector<?>> extends AbstractAlgorithm<Clusterin
    * @param dim the dimensionality of the database
    * @param ids the ids of the database
    */
-  private void initHeap(Heap<IntegerPriorityObject<CASHInterval>> heap, Relation<ParameterizationFunction> relation, int dim, DBIDs ids) {
+  private void initHeap(ObjectHeap<IntegerPriorityObject<CASHInterval>> heap, Relation<ParameterizationFunction> relation, int dim, DBIDs ids) {
     CASHIntervalSplit split = new CASHIntervalSplit(relation, minPts);
 
     // determine minimum and maximum function value of all functions
@@ -479,7 +480,7 @@ public class CASH<V extends NumberVector<?>> extends AbstractAlgorithm<Clusterin
       ModifiableDBIDs intervalIDs = split.determineIDs(ids, alphaInterval, d_mins[i], d_maxs[i]);
       if (intervalIDs != null && intervalIDs.size() >= minPts) {
         CASHInterval rootInterval = new CASHInterval(alphaMin, alphaMax, split, intervalIDs, -1, 0, d_mins[i], d_maxs[i]);
-        heap.add(new IntegerPriorityObject<CASHInterval>(rootInterval.priority(), rootInterval));
+        heap.add(new IntegerPriorityObject<>(rootInterval.priority(), rootInterval));
       }
     }
 
@@ -503,8 +504,8 @@ public class CASH<V extends NumberVector<?>> extends AbstractAlgorithm<Clusterin
    */
   private MaterializedRelation<ParameterizationFunction> buildDB(int dim, Matrix basis, DBIDs ids, Relation<ParameterizationFunction> relation) {
     ProxyDatabase proxy = new ProxyDatabase(ids);
-    SimpleTypeInformation<ParameterizationFunction> type = new SimpleTypeInformation<ParameterizationFunction>(ParameterizationFunction.class);
-    MaterializedRelation<ParameterizationFunction> prep = new MaterializedRelation<ParameterizationFunction>(proxy, type, ids);
+    SimpleTypeInformation<ParameterizationFunction> type = new SimpleTypeInformation<>(ParameterizationFunction.class);
+    MaterializedRelation<ParameterizationFunction> prep = new MaterializedRelation<>(proxy, type, ids);
     proxy.addRelation(prep);
 
     // Project
@@ -566,7 +567,7 @@ public class CASH<V extends NumberVector<?>> extends AbstractAlgorithm<Clusterin
   private double sinusProduct(int start, int end, double[] alpha) {
     double result = 1;
     for (int j = start; j < end; j++) {
-      result *= StrictMath.sin(alpha[j]);
+      result *= Math.sin(alpha[j]);
     }
     return result;
   }
@@ -578,7 +579,7 @@ public class CASH<V extends NumberVector<?>> extends AbstractAlgorithm<Clusterin
    * @param heap the heap storing the intervals
    * @return the next ''best'' interval at maximum level
    */
-  private CASHInterval determineNextIntervalAtMaxLevel(Heap<IntegerPriorityObject<CASHInterval>> heap) {
+  private CASHInterval determineNextIntervalAtMaxLevel(ObjectHeap<IntegerPriorityObject<CASHInterval>> heap) {
     CASHInterval next = doDetermineNextIntervalAtMaxLevel(heap);
     // noise path was chosen
     while (next == null) {
@@ -598,7 +599,7 @@ public class CASH<V extends NumberVector<?>> extends AbstractAlgorithm<Clusterin
    * @param heap the heap storing the intervals
    * @return the next ''best'' interval at maximum level
    */
-  private CASHInterval doDetermineNextIntervalAtMaxLevel(Heap<IntegerPriorityObject<CASHInterval>> heap) {
+  private CASHInterval doDetermineNextIntervalAtMaxLevel(ObjectHeap<IntegerPriorityObject<CASHInterval>> heap) {
     CASHInterval interval = heap.poll().getObject();
     int dim = interval.getDimensionality();
     while (true) {
@@ -632,10 +633,10 @@ public class CASH<V extends NumberVector<?>> extends AbstractAlgorithm<Clusterin
         int comp = interval.getLeftChild().compareTo(interval.getRightChild());
         if (comp < 0) {
           bestInterval = interval.getRightChild();
-          heap.add(new IntegerPriorityObject<CASHInterval>(interval.getLeftChild().priority(), interval.getLeftChild()));
+          heap.add(new IntegerPriorityObject<>(interval.getLeftChild().priority(), interval.getLeftChild()));
         } else {
           bestInterval = interval.getLeftChild();
-          heap.add(new IntegerPriorityObject<CASHInterval>(interval.getRightChild().priority(), interval.getRightChild()));
+          heap.add(new IntegerPriorityObject<>(interval.getRightChild().priority(), interval.getRightChild()));
         }
       } else if (interval.getLeftChild() == null) {
         bestInterval = interval.getRightChild();
@@ -733,8 +734,8 @@ public class CASH<V extends NumberVector<?>> extends AbstractAlgorithm<Clusterin
     DBIDs ids = interval.getIDs();
     ProxyDatabase proxy = new ProxyDatabase(ids);
     int dim = dimensionality(relation);
-    SimpleTypeInformation<DoubleVector> type = new VectorFieldTypeInformation<DoubleVector>(DoubleVector.FACTORY, dim);
-    MaterializedRelation<DoubleVector> prep = new MaterializedRelation<DoubleVector>(proxy, type, ids);
+    SimpleTypeInformation<DoubleVector> type = new VectorFieldTypeInformation<>(DoubleVector.FACTORY, dim);
+    MaterializedRelation<DoubleVector> prep = new MaterializedRelation<>(proxy, type, ids);
     proxy.addRelation(prep);
 
     // Project
@@ -792,8 +793,8 @@ public class CASH<V extends NumberVector<?>> extends AbstractAlgorithm<Clusterin
   private Database buildDerivatorDB(Relation<ParameterizationFunction> relation, DBIDs ids) {
     ProxyDatabase proxy = new ProxyDatabase(ids);
     int dim = dimensionality(relation);
-    SimpleTypeInformation<DoubleVector> type = new VectorFieldTypeInformation<DoubleVector>(DoubleVector.FACTORY, dim);
-    MaterializedRelation<DoubleVector> prep = new MaterializedRelation<DoubleVector>(proxy, type, ids);
+    SimpleTypeInformation<DoubleVector> type = new VectorFieldTypeInformation<>(DoubleVector.FACTORY, dim);
+    MaterializedRelation<DoubleVector> prep = new MaterializedRelation<>(proxy, type, ids);
     proxy.addRelation(prep);
 
     // Project
@@ -837,22 +838,22 @@ public class CASH<V extends NumberVector<?>> extends AbstractAlgorithm<Clusterin
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
       IntParameter minptsP = new IntParameter(MINPTS_ID);
-      minptsP.addConstraint(new GreaterConstraint(0));
+      minptsP.addConstraint(CommonConstraints.GREATER_EQUAL_ONE_INT);
       if (config.grab(minptsP)) {
         minpts = minptsP.getValue();
       }
       IntParameter maxlevelP = new IntParameter(MAXLEVEL_ID);
-      maxlevelP.addConstraint(new GreaterConstraint(0));
+      maxlevelP.addConstraint(CommonConstraints.GREATER_EQUAL_ONE_INT);
       if (config.grab(maxlevelP)) {
         maxlevel = maxlevelP.getValue();
       }
       IntParameter mindimP = new IntParameter(MINDIM_ID, 1);
-      mindimP.addConstraint(new GreaterConstraint(0));
+      mindimP.addConstraint(CommonConstraints.GREATER_EQUAL_ONE_INT);
       if (config.grab(mindimP)) {
         mindim = mindimP.getValue();
       }
       DoubleParameter jitterP = new DoubleParameter(JITTER_ID);
-      jitterP.addConstraint(new GreaterConstraint(0));
+      jitterP.addConstraint(CommonConstraints.GREATER_THAN_ZERO_DOUBLE);
       if (config.grab(jitterP)) {
         jitter = jitterP.getValue();
       }
@@ -864,7 +865,7 @@ public class CASH<V extends NumberVector<?>> extends AbstractAlgorithm<Clusterin
 
     @Override
     protected CASH<NumberVector<?>> makeInstance() {
-      return new CASH<NumberVector<?>>(minpts, maxlevel, mindim, jitter, adjust);
+      return new CASH<>(minpts, maxlevel, mindim, jitter, adjust);
     }
   }
 }
