@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.algorithm.clustering.correlation;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -115,7 +115,7 @@ public class FourC<V extends NumberVector<?>> extends AbstractProjectedDBSCAN<Cl
 
     @Override
     protected FourC<O> makeInstance() {
-      return new FourC<O>(epsilon, minpts, outerdist, lambda);
+      return new FourC<>(epsilon, minpts, outerdist, lambda);
     }
   }
 }
\ No newline at end of file
