@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.algorithm.clustering.gdbscan;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -23,7 +23,8 @@ package de.lmu.ifi.dbs.elki.algorithm.clustering.gdbscan;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-import de.lmu.ifi.dbs.elki.algorithm.AbstractDistanceBasedAlgorithm;
+import de.lmu.ifi.dbs.elki.algorithm.DistanceBasedAlgorithm;
+import de.lmu.ifi.dbs.elki.algorithm.clustering.DBSCAN;
 import de.lmu.ifi.dbs.elki.data.type.SimpleTypeInformation;
 import de.lmu.ifi.dbs.elki.data.type.TypeInformation;
 import de.lmu.ifi.dbs.elki.data.type.TypeUtil;
@@ -32,11 +33,11 @@ import de.lmu.ifi.dbs.elki.database.QueryUtil;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
 import de.lmu.ifi.dbs.elki.database.ids.ModifiableDBIDs;
+import de.lmu.ifi.dbs.elki.database.ids.distance.DistanceDBIDList;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.database.query.range.RangeQuery;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.DistanceFunction;
-import de.lmu.ifi.dbs.elki.distance.distancefunction.EuclideanDistanceFunction;
-import de.lmu.ifi.dbs.elki.distance.distanceresultlist.DistanceDBIDResult;
+import de.lmu.ifi.dbs.elki.distance.distancefunction.minkowski.EuclideanDistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.AbstractParameterizer;
@@ -67,12 +68,12 @@ public class EpsilonNeighborPredicate<O, D extends Distance<D>> implements Neigh
   /**
    * Range to query with
    */
-  D epsilon;
+  protected D epsilon;
 
   /**
    * Distance function to use
    */
-  DistanceFunction<O, D> distFunc;
+  protected DistanceFunction<O, D> distFunc;
 
   /**
    * Full constructor.
@@ -91,7 +92,7 @@ public class EpsilonNeighborPredicate<O, D extends Distance<D>> implements Neigh
   public <T> NeighborPredicate.Instance<T> instantiate(Database database, SimpleTypeInformation<?> type) {
     DistanceQuery<O, D> dq = QueryUtil.getDistanceQuery(database, distFunc);
     RangeQuery<O, D> rq = database.getRangeQuery(dq);
-    return (NeighborPredicate.Instance<T>) new Instance<D>(epsilon, rq, dq.getRelation().getDBIDs());
+    return (NeighborPredicate.Instance<T>) new Instance<>(epsilon, rq, dq.getRelation().getDBIDs());
   }
 
   @Override
@@ -109,7 +110,7 @@ public class EpsilonNeighborPredicate<O, D extends Distance<D>> implements Neigh
    * 
    * @author Erich Schubert
    */
-  public static class Instance<D extends Distance<D>> implements NeighborPredicate.Instance<DistanceDBIDResult<D>> {
+  public static class Instance<D extends Distance<D>> implements NeighborPredicate.Instance<DistanceDBIDList<D>> {
     /**
      * Range to query with
      */
@@ -145,12 +146,12 @@ public class EpsilonNeighborPredicate<O, D extends Distance<D>> implements Neigh
     }
 
     @Override
-    public DistanceDBIDResult<D> getNeighbors(DBIDRef reference) {
+    public DistanceDBIDList<D> getNeighbors(DBIDRef reference) {
       return rq.getRangeForDBID(reference, epsilon);
     }
 
     @Override
-    public void addDBIDs(ModifiableDBIDs ids, DistanceDBIDResult<D> neighbors) {
+    public void addDBIDs(ModifiableDBIDs ids, DistanceDBIDList<D> neighbors) {
       ids.addDBIDs(neighbors);
     }
   }
@@ -177,14 +178,14 @@ public class EpsilonNeighborPredicate<O, D extends Distance<D>> implements Neigh
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
       // Get a distance function.
-      ObjectParameter<DistanceFunction<O, D>> distanceP = new ObjectParameter<DistanceFunction<O, D>>(AbstractDistanceBasedAlgorithm.DISTANCE_FUNCTION_ID, DistanceFunction.class, EuclideanDistanceFunction.class);
+      ObjectParameter<DistanceFunction<O, D>> distanceP = new ObjectParameter<>(DistanceBasedAlgorithm.DISTANCE_FUNCTION_ID, DistanceFunction.class, EuclideanDistanceFunction.class);
       D distanceFactory = null;
       if(config.grab(distanceP)) {
         distfun = distanceP.instantiateClass(config);
         distanceFactory = distfun.getDistanceFactory();
       }
       // Get the epsilon parameter
-      DistanceParameter<D> epsilonP = new DistanceParameter<D>(de.lmu.ifi.dbs.elki.algorithm.clustering.DBSCAN.EPSILON_ID, distanceFactory);
+      DistanceParameter<D> epsilonP = new DistanceParameter<>(DBSCAN.Parameterizer.EPSILON_ID, distanceFactory);
       if(config.grab(epsilonP)) {
         epsilon = epsilonP.getValue();
       }
@@ -192,7 +193,7 @@ public class EpsilonNeighborPredicate<O, D extends Distance<D>> implements Neigh
 
     @Override
     protected EpsilonNeighborPredicate<O, D> makeInstance() {
-      return new EpsilonNeighborPredicate<O, D>(epsilon, distfun);
+      return new EpsilonNeighborPredicate<>(epsilon, distfun);
     }
   }
 }
