@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.algorithm.clustering.kmeans;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -35,6 +35,7 @@ import de.lmu.ifi.dbs.elki.data.model.MeanModel;
 import de.lmu.ifi.dbs.elki.data.type.CombinedTypeInformation;
 import de.lmu.ifi.dbs.elki.data.type.TypeInformation;
 import de.lmu.ifi.dbs.elki.data.type.TypeUtil;
+import de.lmu.ifi.dbs.elki.database.datastore.WritableIntegerDataStore;
 import de.lmu.ifi.dbs.elki.database.ids.ArrayModifiableDBIDs;
 import de.lmu.ifi.dbs.elki.database.ids.DBID;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
@@ -43,9 +44,16 @@ import de.lmu.ifi.dbs.elki.database.ids.ModifiableDBIDs;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.PrimitiveDistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.PrimitiveDoubleDistanceFunction;
+import de.lmu.ifi.dbs.elki.distance.distancefunction.minkowski.EuclideanDistanceFunction;
+import de.lmu.ifi.dbs.elki.distance.distancefunction.minkowski.SquaredEuclideanDistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
+import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.Vector;
 import de.lmu.ifi.dbs.elki.utilities.datastructures.QuickSelect;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.CommonConstraints;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntParameter;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.ObjectParameter;
 
 /**
  * Abstract base class for k-means implementations.
@@ -59,7 +67,7 @@ import de.lmu.ifi.dbs.elki.utilities.datastructures.QuickSelect;
  * @param <D> Distance type
  * @param <M> Cluster model type
  */
-public abstract class AbstractKMeans<V extends NumberVector<?>, D extends Distance<D>, M extends MeanModel<V>> extends AbstractPrimitiveDistanceBasedAlgorithm<NumberVector<?>, D, Clustering<M>> implements KMeans, ClusteringAlgorithm<Clustering<M>> {
+public abstract class AbstractKMeans<V extends NumberVector<?>, D extends Distance<D>, M extends MeanModel<V>> extends AbstractPrimitiveDistanceBasedAlgorithm<NumberVector<?>, D, Clustering<M>> implements KMeans<V, D, M>, ClusteringAlgorithm<Clustering<M>> {
   /**
    * Holds the value of {@link #K_ID}.
    */
@@ -97,9 +105,10 @@ public abstract class AbstractKMeans<V extends NumberVector<?>, D extends Distan
    * @param relation the database to cluster
    * @param means a list of k means
    * @param clusters cluster assignment
+   * @param assignment Current cluster assignment
    * @return true when the object was reassigned
    */
-  protected boolean assignToNearestCluster(Relation<V> relation, List<? extends NumberVector<?>> means, List<? extends ModifiableDBIDs> clusters) {
+  protected boolean assignToNearestCluster(Relation<V> relation, List<? extends NumberVector<?>> means, List<? extends ModifiableDBIDs> clusters, WritableIntegerDataStore assignment) {
     boolean changed = false;
 
     if(getDistanceFunction() instanceof PrimitiveDoubleDistanceFunction) {
@@ -116,18 +125,7 @@ public abstract class AbstractKMeans<V extends NumberVector<?>, D extends Distan
             mindist = dist;
           }
         }
-        if(clusters.get(minIndex).add(iditer)) {
-          changed = true;
-          // Remove from previous cluster
-          // TODO: keep a list of cluster assignments to save this search?
-          for(int i = 0; i < k; i++) {
-            if(i != minIndex) {
-              if(clusters.get(i).remove(iditer)) {
-                break;
-              }
-            }
-          }
-        }
+        changed |= updateAssignment(iditer, clusters, assignment, minIndex);
       }
     }
     else {
@@ -143,23 +141,25 @@ public abstract class AbstractKMeans<V extends NumberVector<?>, D extends Distan
             mindist = dist;
           }
         }
-        if(clusters.get(minIndex).add(iditer)) {
-          changed = true;
-          // Remove from previous cluster
-          // TODO: keep a list of cluster assignments to save this search?
-          for(int i = 0; i < k; i++) {
-            if(i != minIndex) {
-              if(clusters.get(i).remove(iditer)) {
-                break;
-              }
-            }
-          }
-        }
+        changed |= updateAssignment(iditer, clusters, assignment, minIndex);
       }
     }
     return changed;
   }
 
+  protected boolean updateAssignment(DBIDIter iditer, List<? extends ModifiableDBIDs> clusters, WritableIntegerDataStore assignment, int newA) {
+    final int oldA = assignment.intValue(iditer);
+    if(oldA == newA) {
+      return false;
+    }
+    clusters.get(newA).add(iditer);
+    assignment.putInt(iditer, newA);
+    if(oldA >= 0) {
+      clusters.get(oldA).remove(iditer);
+    }
+    return true;
+  }
+
   @Override
   public TypeInformation[] getInputTypeRestriction() {
     return TypeUtil.array(new CombinedTypeInformation(TypeUtil.NUMBER_VECTOR_FIELD, getDistanceFunction().getInputTypeRestriction()));
@@ -174,21 +174,28 @@ public abstract class AbstractKMeans<V extends NumberVector<?>, D extends Distan
    * @return the mean vectors of the given clusters in the given database
    */
   protected List<Vector> means(List<? extends ModifiableDBIDs> clusters, List<? extends NumberVector<?>> means, Relation<V> database) {
-    List<Vector> newMeans = new ArrayList<Vector>(k);
+    // TODO: use Kahan summation for better numerical precision?
+    List<Vector> newMeans = new ArrayList<>(k);
     for(int i = 0; i < k; i++) {
       ModifiableDBIDs list = clusters.get(i);
       Vector mean = null;
       if(list.size() > 0) {
-        double s = 1.0 / list.size();
         DBIDIter iter = list.iter();
-        assert (iter.valid());
-        mean = database.get(iter).getColumnVector().timesEquals(s);
+        // Initialize with first.
+        mean = database.get(iter).getColumnVector();
+        double[] raw = mean.getArrayRef();
         iter.advance();
+        // Update with remaining instances
         for(; iter.valid(); iter.advance()) {
-          mean.plusTimesEquals(database.get(iter).getColumnVector(), s);
+          NumberVector<?> vec = database.get(iter);
+          for(int j = 0; j < mean.getDimensionality(); j++) {
+            raw[j] += vec.doubleValue(j);
+          }
         }
+        mean.timesEquals(1.0 / list.size());
       }
       else {
+        // Keep degenerated means as-is for now.
         mean = means.get(i).getColumnVector();
       }
       newMeans.add(mean);
@@ -207,7 +214,7 @@ public abstract class AbstractKMeans<V extends NumberVector<?>, D extends Distan
   protected List<NumberVector<?>> medians(List<? extends ModifiableDBIDs> clusters, List<? extends NumberVector<?>> medians, Relation<V> database) {
     final int dim = medians.get(0).getDimensionality();
     final SortDBIDsBySingleDimension sorter = new SortDBIDsBySingleDimension(database);
-    List<NumberVector<?>> newMedians = new ArrayList<NumberVector<?>>(k);
+    List<NumberVector<?>> newMedians = new ArrayList<>(k);
     for(int i = 0; i < k; i++) {
       ArrayModifiableDBIDs list = DBIDUtil.newArray(clusters.get(i));
       if(list.size() > 0) {
@@ -238,11 +245,8 @@ public abstract class AbstractKMeans<V extends NumberVector<?>, D extends Distan
     if(newsize == 0) {
       return; // Keep old mean
     }
-    Vector delta = vec.getColumnVector();
-    // Compute difference from mean
-    delta.minusEquals(mean);
-    delta.timesEquals(op / newsize);
-    mean.plusEquals(delta);
+    Vector delta = vec.getColumnVector().minusEquals(mean);
+    mean.plusTimesEquals(delta, op / newsize);
   }
 
   /**
@@ -251,9 +255,10 @@ public abstract class AbstractKMeans<V extends NumberVector<?>, D extends Distan
    * @param relation Relation
    * @param means Means
    * @param clusters Clusters
+   * @param assignment Current cluster assignment
    * @return true when the means have changed
    */
-  protected boolean macQueenIterate(Relation<V> relation, List<Vector> means, List<ModifiableDBIDs> clusters) {
+  protected boolean macQueenIterate(Relation<V> relation, List<Vector> means, List<ModifiableDBIDs> clusters, WritableIntegerDataStore assignment) {
     boolean changed = false;
 
     if(getDistanceFunction() instanceof PrimitiveDoubleDistanceFunction) {
@@ -273,20 +278,7 @@ public abstract class AbstractKMeans<V extends NumberVector<?>, D extends Distan
             mindist = dist;
           }
         }
-        // Update the cluster mean incrementally:
-        for(int i = 0; i < k; i++) {
-          ModifiableDBIDs ci = clusters.get(i);
-          if(i == minIndex) {
-            if(ci.add(iditer)) {
-              incrementalUpdateMean(means.get(i), fv, ci.size(), +1);
-              changed = true;
-            }
-          }
-          else if(ci.remove(iditer)) {
-            incrementalUpdateMean(means.get(i), fv, ci.size() + 1, -1);
-            changed = true;
-          }
-        }
+        changed |= updateMeanAndAssignment(clusters, means, minIndex, fv, iditer, assignment);
       }
     }
     else {
@@ -305,22 +297,111 @@ public abstract class AbstractKMeans<V extends NumberVector<?>, D extends Distan
             mindist = dist;
           }
         }
-        // Update the cluster mean incrementally:
-        for(int i = 0; i < k; i++) {
-          ModifiableDBIDs ci = clusters.get(i);
-          if(i == minIndex) {
-            if(ci.add(iditer)) {
-              incrementalUpdateMean(means.get(i), fv, ci.size(), +1);
-              changed = true;
-            }
-          }
-          else if(ci.remove(iditer)) {
-            incrementalUpdateMean(means.get(i), fv, ci.size() + 1, -1);
-            changed = true;
-          }
-        }
+        changed |= updateMeanAndAssignment(clusters, means, minIndex, fv, iditer, assignment);
       }
     }
     return changed;
   }
+
+  /**
+   * Try to update the cluster assignment.
+   * 
+   * @param clusters Current clusters
+   * @param means Means to update
+   * @param minIndex Cluster to assign to
+   * @param fv Vector
+   * @param iditer Object ID
+   * @param assignment Current cluster assignment
+   * @return {@code true} when assignment changed
+   */
+  private boolean updateMeanAndAssignment(List<ModifiableDBIDs> clusters, List<Vector> means, int minIndex, V fv, DBIDIter iditer, WritableIntegerDataStore assignment) {
+    int cur = assignment.intValue(iditer);
+    if(cur == minIndex) {
+      return false;
+    }
+    final ModifiableDBIDs curclus = clusters.get(minIndex);
+    curclus.add(iditer);
+    incrementalUpdateMean(means.get(minIndex), fv, curclus.size(), +1);
+
+    if(cur >= 0) {
+      ModifiableDBIDs ci = clusters.get(cur);
+      ci.remove(iditer);
+      incrementalUpdateMean(means.get(cur), fv, ci.size() + 1, -1);
+    }
+
+    assignment.putInt(iditer, minIndex);
+    return true;
+  }
+
+  @Override
+  public void setK(int k) {
+    this.k = k;
+  }
+
+  @Override
+  public void setDistanceFunction(PrimitiveDistanceFunction<? super NumberVector<?>, D> distanceFunction) {
+    this.distanceFunction = distanceFunction;
+  }
+
+  /**
+   * Parameterization class.
+   * 
+   * @author Erich Schubert
+   * 
+   * @apiviz.exclude
+   */
+  public abstract static class Parameterizer<V extends NumberVector<?>, D extends Distance<D>> extends AbstractPrimitiveDistanceBasedAlgorithm.Parameterizer<NumberVector<?>, D> {
+    /**
+     * k Parameter.
+     */
+    protected int k;
+
+    /**
+     * Maximum number of iterations.
+     */
+    protected int maxiter;
+
+    /**
+     * Initialization method.
+     */
+    protected KMeansInitialization<V> initializer;
+
+    @Override
+    protected void makeOptions(Parameterization config) {
+      ObjectParameter<PrimitiveDistanceFunction<NumberVector<?>, D>> distanceFunctionP = makeParameterDistanceFunction(SquaredEuclideanDistanceFunction.class, PrimitiveDistanceFunction.class);
+      if(config.grab(distanceFunctionP)) {
+        distanceFunction = distanceFunctionP.instantiateClass(config);
+        if(!(distanceFunction instanceof EuclideanDistanceFunction) && !(distanceFunction instanceof SquaredEuclideanDistanceFunction)) {
+          getLogger().warning("k-means optimizes the sum of squares - it should be used with squared euclidean distance and may stop converging otherwise!");
+        }
+      }
+
+      IntParameter kP = new IntParameter(K_ID);
+      kP.addConstraint(CommonConstraints.GREATER_EQUAL_ONE_INT);
+      if(config.grab(kP)) {
+        k = kP.getValue();
+      }
+
+      ObjectParameter<KMeansInitialization<V>> initialP = new ObjectParameter<>(INIT_ID, KMeansInitialization.class, RandomlyChosenInitialMeans.class);
+      if(config.grab(initialP)) {
+        initializer = initialP.instantiateClass(config);
+      }
+
+      IntParameter maxiterP = new IntParameter(MAXITER_ID, 0);
+      maxiterP.addConstraint(CommonConstraints.GREATER_EQUAL_ZERO_INT);
+      if(config.grab(maxiterP)) {
+        maxiter = maxiterP.getValue();
+      }
+    }
+
+    /**
+     * Get class logger.
+     * 
+     * @return Logger
+     */
+    abstract protected Logging getLogger();
+
+    @Override
+    abstract protected AbstractKMeans<V, D, ?> makeInstance();
+  }
 }
