@@ -1,12 +1,10 @@
 package de.lmu.ifi.dbs.elki.algorithm.clustering.kmeans;
 
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
-
 /*
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -25,12 +23,27 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+import de.lmu.ifi.dbs.elki.algorithm.DistanceBasedAlgorithm;
+import de.lmu.ifi.dbs.elki.algorithm.clustering.ClusteringAlgorithm;
+import de.lmu.ifi.dbs.elki.data.Clustering;
+import de.lmu.ifi.dbs.elki.data.NumberVector;
+import de.lmu.ifi.dbs.elki.data.model.MeanModel;
+import de.lmu.ifi.dbs.elki.database.Database;
+import de.lmu.ifi.dbs.elki.database.relation.Relation;
+import de.lmu.ifi.dbs.elki.distance.distancefunction.PrimitiveDistanceFunction;
+import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
+
 /**
  * Some constants and options shared among kmeans family algorithms.
  * 
  * @author Erich Schubert
+ * 
+ * @param <V> Number vector type
+ * @param <D> Distance type
+ * @param <M> Actual model type
  */
-public interface KMeans {
+public interface KMeans<V extends NumberVector<?>, D extends Distance<?>, M extends MeanModel<V>> extends ClusteringAlgorithm<Clustering<M>>, DistanceBasedAlgorithm<V, D> {
   /**
    * Parameter to specify the initialization method
    */
@@ -52,4 +65,27 @@ public interface KMeans {
    * Parameter to specify the random generator seed.
    */
   public static final OptionID SEED_ID = new OptionID("kmeans.seed", "The random number generator seed.");
-}
\ No newline at end of file
+
+  /**
+   * Run the clustering algorithm.
+   * 
+   * @param database Database to run on.
+   * @param rel Relation to process.
+   * @return Clustering result
+   */
+  Clustering<M> run(Database database, Relation<V> rel);
+
+  /**
+   * Set the value of k. Needed for some types of nested k-means.
+   * 
+   * @param k K parameter
+   */
+  void setK(int k);
+  
+  /**
+   * Set the distance function to use.
+   * 
+   * @param distanceFunction Distance function.
+   */
+  void setDistanceFunction(PrimitiveDistanceFunction<? super NumberVector<?>, D> distanceFunction);
+}
