@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.algorithm.clustering.kmeans;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -26,6 +26,8 @@ import java.util.ArrayList;
 import java.util.List;
 import java.util.Random;
 
+import de.lmu.ifi.dbs.elki.data.NumberVector;
+import de.lmu.ifi.dbs.elki.database.Database;
 import de.lmu.ifi.dbs.elki.database.ids.ArrayDBIDs;
 import de.lmu.ifi.dbs.elki.database.ids.ArrayModifiableDBIDs;
 import de.lmu.ifi.dbs.elki.database.ids.DBID;
@@ -70,20 +72,20 @@ public class KMeansPlusPlusInitialMeans<V, D extends NumberDistance<D, ?>> exten
   }
 
   @Override
-  public List<V> chooseInitialMeans(Relation<V> relation, int k, PrimitiveDistanceFunction<? super V, ?> distanceFunction) {
+  public List<V> chooseInitialMeans(Database database, Relation<V> relation, int k, PrimitiveDistanceFunction<? super NumberVector<?>, ?> distanceFunction) {
     // Get a distance query
     if(!(distanceFunction.getDistanceFactory() instanceof NumberDistance)) {
       throw new AbortException("K-Means++ initialization can only be used with numerical distances.");
     }
     @SuppressWarnings("unchecked")
     final PrimitiveDistanceFunction<? super V, D> distF = (PrimitiveDistanceFunction<? super V, D>) distanceFunction;
-    DistanceQuery<V, D> distQ = relation.getDatabase().getDistanceQuery(relation, distF);
+    DistanceQuery<V, D> distQ = database.getDistanceQuery(relation, distF);
 
     // Chose first mean
-    List<V> means = new ArrayList<V>(k);
+    List<V> means = new ArrayList<>(k);
 
-    Random random = rnd.getRandom();
-    DBID first = DBIDUtil.deref(DBIDUtil.randomSample(relation.getDBIDs(), 1, new Random(random.nextLong())).iter());
+    Random random = rnd.getSingleThreadedRandom();
+    DBID first = DBIDUtil.deref(DBIDUtil.randomSample(relation.getDBIDs(), 1, random).iter());
     means.add(relation.get(first));
 
     ArrayDBIDs ids = DBIDUtil.ensureArray(relation.getDBIDs());
@@ -99,7 +101,7 @@ public class KMeansPlusPlusInitialMeans<V, D extends NumberDistance<D, ?>> exten
       }
       double r = random.nextDouble() * weightsum;
       int pos = 0;
-      while(r > 0 && pos < weights.length) {
+      while(r > 0 && pos < weights.length - 1) {
         r -= weights[pos];
         pos++;
       }
@@ -125,15 +127,15 @@ public class KMeansPlusPlusInitialMeans<V, D extends NumberDistance<D, ?>> exten
   @Override
   public DBIDs chooseInitialMedoids(int k, DistanceQuery<? super V, ?> distQ2) {
     if(!(distQ2.getDistanceFactory() instanceof NumberDistance)) {
-      throw new AbortException("PAM initialization can only be used with numerical distances.");
+      throw new AbortException("K-Means++ initialization initialization can only be used with numerical distances.");
     }
     @SuppressWarnings("unchecked")
     DistanceQuery<? super V, D> distQ = (DistanceQuery<? super V, D>) distQ2;
     // Chose first mean
     ArrayModifiableDBIDs means = DBIDUtil.newArray(k);
 
-    Random random = rnd.getRandom();
-    DBID first = DBIDUtil.deref(DBIDUtil.randomSample(distQ.getRelation().getDBIDs(), 1, new Random(random.nextLong())).iter());
+    Random random = rnd.getSingleThreadedRandom();
+    DBID first = DBIDUtil.deref(DBIDUtil.randomSample(distQ.getRelation().getDBIDs(), 1, random).iter());
     means.add(first);
 
     ArrayDBIDs ids = DBIDUtil.ensureArray(distQ.getRelation().getDBIDs());
@@ -244,7 +246,7 @@ public class KMeansPlusPlusInitialMeans<V, D extends NumberDistance<D, ?>> exten
   public static class Parameterizer<V, D extends NumberDistance<D, ?>> extends AbstractKMeansInitialization.Parameterizer<V> {
     @Override
     protected KMeansPlusPlusInitialMeans<V, D> makeInstance() {
-      return new KMeansPlusPlusInitialMeans<V, D>(rnd);
+      return new KMeansPlusPlusInitialMeans<>(rnd);
     }
   }
 }
\ No newline at end of file
