@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.algorithm.clustering.kmeans;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -46,9 +46,9 @@ import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.PrimitiveDistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.NumberDistance;
 import de.lmu.ifi.dbs.elki.logging.Logging;
+import de.lmu.ifi.dbs.elki.logging.progress.IndefiniteProgress;
 import de.lmu.ifi.dbs.elki.math.Mean;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.GreaterConstraint;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.GreaterEqualConstraint;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.CommonConstraints;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntParameter;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.ObjectParameter;
@@ -118,15 +118,15 @@ public class KMedoidsEM<V, D extends NumberDistance<D, ?>> extends AbstractDista
    * @return result
    */
   public Clustering<MedoidModel> run(Database database, Relation<V> relation) {
-    if (relation.size() <= 0) {
-      return new Clustering<MedoidModel>("k-Medoids Clustering", "kmedoids-clustering");
+    if(relation.size() <= 0) {
+      return new Clustering<>("k-Medoids Clustering", "kmedoids-clustering");
     }
     DistanceQuery<V, D> distQ = database.getDistanceQuery(relation, getDistanceFunction());
     // Choose initial medoids
     ArrayModifiableDBIDs medoids = DBIDUtil.newArray(initializer.chooseInitialMedoids(k, distQ));
     // Setup cluster assignment store
-    List<ModifiableDBIDs> clusters = new ArrayList<ModifiableDBIDs>();
-    for (int i = 0; i < k; i++) {
+    List<ModifiableDBIDs> clusters = new ArrayList<>();
+    for(int i = 0; i < k; i++) {
       clusters.add(DBIDUtil.newHashSet(relation.size() / k));
     }
     Mean[] mdists = Mean.newArray(k);
@@ -135,45 +135,52 @@ public class KMedoidsEM<V, D extends NumberDistance<D, ?>> extends AbstractDista
     // TODO: reuse this information, from the build phase, when possible?
     assignToNearestCluster(medoids, mdists, clusters, distQ);
 
+    IndefiniteProgress prog = LOG.isVerbose() ? new IndefiniteProgress("K-Medoids iteration", LOG) : null;
     // Swap phase
     boolean changed = true;
-    while (changed) {
+    while(changed) {
+      if(prog != null) {
+        prog.incrementProcessed(LOG);
+      }
       changed = false;
       // Try to swap the medoid with a better cluster member:
       int i = 0;
-      for (DBIDIter miter = medoids.iter(); miter.valid(); miter.advance(), i++) {
+      for(DBIDIter miter = medoids.iter(); miter.valid(); miter.advance(), i++) {
         DBID best = null;
         Mean bestm = mdists[i];
-        for (DBIDIter iter = clusters.get(i).iter(); iter.valid(); iter.advance()) {
-          if (DBIDUtil.equal(miter, iter)) {
+        for(DBIDIter iter = clusters.get(i).iter(); iter.valid(); iter.advance()) {
+          if(DBIDUtil.equal(miter, iter)) {
             continue;
           }
           Mean mdist = new Mean();
-          for (DBIDIter iter2 = clusters.get(i).iter(); iter2.valid(); iter2.advance()) {
+          for(DBIDIter iter2 = clusters.get(i).iter(); iter2.valid(); iter2.advance()) {
             mdist.put(distQ.distance(iter, iter2).doubleValue());
           }
-          if (mdist.getMean() < bestm.getMean()) {
+          if(mdist.getMean() < bestm.getMean()) {
             best = DBIDUtil.deref(iter);
             bestm = mdist;
           }
         }
-        if (best != null && !DBIDUtil.equal(miter, best)) {
+        if(best != null && !DBIDUtil.equal(miter, best)) {
           changed = true;
           medoids.set(i, best);
           mdists[i] = bestm;
         }
       }
       // Reassign
-      if (changed) {
+      if(changed) {
         assignToNearestCluster(medoids, mdists, clusters, distQ);
       }
     }
+    if(prog != null) {
+      prog.setCompleted(LOG);
+    }
 
     // Wrap result
-    Clustering<MedoidModel> result = new Clustering<MedoidModel>("k-Medoids Clustering", "kmedoids-clustering");
-    for (int i = 0; i < clusters.size(); i++) {
+    Clustering<MedoidModel> result = new Clustering<>("k-Medoids Clustering", "kmedoids-clustering");
+    for(int i = 0; i < clusters.size(); i++) {
       MedoidModel model = new MedoidModel(medoids.get(i));
-      result.addCluster(new Cluster<MedoidModel>(clusters.get(i), model));
+      result.addToplevelCluster(new Cluster<>(clusters.get(i), model));
     }
     return result;
   }
@@ -192,27 +199,27 @@ public class KMedoidsEM<V, D extends NumberDistance<D, ?>> extends AbstractDista
     boolean changed = false;
 
     double[] dists = new double[k];
-    for (DBIDIter iditer = distQ.getRelation().iterDBIDs(); iditer.valid(); iditer.advance()) {
+    for(DBIDIter iditer = distQ.getRelation().iterDBIDs(); iditer.valid(); iditer.advance()) {
       int minIndex = 0;
       double mindist = Double.POSITIVE_INFINITY;
       {
         int i = 0;
-        for (DBIDIter miter = means.iter(); miter.valid(); miter.advance(), i++) {
+        for(DBIDIter miter = means.iter(); miter.valid(); miter.advance(), i++) {
           dists[i] = distQ.distance(iditer, miter).doubleValue();
-          if (dists[i] < mindist) {
+          if(dists[i] < mindist) {
             minIndex = i;
             mindist = dists[i];
           }
         }
       }
-      if (clusters.get(minIndex).add(iditer)) {
+      if(clusters.get(minIndex).add(iditer)) {
         changed = true;
         mdist[minIndex].put(mindist);
         // Remove from previous cluster
         // TODO: keep a list of cluster assignments to save this search?
-        for (int i = 0; i < k; i++) {
-          if (i != minIndex) {
-            if (clusters.get(i).remove(iditer)) {
+        for(int i = 0; i < k; i++) {
+          if(i != minIndex) {
+            if(clusters.get(i).remove(iditer)) {
               mdist[minIndex].put(dists[i], -1);
               break;
             }
@@ -251,26 +258,26 @@ public class KMedoidsEM<V, D extends NumberDistance<D, ?>> extends AbstractDista
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
       IntParameter kP = new IntParameter(KMeans.K_ID);
-      kP.addConstraint(new GreaterConstraint(0));
-      if (config.grab(kP)) {
+      kP.addConstraint(CommonConstraints.GREATER_EQUAL_ONE_INT);
+      if(config.grab(kP)) {
         k = kP.intValue();
       }
 
-      ObjectParameter<KMedoidsInitialization<V>> initialP = new ObjectParameter<KMedoidsInitialization<V>>(KMeans.INIT_ID, KMedoidsInitialization.class, PAMInitialMeans.class);
-      if (config.grab(initialP)) {
+      ObjectParameter<KMedoidsInitialization<V>> initialP = new ObjectParameter<>(KMeans.INIT_ID, KMedoidsInitialization.class, PAMInitialMeans.class);
+      if(config.grab(initialP)) {
         initializer = initialP.instantiateClass(config);
       }
 
       IntParameter maxiterP = new IntParameter(KMeans.MAXITER_ID, 0);
-      maxiterP.addConstraint(new GreaterEqualConstraint(0));
-      if (config.grab(maxiterP)) {
+      maxiterP.addConstraint(CommonConstraints.GREATER_EQUAL_ZERO_INT);
+      if(config.grab(maxiterP)) {
         maxiter = maxiterP.intValue();
       }
     }
 
     @Override
     protected KMedoidsEM<V, D> makeInstance() {
-      return new KMedoidsEM<V, D>(distanceFunction, k, maxiter, initializer);
+      return new KMedoidsEM<>(distanceFunction, k, maxiter, initializer);
     }
   }
 }
