@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.algorithm.clustering.subspace;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -49,13 +49,13 @@ import de.lmu.ifi.dbs.elki.database.ids.DBID;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
-import de.lmu.ifi.dbs.elki.database.ids.DistanceDBIDPair;
 import de.lmu.ifi.dbs.elki.database.ids.ModifiableDBIDs;
+import de.lmu.ifi.dbs.elki.database.ids.distance.DistanceDBIDList;
+import de.lmu.ifi.dbs.elki.database.ids.distance.DistanceDBIDPair;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.database.query.range.RangeQuery;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.database.relation.RelationUtil;
-import de.lmu.ifi.dbs.elki.distance.distanceresultlist.DistanceDBIDResult;
 import de.lmu.ifi.dbs.elki.distance.distanceresultlist.DistanceDBIDResultUtil;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.DoubleDistance;
 import de.lmu.ifi.dbs.elki.logging.Logging;
@@ -67,7 +67,7 @@ import de.lmu.ifi.dbs.elki.utilities.documentation.Description;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Title;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.GreaterConstraint;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.CommonConstraints;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntParameter;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.RandomParameter;
@@ -148,15 +148,15 @@ public class PROCLUS<V extends NumberVector<?>> extends AbstractProjectedCluster
   public Clustering<SubspaceModel<V>> run(Database database, Relation<V> relation) {
     DistanceQuery<V, DoubleDistance> distFunc = this.getDistanceQuery(database);
     RangeQuery<V, DoubleDistance> rangeQuery = database.getRangeQuery(distFunc);
-    final Random random = rnd.getRandom();
+    final Random random = rnd.getSingleThreadedRandom();
 
-    if(RelationUtil.dimensionality(relation) < l) {
+    if (RelationUtil.dimensionality(relation) < l) {
       throw new IllegalStateException("Dimensionality of data < parameter l! " + "(" + RelationUtil.dimensionality(relation) + " < " + l + ")");
     }
 
     // TODO: use a StepProgress!
     // initialization phase
-    if(LOG.isVerbose()) {
+    if (LOG.isVerbose()) {
       LOG.verbose("1. Initialization phase...");
     }
     int sampleSize = Math.min(relation.size(), k_i * k);
@@ -165,7 +165,7 @@ public class PROCLUS<V extends NumberVector<?>> extends AbstractProjectedCluster
     int medoidSize = Math.min(relation.size(), m_i * k);
     DBIDs medoids = greedy(distFunc, sampleSet, medoidSize, random);
 
-    if(LOG.isDebugging()) {
+    if (LOG.isDebugging()) {
       StringBuilder msg = new StringBuilder();
       msg.append('\n');
       msg.append("sampleSize ").append(sampleSize).append('\n');
@@ -176,7 +176,7 @@ public class PROCLUS<V extends NumberVector<?>> extends AbstractProjectedCluster
     }
 
     // iterative phase
-    if(LOG.isVerbose()) {
+    if (LOG.isVerbose()) {
       LOG.verbose("2. Iterative phase...");
     }
     double bestObjective = Double.POSITIVE_INFINITY;
@@ -184,7 +184,7 @@ public class PROCLUS<V extends NumberVector<?>> extends AbstractProjectedCluster
     ModifiableDBIDs m_bad = null;
     ModifiableDBIDs m_current = initialSet(medoids, k, random);
 
-    if(LOG.isDebugging()) {
+    if (LOG.isDebugging()) {
       StringBuilder msg = new StringBuilder();
       msg.append('\n');
       msg.append("m_c ").append(m_current).append('\n');
@@ -196,12 +196,12 @@ public class PROCLUS<V extends NumberVector<?>> extends AbstractProjectedCluster
     // TODO: Use DataStore and Trove for performance
     Map<DBID, PROCLUSCluster> clusters = null;
     int loops = 0;
-    while(loops < 10) {
+    while (loops < 10) {
       Map<DBID, TIntSet> dimensions = findDimensions(m_current, relation, distFunc, rangeQuery);
       clusters = assignPoints(dimensions, relation);
       double objectiveFunction = evaluateClusters(clusters, dimensions, relation);
 
-      if(objectiveFunction < bestObjective) {
+      if (objectiveFunction < bestObjective) {
         // restart counting loops
         loops = 0;
         bestObjective = objectiveFunction;
@@ -211,32 +211,32 @@ public class PROCLUS<V extends NumberVector<?>> extends AbstractProjectedCluster
 
       m_current = computeM_current(medoids, m_best, m_bad, random);
       loops++;
-      if(cprogress != null) {
+      if (cprogress != null) {
         cprogress.setProcessed(clusters.size(), LOG);
       }
     }
 
-    if(cprogress != null) {
+    if (cprogress != null) {
       cprogress.setCompleted(LOG);
     }
 
     // refinement phase
-    if(LOG.isVerbose()) {
+    if (LOG.isVerbose()) {
       LOG.verbose("3. Refinement phase...");
     }
 
-    List<Pair<V, TIntSet>> dimensions = findDimensions(new ArrayList<PROCLUSCluster>(clusters.values()), relation);
+    List<Pair<V, TIntSet>> dimensions = findDimensions(new ArrayList<>(clusters.values()), relation);
     List<PROCLUSCluster> finalClusters = finalAssignment(dimensions, relation);
 
     // build result
     int numClusters = 1;
-    Clustering<SubspaceModel<V>> result = new Clustering<SubspaceModel<V>>("ProClus clustering", "proclus-clustering");
-    for(PROCLUSCluster c : finalClusters) {
-      Cluster<SubspaceModel<V>> cluster = new Cluster<SubspaceModel<V>>(c.objectIDs);
-      cluster.setModel(new SubspaceModel<V>(new Subspace(c.getDimensions()), c.centroid));
+    Clustering<SubspaceModel<V>> result = new Clustering<>("ProClus clustering", "proclus-clustering");
+    for (PROCLUSCluster c : finalClusters) {
+      Cluster<SubspaceModel<V>> cluster = new Cluster<>(c.objectIDs);
+      cluster.setModel(new SubspaceModel<>(new Subspace(c.getDimensions()), c.centroid));
       cluster.setName("cluster_" + numClusters++);
 
-      result.addCluster(cluster);
+      result.addToplevelCluster(cluster);
     }
     return result;
   }
@@ -257,22 +257,22 @@ public class PROCLUS<V extends NumberVector<?>> extends AbstractProjectedCluster
     // m_1 is random point of S
     DBID m_i = s.remove(random.nextInt(s.size()));
     medoids.add(m_i);
-    if(LOG.isDebugging()) {
+    if (LOG.isDebugging()) {
       LOG.debugFiner("medoids " + medoids);
     }
 
     // compute distances between each point in S and m_i
     // FIXME: don't use maps, so we can work with DBIDRef
-    Map<DBID, DistanceDBIDPair<DoubleDistance>> distances = new HashMap<DBID, DistanceDBIDPair<DoubleDistance>>();
-    for(DBIDIter iter = s.iter(); iter.valid(); iter.advance()) {
+    Map<DBID, DistanceDBIDPair<DoubleDistance>> distances = new HashMap<>();
+    for (DBIDIter iter = s.iter(); iter.valid(); iter.advance()) {
       DBID id = DBIDUtil.deref(iter);
       DoubleDistance dist = distFunc.distance(id, m_i);
       distances.put(id, DBIDUtil.newDistancePair(dist, id));
     }
 
-    for(int i = 1; i < m; i++) {
-      // choose medoid m_i to be far from prevois medoids
-      List<DistanceDBIDPair<DoubleDistance>> d = new ArrayList<DistanceDBIDPair<DoubleDistance>>(distances.values());
+    for (int i = 1; i < m; i++) {
+      // choose medoid m_i to be far from previous medoids
+      List<DistanceDBIDPair<DoubleDistance>> d = new ArrayList<>(distances.values());
       DistanceDBIDResultUtil.sortByDistance(d);
 
       m_i = DBIDUtil.deref(d.get(d.size() - 1));
@@ -281,7 +281,7 @@ public class PROCLUS<V extends NumberVector<?>> extends AbstractProjectedCluster
       distances.remove(m_i);
 
       // compute distances of each point to closest medoid
-      for(DBIDIter iter = s.iter(); iter.valid(); iter.advance()) {
+      for (DBIDIter iter = s.iter(); iter.valid(); iter.advance()) {
         DBID id = DBIDUtil.deref(iter);
         DoubleDistance dist_new = distFunc.distance(id, m_i);
         DoubleDistance dist_old = distances.get(id).getDistance();
@@ -290,7 +290,7 @@ public class PROCLUS<V extends NumberVector<?>> extends AbstractProjectedCluster
         distances.put(id, DBIDUtil.newDistancePair(dist, id));
       }
 
-      if(LOG.isDebugging()) {
+      if (LOG.isDebugging()) {
         LOG.debugFiner("medoids " + medoids);
       }
     }
@@ -309,7 +309,7 @@ public class PROCLUS<V extends NumberVector<?>> extends AbstractProjectedCluster
   private ModifiableDBIDs initialSet(DBIDs sampleSet, int k, Random random) {
     ArrayModifiableDBIDs s = DBIDUtil.newArray(sampleSet);
     ModifiableDBIDs initialSet = DBIDUtil.newHashSet();
-    while(initialSet.size() < k) {
+    while (initialSet.size() < k) {
       DBID next = s.remove(random.nextInt(s.size()));
       initialSet.add(next);
     }
@@ -330,16 +330,15 @@ public class PROCLUS<V extends NumberVector<?>> extends AbstractProjectedCluster
     m_list.removeDBIDs(m_best);
 
     ModifiableDBIDs m_current = DBIDUtil.newHashSet();
-    for(DBIDIter iter = m_best.iter(); iter.valid(); iter.advance()) {
+    for (DBIDIter iter = m_best.iter(); iter.valid(); iter.advance()) {
       DBID m_i = DBIDUtil.deref(iter);
-      if(m_bad.contains(m_i)) {
+      if (m_bad.contains(m_i)) {
         int currentSize = m_current.size();
-        while(m_current.size() == currentSize) {
+        while (m_current.size() == currentSize) {
           DBID next = m_list.remove(random.nextInt(m_list.size()));
           m_current.add(next);
         }
-      }
-      else {
+      } else {
         m_current.add(m_i);
       }
     }
@@ -358,28 +357,28 @@ public class PROCLUS<V extends NumberVector<?>> extends AbstractProjectedCluster
    * @param distFunc the distance function
    * @return a mapping of the medoid's id to its locality
    */
-  private Map<DBID, DistanceDBIDResult<DoubleDistance>> getLocalities(DBIDs medoids, Relation<V> database, DistanceQuery<V, DoubleDistance> distFunc, RangeQuery<V, DoubleDistance> rangeQuery) {
-    Map<DBID, DistanceDBIDResult<DoubleDistance>> result = new HashMap<DBID, DistanceDBIDResult<DoubleDistance>>();
+  private Map<DBID, DistanceDBIDList<DoubleDistance>> getLocalities(DBIDs medoids, Relation<V> database, DistanceQuery<V, DoubleDistance> distFunc, RangeQuery<V, DoubleDistance> rangeQuery) {
+    Map<DBID, DistanceDBIDList<DoubleDistance>> result = new HashMap<>();
 
-    for(DBIDIter iter = medoids.iter(); iter.valid(); iter.advance()) {
+    for (DBIDIter iter = medoids.iter(); iter.valid(); iter.advance()) {
       DBID m = DBIDUtil.deref(iter);
       // determine minimum distance between current medoid m and any other
       // medoid m_i
       DoubleDistance minDist = null;
-      for(DBIDIter iter2 = medoids.iter(); iter2.valid(); iter2.advance()) {
+      for (DBIDIter iter2 = medoids.iter(); iter2.valid(); iter2.advance()) {
         DBID m_i = DBIDUtil.deref(iter2);
-        if(DBIDUtil.equal(m_i, m)) {
+        if (DBIDUtil.equal(m_i, m)) {
           continue;
         }
         DoubleDistance currentDist = distFunc.distance(m, m_i);
-        if(minDist == null || currentDist.compareTo(minDist) < 0) {
+        if (minDist == null || currentDist.compareTo(minDist) < 0) {
           minDist = currentDist;
         }
       }
 
       // determine points in sphere centered at m with radius minDist
       assert minDist != null;
-      DistanceDBIDResult<DoubleDistance> qr = rangeQuery.getRangeForDBID(m, minDist);
+      DistanceDBIDList<DoubleDistance> qr = rangeQuery.getRangeForDBID(m, minDist);
       result.put(m, qr);
     }
 
@@ -398,32 +397,32 @@ public class PROCLUS<V extends NumberVector<?>> extends AbstractProjectedCluster
    */
   private Map<DBID, TIntSet> findDimensions(DBIDs medoids, Relation<V> database, DistanceQuery<V, DoubleDistance> distFunc, RangeQuery<V, DoubleDistance> rangeQuery) {
     // get localities
-    Map<DBID, DistanceDBIDResult<DoubleDistance>> localities = getLocalities(medoids, database, distFunc, rangeQuery);
+    Map<DBID, DistanceDBIDList<DoubleDistance>> localities = getLocalities(medoids, database, distFunc, rangeQuery);
 
     // compute x_ij = avg distance from points in l_i to medoid m_i
     int dim = RelationUtil.dimensionality(database);
-    Map<DBID, double[]> averageDistances = new HashMap<DBID, double[]>();
+    Map<DBID, double[]> averageDistances = new HashMap<>();
 
-    for(DBIDIter iter = medoids.iter(); iter.valid(); iter.advance()) {
+    for (DBIDIter iter = medoids.iter(); iter.valid(); iter.advance()) {
       DBID m_i = DBIDUtil.deref(iter);
       V medoid_i = database.get(m_i);
-      DistanceDBIDResult<DoubleDistance> l_i = localities.get(m_i);
+      DistanceDBIDList<DoubleDistance> l_i = localities.get(m_i);
       double[] x_i = new double[dim];
-      for(DBIDIter qr = l_i.iter(); qr.valid(); qr.advance()) {
+      for (DBIDIter qr = l_i.iter(); qr.valid(); qr.advance()) {
         V o = database.get(qr);
-        for(int d = 0; d < dim; d++) {
+        for (int d = 0; d < dim; d++) {
           x_i[d] += Math.abs(medoid_i.doubleValue(d) - o.doubleValue(d));
         }
       }
-      for(int d = 0; d < dim; d++) {
+      for (int d = 0; d < dim; d++) {
         x_i[d] /= l_i.size();
       }
       averageDistances.put(m_i, x_i);
     }
 
-    Map<DBID, TIntSet> dimensionMap = new HashMap<DBID, TIntSet>();
-    List<CTriple<Double, DBID, Integer>> z_ijs = new ArrayList<CTriple<Double, DBID, Integer>>();
-    for(DBIDIter iter = medoids.iter(); iter.valid(); iter.advance()) {
+    Map<DBID, TIntSet> dimensionMap = new HashMap<>();
+    List<CTriple<Double, DBID, Integer>> z_ijs = new ArrayList<>();
+    for (DBIDIter iter = medoids.iter(); iter.valid(); iter.advance()) {
       DBID m_i = DBIDUtil.deref(iter);
       TIntSet dims_i = new TIntHashSet();
       dimensionMap.put(m_i, dims_i);
@@ -431,33 +430,33 @@ public class PROCLUS<V extends NumberVector<?>> extends AbstractProjectedCluster
       double[] x_i = averageDistances.get(m_i);
       // y_i
       double y_i = 0;
-      for(int j = 0; j < dim; j++) {
+      for (int j = 0; j < dim; j++) {
         y_i += x_i[j];
       }
       y_i /= dim;
 
       // sigma_i
       double sigma_i = 0;
-      for(int j = 0; j < dim; j++) {
+      for (int j = 0; j < dim; j++) {
         double diff = x_i[j] - y_i;
         sigma_i += diff * diff;
       }
       sigma_i /= (dim - 1);
       sigma_i = Math.sqrt(sigma_i);
 
-      for(int j = 0; j < dim; j++) {
-        z_ijs.add(new CTriple<Double, DBID, Integer>((x_i[j] - y_i) / sigma_i, m_i, j));
+      for (int j = 0; j < dim; j++) {
+        z_ijs.add(new CTriple<>((x_i[j] - y_i) / sigma_i, m_i, j));
       }
     }
     Collections.sort(z_ijs);
 
     int max = Math.max(k * l, 2);
-    for(int m = 0; m < max; m++) {
+    for (int m = 0; m < max; m++) {
       CTriple<Double, DBID, Integer> z_ij = z_ijs.get(m);
       TIntSet dims_i = dimensionMap.get(z_ij.getSecond());
       dims_i.add(z_ij.getThird());
 
-      if(LOG.isDebugging()) {
+      if (LOG.isDebugging()) {
         StringBuilder msg = new StringBuilder();
         msg.append('\n');
         msg.append("z_ij ").append(z_ij).append('\n');
@@ -480,61 +479,61 @@ public class PROCLUS<V extends NumberVector<?>> extends AbstractProjectedCluster
   private List<Pair<V, TIntSet>> findDimensions(List<PROCLUSCluster> clusters, Relation<V> database) {
     // compute x_ij = avg distance from points in c_i to c_i.centroid
     int dim = RelationUtil.dimensionality(database);
-    Map<Integer, double[]> averageDistances = new HashMap<Integer, double[]>();
+    Map<Integer, double[]> averageDistances = new HashMap<>();
 
-    for(int i = 0; i < clusters.size(); i++) {
+    for (int i = 0; i < clusters.size(); i++) {
       PROCLUSCluster c_i = clusters.get(i);
       double[] x_i = new double[dim];
-      for(DBIDIter iter = c_i.objectIDs.iter(); iter.valid(); iter.advance()) {
+      for (DBIDIter iter = c_i.objectIDs.iter(); iter.valid(); iter.advance()) {
         V o = database.get(iter);
-        for(int d = 0; d < dim; d++) {
+        for (int d = 0; d < dim; d++) {
           x_i[d] += Math.abs(c_i.centroid.doubleValue(d) - o.doubleValue(d));
         }
       }
-      for(int d = 0; d < dim; d++) {
+      for (int d = 0; d < dim; d++) {
         x_i[d] /= c_i.objectIDs.size();
       }
       averageDistances.put(i, x_i);
     }
 
-    List<CTriple<Double, Integer, Integer>> z_ijs = new ArrayList<CTriple<Double, Integer, Integer>>();
-    for(int i = 0; i < clusters.size(); i++) {
+    List<CTriple<Double, Integer, Integer>> z_ijs = new ArrayList<>();
+    for (int i = 0; i < clusters.size(); i++) {
       double[] x_i = averageDistances.get(i);
       // y_i
       double y_i = 0;
-      for(int j = 0; j < dim; j++) {
+      for (int j = 0; j < dim; j++) {
         y_i += x_i[j];
       }
       y_i /= dim;
 
       // sigma_i
       double sigma_i = 0;
-      for(int j = 0; j < dim; j++) {
+      for (int j = 0; j < dim; j++) {
         double diff = x_i[j] - y_i;
         sigma_i += diff * diff;
       }
       sigma_i /= (dim - 1);
       sigma_i = Math.sqrt(sigma_i);
 
-      for(int j = 0; j < dim; j++) {
-        z_ijs.add(new CTriple<Double, Integer, Integer>((x_i[j] - y_i) / sigma_i, i, j));
+      for (int j = 0; j < dim; j++) {
+        z_ijs.add(new CTriple<>((x_i[j] - y_i) / sigma_i, i, j));
       }
     }
     Collections.sort(z_ijs);
 
     // mapping cluster index -> dimensions
-    Map<Integer, TIntSet> dimensionMap = new HashMap<Integer, TIntSet>();
+    Map<Integer, TIntSet> dimensionMap = new HashMap<>();
     int max = Math.max(k * l, 2);
-    for(int m = 0; m < max; m++) {
+    for (int m = 0; m < max; m++) {
       CTriple<Double, Integer, Integer> z_ij = z_ijs.get(m);
       TIntSet dims_i = dimensionMap.get(z_ij.getSecond());
-      if(dims_i == null) {
+      if (dims_i == null) {
         dims_i = new TIntHashSet();
         dimensionMap.put(z_ij.getSecond(), dims_i);
       }
       dims_i.add(z_ij.getThird());
 
-      if(LOG.isDebugging()) {
+      if (LOG.isDebugging()) {
         StringBuilder msg = new StringBuilder();
         msg.append('\n');
         msg.append("z_ij ").append(z_ij).append('\n');
@@ -544,11 +543,11 @@ public class PROCLUS<V extends NumberVector<?>> extends AbstractProjectedCluster
     }
 
     // mapping cluster -> dimensions
-    List<Pair<V, TIntSet>> result = new ArrayList<Pair<V, TIntSet>>();
-    for(int i : dimensionMap.keySet()) {
+    List<Pair<V, TIntSet>> result = new ArrayList<>();
+    for (int i : dimensionMap.keySet()) {
       TIntSet dims_i = dimensionMap.get(i);
       PROCLUSCluster c_i = clusters.get(i);
-      result.add(new Pair<V, TIntSet>(c_i.centroid, dims_i));
+      result.add(new Pair<>(c_i.centroid, dims_i));
     }
     return result;
   }
@@ -562,19 +561,19 @@ public class PROCLUS<V extends NumberVector<?>> extends AbstractProjectedCluster
    * @return the assignments of the object to the clusters
    */
   private Map<DBID, PROCLUSCluster> assignPoints(Map<DBID, TIntSet> dimensions, Relation<V> database) {
-    Map<DBID, ModifiableDBIDs> clusterIDs = new HashMap<DBID, ModifiableDBIDs>();
-    for(DBID m_i : dimensions.keySet()) {
+    Map<DBID, ModifiableDBIDs> clusterIDs = new HashMap<>();
+    for (DBID m_i : dimensions.keySet()) {
       clusterIDs.put(m_i, DBIDUtil.newHashSet());
     }
 
-    for(DBIDIter it = database.iterDBIDs(); it.valid(); it.advance()) {
+    for (DBIDIter it = database.iterDBIDs(); it.valid(); it.advance()) {
       DBID p_id = DBIDUtil.deref(it);
       V p = database.get(p_id);
       DistanceDBIDPair<DoubleDistance> minDist = null;
-      for(DBID m_i : dimensions.keySet()) {
+      for (DBID m_i : dimensions.keySet()) {
         V m = database.get(m_i);
         DistanceDBIDPair<DoubleDistance> currentDist = DBIDUtil.newDistancePair(manhattanSegmentalDistance(p, m, dimensions.get(m_i)), m_i);
-        if(minDist == null || currentDist.compareByDistance(minDist) < 0) {
+        if (minDist == null || currentDist.compareByDistance(minDist) < 0) {
           minDist = currentDist;
         }
       }
@@ -584,17 +583,17 @@ public class PROCLUS<V extends NumberVector<?>> extends AbstractProjectedCluster
       ids.add(p_id);
     }
 
-    Map<DBID, PROCLUSCluster> clusters = new HashMap<DBID, PROCLUSCluster>();
-    for(DBID m_i : dimensions.keySet()) {
+    Map<DBID, PROCLUSCluster> clusters = new HashMap<>();
+    for (DBID m_i : dimensions.keySet()) {
       ModifiableDBIDs objectIDs = clusterIDs.get(m_i);
-      if(!objectIDs.isEmpty()) {
+      if (!objectIDs.isEmpty()) {
         TIntSet clusterDimensions = dimensions.get(m_i);
         V centroid = Centroid.make(database, objectIDs).toVector(database);
         clusters.put(m_i, new PROCLUSCluster(objectIDs, clusterDimensions, centroid));
       }
     }
 
-    if(LOG.isDebugging()) {
+    if (LOG.isDebugging()) {
       StringBuilder msg = new StringBuilder();
       msg.append('\n');
       msg.append("clusters ").append(clusters).append('\n');
@@ -612,22 +611,22 @@ public class PROCLUS<V extends NumberVector<?>> extends AbstractProjectedCluster
    * @return the assignments of the object to the clusters
    */
   private List<PROCLUSCluster> finalAssignment(List<Pair<V, TIntSet>> dimensions, Relation<V> database) {
-    Map<Integer, ModifiableDBIDs> clusterIDs = new HashMap<Integer, ModifiableDBIDs>();
-    for(int i = 0; i < dimensions.size(); i++) {
+    Map<Integer, ModifiableDBIDs> clusterIDs = new HashMap<>();
+    for (int i = 0; i < dimensions.size(); i++) {
       clusterIDs.put(i, DBIDUtil.newHashSet());
     }
 
-    for(DBIDIter it = database.iterDBIDs(); it.valid(); it.advance()) {
+    for (DBIDIter it = database.iterDBIDs(); it.valid(); it.advance()) {
       DBID p_id = DBIDUtil.deref(it);
       V p = database.get(p_id);
       Pair<DoubleDistance, Integer> minDist = null;
-      for(int i = 0; i < dimensions.size(); i++) {
+      for (int i = 0; i < dimensions.size(); i++) {
         Pair<V, TIntSet> pair_i = dimensions.get(i);
         V c_i = pair_i.first;
         TIntSet dimensions_i = pair_i.second;
         DoubleDistance currentDist = manhattanSegmentalDistance(p, c_i, dimensions_i);
-        if(minDist == null || currentDist.compareTo(minDist.first) < 0) {
-          minDist = new Pair<DoubleDistance, Integer>(currentDist, i);
+        if (minDist == null || currentDist.compareTo(minDist.first) < 0) {
+          minDist = new Pair<>(currentDist, i);
         }
       }
       // add p to cluster with mindist
@@ -636,17 +635,17 @@ public class PROCLUS<V extends NumberVector<?>> extends AbstractProjectedCluster
       ids.add(p_id);
     }
 
-    List<PROCLUSCluster> clusters = new ArrayList<PROCLUSCluster>();
-    for(int i = 0; i < dimensions.size(); i++) {
+    List<PROCLUSCluster> clusters = new ArrayList<>();
+    for (int i = 0; i < dimensions.size(); i++) {
       ModifiableDBIDs objectIDs = clusterIDs.get(i);
-      if(!objectIDs.isEmpty()) {
+      if (!objectIDs.isEmpty()) {
         TIntSet clusterDimensions = dimensions.get(i).second;
         V centroid = Centroid.make(database, objectIDs).toVector(database);
         clusters.add(new PROCLUSCluster(objectIDs, clusterDimensions, centroid));
       }
     }
 
-    if(LOG.isDebugging()) {
+    if (LOG.isDebugging()) {
       StringBuilder msg = new StringBuilder();
       msg.append('\n');
       msg.append("clusters ").append(clusters).append('\n');
@@ -667,7 +666,7 @@ public class PROCLUS<V extends NumberVector<?>> extends AbstractProjectedCluster
    */
   private DoubleDistance manhattanSegmentalDistance(V o1, V o2, TIntSet dimensions) {
     double result = 0;
-    for (TIntIterator iter = dimensions.iterator(); iter.hasNext(); ) {
+    for (TIntIterator iter = dimensions.iterator(); iter.hasNext();) {
       final int d = iter.next();
       result += Math.abs(o1.doubleValue(d) - o2.doubleValue(d));
     }
@@ -685,13 +684,13 @@ public class PROCLUS<V extends NumberVector<?>> extends AbstractProjectedCluster
    */
   private double evaluateClusters(Map<DBID, PROCLUSCluster> clusters, Map<DBID, TIntSet> dimensions, Relation<V> database) {
     double result = 0;
-    for(DBID m_i : clusters.keySet()) {
+    for (DBID m_i : clusters.keySet()) {
       PROCLUSCluster c_i = clusters.get(m_i);
       V centroid_i = c_i.centroid;
 
       TIntSet dims_i = dimensions.get(m_i);
       double w_i = 0;
-      for (TIntIterator iter = dims_i.iterator(); iter.hasNext(); ) {
+      for (TIntIterator iter = dims_i.iterator(); iter.hasNext();) {
         final int j = iter.next();
         w_i += avgDistance(centroid_i, c_i.objectIDs, database, j);
       }
@@ -716,7 +715,7 @@ public class PROCLUS<V extends NumberVector<?>> extends AbstractProjectedCluster
    */
   private double avgDistance(V centroid, DBIDs objectIDs, Relation<V> database, int dimension) {
     Mean avg = new Mean();
-    for(DBIDIter iter = objectIDs.iter(); iter.valid(); iter.advance()) {
+    for (DBIDIter iter = objectIDs.iter(); iter.valid(); iter.advance()) {
       V o = database.get(iter);
       avg.put(Math.abs(centroid.doubleValue(dimension) - o.doubleValue(dimension)));
     }
@@ -733,9 +732,9 @@ public class PROCLUS<V extends NumberVector<?>> extends AbstractProjectedCluster
    */
   private ModifiableDBIDs computeBadMedoids(Map<DBID, PROCLUSCluster> clusters, int threshold) {
     ModifiableDBIDs badMedoids = DBIDUtil.newHashSet();
-    for(DBID m_i : clusters.keySet()) {
+    for (DBID m_i : clusters.keySet()) {
       PROCLUSCluster c_i = clusters.get(m_i);
-      if(c_i.objectIDs.size() < threshold) {
+      if (c_i.objectIDs.size() < threshold) {
         badMedoids.add(m_i);
       }
     }
@@ -791,11 +790,10 @@ public class PROCLUS<V extends NumberVector<?>> extends AbstractProjectedCluster
       StringBuilder result = new StringBuilder();
       result.append("Dimensions: [");
       boolean notFirst = false;
-      for(TIntIterator iter = dimensions.iterator(); iter.hasNext(); ) {
-        if(notFirst) {
+      for (TIntIterator iter = dimensions.iterator(); iter.hasNext();) {
+        if (notFirst) {
           result.append(',');
-        }
-        else {
+        } else {
           notFirst = true;
         }
         result.append(iter.next());
@@ -813,7 +811,7 @@ public class PROCLUS<V extends NumberVector<?>> extends AbstractProjectedCluster
      */
     public BitSet getDimensions() {
       BitSet result = new BitSet();
-      for(TIntIterator iter = dimensions.iterator(); iter.hasNext(); ) {
+      for (TIntIterator iter = dimensions.iterator(); iter.hasNext();) {
         result.set(iter.next());
       }
       return result;
@@ -846,20 +844,20 @@ public class PROCLUS<V extends NumberVector<?>> extends AbstractProjectedCluster
       configL(config);
 
       IntParameter m_iP = new IntParameter(M_I_ID, 10);
-      m_iP.addConstraint(new GreaterConstraint(0));
-      if(config.grab(m_iP)) {
+      m_iP.addConstraint(CommonConstraints.GREATER_EQUAL_ONE_INT);
+      if (config.grab(m_iP)) {
         m_i = m_iP.getValue();
       }
 
       RandomParameter rndP = new RandomParameter(SEED_ID);
-      if(config.grab(rndP)) {
+      if (config.grab(rndP)) {
         rnd = rndP.getValue();
       }
     }
 
     @Override
     protected PROCLUS<V> makeInstance() {
-      return new PROCLUS<V>(k, k_i, l, m_i, rnd);
+      return new PROCLUS<>(k, k_i, l, m_i, rnd);
     }
   }
-}
\ No newline at end of file
+}
