@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.algorithm.clustering.subspace;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -111,7 +111,7 @@ public class PreDeCon<V extends NumberVector<?>> extends AbstractProjectedDBSCAN
 
     @Override
     protected PreDeCon<V> makeInstance() {
-      return new PreDeCon<V>(epsilon, minpts, outerdist, lambda);
+      return new PreDeCon<>(epsilon, minpts, outerdist, lambda);
     }
   }
 }
\ No newline at end of file
