@@ -49,13 +49,14 @@ import de.lmu.ifi.dbs.elki.result.outlier.OutlierResult;
 import de.lmu.ifi.dbs.elki.result.outlier.OutlierScoreMeta;
 import de.lmu.ifi.dbs.elki.utilities.FormatUtil;
 import de.lmu.ifi.dbs.elki.utilities.RandomFactory;
+import de.lmu.ifi.dbs.elki.utilities.datastructures.heap.Heap;
 import de.lmu.ifi.dbs.elki.utilities.datastructures.heap.TopBoundedHeap;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Description;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Title;
 import de.lmu.ifi.dbs.elki.utilities.exceptions.AbortException;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.GreaterEqualConstraint;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.CommonConstraints;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntParameter;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.RandomParameter;
@@ -131,13 +132,13 @@ public class AggarwalYuEvolutionary<V extends NumberVector<?>> extends AbstractA
     final int dbsize = relation.size();
     ArrayList<ArrayList<DBIDs>> ranges = buildRanges(relation);
 
-    Iterable<Individuum> individuums = (new EvolutionarySearch(relation, ranges, m, rnd.getRandom())).run();
+    Heap<Individuum>.UnorderedIter individuums = (new EvolutionarySearch(relation, ranges, m, rnd.getSingleThreadedRandom())).run();
 
     WritableDoubleDataStore outlierScore = DataStoreUtil.makeDoubleStorage(relation.getDBIDs(), DataStoreFactory.HINT_HOT | DataStoreFactory.HINT_STATIC);
-    for(Individuum ind : individuums) {
-      DBIDs ids = computeSubspaceForGene(ind.getGene(), ranges);
+    for(; individuums.valid(); individuums.advance()) {
+      DBIDs ids = computeSubspaceForGene(individuums.get().getGene(), ranges);
       double sparsityC = sparsity(ids.size(), dbsize, k, phi);
-      for (DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
+      for(DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
         double prev = outlierScore.doubleValue(iter);
         if(Double.isNaN(prev) || sparsityC < prev) {
           outlierScore.putDouble(iter, sparsityC);
@@ -154,7 +155,7 @@ public class AggarwalYuEvolutionary<V extends NumberVector<?>> extends AbstractA
       }
       minmax.put(val);
     }
-    Relation<Double> scoreResult = new MaterializedRelation<Double>("AggarwalYuEvolutionary", "aggarwal-yu-outlier", TypeUtil.DOUBLE, outlierScore, relation.getDBIDs());
+    Relation<Double> scoreResult = new MaterializedRelation<>("AggarwalYuEvolutionary", "aggarwal-yu-outlier", TypeUtil.DOUBLE, outlierScore, relation.getDBIDs());
     OutlierScoreMeta meta = new InvertedOutlierScoreMeta(minmax.getMin(), minmax.getMax(), Double.NEGATIVE_INFINITY, 0.0);
     return new OutlierResult(meta, scoreResult);
   }
@@ -214,11 +215,11 @@ public class AggarwalYuEvolutionary<V extends NumberVector<?>> extends AbstractA
       this.random = random;
     }
 
-    public Iterable<Individuum> run() {
+    public Heap<Individuum>.UnorderedIter run() {
       ArrayList<Individuum> pop = initialPopulation(m);
       // best Population
-      TopBoundedHeap<Individuum> bestSol = new TopBoundedHeap<Individuum>(m, Collections.reverseOrder());
-      for (Individuum ind : pop) {
+      TopBoundedHeap<Individuum> bestSol = new TopBoundedHeap<>(m, Collections.reverseOrder());
+      for(Individuum ind : pop) {
         bestSol.add(ind);
       }
 
@@ -232,8 +233,8 @@ public class AggarwalYuEvolutionary<V extends NumberVector<?>> extends AbstractA
         pop = mutation(pop, 0.5, 0.5);
         // Avoid duplicates
         ind: for(Individuum ind : pop) {
-          for (Individuum b : bestSol) {
-            if (b.equals(ind)) {
+          for(Heap<Individuum>.UnorderedIter it = bestSol.unorderedIter(); it.valid(); it.advance()) {
+            if(it.get().equals(ind)) {
               continue ind;
             }
           }
@@ -242,8 +243,8 @@ public class AggarwalYuEvolutionary<V extends NumberVector<?>> extends AbstractA
         if(LOG.isDebuggingFinest()) {
           StringBuilder buf = new StringBuilder();
           buf.append("Top solutions:\n");
-          for(Individuum ind : bestSol) {
-            buf.append(ind.toString()).append('\n');
+          for(Heap<Individuum>.UnorderedIter it = bestSol.unorderedIter(); it.valid(); it.advance()) {
+            buf.append(it.get().toString()).append('\n');
           }
           buf.append("Population:\n");
           for(Individuum ind : pop) {
@@ -257,7 +258,7 @@ public class AggarwalYuEvolutionary<V extends NumberVector<?>> extends AbstractA
           break;
         }
       }
-      return bestSol;
+      return bestSol.unorderedIter();
     }
 
     /**
@@ -315,7 +316,7 @@ public class AggarwalYuEvolutionary<V extends NumberVector<?>> extends AbstractA
      */
     private ArrayList<Individuum> initialPopulation(int popsize) {
       // Initial Population
-      ArrayList<Individuum> population = new ArrayList<Individuum>(popsize);
+      ArrayList<Individuum> population = new ArrayList<>(popsize);
       // fill population
       for(int i = 0; i < popsize; i++) {
         // Random Individual
@@ -357,7 +358,7 @@ public class AggarwalYuEvolutionary<V extends NumberVector<?>> extends AbstractA
       // Relative weight := popsize - position => sum(1..popsize)
       int totalweight = (popsize * (popsize + 1)) >> 1;
       // Survivors
-      ArrayList<Individuum> survivors = new ArrayList<Individuum>(popsize);
+      ArrayList<Individuum> survivors = new ArrayList<>(popsize);
 
       // position of selection
       for(int i = 0; i < popsize; i++) {
@@ -387,11 +388,11 @@ public class AggarwalYuEvolutionary<V extends NumberVector<?>> extends AbstractA
      */
     private ArrayList<Individuum> mutation(ArrayList<Individuum> population, double perc1, double perc2) {
       // the Mutations
-      ArrayList<Individuum> mutations = new ArrayList<Individuum>();
+      ArrayList<Individuum> mutations = new ArrayList<>();
       // Set of Positions which are don't care in the String
-      TreeSet<Integer> Q = new TreeSet<Integer>();
+      TreeSet<Integer> Q = new TreeSet<>();
       // Set of Positions which are not don't care in the String
-      TreeSet<Integer> R = new TreeSet<Integer>();
+      TreeSet<Integer> R = new TreeSet<>();
 
       // for each individuum
       for(int j = 0; j < population.size(); j++) {
@@ -470,7 +471,7 @@ public class AggarwalYuEvolutionary<V extends NumberVector<?>> extends AbstractA
      */
     private ArrayList<Individuum> crossoverOptimized(ArrayList<Individuum> population) {
       // Crossover Set of population Set
-      ArrayList<Individuum> crossover = new ArrayList<Individuum>();
+      ArrayList<Individuum> crossover = new ArrayList<>();
 
       for(int i = 0; i < population.size() - 1; i += 2) {
         Pair<Individuum, Individuum> recombine = recombineOptimized(population.get(i), population.get(i + 1));
@@ -496,9 +497,9 @@ public class AggarwalYuEvolutionary<V extends NumberVector<?>> extends AbstractA
     private Pair<Individuum, Individuum> recombineOptimized(Individuum parent1, Individuum parent2) {
       Pair<Individuum, Individuum> recombinePair;
       // Set of Positions in which either s1 or s2 are don't care
-      ArrayList<Integer> Q = new ArrayList<Integer>(dim);
+      ArrayList<Integer> Q = new ArrayList<>(dim);
       // Set of Positions in which neither s1 or s2 is don't care
-      ArrayList<Integer> R = new ArrayList<Integer>(dim);
+      ArrayList<Integer> R = new ArrayList<>(dim);
 
       for(int i = 0; i < dim; i++) {
         if((parent1.getGene()[i] == DONT_CARE) && (parent2.getGene()[i] != DONT_CARE)) {
@@ -567,7 +568,7 @@ public class AggarwalYuEvolutionary<V extends NumberVector<?>> extends AbstractA
       }
       final Individuum i1 = makeIndividuum(b);
       final Individuum i2 = makeIndividuum(comp);
-      recombinePair = new Pair<Individuum, Individuum>(i1, i2);
+      recombinePair = new Pair<>(i1, i2);
 
       return recombinePair;
     }
@@ -610,7 +611,7 @@ public class AggarwalYuEvolutionary<V extends NumberVector<?>> extends AbstractA
    * Individuum for the evolutionary search.
    * 
    * @author Erich Schubert
-   *
+   * 
    * @apiviz.exclude de.lmu.ifi.dbs.elki.utilities.pairs.FCPair
    */
   private static class Individuum extends FCPair<Double, int[]> {
@@ -707,7 +708,7 @@ public class AggarwalYuEvolutionary<V extends NumberVector<?>> extends AbstractA
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
       final IntParameter mP = new IntParameter(M_ID);
-      mP.addConstraint(new GreaterEqualConstraint(2));
+      mP.addConstraint(CommonConstraints.GREATER_THAN_ONE_INT);
       if(config.grab(mP)) {
         m = mP.getValue();
       }
@@ -719,7 +720,7 @@ public class AggarwalYuEvolutionary<V extends NumberVector<?>> extends AbstractA
 
     @Override
     protected AggarwalYuEvolutionary<V> makeInstance() {
-      return new AggarwalYuEvolutionary<V>(k, phi, m, rnd);
+      return new AggarwalYuEvolutionary<>(k, phi, m, rnd);
     }
   }
 }
