@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.algorithm.statistics;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -94,7 +94,11 @@ public class AddSingleScale implements Algorithm {
       for(DBIDIter iditer = rel.iterDBIDs(); iditer.valid(); iditer.advance()) {
         NumberVector<?> vec = rel.get(iditer);
         for(int d = 0; d < dim; d++) {
-          mm.put(vec.doubleValue(d));
+          final double val = vec.doubleValue(d);
+          if(val != val) {
+            continue; // NaN
+          }
+          mm.put(val);
         }
       }
       LinearScale scale = new LinearScale(mm.getMin(), mm.getMax());
