@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.algorithm.statistics;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -62,7 +62,7 @@ import de.lmu.ifi.dbs.elki.utilities.documentation.Description;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Title;
 import de.lmu.ifi.dbs.elki.utilities.exceptions.ExceptionMessages;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.GreaterEqualConstraint;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.CommonConstraints;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.OnlyOneIsAllowedToBeSetGlobalConstraint;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.Flag;
@@ -159,24 +159,26 @@ public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> ex
     MeanVariance modif = new MeanVariance();
     // Histogram
     final ObjHistogram<long[]> histogram;
-    if (stepprog != null) {
+    if(stepprog != null) {
       stepprog.beginStep(1, "Prepare histogram.", LOG);
     }
-    if (exact) {
+    if(exact) {
       gminmax = exactMinMax(relation, distFunc);
       histogram = new LongArrayStaticHistogram(numbin, gminmax.getMin(), gminmax.getMax(), 2);
-    } else if (sampling) {
+    }
+    else if(sampling) {
       gminmax = sampleMinMax(relation, distFunc);
       histogram = new LongArrayStaticHistogram(numbin, gminmax.getMin(), gminmax.getMax(), 2);
-    } else {
+    }
+    else {
       histogram = new AbstractObjDynamicHistogram<long[]>(numbin) {
         @Override
         protected long[] downsample(Object[] data, int start, int end, int size) {
           long[] ret = new long[2];
-          for (int i = start; i < end; i++) {
+          for(int i = start; i < end; i++) {
             long[] existing = (long[]) data[i];
-            if (existing != null) {
-              for (int c = 0; c < 2; c++) {
+            if(existing != null) {
+              for(int c = 0; c < 2; c++) {
                 ret[c] += existing[c];
               }
             }
@@ -186,7 +188,7 @@ public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> ex
 
         @Override
         protected long[] aggregate(long[] first, long[] second) {
-          for (int c = 0; c < 2; c++) {
+          for(int c = 0; c < 2; c++) {
             first[c] += second[c];
           }
           return first;
@@ -204,20 +206,20 @@ public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> ex
       };
     }
 
-    if (stepprog != null) {
+    if(stepprog != null) {
       stepprog.beginStep(2, "Build histogram.", LOG);
     }
     final FiniteProgress progress = LOG.isVerbose() ? new FiniteProgress("Distance computations", relation.size(), LOG) : null;
     // iterate per cluster
     final long[] incFirst = new long[] { 1L, 0L };
     final long[] incSecond = new long[] { 0L, 1L };
-    for (Cluster<?> c1 : split) {
-      for (DBIDIter id1 = c1.getIDs().iter(); id1.valid(); id1.advance()) {
+    for(Cluster<?> c1 : split) {
+      for(DBIDIter id1 = c1.getIDs().iter(); id1.valid(); id1.advance()) {
         // in-cluster distances
         DoubleMinMax iminmax = new DoubleMinMax();
-        for (DBIDIter iter2 = c1.getIDs().iter(); iter2.valid(); iter2.advance()) {
+        for(DBIDIter iter2 = c1.getIDs().iter(); iter2.valid(); iter2.advance()) {
           // skip the point itself.
-          if (DBIDUtil.equal(id1, iter2)) {
+          if(DBIDUtil.equal(id1, iter2)) {
             continue;
           }
           double d = distFunc.distance(id1, iter2).doubleValue();
@@ -236,13 +238,13 @@ public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> ex
 
         // other-cluster distances
         DoubleMinMax ominmax = new DoubleMinMax();
-        for (Cluster<?> c2 : split) {
-          if (c2 == c1) {
+        for(Cluster<?> c2 : split) {
+          if(c2 == c1) {
             continue;
           }
-          for (DBIDIter iter2 = c2.getIDs().iter(); iter2.valid(); iter2.advance()) {
+          for(DBIDIter iter2 = c2.getIDs().iter(); iter2.valid(); iter2.advance()) {
             // skip the point itself (shouldn't happen though)
-            if (DBIDUtil.equal(id1, iter2)) {
+            if(DBIDUtil.equal(id1, iter2)) {
               continue;
             }
             double d = distFunc.distance(id1, iter2).doubleValue();
@@ -259,33 +261,33 @@ public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> ex
         // min/max
         gominmax.put(ominmax.getMin());
         gominmax.put(ominmax.getMax());
-        if (progress != null) {
+        if(progress != null) {
           progress.incrementProcessed(LOG);
         }
       }
     }
-    if (progress != null) {
+    if(progress != null) {
       progress.ensureCompleted(LOG);
     }
     // Update values (only needed for sampling case).
     gminmax.setFirst(Math.min(giminmax.getMin(), gominmax.getMin()));
     gminmax.setSecond(Math.max(giminmax.getMax(), gominmax.getMax()));
 
-    if (stepprog != null) {
+    if(stepprog != null) {
       stepprog.setCompleted(LOG);
     }
 
     // count the number of samples we have in the data
     long inum = 0;
     long onum = 0;
-    for (ObjHistogram.Iter<long[]> iter = histogram.iter(); iter.valid(); iter.advance()) {
+    for(ObjHistogram.Iter<long[]> iter = histogram.iter(); iter.valid(); iter.advance()) {
       inum += iter.getValue()[0];
       onum += iter.getValue()[1];
     }
     long bnum = inum + onum;
 
-    Collection<DoubleVector> binstat = new ArrayList<DoubleVector>(numbin);
-    for (ObjHistogram.Iter<long[]> iter = histogram.iter(); iter.valid(); iter.advance()) {
+    Collection<DoubleVector> binstat = new ArrayList<>(numbin);
+    for(ObjHistogram.Iter<long[]> iter = histogram.iter(); iter.valid(); iter.advance()) {
       final long[] value = iter.getValue();
       final double icof = (inum == 0) ? 0 : ((double) value[0]) / inum / histogram.getBinsize();
       final double icaf = ((double) value[0]) / bnum / histogram.getBinsize();
@@ -294,7 +296,7 @@ public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> ex
       DoubleVector row = new DoubleVector(new double[] { iter.getCenter(), icof, icaf, ocof, ocaf });
       binstat.add(row);
     }
-    HistogramResult<DoubleVector> result = new HistogramResult<DoubleVector>("Distance Histogram", "distance-histogram", binstat);
+    HistogramResult<DoubleVector> result = new HistogramResult<>("Distance Histogram", "distance-histogram", binstat);
 
     result.addHeader("Absolute minimum distance (abs): " + gminmax.getMin());
     result.addHeader("Absolute maximum distance (abs): " + gminmax.getMax());
@@ -319,34 +321,34 @@ public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> ex
     Random rnd = new Random();
     // estimate minimum and maximum.
     int k = (int) Math.max(25, Math.pow(relation.size(), 0.2));
-    TreeSet<DoubleDBIDPair> minhotset = new TreeSet<DoubleDBIDPair>();
-    TreeSet<DoubleDBIDPair> maxhotset = new TreeSet<DoubleDBIDPair>(Collections.reverseOrder());
+    TreeSet<DoubleDBIDPair> minhotset = new TreeSet<>();
+    TreeSet<DoubleDBIDPair> maxhotset = new TreeSet<>(Collections.reverseOrder());
 
     int randomsize = (int) Math.max(25, Math.pow(relation.size(), 0.2));
     double rprob = ((double) randomsize) / size;
     ArrayModifiableDBIDs randomset = DBIDUtil.newArray(randomsize);
 
     DBIDIter iter = relation.iterDBIDs();
-    if (!iter.valid()) {
+    if(!iter.valid()) {
       throw new IllegalStateException(ExceptionMessages.DATABASE_EMPTY);
     }
     DBID firstid = DBIDUtil.deref(iter);
     iter.advance();
     minhotset.add(DBIDUtil.newPair(Double.MAX_VALUE, firstid));
     maxhotset.add(DBIDUtil.newPair(Double.MIN_VALUE, firstid));
-    for (; iter.valid(); iter.advance()) {
+    for(; iter.valid(); iter.advance()) {
       // generate candidates for min distance.
-      ArrayList<DoubleDBIDPair> np = new ArrayList<DoubleDBIDPair>(k * 2 + randomsize * 2);
-      for (DoubleDBIDPair pair : minhotset) {
+      ArrayList<DoubleDBIDPair> np = new ArrayList<>(k * 2 + randomsize * 2);
+      for(DoubleDBIDPair pair : minhotset) {
         // skip the object itself
-        if (DBIDUtil.equal(iter, pair)) {
+        if(DBIDUtil.equal(iter, pair)) {
           continue;
         }
         double d = distFunc.distance(iter, pair).doubleValue();
         np.add(DBIDUtil.newPair(d, iter));
         np.add(DBIDUtil.newPair(d, pair));
       }
-      for (DBIDIter iter2 = randomset.iter(); iter2.valid(); iter2.advance()) {
+      for(DBIDIter iter2 = randomset.iter(); iter2.valid(); iter2.advance()) {
         double d = distFunc.distance(iter, iter2).doubleValue();
         np.add(DBIDUtil.newPair(d, iter));
         np.add(DBIDUtil.newPair(d, iter2));
@@ -355,17 +357,17 @@ public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> ex
       shrinkHeap(minhotset, k);
 
       // generate candidates for max distance.
-      ArrayList<DoubleDBIDPair> np2 = new ArrayList<DoubleDBIDPair>(k * 2 + randomsize * 2);
-      for (DoubleDBIDPair pair : minhotset) {
+      ArrayList<DoubleDBIDPair> np2 = new ArrayList<>(k * 2 + randomsize * 2);
+      for(DoubleDBIDPair pair : minhotset) {
         // skip the object itself
-        if (DBIDUtil.equal(iter, pair)) {
+        if(DBIDUtil.equal(iter, pair)) {
           continue;
         }
         double d = distFunc.distance(iter, pair).doubleValue();
         np2.add(DBIDUtil.newPair(d, iter));
         np2.add(DBIDUtil.newPair(d, pair));
       }
-      for (DBIDIter iter2 = randomset.iter(); iter2.valid(); iter2.advance()) {
+      for(DBIDIter iter2 = randomset.iter(); iter2.valid(); iter2.advance()) {
         double d = distFunc.distance(iter, iter2).doubleValue();
         np.add(DBIDUtil.newPair(d, iter));
         np.add(DBIDUtil.newPair(d, iter2));
@@ -374,9 +376,10 @@ public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> ex
       shrinkHeap(maxhotset, k);
 
       // update random set
-      if (randomset.size() < randomsize) {
+      if(randomset.size() < randomsize) {
         randomset.add(iter);
-      } else if (rnd.nextDouble() < rprob) {
+      }
+      else if(rnd.nextDouble() < rprob) {
         randomset.set((int) Math.floor(rnd.nextDouble() * randomsize), iter);
       }
     }
@@ -393,10 +396,10 @@ public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> ex
   private DoubleMinMax exactMinMax(Relation<O> relation, DistanceQuery<O, D> distFunc) {
     DoubleMinMax minmax = new DoubleMinMax();
     // find exact minimum and maximum first.
-    for (DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
-      for (DBIDIter iditer2 = relation.iterDBIDs(); iditer2.valid(); iditer2.advance()) {
+    for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
+      for(DBIDIter iditer2 = relation.iterDBIDs(); iditer2.valid(); iditer2.advance()) {
         // skip the point itself.
-        if (DBIDUtil.equal(iditer, iditer2)) {
+        if(DBIDUtil.equal(iditer, iditer2)) {
           continue;
         }
         double d = distFunc.distance(iditer, iditer2).doubleValue();
@@ -416,11 +419,12 @@ public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> ex
     // drop duplicates
     ModifiableDBIDs seenids = DBIDUtil.newHashSet(2 * k);
     int cnt = 0;
-    for (Iterator<DoubleDBIDPair> i = hotset.iterator(); i.hasNext();) {
+    for(Iterator<DoubleDBIDPair> i = hotset.iterator(); i.hasNext();) {
       DoubleDBIDPair p = i.next();
-      if (cnt > k || seenids.contains(p)) {
+      if(cnt > k || seenids.contains(p)) {
         i.remove();
-      } else {
+      }
+      else {
         seenids.add(p);
         cnt++;
       }
@@ -464,22 +468,22 @@ public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> ex
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
       final IntParameter numbinP = new IntParameter(HISTOGRAM_BINS_ID, 20);
-      numbinP.addConstraint(new GreaterEqualConstraint(2));
-      if (config.grab(numbinP)) {
+      numbinP.addConstraint(CommonConstraints.GREATER_THAN_ONE_INT);
+      if(config.grab(numbinP)) {
         numbin = numbinP.getValue();
       }
 
       final Flag exactF = new Flag(EXACT_ID);
-      if (config.grab(exactF)) {
+      if(config.grab(exactF)) {
         exact = exactF.getValue();
       }
 
       final Flag samplingF = new Flag(SAMPLING_ID);
-      if (config.grab(samplingF)) {
+      if(config.grab(samplingF)) {
         sampling = samplingF.getValue();
       }
 
-      ArrayList<Parameter<?>> exclusive = new ArrayList<Parameter<?>>();
+      ArrayList<Parameter<?>> exclusive = new ArrayList<>();
       exclusive.add(exactF);
       exclusive.add(samplingF);
       config.checkConstraint(new OnlyOneIsAllowedToBeSetGlobalConstraint(exclusive));
@@ -487,7 +491,7 @@ public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> ex
 
     @Override
     protected DistanceStatisticsWithClasses<O, D> makeInstance() {
-      return new DistanceStatisticsWithClasses<O, D>(distanceFunction, numbin, exact, sampling);
+      return new DistanceStatisticsWithClasses<>(distanceFunction, numbin, exact, sampling);
     }
   }
 }
