@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.application;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -27,6 +27,7 @@ import java.io.File;
 import java.util.Collection;
 
 import de.lmu.ifi.dbs.elki.logging.Logging;
+import de.lmu.ifi.dbs.elki.logging.Logging.Level;
 import de.lmu.ifi.dbs.elki.logging.LoggingConfiguration;
 import de.lmu.ifi.dbs.elki.logging.LoggingUtil;
 import de.lmu.ifi.dbs.elki.utilities.ClassGenericsUtil;
@@ -60,6 +61,7 @@ import de.lmu.ifi.dbs.elki.utilities.pairs.Pair;
  * @author Erich Schubert
  * 
  * @apiviz.uses LoggingConfiguration oneway
+ * @apiviz.excludeSubtypes
  */
 public abstract class AbstractApplication implements Parameterizable {
   /**
@@ -75,7 +77,12 @@ public abstract class AbstractApplication implements Parameterizable {
   /**
    * Information for citation and version.
    */
-  public static final String INFORMATION = "ELKI Version 0.5.5 (2012, December)" + NEWLINE + NEWLINE + "published in:" + NEWLINE + "E. Achtert, S. Goldhofer, H.-P. Kriegel, E. Schubert, A. Zimek:" + NEWLINE + "Evaluation of Clusterings – Metrics and Visual Support." + NEWLINE + "In Proceedings of the 28th" + NEWLINE + "International Conference on Data Engineering (ICDE), Washington, DC, 2012." + NEWLINE;
+  public static final String INFORMATION = "ELKI Version 0.6.0 (2014, January)" + NEWLINE + NEWLINE //
+      + "published in:" + NEWLINE //
+      + "Elke Achtert, Hans-Peter Kriegel, Erich Schubert, Arthur Zimek:" + NEWLINE //
+      + "Interactive Data Mining with 3D-Parallel-Coordinate-Trees." + NEWLINE //
+      + "In Proceedings of the ACM International Conference on " + NEWLINE //
+      + "Management of Data (SIGMOD), New York City, NY, 2013." + NEWLINE;
 
   /**
    * Constructor.
@@ -93,10 +100,11 @@ public abstract class AbstractApplication implements Parameterizable {
    * @param args the arguments to run this application with
    */
   public static void runCLIApplication(Class<?> cls, String[] args) {
-    final Flag helpF = new Flag(OptionID.HELP);
-    final Flag helpLongF = new Flag(OptionID.HELP_LONG);
-    final ClassParameter<Object> descriptionP = new ClassParameter<Object>(OptionID.DESCRIPTION, Object.class, true);
-    final StringParameter debugP = new StringParameter(OptionID.DEBUG);
+    final Flag helpF = new Flag(Parameterizer.HELP_ID);
+    final Flag helpLongF = new Flag(Parameterizer.HELP_LONG_ID);
+    final ClassParameter<Object> descriptionP = new ClassParameter<>(Parameterizer.DESCRIPTION_ID, Object.class, true);
+    final StringParameter debugP = new StringParameter(Parameterizer.DEBUG_ID);
+    final Flag verboseF = new Flag(Parameterizer.VERBOSE_ID);
     debugP.setOptional(true);
 
     SerializedParameterization params = new SerializedParameterization(args);
@@ -105,45 +113,60 @@ public abstract class AbstractApplication implements Parameterizable {
       params.grab(helpLongF);
       params.grab(descriptionP);
       params.grab(debugP);
-      if (descriptionP.isDefined()) {
+      if(descriptionP.isDefined()) {
         params.clearErrors();
         printDescription(descriptionP.getValue());
         return;
       }
       // Fail silently on errors.
-      if (params.getErrors().size() > 0) {
+      if(params.getErrors().size() > 0) {
         params.logAndClearReportedErrors();
         return;
       }
-      if (debugP.isDefined()) {
+      if(debugP.isDefined()) {
         LoggingUtil.parseDebugParameter(debugP);
       }
-    } catch (Exception e) {
+    }
+    catch(Exception e) {
       printErrorMessage(e);
       return;
     }
     try {
       TrackParameters config = new TrackParameters(params);
+      if(config.grab(verboseF) && verboseF.isTrue()) {
+        // Extra verbosity by repeating the flag:
+        final Flag verbose2F = new Flag(Parameterizer.VERBOSE_ID);
+        if(config.grab(verbose2F) && verbose2F.isTrue()) {
+          LoggingConfiguration.setVerbose(Level.VERYVERBOSE);
+        }
+        else {
+          LoggingConfiguration.setVerbose(Level.VERBOSE);
+        }
+      }
       AbstractApplication task = ClassGenericsUtil.tryInstantiate(AbstractApplication.class, cls, config);
 
-      if ((helpF.isDefined() && helpF.getValue()) || (helpLongF.isDefined() && helpLongF.getValue())) {
-        LoggingConfiguration.setVerbose(true);
+      if((helpF.isDefined() && helpF.getValue()) || (helpLongF.isDefined() && helpLongF.getValue())) {
+        LoggingConfiguration.setVerbose(Level.VERBOSE);
         LOG.verbose(usage(config.getAllParameters()));
-      } else {
+      }
+      else {
         params.logUnusedParameters();
-        if (params.getErrors().size() > 0) {
-          LoggingConfiguration.setVerbose(true);
+        if(params.getErrors().size() > 0) {
+          LoggingConfiguration.setVerbose(Level.VERBOSE);
           LOG.verbose("The following configuration errors prevented execution:\n");
-          for (ParameterException e : params.getErrors()) {
+          for(ParameterException e : params.getErrors()) {
             LOG.verbose(e.getMessage());
           }
           LOG.verbose("\n");
           LOG.verbose("Stopping execution because of configuration errors.");
-        } else {
+          System.exit(1);
+        }
+        else {
           task.run();
         }
       }
-    } catch (Exception e) {
+    }
+    catch(Exception e) {
       printErrorMessage(e);
     }
   }
@@ -172,15 +195,18 @@ public abstract class AbstractApplication implements Parameterizable {
    * @param e Error Exception.
    */
   protected static void printErrorMessage(Exception e) {
-    if (e instanceof AbortException) {
+    if(e instanceof AbortException) {
       // ensure we actually show the message:
-      LoggingConfiguration.setVerbose(true);
+      LoggingConfiguration.setVerbose(Level.VERBOSE);
       LOG.verbose(e.getMessage());
-    } else if (e instanceof UnspecifiedParameterException) {
+    }
+    else if(e instanceof UnspecifiedParameterException) {
       LOG.error(e.getMessage());
-    } else if (e instanceof ParameterException) {
+    }
+    else if(e instanceof ParameterException) {
       LOG.error(e.getMessage());
-    } else {
+    }
+    else {
       LOG.exception(e);
     }
   }
@@ -189,8 +215,8 @@ public abstract class AbstractApplication implements Parameterizable {
    * Print the description for the given parameter
    */
   private static void printDescription(Class<?> descriptionClass) {
-    if (descriptionClass != null) {
-      LoggingConfiguration.setVerbose(true);
+    if(descriptionClass != null) {
+      LoggingConfiguration.setVerbose(Level.VERBOSE);
       LOG.verbose(OptionUtil.describeParameterizable(new StringBuilder(), descriptionClass, FormatUtil.getConsoleWidth(), "    ").toString());
     }
   }
@@ -227,25 +253,45 @@ public abstract class AbstractApplication implements Parameterizable {
      */
     public static final OptionID INPUT_ID = new OptionID("app.in", "");
 
-    @Override
-    protected void makeOptions(Parameterization config) {
-      super.makeOptions(config);
-      configVerbose(config);
-      // Note: we do not run the other methods by default.
-      // Only verbose will always be present!
-    }
+    /**
+     * Flag to obtain help-message.
+     * <p>
+     * Key: {@code -h}
+     * </p>
+     */
+    public static final OptionID HELP_ID = new OptionID("h", "Request a help-message, either for the main-routine or for any specified algorithm. " + "Causes immediate stop of the program.");
 
     /**
-     * Get the verbose parameter.
-     * 
-     * @param config Parameterization
+     * Flag to obtain help-message.
+     * <p>
+     * Key: {@code -help}
+     * </p>
      */
-    protected void configVerbose(Parameterization config) {
-      final Flag verboseF = new Flag(OptionID.VERBOSE_FLAG);
-      if (config.grab(verboseF)) {
-        LoggingConfiguration.setVerbose(verboseF.isTrue());
-      }
-    }
+    public static final OptionID HELP_LONG_ID = new OptionID("help", "Request a help-message, either for the main-routine or for any specified algorithm. " + "Causes immediate stop of the program.");
+
+    /**
+     * Optional Parameter to specify a class to obtain a description for.
+     * <p>
+     * Key: {@code -description}
+     * </p>
+     */
+    public static final OptionID DESCRIPTION_ID = new OptionID("description", "Class to obtain a description of. " + "Causes immediate stop of the program.");
+
+    /**
+     * Optional Parameter to specify a class to enable debugging for.
+     * <p>
+     * Key: {@code -enableDebug}
+     * </p>
+     */
+    public static final OptionID DEBUG_ID = new OptionID("enableDebug", "Parameter to enable debugging for particular packages.");
+
+    /**
+     * Flag to allow verbose messages while running the application.
+     * <p>
+     * Key: {@code -verbose}
+     * </p>
+     */
+    public static final OptionID VERBOSE_ID = new OptionID("verbose", "Enable verbose messages.");
 
     /**
      * Get the output file parameter.
@@ -267,7 +313,7 @@ public abstract class AbstractApplication implements Parameterizable {
     protected File getParameterOutputFile(Parameterization config, String description) {
       final FileParameter outputP = new FileParameter(OUTPUT_ID, FileParameter.FileType.OUTPUT_FILE);
       outputP.setShortDescription(description);
-      if (config.grab(outputP)) {
+      if(config.grab(outputP)) {
         return outputP.getValue();
       }
       return null;
@@ -293,7 +339,7 @@ public abstract class AbstractApplication implements Parameterizable {
     protected File getParameterInputFile(Parameterization config, String description) {
       final FileParameter inputP = new FileParameter(INPUT_ID, FileParameter.FileType.INPUT_FILE);
       inputP.setShortDescription(description);
-      if (config.grab(inputP)) {
+      if(config.grab(inputP)) {
         return inputP.getValue();
       }
       return null;
