@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.application.greedyensemble;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -30,17 +30,18 @@ import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
 
 import de.lmu.ifi.dbs.elki.algorithm.AbstractAlgorithm;
-import de.lmu.ifi.dbs.elki.algorithm.outlier.ABOD;
 import de.lmu.ifi.dbs.elki.algorithm.outlier.KNNOutlier;
 import de.lmu.ifi.dbs.elki.algorithm.outlier.KNNWeightOutlier;
-import de.lmu.ifi.dbs.elki.algorithm.outlier.LDOF;
-import de.lmu.ifi.dbs.elki.algorithm.outlier.LOF;
-import de.lmu.ifi.dbs.elki.algorithm.outlier.LoOP;
+import de.lmu.ifi.dbs.elki.algorithm.outlier.lof.LDF;
+import de.lmu.ifi.dbs.elki.algorithm.outlier.lof.LDOF;
+import de.lmu.ifi.dbs.elki.algorithm.outlier.lof.LOF;
+import de.lmu.ifi.dbs.elki.algorithm.outlier.lof.LoOP;
+import de.lmu.ifi.dbs.elki.algorithm.outlier.lof.SimplifiedLOF;
 import de.lmu.ifi.dbs.elki.algorithm.outlier.trivial.ByLabelOutlier;
 import de.lmu.ifi.dbs.elki.algorithm.outlier.trivial.TrivialAllOutlier;
 import de.lmu.ifi.dbs.elki.algorithm.outlier.trivial.TrivialNoOutlier;
 import de.lmu.ifi.dbs.elki.application.AbstractApplication;
-import de.lmu.ifi.dbs.elki.data.DoubleVector;
+import de.lmu.ifi.dbs.elki.data.NumberVector;
 import de.lmu.ifi.dbs.elki.database.Database;
 import de.lmu.ifi.dbs.elki.database.QueryUtil;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
@@ -50,28 +51,24 @@ import de.lmu.ifi.dbs.elki.database.query.knn.KNNQuery;
 import de.lmu.ifi.dbs.elki.database.query.knn.PreprocessorKNNQuery;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.DistanceFunction;
-import de.lmu.ifi.dbs.elki.distance.distancefunction.EuclideanDistanceFunction;
-import de.lmu.ifi.dbs.elki.distance.distancevalue.DoubleDistance;
+import de.lmu.ifi.dbs.elki.distance.distancefunction.minkowski.EuclideanDistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.NumberDistance;
-import de.lmu.ifi.dbs.elki.distance.similarityfunction.kernel.PolynomialKernelFunction;
 import de.lmu.ifi.dbs.elki.index.preprocessed.knn.MaterializeKNNPreprocessor;
 import de.lmu.ifi.dbs.elki.logging.Logging;
-import de.lmu.ifi.dbs.elki.result.Result;
-import de.lmu.ifi.dbs.elki.result.ResultHierarchy;
+import de.lmu.ifi.dbs.elki.math.statistics.kernelfunctions.GaussianKernelDensityFunction;
 import de.lmu.ifi.dbs.elki.result.outlier.OutlierResult;
 import de.lmu.ifi.dbs.elki.utilities.Base64;
 import de.lmu.ifi.dbs.elki.utilities.FormatUtil;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
 import de.lmu.ifi.dbs.elki.utilities.exceptions.AbortException;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.GreaterConstraint;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.CommonConstraints;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntParameter;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.ObjectParameter;
 import de.lmu.ifi.dbs.elki.utilities.scaling.IdentityScaling;
 import de.lmu.ifi.dbs.elki.utilities.scaling.ScalingFunction;
-import de.lmu.ifi.dbs.elki.utilities.scaling.outlier.MinusLogStandardDeviationScaling;
-import de.lmu.ifi.dbs.elki.utilities.scaling.outlier.StandardDeviationScaling;
+import de.lmu.ifi.dbs.elki.utilities.scaling.outlier.OutlierScalingFunction;
 import de.lmu.ifi.dbs.elki.workflow.InputStep;
 
 /**
@@ -90,7 +87,7 @@ import de.lmu.ifi.dbs.elki.workflow.InputStep;
  * @author Erich Schubert
  */
 @Reference(authors = "E. Schubert, R. Wojdanowski, A. Zimek, H.-P. Kriegel", title = "On Evaluation of Outlier Rankings and Outlier Scores", booktitle = "Proc. 12th SIAM International Conference on Data Mining (SDM), Anaheim, CA, 2012.")
-public class ComputeKNNOutlierScores<O, D extends NumberDistance<D, ?>> extends AbstractApplication {
+public class ComputeKNNOutlierScores<O extends NumberVector<?>, D extends NumberDistance<D, ?>> extends AbstractApplication {
   /**
    * Our logger class.
    */
@@ -132,9 +129,9 @@ public class ComputeKNNOutlierScores<O, D extends NumberDistance<D, ?>> extends
   ByLabelOutlier bylabel;
 
   /**
-   * Include ABOD in the experiments.
+   * Scaling function.
    */
-  boolean runabod = false;
+  ScalingFunction scaling;
 
   /**
    * Constructor.
@@ -146,8 +143,9 @@ public class ComputeKNNOutlierScores<O, D extends NumberDistance<D, ?>> extends
    * @param maxk Maximum k value
    * @param bylabel By label outlier (reference)
    * @param outfile Output file
+   * @param scaling Scaling function
    */
-  public ComputeKNNOutlierScores(InputStep inputstep, DistanceFunction<? super O, D> distf, int startk, int stepk, int maxk, ByLabelOutlier bylabel, File outfile) {
+  public ComputeKNNOutlierScores(InputStep inputstep, DistanceFunction<? super O, D> distf, int startk, int stepk, int maxk, ByLabelOutlier bylabel, File outfile, ScalingFunction scaling) {
     super();
     this.distf = distf;
     this.startk = startk;
@@ -156,18 +154,24 @@ public class ComputeKNNOutlierScores<O, D extends NumberDistance<D, ?>> extends
     this.inputstep = inputstep;
     this.bylabel = bylabel;
     this.outfile = outfile;
+    this.scaling = scaling;
   }
 
   @Override
   public void run() {
     final Database database = inputstep.getDatabase();
     final Relation<O> relation = database.getRelation(distf.getInputTypeRestriction());
-    LOG.verbose("Running preprocessor ...");
-    MaterializeKNNPreprocessor<O, D> preproc = new MaterializeKNNPreprocessor<O, D>(relation, distf, maxk + 2);
-    database.addIndex(preproc);
 
-    // Test that we did get a proper index query
-    KNNQuery<O, D> knnq = QueryUtil.getKNNQuery(relation, distf);
+    // If there is no kNN preprocessor already, then precompute.
+    KNNQuery<O, D> knnq = QueryUtil.getKNNQuery(relation, distf, maxk + 2);
+    if(!(knnq instanceof PreprocessorKNNQuery)) {
+      LOG.verbose("Running preprocessor ...");
+      MaterializeKNNPreprocessor<O, D> preproc = new MaterializeKNNPreprocessor<>(relation, distf, maxk + 2);
+      database.addIndex(preproc);
+    }
+
+    // Test that we now get a proper index query
+    knnq = QueryUtil.getKNNQuery(relation, distf, maxk + 2);
     if(!(knnq instanceof PreprocessorKNNQuery)) {
       LOG.warning("Not using preprocessor knn query -- KNN queries using class: " + knnq.getClass());
     }
@@ -185,8 +189,8 @@ public class ComputeKNNOutlierScores<O, D extends NumberDistance<D, ?>> extends
     {
       try {
         MessageDigest md = MessageDigest.getInstance("MD5");
-        for (DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
-          md.update(" ".getBytes());
+        for(DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
+          md.update((byte) ' ');
           md.update(DBIDUtil.toString(iter).getBytes());
         }
         fout.append("# DBID-series MD5:");
@@ -204,7 +208,8 @@ public class ComputeKNNOutlierScores<O, D extends NumberDistance<D, ?>> extends
       writeResult(fout, ids, bylabelresult, new IdentityScaling(), "bylabel");
     }
     // No/all outliers "results"
-    {
+    boolean withdummy = false;
+    if(withdummy) {
       OutlierResult noresult = (new TrivialNoOutlier()).run(database);
       writeResult(fout, ids, noresult, new IdentityScaling(), "no-outliers");
       OutlierResult allresult = (new TrivialAllOutlier()).run(database);
@@ -216,13 +221,10 @@ public class ComputeKNNOutlierScores<O, D extends NumberDistance<D, ?>> extends
     runForEachK(new AlgRunner() {
       @Override
       public void run(int k, String kstr) {
-        KNNOutlier<O, D> knn = new KNNOutlier<O, D>(distf, k);
+        KNNOutlier<O, D> knn = new KNNOutlier<>(distf, k);
         OutlierResult knnresult = knn.run(database, relation);
-        // Setup scaling
-        StandardDeviationScaling scaling = new StandardDeviationScaling();
-        scaling.prepare(knnresult);
         writeResult(fout, ids, knnresult, scaling, "KNN-" + kstr);
-        detachResult(database, knnresult);
+        database.getHierarchy().removeSubtree(knnresult);
       }
     });
     // KNN Weight
@@ -230,13 +232,10 @@ public class ComputeKNNOutlierScores<O, D extends NumberDistance<D, ?>> extends
     runForEachK(new AlgRunner() {
       @Override
       public void run(int k, String kstr) {
-        KNNWeightOutlier<O, D> knnw = new KNNWeightOutlier<O, D>(distf, k);
+        KNNWeightOutlier<O, D> knnw = new KNNWeightOutlier<>(distf, k);
         OutlierResult knnresult = knnw.run(database, relation);
-        // Setup scaling
-        StandardDeviationScaling scaling = new StandardDeviationScaling();
-        scaling.prepare(knnresult);
         writeResult(fout, ids, knnresult, scaling, "KNNW-" + kstr);
-        detachResult(database, knnresult);
+        database.getHierarchy().removeSubtree(knnresult);
       }
     });
     // Run LOF
@@ -244,13 +243,21 @@ public class ComputeKNNOutlierScores<O, D extends NumberDistance<D, ?>> extends
     runForEachK(new AlgRunner() {
       @Override
       public void run(int k, String kstr) {
-        LOF<O, D> lof = new LOF<O, D>(k, distf, distf);
-        OutlierResult lofresult = lof.run(relation);
-        // Setup scaling
-        StandardDeviationScaling scaling = new StandardDeviationScaling(1.0, 1.0);
-        scaling.prepare(lofresult);
+        LOF<O, D> lof = new LOF<>(k, distf);
+        OutlierResult lofresult = lof.run(database, relation);
         writeResult(fout, ids, lofresult, scaling, "LOF-" + kstr);
-        detachResult(database, lofresult);
+        database.getHierarchy().removeSubtree(lofresult);
+      }
+    });
+    // Run Simplified-LOF
+    LOG.verbose("Running Simplified-LOF");
+    runForEachK(new AlgRunner() {
+      @Override
+      public void run(int k, String kstr) {
+        SimplifiedLOF<O, D> lof = new SimplifiedLOF<>(k, distf);
+        OutlierResult lofresult = lof.run(database, relation);
+        writeResult(fout, ids, lofresult, scaling, "Simplified-LOF-" + kstr);
+        database.getHierarchy().removeSubtree(lofresult);
       }
     });
     // LoOP
@@ -258,64 +265,37 @@ public class ComputeKNNOutlierScores<O, D extends NumberDistance<D, ?>> extends
     runForEachK(new AlgRunner() {
       @Override
       public void run(int k, String kstr) {
-        LoOP<O, D> loop = new LoOP<O, D>(k, k, distf, distf, 1.0);
+        LoOP<O, D> loop = new LoOP<>(k, k, distf, distf, 1.0);
         OutlierResult loopresult = loop.run(database, relation);
-        writeResult(fout, ids, loopresult, new IdentityScaling(), "LOOP-" + kstr);
-        detachResult(database, loopresult);
+        writeResult(fout, ids, loopresult, scaling, "LOOP-" + kstr);
+        database.getHierarchy().removeSubtree(loopresult);
       }
     });
     // LDOF
-    LOG.verbose("Running LDOF");
+    boolean runldof = false;
+    if(runldof) {
+      LOG.verbose("Running LDOF");
+      runForEachK(new AlgRunner() {
+        @Override
+        public void run(int k, String kstr) {
+          LDOF<O, D> ldof = new LDOF<>(distf, k + 1);
+          OutlierResult ldofresult = ldof.run(database, relation);
+          writeResult(fout, ids, ldofresult, scaling, "LDOF-" + kstr);
+          database.getHierarchy().removeSubtree(ldofresult);
+        }
+      });
+    }
+    // Run LDF
+    LOG.verbose("Running LDF");
     runForEachK(new AlgRunner() {
       @Override
       public void run(int k, String kstr) {
-        LDOF<O, D> ldof = new LDOF<O, D>(distf, k);
-        OutlierResult ldofresult = ldof.run(database, relation);
-        // Setup scaling
-        StandardDeviationScaling scaling = new StandardDeviationScaling(1.0, 1.0);
-        scaling.prepare(ldofresult);
-        writeResult(fout, ids, ldofresult, scaling, "LDOF-" + kstr);
-        detachResult(database, ldofresult);
+        LDF<O, D> ldf = new LDF<>(k, distf, GaussianKernelDensityFunction.KERNEL, 2, .1);
+        OutlierResult ldfresult = ldf.run(database, relation);
+        writeResult(fout, ids, ldfresult, scaling, "LDF-" + kstr);
+        database.getHierarchy().removeSubtree(ldfresult);
       }
     });
-    // ABOD
-    if(runabod && relation.size() < 10000) {
-      try {
-        final PolynomialKernelFunction poly = new PolynomialKernelFunction(PolynomialKernelFunction.DEFAULT_DEGREE);
-        @SuppressWarnings("unchecked")
-        final DistanceFunction<DoubleVector, DoubleDistance> df = DistanceFunction.class.cast(distf);
-        LOG.verbose("Running ABOD");
-        runForEachK(new AlgRunner() {
-          @Override
-          public void run(int k, String kstr) {
-            ABOD<DoubleVector> abod = new ABOD<DoubleVector>(k, poly, df);
-            OutlierResult abodresult = abod.run(database);
-            // Setup scaling
-            StandardDeviationScaling scaling = new MinusLogStandardDeviationScaling(null, 1.0);
-            scaling.prepare(abodresult);
-            writeResult(fout, ids, abodresult, scaling, "ABOD-" + kstr);
-            detachResult(database, abodresult);
-          }
-        });
-      }
-      catch(ClassCastException e) {
-        // ABOD might just be not appropriate.
-        LOG.warning("Running ABOD failed - probably not appropriate to this data type / distance?", e);
-      }
-    }
-  }
-
-  /**
-   * Avoid that (future changes?) keep a reference to the result.
-   * 
-   * @param database Database
-   * @param discardresult Result to discard.
-   */
-  void detachResult(Database database, OutlierResult discardresult) {
-    final ResultHierarchy hier = database.getHierarchy();
-    for(Result parent : hier.getParents(discardresult)) {
-      hier.remove(parent, discardresult);
-    }
   }
 
   /**
@@ -324,15 +304,21 @@ public class ComputeKNNOutlierScores<O, D extends NumberDistance<D, ?>> extends
    * @param out Output stream
    * @param ids DBIDs
    * @param result Outlier result
-   * @param scaling Scaling function.
+   * @param scaling Scaling function
    * @param label Identification label
    */
   void writeResult(PrintStream out, DBIDs ids, OutlierResult result, ScalingFunction scaling, String label) {
+    if(scaling instanceof OutlierScalingFunction) {
+      ((OutlierScalingFunction) scaling).prepare(result);
+    }
     out.append(label);
     Relation<Double> scores = result.getScores();
-    for (DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
-      final double value = scaling.getScaled(scores.get(iter));
-      out.append(" ").append(FormatUtil.format(value, FormatUtil.NF8));
+    for(DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
+      double value = scores.get(iter);
+      if(scaling != null) {
+        value = scaling.getScaled(value);
+      }
+      out.append(' ').append(FormatUtil.NF.format(value));
     }
     out.append(FormatUtil.NEWLINE);
   }
@@ -369,7 +355,7 @@ public class ComputeKNNOutlierScores<O, D extends NumberDistance<D, ?>> extends
    * 
    * @apiviz.exclude
    */
-  public static class Parameterizer<O, D extends NumberDistance<D, ?>> extends AbstractApplication.Parameterizer {
+  public static class Parameterizer<O extends NumberVector<?>, D extends NumberDistance<D, ?>> extends AbstractApplication.Parameterizer {
     /**
      * Option ID for k step size.
      */
@@ -385,6 +371,11 @@ public class ComputeKNNOutlierScores<O, D extends NumberDistance<D, ?>> extends
      */
     public static final OptionID MAXK_ID = new OptionID("maxk", "Maximum value for k.");
 
+    /**
+     * Option ID for scaling class
+     */
+    public static final OptionID SCALING_ID = new OptionID("scaling", "Scaling function.");
+
     /**
      * k step size
      */
@@ -415,6 +406,11 @@ public class ComputeKNNOutlierScores<O, D extends NumberDistance<D, ?>> extends
      */
     ByLabelOutlier bylabel;
 
+    /**
+     * Scaling function.
+     */
+    ScalingFunction scaling = null;
+
     /**
      * Output destination file
      */
@@ -432,7 +428,7 @@ public class ComputeKNNOutlierScores<O, D extends NumberDistance<D, ?>> extends
       }
       // k parameters
       IntParameter stepkP = new IntParameter(STEPK_ID);
-      stepkP.addConstraint(new GreaterConstraint(0));
+      stepkP.addConstraint(CommonConstraints.GREATER_EQUAL_ONE_INT);
       if(config.grab(stepkP)) {
         stepk = stepkP.getValue();
       }
@@ -445,18 +441,24 @@ public class ComputeKNNOutlierScores<O, D extends NumberDistance<D, ?>> extends
         startk = stepk;
       }
       IntParameter maxkP = new IntParameter(MAXK_ID);
-      maxkP.addConstraint(new GreaterConstraint(0));
+      maxkP.addConstraint(CommonConstraints.GREATER_EQUAL_ONE_INT);
       if(config.grab(maxkP)) {
         maxk = maxkP.getValue();
       }
       bylabel = config.tryInstantiate(ByLabelOutlier.class);
       // Output
       outfile = super.getParameterOutputFile(config, "File to output the resulting score vectors to.");
+
+      ObjectParameter<ScalingFunction> scalingP = new ObjectParameter<>(SCALING_ID, ScalingFunction.class);
+      scalingP.setOptional(true);
+      if(config.grab(scalingP)) {
+        scaling = scalingP.instantiateClass(config);
+      }
     }
 
     @Override
-    protected AbstractApplication makeInstance() {
-      return new ComputeKNNOutlierScores<O, D>(inputstep, distf, startk, stepk, maxk, bylabel, outfile);
+    protected ComputeKNNOutlierScores<O, D> makeInstance() {
+      return new ComputeKNNOutlierScores<>(inputstep, distf, startk, stepk, maxk, bylabel, outfile, scaling);
     }
   }
 
@@ -466,6 +468,6 @@ public class ComputeKNNOutlierScores<O, D extends NumberDistance<D, ?>> extends
    * @param args Command line parameters.
    */
   public static void main(String[] args) {
-    ComputeKNNOutlierScores.runCLIApplication(ComputeKNNOutlierScores.class, args);
+    runCLIApplication(ComputeKNNOutlierScores.class, args);
   }
-}
\ No newline at end of file
+}
