@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.application.greedyensemble;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -24,11 +24,7 @@ package de.lmu.ifi.dbs.elki.application.greedyensemble;
  */
 
 import java.awt.image.BufferedImage;
-import java.util.Arrays;
-import java.util.Collections;
 import java.util.List;
-import java.util.Set;
-import java.util.TreeSet;
 
 import org.apache.batik.util.SVGConstants;
 
@@ -47,15 +43,20 @@ import de.lmu.ifi.dbs.elki.evaluation.roc.ROC;
 import de.lmu.ifi.dbs.elki.evaluation.similaritymatrix.ComputeSimilarityMatrixImage;
 import de.lmu.ifi.dbs.elki.evaluation.similaritymatrix.ComputeSimilarityMatrixImage.SimilarityMatrix;
 import de.lmu.ifi.dbs.elki.logging.Logging;
+import de.lmu.ifi.dbs.elki.logging.progress.FiniteProgress;
 import de.lmu.ifi.dbs.elki.math.DoubleMinMax;
 import de.lmu.ifi.dbs.elki.math.geometry.XYCurve;
+import de.lmu.ifi.dbs.elki.math.linearalgebra.Vector;
 import de.lmu.ifi.dbs.elki.result.ResultUtil;
 import de.lmu.ifi.dbs.elki.utilities.DatabaseUtil;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
+import de.lmu.ifi.dbs.elki.utilities.ensemble.EnsembleVoting;
+import de.lmu.ifi.dbs.elki.utilities.ensemble.EnsembleVotingMean;
 import de.lmu.ifi.dbs.elki.utilities.exceptions.AbortException;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
-import de.lmu.ifi.dbs.elki.utilities.pairs.DoubleIntPair;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.ObjectParameter;
 import de.lmu.ifi.dbs.elki.utilities.scaling.LinearScaling;
+import de.lmu.ifi.dbs.elki.utilities.scaling.ScalingFunction;
 import de.lmu.ifi.dbs.elki.visualization.VisualizationTask;
 import de.lmu.ifi.dbs.elki.visualization.VisualizerContext;
 import de.lmu.ifi.dbs.elki.visualization.VisualizerParameterizer;
@@ -104,89 +105,103 @@ public class VisualizePairwiseGainMatrix extends AbstractApplication {
    */
   private VisualizerParameterizer vispar;
 
+  /**
+   * Outlier scaling to apply during preprocessing.
+   */
+  private ScalingFunction prescaling;
+
+  /**
+   * Ensemble voting function.
+   */
+  private EnsembleVoting voting;
+
   /**
    * Constructor.
    * 
    * @param inputstep Input step
+   * @param prescaling Scaling function for input scores.
+   * @param voting Voting function
    * @param vispar Visualizer parameterizer
    */
-  public VisualizePairwiseGainMatrix(InputStep inputstep, VisualizerParameterizer vispar) {
+  public VisualizePairwiseGainMatrix(InputStep inputstep, ScalingFunction prescaling, EnsembleVoting voting, VisualizerParameterizer vispar) {
     super();
     this.inputstep = inputstep;
+    this.prescaling = prescaling;
+    this.voting = voting;
     this.vispar = vispar;
   }
 
   @Override
   public void run() {
     final Database database = inputstep.getDatabase();
-    final Relation<NumberVector<?>> relation = database.getRelation(TypeUtil.NUMBER_VECTOR_FIELD);
+    Relation<NumberVector<?>> relation = database.getRelation(TypeUtil.NUMBER_VECTOR_FIELD);
     final Relation<String> labels = DatabaseUtil.guessLabelRepresentation(database);
     final DBID firstid = DBIDUtil.deref(labels.iterDBIDs());
     final String firstlabel = labels.get(firstid);
-    if(!firstlabel.matches("bylabel")) {
+    if (!firstlabel.matches(".*by.?label.*")) {
       throw new AbortException("No 'by label' reference outlier found, which is needed for weighting!");
     }
+    relation = GreedyEnsembleExperiment.applyPrescaling(prescaling, relation, firstid);
 
     // Dimensionality and reference vector
     final int dim = RelationUtil.dimensionality(relation);
     final NumberVector<?> refvec = relation.get(firstid);
 
     // Build the truth vector
-    Set<Integer> pos = new TreeSet<Integer>();
-    final DoubleIntPair[] combined = new DoubleIntPair[dim];
-    {
-      for(int d = 0; d < dim; d++) {
-        combined[d] = new DoubleIntPair(0, d);
-        if(refvec.doubleValue(d) > 0) {
-          pos.add(d);
-        }
-      }
-    }
+    ROC.VectorNonZero pos = new ROC.VectorNonZero(refvec);
 
     ArrayModifiableDBIDs ids = DBIDUtil.newArray(relation.getDBIDs());
     ids.remove(firstid);
+    ids.sort();
     final int size = ids.size();
 
     double[][] data = new double[size][size];
-    DoubleMinMax minmax = new DoubleMinMax();
+    DoubleMinMax minmax = new DoubleMinMax(), commax = new DoubleMinMax();
 
     {
+      FiniteProgress prog = LOG.isVerbose() ? new FiniteProgress("Computing ensemble gain.", size * (size + 1) >> 1, LOG) : null;
+      double[] buf = new double[2]; // Vote combination buffer.
       int a = 0;
-      for(DBIDIter id = ids.iter(); id.valid(); id.advance(), a++) {
+      for (DBIDIter id = ids.iter(); id.valid(); id.advance(), a++) {
         final NumberVector<?> veca = relation.get(id);
         // Direct AUC score:
         {
-          for(int d = 0; d < dim; d++) {
-            combined[d].first = veca.doubleValue(d);
-            combined[d].second = d;
-          }
-          Arrays.sort(combined, Collections.reverseOrder(DoubleIntPair.BYFIRST_COMPARATOR));
-          double auc = XYCurve.areaUnderCurve(ROC.materializeROC(dim, pos, Arrays.asList(combined).iterator()));
+          double auc = XYCurve.areaUnderCurve(ROC.materializeROC(pos, new ROC.DecreasingVectorIter(veca)));
           data[a][a] = auc;
           // minmax.put(auc);
-          // logger.verbose(auc + " " + labels.get(ids.get(a)));
+          if (prog != null) {
+            prog.incrementProcessed(LOG);
+          }
         }
         // Compare to others, exploiting symmetry
         DBIDArrayIter id2 = ids.iter();
         id2.seek(a + 1);
-        for(int b = a + 1; b < size; b++, id2.advance()) {
+        for (int b = a + 1; b < size; b++, id2.advance()) {
           final NumberVector<?> vecb = relation.get(id2);
-          for(int d = 0; d < dim; d++) {
-            combined[d].first = veca.doubleValue(d) + vecb.doubleValue(d);
-            combined[d].second = d;
+          double[] combined = new double[dim];
+          for (int d = 0; d < dim; d++) {
+            buf[0] = veca.doubleValue(d);
+            buf[1] = vecb.doubleValue(d);
+            combined[d] = voting.combine(buf);
           }
-          Arrays.sort(combined, Collections.reverseOrder(DoubleIntPair.BYFIRST_COMPARATOR));
-          double auc = XYCurve.areaUnderCurve(ROC.materializeROC(dim, pos, Arrays.asList(combined).iterator()));
+          double auc = XYCurve.areaUnderCurve(ROC.materializeROC(pos, new ROC.DecreasingVectorIter(new Vector(combined))));
           // logger.verbose(auc + " " + labels.get(ids.get(a)) + " " +
           // labels.get(ids.get(b)));
           data[a][b] = auc;
           data[b][a] = auc;
+          commax.put(data[a][b]);
           // minmax.put(auc);
+          if (prog != null) {
+            prog.incrementProcessed(LOG);
+          }
         }
       }
+      if (prog != null) {
+        prog.ensureCompleted(LOG);
+      }
     }
-    for(int a = 0; a < size; a++) {
-      for(int b = a + 1; b < size; b++) {
+    for (int a = 0; a < size; a++) {
+      for (int b = a + 1; b < size; b++) {
         double ref = Math.max(data[a][a], data[b][b]);
         data[a][b] = (data[a][b] - ref) / (1 - ref);
         data[b][a] = (data[b][a] - ref) / (1 - ref);
@@ -195,42 +210,35 @@ public class VisualizePairwiseGainMatrix extends AbstractApplication {
         minmax.put(data[a][b]);
       }
     }
-    for(int a = 0; a < size; a++) {
+    for (int a = 0; a < size; a++) {
       data[a][a] = 0;
     }
 
-    LOG.verbose(minmax.toString());
+    LOG.verbose("Gain: " + minmax.toString() + " AUC: " + commax.toString());
 
     boolean hasneg = (minmax.getMin() < -1E-3);
     LinearScaling scale;
-    if(!hasneg) {
-      scale = new LinearScaling(minmax);
-    }
-    else {
+    if (!hasneg) {
+      scale = LinearScaling.fromMinMax(0., minmax.getMax());
+    } else {
       scale = LinearScaling.fromMinMax(0.0, Math.max(minmax.getMax(), -minmax.getMin()));
     }
+    scale = LinearScaling.fromMinMax(0., .5);
 
     BufferedImage img = new BufferedImage(size, size, BufferedImage.TYPE_INT_RGB);
-    for(int x = 0; x < size; x++) {
-      for(int y = x; y < size; y++) {
+    for (int x = 0; x < size; x++) {
+      for (int y = x; y < size; y++) {
         double val = data[x][y];
-        val = scale.getScaled(val);
+        val = Math.max(-1, Math.min(1., scale.getScaled(val)));
         // Compute color:
         final int col;
         {
-          if(!hasneg) {
-            int ival = 0xFF & (int) (255 * Math.max(0, val));
-            col = 0xff000000 | (ival << 16) | (ival << 8) | ival;
-          }
-          else {
-            if(val >= 0) {
-              int ival = 0xFF & (int) (255 * val);
-              col = 0xff000000 | (ival << 8);
-            }
-            else {
-              int ival = 0xFF & (int) (255 * -val);
-              col = 0xff000000 | (ival << 16);
-            }
+          if (val >= 0) {
+            int ival = 0xFF & (int) (255 * val);
+            col = 0xff000000 | (ival << 8);
+          } else {
+            int ival = 0xFF & (int) (255 * -val);
+            col = 0xff000000 | (ival << 16);
           }
         }
         img.setRGB(x, y, col);
@@ -247,8 +255,8 @@ public class VisualizePairwiseGainMatrix extends AbstractApplication {
     factory.processNewResult(database, database);
 
     List<VisualizationTask> tasks = ResultUtil.filterResults(database, VisualizationTask.class);
-    for(VisualizationTask task : tasks) {
-      if(task.getFactory() == factory) {
+    for (VisualizationTask task : tasks) {
+      if (task.getFactory() == factory) {
         showVisualization(context, factory, task);
       }
     }
@@ -284,13 +292,23 @@ public class VisualizePairwiseGainMatrix extends AbstractApplication {
     /**
      * Data source.
      */
-    InputStep inputstep;
+    private InputStep inputstep;
 
     /**
      * Parameterizer for visualizers.
      */
     private VisualizerParameterizer vispar;
 
+    /**
+     * Outlier scaling to apply during preprocessing.
+     */
+    private ScalingFunction prescaling;
+
+    /**
+     * Voring function.
+     */
+    private EnsembleVoting voting;
+
     @Override
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
@@ -298,11 +316,23 @@ public class VisualizePairwiseGainMatrix extends AbstractApplication {
       inputstep = config.tryInstantiate(InputStep.class);
       // Visualization options
       vispar = config.tryInstantiate(VisualizerParameterizer.class);
+
+      // Prescaling
+      ObjectParameter<ScalingFunction> prescalingP = new ObjectParameter<>(GreedyEnsembleExperiment.Parameterizer.PRESCALING_ID, ScalingFunction.class);
+      prescalingP.setOptional(true);
+      if (config.grab(prescalingP)) {
+        prescaling = prescalingP.instantiateClass(config);
+      }
+
+      ObjectParameter<EnsembleVoting> votingP = new ObjectParameter<>(GreedyEnsembleExperiment.Parameterizer.VOTING_ID, EnsembleVoting.class, EnsembleVotingMean.class);
+      if (config.grab(votingP)) {
+        voting = votingP.instantiateClass(config);
+      }
     }
 
     @Override
-    protected AbstractApplication makeInstance() {
-      return new VisualizePairwiseGainMatrix(inputstep, vispar);
+    protected VisualizePairwiseGainMatrix makeInstance() {
+      return new VisualizePairwiseGainMatrix(inputstep, prescaling, voting, vispar);
     }
   }
 
@@ -312,6 +342,6 @@ public class VisualizePairwiseGainMatrix extends AbstractApplication {
    * @param args Command line parameters.
    */
   public static void main(String[] args) {
-    VisualizePairwiseGainMatrix.runCLIApplication(VisualizePairwiseGainMatrix.class, args);
+    runCLIApplication(VisualizePairwiseGainMatrix.class, args);
   }
-}
\ No newline at end of file
+}
