@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.application.internal;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -67,7 +67,7 @@ public class CheckELKIServices {
   /**
    * Package to skip matches in - unreleased code.
    */
-  private String[] skippackages = { "experimentalcode." };
+  private String[] skippackages = { "experimentalcode.", "project." };
 
   /**
    * Main method.
@@ -126,11 +126,11 @@ public class CheckELKIServices {
       cls = Class.forName(prop);
     }
     catch(ClassNotFoundException e) {
-      LOG.warning("Property is not a class name: " + prop);
+      LOG.warning("Service file name is not a class name: " + prop);
       return;
     }
     List<Class<?>> impls = InspectionUtil.findAllImplementations(cls, false);
-    HashSet<String> names = new HashSet<String>();
+    HashSet<String> names = new HashSet<>();
     for(Class<?> c2 : impls) {
       boolean skip = false;
       for(String pkg : skippackages) {
@@ -176,7 +176,7 @@ public class CheckELKIServices {
     }
     if(names.size() > 0) {
       // format for copy & paste to properties file:
-      ArrayList<String> sorted = new ArrayList<String>(names);
+      ArrayList<String> sorted = new ArrayList<>(names);
       // TODO: sort by package, then classname
       Collections.sort(sorted);
       if(!update) {
