@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.data;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -82,7 +82,7 @@ public class ModifiableHyperBoundingBox extends HyperBoundingBox {
    *        <code>dimension</code>
    */
   public void setMax(int dimension, double value) {
-    max[dimension - 1] = value;
+    max[dimension] = value;
   }
 
   /**
@@ -95,7 +95,7 @@ public class ModifiableHyperBoundingBox extends HyperBoundingBox {
    *        <code>dimension</code>
    */
   public void setMin(int dimension, double value) {
-    max[dimension - 1] = value;
+    max[dimension] = value;
   }
 
   /**
