@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.data;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -23,15 +23,15 @@ package de.lmu.ifi.dbs.elki.data;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-import gnu.trove.impl.unmodifiable.TUnmodifiableIntDoubleMap;
 import gnu.trove.iterator.TIntDoubleIterator;
 import gnu.trove.map.TIntDoubleMap;
-import gnu.trove.map.hash.TIntDoubleHashMap;
 
+import java.io.IOException;
+import java.nio.ByteBuffer;
 import java.util.Arrays;
-import java.util.BitSet;
 
 import de.lmu.ifi.dbs.elki.math.linearalgebra.Vector;
+import de.lmu.ifi.dbs.elki.persistent.ByteArrayUtil;
 import de.lmu.ifi.dbs.elki.persistent.ByteBufferSerializer;
 import de.lmu.ifi.dbs.elki.utilities.datastructures.arraylike.ArrayAdapter;
 import de.lmu.ifi.dbs.elki.utilities.datastructures.arraylike.NumberArrayAdapter;
@@ -47,13 +47,17 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.AbstractParameterizer;
  * 
  * @author Arthur Zimek
  */
-// TODO: implement ByteArraySerializer<SparseDoubleVector>
 public class SparseDoubleVector extends AbstractNumberVector<Double> implements SparseNumberVector<Double> {
   /**
    * Static instance.
    */
   public static final SparseDoubleVector.Factory FACTORY = new SparseDoubleVector.Factory();
 
+  /**
+   * Serializer using varint encoding.
+   */
+  public static final ByteBufferSerializer<SparseDoubleVector> VARIABLE_SERIALIZER = new VariableSerializer();
+
   /**
    * Indexes of values.
    */
@@ -94,7 +98,7 @@ public class SparseDoubleVector extends AbstractNumberVector<Double> implements
    *         zero is bigger than the given dimensionality)
    */
   public SparseDoubleVector(TIntDoubleMap values, int dimensionality) throws IllegalArgumentException {
-    if (values.size() > dimensionality) {
+    if(values.size() > dimensionality) {
       throw new IllegalArgumentException("values.size() > dimensionality!");
     }
 
@@ -103,7 +107,7 @@ public class SparseDoubleVector extends AbstractNumberVector<Double> implements
     // Import and sort the indexes
     {
       TIntDoubleIterator iter = values.iterator();
-      for (int i = 0; iter.hasNext(); i++) {
+      for(int i = 0; iter.hasNext(); i++) {
         iter.advance();
         this.indexes[i] = iter.key();
       }
@@ -111,13 +115,13 @@ public class SparseDoubleVector extends AbstractNumberVector<Double> implements
     }
     // Import the values accordingly
     {
-      for (int i = 0; i < values.size(); i++) {
+      for(int i = 0; i < values.size(); i++) {
         this.values[i] = values.get(this.indexes[i]);
       }
     }
     this.dimensionality = dimensionality;
     final int maxdim = getMaxDim();
-    if (maxdim > dimensionality) {
+    if(maxdim > dimensionality) {
       throw new IllegalArgumentException("Given dimensionality " + dimensionality + " is too small w.r.t. the given values (occurring maximum: " + maxdim + ").");
     }
   }
@@ -128,9 +132,10 @@ public class SparseDoubleVector extends AbstractNumberVector<Double> implements
    * @return the maximum dimensionality seen
    */
   private int getMaxDim() {
-    if (this.indexes.length == 0) {
+    if(this.indexes.length == 0) {
       return 0;
-    } else {
+    }
+    else {
       return this.indexes[this.indexes.length - 1];
     }
   }
@@ -150,8 +155,8 @@ public class SparseDoubleVector extends AbstractNumberVector<Double> implements
     // Count the number of non-zero entries
     int size = 0;
     {
-      for (int i = 0; i < values.length; i++) {
-        if (values[i] != 0.0f) {
+      for(int i = 0; i < values.length; i++) {
+        if(values[i] != 0.0f) {
           size++;
         }
       }
@@ -162,10 +167,10 @@ public class SparseDoubleVector extends AbstractNumberVector<Double> implements
     // Copy the values
     {
       int pos = 0;
-      for (int i = 0; i < values.length; i++) {
+      for(int i = 0; i < values.length; i++) {
         double value = values[i];
-        if (value != 0.0f) {
-          this.indexes[pos] = i + 1;
+        if(value != 0.0f) {
+          this.indexes[pos] = i;
           this.values[pos] = value;
           pos++;
         }
@@ -190,7 +195,7 @@ public class SparseDoubleVector extends AbstractNumberVector<Double> implements
   @Override
   public void setDimensionality(int dimensionality) throws IllegalArgumentException {
     final int maxdim = getMaxDim();
-    if (maxdim > dimensionality) {
+    if(maxdim > dimensionality) {
       throw new IllegalArgumentException("Given dimensionality " + dimensionality + " is too small w.r.t. the given values (occurring maximum: " + maxdim + ").");
     }
     this.dimensionality = dimensionality;
@@ -200,29 +205,34 @@ public class SparseDoubleVector extends AbstractNumberVector<Double> implements
   @Deprecated
   public Double getValue(int dimension) {
     int pos = Arrays.binarySearch(this.indexes, dimension);
-    if (pos >= 0) {
+    if(pos >= 0) {
       return values[pos];
-    } else {
+    }
+    else {
       return 0.0;
     }
   }
 
   @Override
+  @Deprecated
   public double doubleValue(int dimension) {
     int pos = Arrays.binarySearch(this.indexes, dimension);
-    if (pos >= 0) {
+    if(pos >= 0) {
       return values[pos];
-    } else {
+    }
+    else {
       return 0.0;
     }
   }
 
   @Override
+  @Deprecated
   public long longValue(int dimension) {
     int pos = Arrays.binarySearch(this.indexes, dimension);
-    if (pos >= 0) {
+    if(pos >= 0) {
       return (long) values[pos];
-    } else {
+    }
+    else {
       return 0;
     }
   }
@@ -258,7 +268,7 @@ public class SparseDoubleVector extends AbstractNumberVector<Double> implements
   public String toString() {
     StringBuilder featureLine = new StringBuilder();
     featureLine.append(this.indexes.length);
-    for (int i = 0; i < this.indexes.length; i++) {
+    for(int i = 0; i < this.indexes.length; i++) {
       featureLine.append(ATTRIBUTE_SEPARATOR);
       featureLine.append(this.indexes[i]);
       featureLine.append(ATTRIBUTE_SEPARATOR);
@@ -275,12 +285,62 @@ public class SparseDoubleVector extends AbstractNumberVector<Double> implements
    */
   private double[] getValues() {
     double[] vals = new double[dimensionality];
-    for (int i = 0; i < indexes.length; i++) {
+    for(int i = 0; i < indexes.length; i++) {
       vals[this.indexes[i]] = this.values[i];
     }
     return vals;
   }
 
+  @Override
+  public int iter() {
+    return 0;
+  }
+
+  @Override
+  public int iterDim(int iter) {
+    return indexes[iter];
+  }
+
+  @Override
+  public int iterAdvance(int iter) {
+    return iter + 1;
+  }
+
+  @Override
+  public boolean iterValid(int iter) {
+    return iter < indexes.length;
+  }
+
+  @Override
+  public double iterDoubleValue(int iter) {
+    return values[iter];
+  }
+
+  @Override
+  public float iterFloatValue(int iter) {
+    return (float) values[iter];
+  }
+
+  @Override
+  public int iterIntValue(int iter) {
+    return (int) values[iter];
+  }
+
+  @Override
+  public short iterShortValue(int iter) {
+    return (short) values[iter];
+  }
+
+  @Override
+  public long iterLongValue(int iter) {
+    return (long) values[iter];
+  }
+
+  @Override
+  public byte iterByteValue(int iter) {
+    return (byte) values[iter];
+  }
+
   /**
    * Factory class.
    * 
@@ -293,10 +353,10 @@ public class SparseDoubleVector extends AbstractNumberVector<Double> implements
     public <A> SparseDoubleVector newFeatureVector(A array, ArrayAdapter<Double, A> adapter) {
       int dim = adapter.size(array);
       double[] values = new double[dim];
-      for (int i = 0; i < dim; i++) {
+      for(int i = 0; i < dim; i++) {
         values[i] = adapter.get(array, i);
       }
-      // TODO: inefficient
+      // TODO: improve efficiency
       return new SparseDoubleVector(values);
     }
 
@@ -304,10 +364,10 @@ public class SparseDoubleVector extends AbstractNumberVector<Double> implements
     public <A> SparseDoubleVector newNumberVector(A array, NumberArrayAdapter<?, ? super A> adapter) {
       int dim = adapter.size(array);
       double[] values = new double[dim];
-      for (int i = 0; i < dim; i++) {
+      for(int i = 0; i < dim; i++) {
         values[i] = adapter.getDouble(array, i);
       }
-      // TODO: inefficient
+      // TODO: improve efficiency
       return new SparseDoubleVector(values);
     }
 
@@ -318,10 +378,9 @@ public class SparseDoubleVector extends AbstractNumberVector<Double> implements
 
     @Override
     public ByteBufferSerializer<SparseDoubleVector> getDefaultSerializer() {
-      // FIXME: add a serializer
-      return null;
+      return VARIABLE_SERIALIZER;
     }
-    
+
     @Override
     public Class<? super SparseDoubleVector> getRestrictionClass() {
       return SparseDoubleVector.class;
@@ -342,17 +401,47 @@ public class SparseDoubleVector extends AbstractNumberVector<Double> implements
     }
   }
 
-  @Override
-  public BitSet getNotNullMask() {
-    BitSet b = new BitSet();
-    for (int key : indexes) {
-      b.set(key);
-    }
-    return b;
-  }
-
   /**
-   * Empty map.
+   * Serialization class using VarInt encodings.
+   * 
+   * @author Erich Schubert
+   * 
+   * @apiviz.uses SparseDoubleVector - - «serializes»
    */
-  public static final TIntDoubleMap EMPTYMAP = new TUnmodifiableIntDoubleMap(new TIntDoubleHashMap());
+  public static class VariableSerializer implements ByteBufferSerializer<SparseDoubleVector> {
+    @Override
+    public SparseDoubleVector fromByteBuffer(ByteBuffer buffer) throws IOException {
+      final int dimensionality = ByteArrayUtil.readUnsignedVarint(buffer);
+      final int nonzero = ByteArrayUtil.readUnsignedVarint(buffer);
+      final int[] dims = new int[nonzero];
+      final double[] values = new double[nonzero];
+      for(int i = 0; i < nonzero; i++) {
+        dims[i] = ByteArrayUtil.readUnsignedVarint(buffer);
+        values[i] = buffer.getDouble();
+      }
+      return new SparseDoubleVector(dims, values, dimensionality);
+    }
+
+    @Override
+    public void toByteBuffer(ByteBuffer buffer, SparseDoubleVector vec) throws IOException {
+      ByteArrayUtil.writeUnsignedVarint(buffer, vec.dimensionality);
+      ByteArrayUtil.writeUnsignedVarint(buffer, vec.values.length);
+      for(int i = 0; i < vec.values.length; i++) {
+        ByteArrayUtil.writeUnsignedVarint(buffer, vec.indexes[i]);
+        buffer.putDouble(vec.values[i]);
+      }
+    }
+
+    @Override
+    public int getByteSize(SparseDoubleVector vec) {
+      int sum = 0;
+      sum += ByteArrayUtil.getUnsignedVarintSize(vec.dimensionality);
+      sum += ByteArrayUtil.getUnsignedVarintSize(vec.values.length);
+      for(int d : vec.indexes) {
+        sum += ByteArrayUtil.getUnsignedVarintSize(d);
+      }
+      sum += vec.values.length * ByteArrayUtil.SIZE_DOUBLE;
+      return sum;
+    }
+  }
 }
