@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.data.images;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -29,8 +29,7 @@ import java.util.List;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.AbstractParameterizer;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.WrongParameterValueException;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.GreaterEqualConstraint;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.ListEachConstraint;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.CommonConstraints;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.ListSizeConstraint;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntListParameter;
@@ -127,7 +126,7 @@ public class ComputeHSBColorHistogram extends AbstractComputeColorHistogram {
       super.makeOptions(config);
       final IntListParameter param = new IntListParameter(BINSPERPLANE_ID);
       param.addConstraint(new ListSizeConstraint(3));
-      param.addConstraint(new ListEachConstraint<Integer>(new GreaterEqualConstraint(1)));
+      param.addConstraint(CommonConstraints.GREATER_EQUAL_ONE_INT_LIST);
 
       if(config.grab(param)) {
         List<Integer> quant = param.getValue();
