@@ -1,9 +1,10 @@
 package de.lmu.ifi.dbs.elki.data.projection;
+
 /*
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -29,9 +30,15 @@ import de.lmu.ifi.dbs.elki.data.type.SimpleTypeInformation;
 import de.lmu.ifi.dbs.elki.data.type.TypeInformation;
 import de.lmu.ifi.dbs.elki.data.type.VectorFieldTypeInformation;
 import de.lmu.ifi.dbs.elki.data.type.VectorTypeInformation;
+import de.lmu.ifi.dbs.elki.datasource.filter.transform.NumberVectorFeatureSelectionFilter;
 import de.lmu.ifi.dbs.elki.utilities.datastructures.arraylike.ArrayAdapter;
 import de.lmu.ifi.dbs.elki.utilities.datastructures.arraylike.ArrayLikeUtil;
 import de.lmu.ifi.dbs.elki.utilities.datastructures.arraylike.SubsetArrayAdapter;
+import de.lmu.ifi.dbs.elki.utilities.exceptions.AbortException;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.AbstractParameterizer;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.CommonConstraints;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntListParameter;
 
 /**
  * Projection class for number vectors.
@@ -61,16 +68,14 @@ public class FeatureSelection<V extends FeatureVector<F>, F> implements Projecti
    * Array adapter.
    */
   protected ArrayAdapter<F, V> adapter;
-  
+
   /**
    * Constructor.
    * 
    * @param dims Dimensions
-   * @param factory Object factory
    */
-  public FeatureSelection(int[] dims, FeatureVector.Factory<V, F> factory) {
-    this.adapter = new SubsetArrayAdapter<F, V>(getAdapter(factory), dims);
-    this.factory = factory;
+  public FeatureSelection(int[] dims) {
+    this.adapter = new SubsetArrayAdapter<>(getAdapter(factory), dims);
     this.dimensionality = dims.length;
 
     int mind = 0;
@@ -80,11 +85,21 @@ public class FeatureSelection<V extends FeatureVector<F>, F> implements Projecti
     this.mindim = mind;
   }
 
+  @SuppressWarnings("unchecked")
+  @Override
+  public void initialize(SimpleTypeInformation<V> in) {
+    final VectorFieldTypeInformation<V> vin = (VectorFieldTypeInformation<V>) in;
+    factory = (FeatureVector.Factory<V, F>) vin.getFactory();
+    if(vin.getDimensionality() < mindim) {
+      throw new AbortException("Data does not have enough dimensions for this projection!");
+    }
+  }
+
   @Override
   public V project(V data) {
     return factory.newFeatureVector(data, adapter);
   }
-  
+
   /**
    * Choose the best adapter for this.
    * 
@@ -103,15 +118,43 @@ public class FeatureSelection<V extends FeatureVector<F>, F> implements Projecti
 
   @Override
   public SimpleTypeInformation<V> getOutputDataTypeInformation() {
-    @SuppressWarnings("unchecked")
-    final Class<V> cls = (Class<V>) factory.getClass();
-    return new VectorFieldTypeInformation<V>(cls, dimensionality);
+    return new VectorFieldTypeInformation<>(factory, dimensionality);
   }
 
   @Override
   public TypeInformation getInputDataTypeInformation() {
     @SuppressWarnings("unchecked")
-    final Class<V> cls = (Class<V>) factory.getClass();
-    return new VectorTypeInformation<V>(cls, mindim, Integer.MAX_VALUE);
+    final Class<V> cls = (Class<V>) factory.getRestrictionClass();
+    return new VectorTypeInformation<>(cls, mindim, Integer.MAX_VALUE);
   }
-}
\ No newline at end of file
+
+  /**
+   * Parameterization class.
+   * 
+   * @author Erich Schubert
+   * 
+   * @apiviz.exclude
+   */
+  public static class Parameterizer<V extends FeatureVector<F>, F> extends AbstractParameterizer {
+    /**
+     * Dimensions to select.
+     */
+    int[] dims = null;
+
+    @Override
+    protected void makeOptions(Parameterization config) {
+      super.makeOptions(config);
+
+      IntListParameter selectedAttributesP = new IntListParameter(NumberVectorFeatureSelectionFilter.Parameterizer.SELECTED_ATTRIBUTES_ID);
+      selectedAttributesP.addConstraint(CommonConstraints.GREATER_EQUAL_ZERO_INT_LIST);
+      if(config.grab(selectedAttributesP)) {
+        dims = ArrayLikeUtil.toPrimitiveIntegerArray(selectedAttributesP.getValue());
+      }
+    }
+
+    @Override
+    protected FeatureSelection<V, F> makeInstance() {
+      return new FeatureSelection<>(dims);
+    }
+  }
+}
