@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.data.projection;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -30,10 +30,18 @@ import de.lmu.ifi.dbs.elki.data.type.SimpleTypeInformation;
 import de.lmu.ifi.dbs.elki.data.type.TypeInformation;
 import de.lmu.ifi.dbs.elki.data.type.VectorFieldTypeInformation;
 import de.lmu.ifi.dbs.elki.data.type.VectorTypeInformation;
+import de.lmu.ifi.dbs.elki.datasource.filter.transform.NumberVectorFeatureSelectionFilter;
+import de.lmu.ifi.dbs.elki.utilities.exceptions.AbortException;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.AbstractParameterizer;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.CommonConstraints;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntListParameter;
 
 /**
  * Projection class for number vectors.
  * 
+ * FIXME: Use int[] instead of a BitSet, to allow reordering?
+ * 
  * @author Erich Schubert
  * 
  * @param <V> Vector type
@@ -63,12 +71,10 @@ public class NumericalFeatureSelection<V extends NumberVector<?>> implements Pro
    * Constructor.
    * 
    * @param bits Dimensions
-   * @param factory Object factory
    */
-  public NumericalFeatureSelection(BitSet bits, NumberVector.Factory<V, ?> factory) {
+  public NumericalFeatureSelection(BitSet bits) {
     super();
     this.bits = bits;
-    this.factory = factory;
     this.dimensionality = bits.cardinality();
 
     int mind = 0;
@@ -78,6 +84,15 @@ public class NumericalFeatureSelection<V extends NumberVector<?>> implements Pro
     this.mindim = mind;
   }
 
+  @Override
+  public void initialize(SimpleTypeInformation<V> in) {
+    final VectorFieldTypeInformation<V> vin = (VectorFieldTypeInformation<V>) in;
+    factory = (NumberVector.Factory<V, ?>) vin.getFactory();
+    if(vin.getDimensionality() < mindim) {
+      throw new AbortException("Data does not have enough dimensions for this projection!");
+    }
+  }
+
   @Override
   public V project(V data) {
     double[] dbl = new double[dimensionality];
@@ -89,15 +104,44 @@ public class NumericalFeatureSelection<V extends NumberVector<?>> implements Pro
 
   @Override
   public SimpleTypeInformation<V> getOutputDataTypeInformation() {
-    @SuppressWarnings("unchecked")
-    final Class<V> cls = (Class<V>) factory.getRestrictionClass();
-    return new VectorFieldTypeInformation<V>(cls, dimensionality);
+    return new VectorFieldTypeInformation<>(factory, dimensionality);
   }
 
   @Override
   public TypeInformation getInputDataTypeInformation() {
-    @SuppressWarnings("unchecked")
-    final Class<V> cls = (Class<V>) factory.getRestrictionClass();
-    return new VectorTypeInformation<V>(cls, mindim, Integer.MAX_VALUE);
+    return new VectorTypeInformation<>(NumberVector.class, mindim, Integer.MAX_VALUE);
   }
-}
\ No newline at end of file
+
+  /**
+   * Parameterization class.
+   * 
+   * @author Erich Schubert
+   * 
+   * @apiviz.exclude
+   */
+  public static class Parameterizer<V extends NumberVector<?>> extends AbstractParameterizer {
+    /**
+     * Dimensions to select.
+     */
+    BitSet dims = new BitSet();
+
+    @Override
+    protected void makeOptions(Parameterization config) {
+      super.makeOptions(config);
+
+      IntListParameter selectedAttributesP = new IntListParameter(NumberVectorFeatureSelectionFilter.Parameterizer.SELECTED_ATTRIBUTES_ID);
+      selectedAttributesP.addConstraint(CommonConstraints.GREATER_EQUAL_ZERO_INT_LIST);
+      if(config.grab(selectedAttributesP)) {
+        dims.clear();
+        for(int in : selectedAttributesP.getValue()) {
+          dims.set(in);
+        }
+      }
+    }
+
+    @Override
+    protected NumericalFeatureSelection<V> makeInstance() {
+      return new NumericalFeatureSelection<>(dims);
+    }
+  }
+}
