@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.data.synthetic.bymodel;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -30,7 +30,7 @@ import java.util.Random;
 import de.lmu.ifi.dbs.elki.data.model.Model;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.AffineTransformation;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.Vector;
-import de.lmu.ifi.dbs.elki.math.statistics.distribution.DistributionWithRandom;
+import de.lmu.ifi.dbs.elki.math.statistics.distribution.Distribution;
 import de.lmu.ifi.dbs.elki.utilities.exceptions.UnableToComplyException;
 
 /**
@@ -40,14 +40,14 @@ import de.lmu.ifi.dbs.elki.utilities.exceptions.UnableToComplyException;
  * 
  * @author Erich Schubert
  * 
- * @apiviz.composedOf DistributionWithRandom
+ * @apiviz.composedOf Distribution
  * @apiviz.composedOf AffineTransformation
  */
 public class GeneratorSingleCluster implements GeneratorInterfaceDynamic, Model {
   /**
    * The distribution generators for each axis
    */
-  private List<DistributionWithRandom> axes = new ArrayList<DistributionWithRandom>();
+  private List<Distribution> axes = new ArrayList<>();
 
   /**
    * The transformation matrix
@@ -121,7 +121,7 @@ public class GeneratorSingleCluster implements GeneratorInterfaceDynamic, Model
    * @throws UnableToComplyException thrown when no new generators may be added
    *         anymore
    */
-  public void addGenerator(DistributionWithRandom gen) throws UnableToComplyException {
+  public void addGenerator(Distribution gen) throws UnableToComplyException {
     if(trans != null) {
       throw new UnableToComplyException("Generators may no longer be added when transformations have been applied.");
     }
@@ -231,11 +231,11 @@ public class GeneratorSingleCluster implements GeneratorInterfaceDynamic, Model
    */
   @Override
   public List<Vector> generate(int count) throws UnableToComplyException {
-    ArrayList<Vector> result = new ArrayList<Vector>(count);
+    ArrayList<Vector> result = new ArrayList<>(count);
     while(result.size() < count) {
       double[] d = new double[dim];
       int i = 0;
-      for(DistributionWithRandom axis : axes) {
+      for(Distribution axis : axes) {
         d[i] = axis.nextRandom();
         i++;
       }
@@ -269,7 +269,7 @@ public class GeneratorSingleCluster implements GeneratorInterfaceDynamic, Model
 
     double density = 1.0;
     int i = 0;
-    for(DistributionWithRandom axis : axes) {
+    for(Distribution axis : axes) {
       density = density * axis.pdf(o.get(i));
       i++;
     }
@@ -400,7 +400,7 @@ public class GeneratorSingleCluster implements GeneratorInterfaceDynamic, Model
    * @param i Generator axis i
    * @return Distribution
    */
-  public DistributionWithRandom getDistribution(int i) {
+  public Distribution getDistribution(int i) {
     return axes.get(i);
   }
 }
\ No newline at end of file
