@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.database;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -25,13 +25,13 @@ package de.lmu.ifi.dbs.elki.database;
 
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.database.query.distance.PrimitiveDistanceQuery;
+import de.lmu.ifi.dbs.elki.database.query.knn.DoubleOptimizedDistanceKNNQuery;
 import de.lmu.ifi.dbs.elki.database.query.knn.KNNQuery;
-import de.lmu.ifi.dbs.elki.database.query.knn.LinearScanKNNQuery;
+import de.lmu.ifi.dbs.elki.database.query.knn.LinearScanDistanceKNNQuery;
 import de.lmu.ifi.dbs.elki.database.query.knn.LinearScanPrimitiveDistanceKNNQuery;
-import de.lmu.ifi.dbs.elki.database.query.knn.LinearScanRawDoubleDistanceKNNQuery;
+import de.lmu.ifi.dbs.elki.database.query.range.DoubleOptimizedDistanceRangeQuery;
+import de.lmu.ifi.dbs.elki.database.query.range.LinearScanDistanceRangeQuery;
 import de.lmu.ifi.dbs.elki.database.query.range.LinearScanPrimitiveDistanceRangeQuery;
-import de.lmu.ifi.dbs.elki.database.query.range.LinearScanRangeQuery;
-import de.lmu.ifi.dbs.elki.database.query.range.LinearScanRawDoubleDistanceRangeQuery;
 import de.lmu.ifi.dbs.elki.database.query.range.RangeQuery;
 import de.lmu.ifi.dbs.elki.database.query.rknn.RKNNQuery;
 import de.lmu.ifi.dbs.elki.database.query.similarity.SimilarityQuery;
@@ -127,6 +127,7 @@ public final class QueryUtil {
    * <li>{@link de.lmu.ifi.dbs.elki.database.query.DatabaseQuery#HINT_BULK} bulk
    * query needed</li>
    * </ul>
+   * 
    * @param relation Relation used
    * @param distanceFunction Distance function
    * @param hints Optimizer hints
@@ -179,6 +180,7 @@ public final class QueryUtil {
    * <li>{@link de.lmu.ifi.dbs.elki.database.query.DatabaseQuery#HINT_BULK} bulk
    * query needed</li>
    * </ul>
+   * 
    * @param relation Relation used
    * @param distanceFunction Distance function
    * @param hints Optimizer hints
@@ -205,6 +207,7 @@ public final class QueryUtil {
    * <li>{@link de.lmu.ifi.dbs.elki.database.query.DatabaseQuery#HINT_BULK} bulk
    * query needed</li>
    * </ul>
+   * 
    * @param relation Relation used
    * @param distanceFunction Distance function
    * @param hints Optimizer hints
@@ -233,17 +236,17 @@ public final class QueryUtil {
       if(distanceQuery.getDistanceFunction() instanceof PrimitiveDoubleDistanceFunction) {
         final PrimitiveDistanceQuery<O, ?> pdq = (PrimitiveDistanceQuery<O, ?>) distanceQuery;
         @SuppressWarnings("unchecked")
-        final KNNQuery<O, ?> knnQuery = new LinearScanRawDoubleDistanceKNNQuery<O>((PrimitiveDistanceQuery<O, DoubleDistance>) pdq);
+        final KNNQuery<O, ?> knnQuery = new DoubleOptimizedDistanceKNNQuery<>((PrimitiveDistanceQuery<O, DoubleDistance>) pdq);
         @SuppressWarnings("unchecked")
         final KNNQuery<O, D> castQuery = (KNNQuery<O, D>) knnQuery;
         return castQuery;
       }
       else {
         final PrimitiveDistanceQuery<O, D> pdq = (PrimitiveDistanceQuery<O, D>) distanceQuery;
-        return new LinearScanPrimitiveDistanceKNNQuery<O, D>(pdq);
+        return new LinearScanPrimitiveDistanceKNNQuery<>(pdq);
       }
     }
-    return new LinearScanKNNQuery<O, D>(distanceQuery);
+    return new LinearScanDistanceKNNQuery<>(distanceQuery);
   }
 
   /**
@@ -260,16 +263,16 @@ public final class QueryUtil {
       if(distanceQuery.getDistanceFunction() instanceof PrimitiveDoubleDistanceFunction) {
         final PrimitiveDistanceQuery<O, ?> pdq = (PrimitiveDistanceQuery<O, ?>) distanceQuery;
         @SuppressWarnings("unchecked")
-        final RangeQuery<O, ?> knnQuery = new LinearScanRawDoubleDistanceRangeQuery<O>((PrimitiveDistanceQuery<O, DoubleDistance>) pdq);
+        final RangeQuery<O, ?> knnQuery = new DoubleOptimizedDistanceRangeQuery<>((PrimitiveDistanceQuery<O, DoubleDistance>) pdq);
         @SuppressWarnings("unchecked")
         final RangeQuery<O, D> castQuery = (RangeQuery<O, D>) knnQuery;
         return castQuery;
       }
       else {
         final PrimitiveDistanceQuery<O, D> pdq = (PrimitiveDistanceQuery<O, D>) distanceQuery;
-        return new LinearScanPrimitiveDistanceRangeQuery<O, D>(pdq);
+        return new LinearScanPrimitiveDistanceRangeQuery<>(pdq);
       }
     }
-    return new LinearScanRangeQuery<O, D>(distanceQuery);
+    return new LinearScanDistanceRangeQuery<>(distanceQuery);
   }
-}
\ No newline at end of file
+}
