@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.database.datastore.memory;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -67,7 +67,7 @@ public class ArrayDoubleStore implements WritableDoubleDataStore {
   public ArrayDoubleStore(int size, DataStoreIDMap idmap, double def) {
     super();
     this.data = new double[size];
-    if(def != 0) {
+    if (def != 0) {
       Arrays.fill(this.data, def);
     }
     this.idmap = idmap;
@@ -76,12 +76,7 @@ public class ArrayDoubleStore implements WritableDoubleDataStore {
   @Override
   @Deprecated
   public Double get(DBIDRef id) {
-    try {
-      return Double.valueOf(data[idmap.mapDBIDToOffset(id)]);
-    }
-    catch(ArrayIndexOutOfBoundsException e) {
-      return null;
-    }
+    return Double.valueOf(data[idmap.mapDBIDToOffset(id)]);
   }
 
   @Override
@@ -134,4 +129,4 @@ public class ArrayDoubleStore implements WritableDoubleDataStore {
   public String getShortName() {
     return "raw";
   }
-}
\ No newline at end of file
+}
