@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.database.datastore.memory;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -58,7 +58,7 @@ public class MapIntegerDBIDStore<T> implements WritableDataStore<T> {
    */
   public MapIntegerDBIDStore() {
     super();
-    this.data = new TIntObjectHashMap<T>();
+    this.data = new TIntObjectHashMap<>();
   }
 
   /**
@@ -67,7 +67,7 @@ public class MapIntegerDBIDStore<T> implements WritableDataStore<T> {
    * @param size Expected size
    */
   public MapIntegerDBIDStore(int size) {
-    this.data = new TIntObjectHashMap<T>(size);
+    this.data = new TIntObjectHashMap<>(size);
   }
 
   @Override
