@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.database.ids;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -47,7 +47,7 @@ public interface ArrayDBIDs extends DBIDs {
    * @param index Position
    * @param var Variable to assign the value to.
    */
-  public void assign(int index, DBIDVar var);
+  public void assignVar(int index, DBIDVar var);
 
   /**
    * Iterable
@@ -76,4 +76,13 @@ public interface ArrayDBIDs extends DBIDs {
    * @return Offset of key
    */
   public int binarySearch(DBIDRef key);
+
+  /**
+   * Slice a subarray (as view, not copy!)
+   * 
+   * @param begin Begin (inclusive)
+   * @param end End (exclusive)
+   * @return Array slice.
+   */
+  public ArrayDBIDs slice(int begin, int end);
 }
