@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.database.ids;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -25,15 +25,24 @@ package de.lmu.ifi.dbs.elki.database.ids;
 
 import java.util.Random;
 
+import de.lmu.ifi.dbs.elki.database.ids.distance.DistanceDBIDPair;
+import de.lmu.ifi.dbs.elki.database.ids.distance.DoubleDistanceDBIDPair;
+import de.lmu.ifi.dbs.elki.database.ids.distance.DoubleDistanceKNNHeap;
+import de.lmu.ifi.dbs.elki.database.ids.distance.DoubleDistanceKNNList;
+import de.lmu.ifi.dbs.elki.database.ids.distance.KNNHeap;
+import de.lmu.ifi.dbs.elki.database.ids.distance.KNNList;
+import de.lmu.ifi.dbs.elki.database.ids.generic.DoubleDistanceKNNSubList;
+import de.lmu.ifi.dbs.elki.database.ids.generic.KNNSubList;
 import de.lmu.ifi.dbs.elki.database.ids.generic.UnmodifiableArrayDBIDs;
 import de.lmu.ifi.dbs.elki.database.ids.generic.UnmodifiableDBIDs;
+import de.lmu.ifi.dbs.elki.database.ids.integer.IntegerArrayDBIDs;
 import de.lmu.ifi.dbs.elki.database.ids.integer.IntegerDBIDs;
-import de.lmu.ifi.dbs.elki.database.ids.integer.TroveArrayDBIDs;
 import de.lmu.ifi.dbs.elki.database.ids.integer.UnmodifiableIntegerArrayDBIDs;
 import de.lmu.ifi.dbs.elki.database.ids.integer.UnmodifiableIntegerDBIDs;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
 import de.lmu.ifi.dbs.elki.persistent.ByteBufferSerializer;
 import de.lmu.ifi.dbs.elki.utilities.RandomFactory;
+import de.lmu.ifi.dbs.elki.utilities.UnsafeRandom;
 
 /**
  * DBID Utility functions.
@@ -319,9 +328,33 @@ public final class DBIDUtil {
    * @return size
    */
   public static int intersectionSize(DBIDs first, DBIDs second) {
-    if (first.size() > second.size()) {
-      return intersectionSize(second, first);
+    // If exactly one is a Set, use it as second parameter.
+    if (second instanceof SetDBIDs) {
+      if (!(first instanceof SetDBIDs)) {
+        return internalIntersectionSize(first, second);
+      }
+    } else {
+      if (first instanceof SetDBIDs) {
+        return internalIntersectionSize(second, first);
+      }
     }
+    // Both are the same type: both set or both non set.
+    // Smaller goes first.
+    if (first.size() <= second.size()) {
+      return internalIntersectionSize(first, second);
+    } else {
+      return internalIntersectionSize(second, first);
+    }
+  }
+
+  /**
+   * Compute the set intersection size of two sets.
+   * 
+   * @param first First set
+   * @param second Second set
+   * @return size
+   */
+  private static int internalIntersectionSize(DBIDs first, DBIDs second) {
     int c = 0;
     for (DBIDIter it = first.iter(); it.valid(); it.advance()) {
       if (second.contains(it)) {
@@ -398,8 +431,8 @@ public final class DBIDUtil {
     if (existing instanceof StaticDBIDs) {
       return (StaticDBIDs) existing;
     }
-    if (existing instanceof TroveArrayDBIDs) {
-      return new UnmodifiableIntegerArrayDBIDs((TroveArrayDBIDs) existing);
+    if (existing instanceof IntegerArrayDBIDs) {
+      return new UnmodifiableIntegerArrayDBIDs((IntegerArrayDBIDs) existing);
     }
     if (existing instanceof IntegerDBIDs) {
       return new UnmodifiableIntegerDBIDs((IntegerDBIDs) existing);
@@ -504,15 +537,38 @@ public final class DBIDUtil {
   }
 
   /**
-   * Produce a random sample of the given DBIDs.
+   * Create an appropriate heap for the distance type.
    * 
-   * @param source Original DBIDs
-   * @param k k Parameter
-   * @param rnd Random generator
-   * @return new DBIDs
+   * This will use a double heap if appropriate.
+   * 
+   * @param distancetype distance prototype
+   * @param k K value
+   * @param <D> distance type
+   * @return New heap of size k, appropriate for this distance type.
    */
-  public static ModifiableDBIDs randomSample(DBIDs source, int k, RandomFactory rnd) {
-    return randomSample(source, k, rnd.getRandom());
+  public static <D extends Distance<D>> KNNHeap<D> newHeap(D distancetype, int k) {
+    return DBIDFactory.FACTORY.newHeap(distancetype, k);
+  }
+
+  /**
+   * Create an appropriate heap for double distances.
+   * 
+   * @param k K value
+   * @return New heap of size k, appropriate for this distance type.
+   */
+  public static DoubleDistanceKNNHeap newDoubleDistanceHeap(int k) {
+    return DBIDFactory.FACTORY.newDoubleDistanceHeap(k);
+  }
+
+  /**
+   * Build a new heap from a given list.
+   * 
+   * @param exist Existing result
+   * @param <D> Distance type
+   * @return New heap
+   */
+  public static <D extends Distance<D>> KNNHeap<D> newHeap(KNNList<D> exist) {
+    return DBIDFactory.FACTORY.newHeap(exist);
   }
 
   /**
@@ -522,7 +578,7 @@ public final class DBIDUtil {
    * @param rnd Random generator
    */
   public static void randomShuffle(ArrayModifiableDBIDs ids, RandomFactory rnd) {
-    randomShuffle(ids, rnd.getRandom(), ids.size());
+    randomShuffle(ids, rnd.getSingleThreadedRandom(), ids.size());
   }
 
   /**
@@ -578,6 +634,18 @@ public final class DBIDUtil {
     }
   }
 
+  /**
+   * Produce a random sample of the given DBIDs.
+   * 
+   * @param source Original DBIDs
+   * @param k k Parameter
+   * @param rnd Random generator
+   * @return new DBIDs
+   */
+  public static ModifiableDBIDs randomSample(DBIDs source, int k, RandomFactory rnd) {
+    return randomSample(source, k, rnd.getSingleThreadedRandom());
+  }
+
   /**
    * Produce a random sample of the given DBIDs.
    * 
@@ -587,12 +655,13 @@ public final class DBIDUtil {
    * @return new DBIDs
    */
   public static ModifiableDBIDs randomSample(DBIDs source, int k, Random random) {
-    if (k <= 0 || k > source.size()) {
+    if (k < 0 || k > source.size()) {
       throw new IllegalArgumentException("Illegal value for size of random sample: " + k + " > " + source.size() + " or < 0");
     }
     if (random == null) {
-      random = new Random();
+      random = new UnsafeRandom(); // Fast, and we're single-threaded here anyway.
     }
+    
     // TODO: better balancing for different sizes
     // Two methods: constructive vs. destructive
     if (k < source.size() >> 1) {
@@ -614,4 +683,63 @@ public final class DBIDUtil {
       return sample;
     }
   }
+
+  /**
+   * Randomly split IDs into {@code p} partitions of almost-equal size.
+   * 
+   * @param ids Original DBIDs
+   * @param p Desired number of partitions.
+   * @param rnd Random generator
+   */
+  public static ArrayDBIDs[] randomSplit(DBIDs ids, int p, RandomFactory rnd) {
+    return randomSplit(ids, p, rnd.getSingleThreadedRandom());
+  }
+
+  /**
+   * Randomly split IDs into {@code p} partitions of almost-equal size.
+   * 
+   * @param oids Original DBIDs
+   * @param p Desired number of partitions.
+   * @param random Random generator
+   */
+  public static ArrayDBIDs[] randomSplit(DBIDs oids, int p, Random random) {
+    if (random == null) {
+      random = new UnsafeRandom(); // Fast, and we're single-threaded here anyway.
+    }
+    ArrayModifiableDBIDs ids = newArray(oids);
+    final int size = ids.size();
+    ArrayDBIDs[] split = new ArrayDBIDs[p];
+    // Shuffle
+    for (int i = 1; i < size; i++) {
+      ids.swap(i - 1, i + random.nextInt(size - i));
+    }
+    final int minsize = size / p; // Floor.
+    final int extra = size % p; // Remainder
+    for (int beg = 0, part = 0; part < p; part++) {
+      // First partitions are smaller, last partitions are larger.
+      final int psize = minsize + ((part < extra) ? 1 : 0);
+      split[part] = ids.slice(beg, beg + psize);
+      beg += psize;
+    }
+    return split;
+  }
+
+  /**
+   * Get a subset of the KNN result.
+   * 
+   * @param list Existing list
+   * @param k k
+   * @param <D> distance type
+   * @return Subset
+   */
+  @SuppressWarnings("unchecked")
+  public static <D extends Distance<D>> KNNList<D> subList(KNNList<D> list, int k) {
+    if (k >= list.size()) {
+      return list;
+    }
+    if (list instanceof DoubleDistanceKNNList) {
+      return (KNNList<D>) new DoubleDistanceKNNSubList((DoubleDistanceKNNList) list, k);
+    }
+    return new KNNSubList<>(list, k);
+  }
 }
