@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.database.ids.generic;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -68,7 +68,7 @@ public class UnmodifiableArrayDBIDs implements ArrayStaticDBIDs {
   @Override
   public DBIDArrayIter iter() {
     DBIDArrayIter it = inner.iter();
-    if(it instanceof DBIDMIter) {
+    if (it instanceof DBIDMIter) {
       return new UnmodifiableDBIDArrayIter(it);
     }
     return it;
@@ -90,8 +90,8 @@ public class UnmodifiableArrayDBIDs implements ArrayStaticDBIDs {
   }
 
   @Override
-  public void assign(int index, DBIDVar var) {
-    inner.assign(index, var);
+  public void assignVar(int index, DBIDVar var) {
+    inner.assignVar(index, var);
   }
 
   @Override
@@ -99,6 +99,11 @@ public class UnmodifiableArrayDBIDs implements ArrayStaticDBIDs {
     return inner.binarySearch(key);
   }
 
+  @Override
+  public ArrayDBIDs slice(int begin, int end) {
+    return new UnmodifiableArrayDBIDs(inner.slice(begin, end));
+  }
+
   /**
    * Make an existing DBIDMIter unmodifiable.
    * 
@@ -155,4 +160,4 @@ public class UnmodifiableArrayDBIDs implements ArrayStaticDBIDs {
       return it.getOffset();
     }
   }
-}
\ No newline at end of file
+}
