@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.database.query.distance;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -35,7 +35,7 @@ import de.lmu.ifi.dbs.elki.distance.similarityfunction.PrimitiveSimilarityFuncti
  * 
  * @author Erich Schubert
  * 
- * @apiviz.uses de.lmu.ifi.dbs.elki.distance.similarityfunction.PrimitiveSimilarityFunction
+ * @apiviz.uses PrimitiveSimilarityFunction
  * 
  * @param <O> Object type
  * @param <D> Distance type
@@ -83,4 +83,9 @@ public class PrimitiveDistanceSimilarityQuery<O, D extends Distance<D>> extends
   public D similarity(O o1, O o2) {
     return this.similarityFunction.similarity(o1, o2);
   }
-}
\ No newline at end of file
+
+  @Override
+  public PrimitiveSimilarityFunction<? super O, D> getSimilarityFunction() {
+    return similarityFunction;
+  }
+}
