@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.database.relation;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -48,7 +48,7 @@ public class ProxyView<O> extends AbstractHierarchicalResult implements Relation
   /**
    * The DBIDs we contain
    */
-  private final DBIDs idview;
+  private DBIDs idview;
 
   /**
    * The wrapped representation where we get the IDs from.
@@ -82,7 +82,7 @@ public class ProxyView<O> extends AbstractHierarchicalResult implements Relation
    * @return Instance
    */
   public static <O> ProxyView<O> wrap(Database database, DBIDs idview, Relation<O> inner) {
-    return new ProxyView<O>(database, idview, inner);
+    return new ProxyView<>(database, idview, inner);
   }
 
   @Override
@@ -90,7 +90,7 @@ public class ProxyView<O> extends AbstractHierarchicalResult implements Relation
     assert (idview.contains(id)) : "Accessing object not included in view.";
     return inner.get(id);
   }
-  
+
   @Override
   public void set(DBIDRef id, O val) {
     assert (idview.contains(id)) : "Accessing object not included in view.";
@@ -131,4 +131,13 @@ public class ProxyView<O> extends AbstractHierarchicalResult implements Relation
   public String getShortName() {
     return "partition";
   }
-}
\ No newline at end of file
+
+  /**
+   * Set the DBIDs to use.
+   * 
+   * @param ids DBIDs
+   */
+  public void setDBIDs(DBIDs ids) {
+    this.idview = ids;
+  }
+}
