@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.datasource;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -52,22 +52,6 @@ public abstract class AbstractDatabaseConnection implements DatabaseConnection {
    */
   public static final String LABEL_CONCATENATION = " ";
 
-  /**
-   * Filters to apply to the input data.
-   * <p>
-   * Key: {@code -dbc.filter}
-   * </p>
-   */
-  public static final OptionID FILTERS_ID = new OptionID("dbc.filter", "The filters to apply to the input data.");
-
-  /**
-   * Parameter to specify the parser to provide a database.
-   * <p>
-   * Key: {@code -dbc.parser}
-   * </p>
-   */
-  public static final OptionID PARSER_ID = new OptionID("dbc.parser", "Parser to provide the database.");
-
   /**
    * The filters to invoke
    */
@@ -183,6 +167,22 @@ public abstract class AbstractDatabaseConnection implements DatabaseConnection {
    * @apiviz.exclude
    */
   public abstract static class Parameterizer extends AbstractParameterizer {
+    /**
+     * Filters to apply to the input data.
+     * <p>
+     * Key: {@code -dbc.filter}
+     * </p>
+     */
+    public static final OptionID FILTERS_ID = new OptionID("dbc.filter", "The filters to apply to the input data.");
+
+    /**
+     * Parameter to specify the parser to provide a database.
+     * <p>
+     * Key: {@code -dbc.parser}
+     * </p>
+     */
+    public static final OptionID PARSER_ID = new OptionID("dbc.parser", "Parser to provide the database.");
+
     /**
      * Filters
      */
@@ -199,7 +199,7 @@ public abstract class AbstractDatabaseConnection implements DatabaseConnection {
      * @param config Parameterization
      */
     protected void configFilters(Parameterization config) {
-      final ObjectListParameter<ObjectFilter> filterParam = new ObjectListParameter<ObjectFilter>(FILTERS_ID, ObjectFilter.class, true);
+      final ObjectListParameter<ObjectFilter> filterParam = new ObjectListParameter<>(FILTERS_ID, ObjectFilter.class, true);
       if(config.grab(filterParam)) {
         filters = filterParam.instantiateClasses(config);
       }
@@ -213,7 +213,7 @@ public abstract class AbstractDatabaseConnection implements DatabaseConnection {
      * @param parserDefaultValueClass Default value
      */
     protected void configParser(Parameterization config, Class<?> parserRestrictionClass, Class<?> parserDefaultValueClass) {
-      ObjectParameter<Parser> parserParam = new ObjectParameter<Parser>(PARSER_ID, parserRestrictionClass, parserDefaultValueClass);
+      ObjectParameter<Parser> parserParam = new ObjectParameter<>(PARSER_ID, parserRestrictionClass, parserDefaultValueClass);
       if(config.grab(parserParam)) {
         parser = parserParam.instantiateClass(config);
       }
