@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.datasource;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -50,11 +50,12 @@ import de.lmu.ifi.dbs.elki.data.synthetic.bymodel.GeneratorStatic;
 import de.lmu.ifi.dbs.elki.datasource.bundle.MultipleObjectsBundle;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.Vector;
-import de.lmu.ifi.dbs.elki.math.statistics.distribution.DistributionWithRandom;
+import de.lmu.ifi.dbs.elki.math.statistics.distribution.Distribution;
 import de.lmu.ifi.dbs.elki.math.statistics.distribution.GammaDistribution;
 import de.lmu.ifi.dbs.elki.math.statistics.distribution.HaltonUniformDistribution;
 import de.lmu.ifi.dbs.elki.math.statistics.distribution.NormalDistribution;
 import de.lmu.ifi.dbs.elki.math.statistics.distribution.UniformDistribution;
+import de.lmu.ifi.dbs.elki.utilities.FormatUtil;
 import de.lmu.ifi.dbs.elki.utilities.exceptions.AbortException;
 import de.lmu.ifi.dbs.elki.utilities.exceptions.UnableToComplyException;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.AbstractParameterizer;
@@ -341,7 +342,7 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
 
     String dcostr = ((Element) cur).getAttribute(ATTR_DENSITY);
     if (dcostr != null && dcostr.length() > 0) {
-      overweight = Double.parseDouble(dcostr);
+      overweight = FormatUtil.parseDouble(dcostr);
     }
 
     if (size < 0) {
@@ -394,16 +395,16 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
 
     String minstr = ((Element) cur).getAttribute(ATTR_MIN);
     if (minstr != null && minstr.length() > 0) {
-      min = Double.parseDouble(minstr);
+      min = FormatUtil.parseDouble(minstr);
     }
     String maxstr = ((Element) cur).getAttribute(ATTR_MAX);
     if (maxstr != null && maxstr.length() > 0) {
-      max = Double.parseDouble(maxstr);
+      max = FormatUtil.parseDouble(maxstr);
     }
 
     // *** new uniform generator
     Random random = cluster.getNewRandomGenerator();
-    DistributionWithRandom generator = new UniformDistribution(min, max, random);
+    Distribution generator = new UniformDistribution(min, max, random);
     cluster.addGenerator(generator);
 
     // TODO: check for unknown attributes.
@@ -428,16 +429,16 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
     double stddev = 1.0;
     String meanstr = ((Element) cur).getAttribute(ATTR_MEAN);
     if (meanstr != null && meanstr.length() > 0) {
-      mean = Double.parseDouble(meanstr);
+      mean = FormatUtil.parseDouble(meanstr);
     }
     String stddevstr = ((Element) cur).getAttribute(ATTR_STDDEV);
     if (stddevstr != null && stddevstr.length() > 0) {
-      stddev = Double.parseDouble(stddevstr);
+      stddev = FormatUtil.parseDouble(stddevstr);
     }
 
     // *** New normal distribution generator
     Random random = cluster.getNewRandomGenerator();
-    DistributionWithRandom generator = new NormalDistribution(mean, stddev, random);
+    Distribution generator = new NormalDistribution(mean, stddev, random);
     cluster.addGenerator(generator);
 
     // TODO: check for unknown attributes.
@@ -462,16 +463,16 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
     double theta = 1.0;
     String kstr = ((Element) cur).getAttribute(ATTR_K);
     if (kstr != null && kstr.length() > 0) {
-      k = Double.parseDouble(kstr);
+      k = FormatUtil.parseDouble(kstr);
     }
     String thetastr = ((Element) cur).getAttribute(ATTR_THETA);
     if (thetastr != null && thetastr.length() > 0) {
-      theta = Double.parseDouble(thetastr);
+      theta = FormatUtil.parseDouble(thetastr);
     }
 
     // *** New normal distribution generator
     Random random = cluster.getNewRandomGenerator();
-    DistributionWithRandom generator = new GammaDistribution(k, theta, random);
+    Distribution generator = new GammaDistribution(k, theta, random);
     cluster.addGenerator(generator);
 
     // TODO: check for unknown attributes.
@@ -497,16 +498,16 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
 
     String minstr = ((Element) cur).getAttribute(ATTR_MIN);
     if (minstr != null && minstr.length() > 0) {
-      min = Double.parseDouble(minstr);
+      min = FormatUtil.parseDouble(minstr);
     }
     String maxstr = ((Element) cur).getAttribute(ATTR_MAX);
     if (maxstr != null && maxstr.length() > 0) {
-      max = Double.parseDouble(maxstr);
+      max = FormatUtil.parseDouble(maxstr);
     }
 
     // *** new uniform generator
     Random random = cluster.getNewRandomGenerator();
-    DistributionWithRandom generator = new HaltonUniformDistribution(min, max, random);
+    Distribution generator = new HaltonUniformDistribution(min, max, random);
     cluster.addGenerator(generator);
 
     // TODO: check for unknown attributes.
@@ -541,7 +542,7 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
     }
     String anstr = ((Element) cur).getAttribute(ATTR_ANGLE);
     if (anstr != null && anstr.length() > 0) {
-      angle = Double.parseDouble(anstr);
+      angle = FormatUtil.parseDouble(anstr);
     }
     if (axis1 <= 0 || axis1 > cluster.getDim()) {
       throw new UnableToComplyException("Invalid axis1 number given in specification file.");
@@ -645,7 +646,7 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
       throw new UnableToComplyException("No cluster name given in specification file.");
     }
 
-    ArrayList<Vector> points = new ArrayList<Vector>();
+    ArrayList<Vector> points = new ArrayList<>();
     // TODO: check for unknown attributes.
     XMLNodeIterator iter = new XMLNodeIterator(cur.getFirstChild());
     while (iter.hasNext()) {
@@ -709,7 +710,7 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
     double[] d = new double[entries.length];
     for (int i = 0; i < entries.length; i++) {
       try {
-        d[i] = Double.parseDouble(entries[i]);
+        d[i] = FormatUtil.parseDouble(entries[i]);
       } catch (NumberFormatException e) {
         throw new UnableToComplyException("Could not parse vector.");
       }
@@ -749,7 +750,7 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
         specfile = cfgparam.getValue();
       }
       // Cluster size scaling
-      final DoubleParameter scalepar = new DoubleParameter(SIZE_SCALE_ID, Double.valueOf(1.0));
+      final DoubleParameter scalepar = new DoubleParameter(SIZE_SCALE_ID, 1.);
       if (config.grab(scalepar)) {
         sizescale = scalepar.getValue().doubleValue();
       }
@@ -757,7 +758,7 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
       final RandomParameter rndP = new RandomParameter(RANDOMSEED_ID);
       if (config.grab(rndP)) {
         // TODO: use RandomFactory in cluster
-        clusterRandom = rndP.getValue().getRandom();
+        clusterRandom = rndP.getValue().getSingleThreadedRandom();
       }
     }
 
