@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.datasource;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -82,11 +82,11 @@ public class RandomDoubleVectorDatabaseConnection extends AbstractDatabaseConnec
 
   @Override
   public MultipleObjectsBundle loadData() {
-    VectorFieldTypeInformation<DoubleVector> type = new VectorFieldTypeInformation<DoubleVector>(DoubleVector.FACTORY, dim);
-    List<DoubleVector> vectors = new ArrayList<DoubleVector>(size);
+    VectorFieldTypeInformation<DoubleVector> type = new VectorFieldTypeInformation<>(DoubleVector.FACTORY, dim);
+    List<DoubleVector> vectors = new ArrayList<>(size);
 
     // Setup random generator
-    final Random rand = rnd.getRandom();
+    final Random rand = rnd.getSingleThreadedRandom();
 
     // Produce random vectors
     for(int i = 0; i < size; i++) {
