@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.datasource.bundle;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -50,7 +50,7 @@ public class SingleObjectBundle implements ObjectBundle {
    * Constructor.
    */
   public SingleObjectBundle() {
-    this(new BundleMeta(), new ArrayList<Object>(5));
+    this(new BundleMeta(), new ArrayList<>(5));
   }
 
   /**
