@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.datasource.filter;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -112,8 +112,9 @@ public class ByLabelFilter extends AbstractStreamFilter {
           Object l = source.data(lblcol);
           if(l instanceof LabelList) {
             boolean good = false;
-            for(String label : (LabelList) l) {
-              if(pattern.matcher(label).matches()) {
+            final LabelList ll = (LabelList) l;
+            for(int i = 0; i < ll.size(); i++) {
+              if(pattern.matcher(ll.get(i)).matches()) {
                 good = true;
                 break;
               }
