@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.datasource.filter;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -97,7 +97,7 @@ public class NoMissingValuesFilter extends AbstractStreamFilter {
 
     MultipleObjectsBundle bundle = new MultipleObjectsBundle();
     for(int j = 0; j < objects.metaLength(); j++) {
-      bundle.appendColumn(objects.meta(j), new ArrayList<Object>());
+      bundle.appendColumn(objects.meta(j), new ArrayList<>());
     }
     for(int i = 0; i < objects.dataLength(); i++) {
       boolean good = true;
