@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.datasource.filter;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -27,6 +27,7 @@ import de.lmu.ifi.dbs.elki.data.SparseNumberVector;
 import de.lmu.ifi.dbs.elki.data.type.SimpleTypeInformation;
 import de.lmu.ifi.dbs.elki.data.type.TypeUtil;
 import de.lmu.ifi.dbs.elki.data.type.VectorFieldTypeInformation;
+import de.lmu.ifi.dbs.elki.logging.Logging;
 
 /**
  * Class that turns sparse float vectors into a proper vector field, by setting
@@ -37,6 +38,11 @@ import de.lmu.ifi.dbs.elki.data.type.VectorFieldTypeInformation;
  * @param <V> Vector type
  */
 public class SparseVectorFieldFilter<V extends SparseNumberVector<?>> extends AbstractConversionFilter<V, V> {
+  /**
+   * Class logger.
+   */
+  private static final Logging LOG = Logging.getLogger(SparseVectorFieldFilter.class);
+
   /**
    * Maximum dimension.
    */
@@ -74,6 +80,11 @@ public class SparseVectorFieldFilter<V extends SparseNumberVector<?>> extends Ab
   @Override
   protected SimpleTypeInformation<? super V> convertedType(SimpleTypeInformation<V> in) {
     SparseNumberVector.Factory<V, ?> factory = (SparseNumberVector.Factory<V, ?>) FilterUtil.guessFactory(in);
-    return new VectorFieldTypeInformation<V>(factory, maxdim);
+    return new VectorFieldTypeInformation<>(factory, maxdim);
   }
-}
\ No newline at end of file
+
+  @Override
+  protected Logging getLogger() {
+    return LOG;
+  }
+}
