@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.datasource.filter.normalization;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -25,7 +25,6 @@ package de.lmu.ifi.dbs.elki.datasource.filter.normalization;
 
 import de.lmu.ifi.dbs.elki.data.NumberVector;
 import de.lmu.ifi.dbs.elki.data.type.SimpleTypeInformation;
-import de.lmu.ifi.dbs.elki.datasource.bundle.MultipleObjectsBundle;
 import de.lmu.ifi.dbs.elki.datasource.filter.AbstractVectorConversionFilter;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.LinearEquationSystem;
 
@@ -50,11 +49,6 @@ public abstract class AbstractNormalization<O extends NumberVector<?>> extends A
     return in;
   }
 
-  @Override
-  public MultipleObjectsBundle normalizeObjects(MultipleObjectsBundle objects) {
-    return super.filter(objects);
-  }
-
   @Override
   public LinearEquationSystem transform(LinearEquationSystem linearEquationSystem) {
     // FIXME: implement.
@@ -65,4 +59,4 @@ public abstract class AbstractNormalization<O extends NumberVector<?>> extends A
   public String toString() {
     return getClass().getName();
   }
-}
\ No newline at end of file
+}
