@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.datasource.filter.normalization;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -25,7 +25,6 @@ package de.lmu.ifi.dbs.elki.datasource.filter.normalization;
 
 import de.lmu.ifi.dbs.elki.data.NumberVector;
 import de.lmu.ifi.dbs.elki.data.type.SimpleTypeInformation;
-import de.lmu.ifi.dbs.elki.datasource.bundle.MultipleObjectsBundle;
 import de.lmu.ifi.dbs.elki.datasource.filter.AbstractVectorStreamConversionFilter;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.LinearEquationSystem;
 
@@ -49,11 +48,6 @@ public abstract class AbstractStreamNormalization<O extends NumberVector<?>> ext
     initializeOutputType(in);
     return in;
   }
-  
-  @Override
-  public MultipleObjectsBundle normalizeObjects(MultipleObjectsBundle objects) {
-    return super.filter(objects);
-  }
 
   @Override
   public LinearEquationSystem transform(LinearEquationSystem linearEquationSystem) {
@@ -67,4 +61,4 @@ public abstract class AbstractStreamNormalization<O extends NumberVector<?>> ext
     result.append("normalization class: ").append(getClass().getName());
     return result.toString();
   }
-}
\ No newline at end of file
+}
