@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.datasource.filter.normalization;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -26,6 +26,7 @@ package de.lmu.ifi.dbs.elki.datasource.filter.normalization;
 import de.lmu.ifi.dbs.elki.data.NumberVector;
 import de.lmu.ifi.dbs.elki.data.type.SimpleTypeInformation;
 import de.lmu.ifi.dbs.elki.data.type.TypeUtil;
+import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.math.statistics.distribution.NormalDistribution;
 
 /**
@@ -39,6 +40,11 @@ import de.lmu.ifi.dbs.elki.math.statistics.distribution.NormalDistribution;
  * @apiviz.uses NumberVector
  */
 public class AttributeWiseErfNormalization<O extends NumberVector<?>> extends AbstractNormalization<O> {
+  /**
+   * Class logger.
+   */
+  private static final Logging LOG = Logging.getLogger(AttributeWiseErfNormalization.class);
+
   /**
    * Constructor.
    */
@@ -54,7 +60,7 @@ public class AttributeWiseErfNormalization<O extends NumberVector<?>> extends Ab
   @Override
   protected O filterSingleObject(O obj) {
     double[] val = new double[obj.getDimensionality()];
-    for(int i = 0; i < val.length; i++) {
+    for (int i = 0; i < val.length; i++) {
       val[i] = NormalDistribution.erf(obj.doubleValue(i));
     }
     return factory.newNumberVector(val);
@@ -64,4 +70,9 @@ public class AttributeWiseErfNormalization<O extends NumberVector<?>> extends Ab
   protected SimpleTypeInformation<? super O> getInputTypeRestriction() {
     return TypeUtil.NUMBER_VECTOR_FIELD;
   }
+
+  @Override
+  protected Logging getLogger() {
+    return LOG;
+  }
 }
