@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.datasource.filter.normalization;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -23,11 +23,10 @@ package de.lmu.ifi.dbs.elki.datasource.filter.normalization;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-import java.util.ArrayList;
-
 import de.lmu.ifi.dbs.elki.data.NumberVector;
 import de.lmu.ifi.dbs.elki.data.type.SimpleTypeInformation;
 import de.lmu.ifi.dbs.elki.data.type.TypeUtil;
+import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.LinearEquationSystem;
 import de.lmu.ifi.dbs.elki.utilities.FormatUtil;
 import de.lmu.ifi.dbs.elki.utilities.datastructures.arraylike.ArrayLikeUtil;
@@ -37,8 +36,6 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.AllOrNoneMustBeS
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.EqualSizeGlobalConstraint;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.DoubleListParameter;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.ListParameter;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.Parameter;
 
 /**
  * Class to perform and undo a normalization on real vectors with respect to
@@ -51,6 +48,11 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.Parameter;
  */
 // TODO: extract superclass AbstractAttributeWiseNormalization
 public class AttributeWiseMinMaxNormalization<V extends NumberVector<?>> extends AbstractNormalization<V> {
+  /**
+   * Class logger.
+   */
+  private static final Logging LOG = Logging.getLogger(AttributeWiseMinMaxNormalization.class);
+
   /**
    * Parameter for minimum.
    */
@@ -186,12 +188,17 @@ public class AttributeWiseMinMaxNormalization<V extends NumberVector<?>> extends
     result.append("normalization maxima: ").append(FormatUtil.format(maxima));
     return result.toString();
   }
-  
+
   @Override
   protected SimpleTypeInformation<? super V> getInputTypeRestriction() {
     return TypeUtil.NUMBER_VECTOR_FIELD;
   }
 
+  @Override
+  protected Logging getLogger() {
+    return LOG;
+  }
+
   /**
    * Parameterization class.
    * 
@@ -222,20 +229,13 @@ public class AttributeWiseMinMaxNormalization<V extends NumberVector<?>> extends
         maxima = ArrayLikeUtil.toPrimitiveDoubleArray(maximaP.getValue());
       }
 
-      ArrayList<Parameter<?>> global_1 = new ArrayList<Parameter<?>>();
-      global_1.add(minimaP);
-      global_1.add(maximaP);
-      config.checkConstraint(new AllOrNoneMustBeSetGlobalConstraint(global_1));
-
-      ArrayList<ListParameter<?>> global = new ArrayList<ListParameter<?>>();
-      global.add(minimaP);
-      global.add(maximaP);
-      config.checkConstraint(new EqualSizeGlobalConstraint(global));
+      config.checkConstraint(new AllOrNoneMustBeSetGlobalConstraint(minimaP, maximaP));
+      config.checkConstraint(new EqualSizeGlobalConstraint(minimaP, maximaP));
     }
 
     @Override
     protected AttributeWiseMinMaxNormalization<V> makeInstance() {
-      return new AttributeWiseMinMaxNormalization<V>(minima, maxima);
+      return new AttributeWiseMinMaxNormalization<>(minima, maxima);
     }
   }
-}
\ No newline at end of file
+}
