@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.datasource.filter.normalization;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -24,10 +24,8 @@ package de.lmu.ifi.dbs.elki.datasource.filter.normalization;
  */
 
 import gnu.trove.map.hash.TIntDoubleHashMap;
-
-import java.util.BitSet;
-
 import de.lmu.ifi.dbs.elki.data.SparseNumberVector;
+import de.lmu.ifi.dbs.elki.logging.Logging;
 
 /**
  * Perform full TF-IDF Normalization as commonly used in text mining.
@@ -43,6 +41,11 @@ import de.lmu.ifi.dbs.elki.data.SparseNumberVector;
  * @param <V> Vector type
  */
 public class TFIDFNormalization<V extends SparseNumberVector<?>> extends InverseDocumentFrequencyNormalization<V> {
+  /**
+   * Class logger.
+   */
+  private static final Logging LOG = Logging.getLogger(TFIDFNormalization.class);
+
   /**
    * Constructor.
    */
@@ -52,18 +55,23 @@ public class TFIDFNormalization<V extends SparseNumberVector<?>> extends Inverse
 
   @Override
   protected V filterSingleObject(V featureVector) {
-    BitSet b = featureVector.getNotNullMask();
     double sum = 0.0;
-    for(int i = b.nextSetBit(0); i >= 0; i = b.nextSetBit(i + 1)) {
-      sum += featureVector.doubleValue(i);
+    for(int it = featureVector.iter(); featureVector.iterValid(it); it = featureVector.iterAdvance(it)) {
+      sum += featureVector.iterDoubleValue(it);
     }
     if(sum <= 0) {
       sum = 1.0;
     }
     TIntDoubleHashMap vals = new TIntDoubleHashMap();
-    for(int i = b.nextSetBit(0); i >= 0; i = b.nextSetBit(i + 1)) {
-      vals.put(i, (float) (featureVector.doubleValue(i) / sum * idf.get(i)));
+    for(int it = featureVector.iter(); featureVector.iterValid(it); it = featureVector.iterAdvance(it)) {
+      final int dim = featureVector.iterDim(it);
+      vals.put(dim, featureVector.iterDoubleValue(it) / sum * idf.get(dim));
     }
     return ((SparseNumberVector.Factory<V, ?>) factory).newNumberVector(vals, featureVector.getDimensionality());
   }
+
+  @Override
+  protected Logging getLogger() {
+    return LOG;
+  }
 }
\ No newline at end of file
