@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.datasource.parser;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -24,7 +24,7 @@ package de.lmu.ifi.dbs.elki.datasource.parser;
  */
 
 import gnu.trove.iterator.TIntObjectIterator;
-import gnu.trove.map.hash.TIntFloatHashMap;
+import gnu.trove.map.hash.TIntDoubleHashMap;
 import gnu.trove.map.hash.TIntObjectHashMap;
 
 import java.io.BufferedReader;
@@ -41,13 +41,14 @@ import de.lmu.ifi.dbs.elki.data.DoubleVector;
 import de.lmu.ifi.dbs.elki.data.ExternalID;
 import de.lmu.ifi.dbs.elki.data.LabelList;
 import de.lmu.ifi.dbs.elki.data.SimpleClassLabel;
-import de.lmu.ifi.dbs.elki.data.SparseFloatVector;
+import de.lmu.ifi.dbs.elki.data.SparseDoubleVector;
 import de.lmu.ifi.dbs.elki.data.type.SimpleTypeInformation;
 import de.lmu.ifi.dbs.elki.data.type.TypeInformation;
 import de.lmu.ifi.dbs.elki.data.type.TypeUtil;
 import de.lmu.ifi.dbs.elki.data.type.VectorFieldTypeInformation;
 import de.lmu.ifi.dbs.elki.datasource.bundle.MultipleObjectsBundle;
 import de.lmu.ifi.dbs.elki.logging.Logging;
+import de.lmu.ifi.dbs.elki.utilities.FormatUtil;
 import de.lmu.ifi.dbs.elki.utilities.exceptions.AbortException;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.AbstractParameterizer;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
@@ -60,7 +61,11 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.PatternParameter;
  * This parser is quite hackish, and contains lots of not yet configurable
  * magic.
  * 
- * TODO: Sparse vectors are not yet fully supported.
+ * TODO: Allow configuration of the vector types (double, float)
+ * 
+ * TODO: when encountering integer columns, produce integer vectors.
+ * 
+ * TODO: allow optional class labels.
  * 
  * @author Erich Schubert
  */
@@ -120,6 +125,11 @@ public class ArffParser implements Parser {
    */
   Pattern magic_class;
 
+  /**
+   * (Reused) buffer for building label lists.
+   */
+  ArrayList<String> labels = new ArrayList<>();
+
   /**
    * Constructor.
    * 
@@ -146,8 +156,8 @@ public class ArffParser implements Parser {
   public MultipleObjectsBundle parse(InputStream instream) {
     try {
       BufferedReader br = new BufferedReader(new InputStreamReader(instream));
-      ArrayList<String> names = new ArrayList<String>();
-      ArrayList<String> types = new ArrayList<String>();
+      ArrayList<String> names = new ArrayList<>();
+      ArrayList<String> types = new ArrayList<>();
 
       readHeader(br);
       parseAttributeStatements(br, names, types);
@@ -205,7 +215,7 @@ public class ArffParser implements Parser {
 
   private Object[] loadSparseInstance(StreamTokenizer tokenizer, int[] targ, int[] dimsize, TypeInformation[] elkitypes, int metaLength) throws IOException {
     // logger.warning("Sparse instance.");
-    TIntObjectHashMap<Object> map = new TIntObjectHashMap<Object>();
+    TIntObjectHashMap<Object> map = new TIntObjectHashMap<>();
     while(true) {
       nextToken(tokenizer);
       assert (tokenizer.ttype != StreamTokenizer.TT_EOF && tokenizer.ttype != StreamTokenizer.TT_EOL);
@@ -216,19 +226,21 @@ public class ArffParser implements Parser {
       }
       else {
         // sparse token
-        if(tokenizer.ttype != StreamTokenizer.TT_NUMBER) {
-          throw new AbortException("Unexpected token type encountered: " + tokenizer.toString());
+        if(tokenizer.ttype != StreamTokenizer.TT_WORD) {
+          throw new AbortException("Unexpected token type encountered: " + tokenizer.toString() + " type: " + tokenizer.ttype);
         }
-        int dim = (int) tokenizer.nval;
+        int dim = Integer.valueOf(tokenizer.sval);
         if(map.containsKey(dim)) {
           throw new AbortException("Duplicate key in sparse vector: " + tokenizer.toString());
         }
         nextToken(tokenizer);
-        if(tokenizer.ttype == StreamTokenizer.TT_NUMBER) {
-          map.put(dim, Double.valueOf(tokenizer.nval));
-        }
-        else if(tokenizer.ttype == StreamTokenizer.TT_WORD) {
-          map.put(dim, tokenizer.sval);
+        if(tokenizer.ttype == StreamTokenizer.TT_WORD) {
+          if(TypeUtil.NUMBER_VECTOR_FIELD.equals(elkitypes[targ[dim]])) {
+            map.put(dim, FormatUtil.parseDouble(tokenizer.sval));
+          }
+          else {
+            map.put(dim, tokenizer.sval);
+          }
         }
         else {
           throw new AbortException("Unexpected token type encountered: " + tokenizer.toString());
@@ -247,7 +259,7 @@ public class ArffParser implements Parser {
       }
       assert (s >= 0);
       if(TypeUtil.NUMBER_VECTOR_FIELD.equals(elkitypes[out])) {
-        TIntFloatHashMap f = new TIntFloatHashMap(dimsize[out]);
+        TIntDoubleHashMap f = new TIntDoubleHashMap(dimsize[out]);
         for(TIntObjectIterator<Object> iter = map.iterator(); iter.hasNext();) {
           iter.advance();
           int i = iter.key();
@@ -258,13 +270,13 @@ public class ArffParser implements Parser {
             break;
           }
           double v = ((Double) iter.value()).doubleValue();
-          f.put(i - s + 1, (float) v);
+          f.put(i - s, v);
         }
-        data[out] = new SparseFloatVector(f, dimsize[out]);
+        data[out] = new SparseDoubleVector(f, dimsize[out]);
       }
       else if(TypeUtil.LABELLIST.equals(elkitypes[out])) {
         // Build a label list out of successive labels
-        LabelList ll = new LabelList(1);
+        labels.clear();
         for(TIntObjectIterator<Object> iter = map.iterator(); iter.hasNext();) {
           iter.advance();
           int i = iter.key();
@@ -275,12 +287,12 @@ public class ArffParser implements Parser {
             break;
           }
           String v = (String) iter.value();
-          if(ll.size() < i - s) {
+          if(labels.size() < i - s) {
             LOG.warning("Sparse consecutive labels are currently not correctly supported.");
           }
-          ll.add(v);
+          labels.add(v);
         }
-        data[out] = ll;
+        data[out] = LabelList.make(labels);
       }
       else if(TypeUtil.EXTERNALID.equals(elkitypes[out])) {
         String val = (String) map.get(s);
@@ -292,10 +304,10 @@ public class ArffParser implements Parser {
         }
       }
       else if(TypeUtil.CLASSLABEL.equals(elkitypes[out])) {
-        String val = (String) map.get(s);
+        Object val = map.get(s);
         if(val != null) {
           // TODO: support other class label types.
-          ClassLabel lbl = new SimpleClassLabel(val);
+          ClassLabel lbl = new SimpleClassLabel(String.valueOf(val));
           data[out] = lbl;
         }
         else {
@@ -321,7 +333,7 @@ public class ArffParser implements Parser {
           }
           else if(tokenizer.ttype == StreamTokenizer.TT_WORD) {
             try {
-              cur[k] = Double.parseDouble(tokenizer.sval);
+              cur[k] = FormatUtil.parseDouble(tokenizer.sval);
             }
             catch(NumberFormatException e) {
               throw new AbortException("Expected number value, got: " + tokenizer.sval);
@@ -336,15 +348,15 @@ public class ArffParser implements Parser {
       }
       else if(TypeUtil.LABELLIST.equals(etyp[out])) {
         // Build a label list out of successive labels
-        LabelList ll = new LabelList(dimsize[out]);
+        labels.clear();
         for(int k = 0; k < dimsize[out]; k++) {
           if(tokenizer.ttype != StreamTokenizer.TT_WORD) {
             throw new AbortException("Expected word token, got: " + tokenizer.toString());
           }
-          ll.add(tokenizer.sval);
+          labels.add(tokenizer.sval);
           nextToken(tokenizer);
         }
-        data[out] = ll;
+        data[out] = LabelList.make(labels);
       }
       else if(TypeUtil.EXTERNALID.equals(etyp[out])) {
         if(tokenizer.ttype != StreamTokenizer.TT_WORD) {
@@ -381,7 +393,7 @@ public class ArffParser implements Parser {
     {
       tokenizer.resetSyntax();
       tokenizer.whitespaceChars(0, ' ');
-      tokenizer.ordinaryChars('0', '9');
+      tokenizer.ordinaryChars('0', '9'); // Do not parse numbers
       tokenizer.ordinaryChar('-');
       tokenizer.ordinaryChar('.');
       tokenizer.wordChars(' ' + 1, '\u00FF');
@@ -421,12 +433,12 @@ public class ArffParser implements Parser {
           labels[i] = names.get(out + i);
         }
         if(!sparse) {
-          VectorFieldTypeInformation<DoubleVector> type = new VectorFieldTypeInformation<DoubleVector>(DoubleVector.FACTORY, dimsize[out], labels);
+          VectorFieldTypeInformation<DoubleVector> type = new VectorFieldTypeInformation<>(DoubleVector.FACTORY, dimsize[out], labels);
           bundle.appendColumn(type, new ArrayList<DoubleVector>());
         }
         else {
-          VectorFieldTypeInformation<SparseFloatVector> type = new VectorFieldTypeInformation<SparseFloatVector>(SparseFloatVector.FACTORY, dimsize[out], labels);
-          bundle.appendColumn(type, new ArrayList<SparseFloatVector>());
+          VectorFieldTypeInformation<SparseDoubleVector> type = new VectorFieldTypeInformation<>(SparseDoubleVector.FACTORY, dimsize[out], labels);
+          bundle.appendColumn(type, new ArrayList<SparseDoubleVector>());
         }
       }
       else if(TypeUtil.LABELLIST.equals(etyp[out])) {
@@ -434,13 +446,13 @@ public class ArffParser implements Parser {
         for(int i = 1; i < dimsize[out]; i++) {
           label.append(' ').append(names.get(out + i));
         }
-        bundle.appendColumn(new SimpleTypeInformation<LabelList>(LabelList.class, label.toString()), new ArrayList<LabelList>());
+        bundle.appendColumn(new SimpleTypeInformation<>(LabelList.class, label.toString()), new ArrayList<LabelList>());
       }
       else if(TypeUtil.EXTERNALID.equals(etyp[out])) {
-        bundle.appendColumn(new SimpleTypeInformation<ExternalID>(ExternalID.class, names.get(out)), new ArrayList<ExternalID>());
+        bundle.appendColumn(new SimpleTypeInformation<>(ExternalID.class, names.get(out)), new ArrayList<ExternalID>());
       }
       else if(TypeUtil.CLASSLABEL.equals(etyp[out])) {
-        bundle.appendColumn(new SimpleTypeInformation<ClassLabel>(ClassLabel.class, names.get(out)), new ArrayList<ClassLabel>());
+        bundle.appendColumn(new SimpleTypeInformation<>(ClassLabel.class, names.get(out)), new ArrayList<ClassLabel>());
       }
       else {
         throw new AbortException("Unsupported type for column " + in + "->" + out + ": " + ((etyp[out] != null) ? etyp[out].toString() : "null"));
