@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.datasource.parser;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -28,10 +28,10 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.util.ArrayList;
+import java.util.BitSet;
 import java.util.List;
 import java.util.regex.Pattern;
 
-import de.lmu.ifi.dbs.elki.data.Bit;
 import de.lmu.ifi.dbs.elki.data.BitVector;
 import de.lmu.ifi.dbs.elki.data.LabelList;
 import de.lmu.ifi.dbs.elki.data.type.TypeUtil;
@@ -63,11 +63,12 @@ public class BitVectorLabelParser extends AbstractParser implements Parser {
   /**
    * Constructor.
    * 
-   * @param colSep
-   * @param quoteChar
+   * @param colSep Column separator
+   * @param quoteChars Quotation character
+   * @param comment Comment pattern
    */
-  public BitVectorLabelParser(Pattern colSep, char quoteChar) {
-    super(colSep, quoteChar);
+  public BitVectorLabelParser(Pattern colSep, String quoteChars, Pattern comment) {
+    super(colSep, quoteChars, comment);
   }
 
   @Override
@@ -75,38 +76,39 @@ public class BitVectorLabelParser extends AbstractParser implements Parser {
     BufferedReader reader = new BufferedReader(new InputStreamReader(in));
     int lineNumber = 0;
     int dimensionality = -1;
-    List<BitVector> vectors = new ArrayList<BitVector>();
-    List<LabelList> labels = new ArrayList<LabelList>();
+    List<BitVector> vectors = new ArrayList<>();
+    List<LabelList> labels = new ArrayList<>();
+    ArrayList<String> ll = new ArrayList<>();
     try {
       for(String line; (line = reader.readLine()) != null; lineNumber++) {
-        if(!line.startsWith(COMMENT) && line.length() > 0) {
-          List<String> entries = tokenize(line);
-          // FIXME: use more efficient storage right away?
-          List<Bit> attributes = new ArrayList<Bit>();
-          LabelList ll = null;
-          for(String entry : entries) {
-            try {
-              Bit attribute = Bit.valueOf(entry);
-              attributes.add(attribute);
-            }
-            catch(NumberFormatException e) {
-              if(ll == null) {
-                ll = new LabelList(1);
-              }
-              ll.add(entry);
-            }
-          }
-
-          if(dimensionality < 0) {
-            dimensionality = attributes.size();
-          }
-          else if(dimensionality != attributes.size()) {
-            throw new IllegalArgumentException("Differing dimensionality in line " + lineNumber + ".");
-          }
-
-          vectors.add(new BitVector(attributes.toArray(new Bit[attributes.size()])));
-          labels.add(ll);
+        // Skip empty lines and comments
+        if(line.length() <= 0 || (comment != null && comment.matcher(line).matches())) {
+          continue;
         }
+        BitSet bitSet = new BitSet();
+        ll.clear();
+        int i = 0;
+        for(tokenizer.initialize(line, 0, lengthWithoutLinefeed(line)); tokenizer.valid(); tokenizer.advance()) {
+          try {
+            if(tokenizer.getLongBase10() > 0) {
+              bitSet.set(i);
+            }
+            ++i;
+          }
+          catch(NumberFormatException e) {
+            ll.add(tokenizer.getSubstring());
+          }
+        }
+
+        if(dimensionality < 0) {
+          dimensionality = i;
+        }
+        else if(dimensionality != i) {
+          throw new IllegalArgumentException("Differing dimensionality in line " + lineNumber + ".");
+        }
+
+        vectors.add(new BitVector(bitSet, dimensionality));
+        labels.add(LabelList.make(ll));
       }
     }
     catch(IOException e) {
@@ -116,7 +118,7 @@ public class BitVectorLabelParser extends AbstractParser implements Parser {
   }
 
   protected VectorFieldTypeInformation<BitVector> getTypeInformation(int dimensionality) {
-    return new VectorFieldTypeInformation<BitVector>(BitVector.FACTORY, dimensionality);
+    return new VectorFieldTypeInformation<>(BitVector.FACTORY, dimensionality);
   }
 
   @Override
@@ -134,7 +136,7 @@ public class BitVectorLabelParser extends AbstractParser implements Parser {
   public static class Parameterizer extends AbstractParser.Parameterizer {
     @Override
     protected BitVectorLabelParser makeInstance() {
-      return new BitVectorLabelParser(colSep, quoteChar);
+      return new BitVectorLabelParser(colSep, quoteChars, comment);
     }
   }
-}
\ No newline at end of file
+}
