@@ -25,8 +25,8 @@ package de.lmu.ifi.dbs.elki.datasource.parser;
 
 import gnu.trove.map.hash.TIntDoubleHashMap;
 
+import java.util.ArrayList;
 import java.util.BitSet;
-import java.util.List;
 import java.util.regex.Pattern;
 
 import de.lmu.ifi.dbs.elki.data.LabelList;
@@ -34,6 +34,7 @@ import de.lmu.ifi.dbs.elki.data.SparseFloatVector;
 import de.lmu.ifi.dbs.elki.data.SparseNumberVector;
 import de.lmu.ifi.dbs.elki.data.type.SimpleTypeInformation;
 import de.lmu.ifi.dbs.elki.data.type.VectorFieldTypeInformation;
+import de.lmu.ifi.dbs.elki.data.type.VectorTypeInformation;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Description;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Title;
@@ -85,76 +86,78 @@ public class SparseNumberVectorLabelParser<V extends SparseNumberVector<?>> exte
    */
   private static final Logging LOG = Logging.getLogger(SparseNumberVectorLabelParser.class);
 
-  /**
-   * Holds the dimensionality of the parsed data which is the maximum occurring
-   * index of any attribute.
-   */
-  private int maxdim = -1;
-
   /**
    * Same as {@link #factory}, but subtype.
    */
   private SparseNumberVector.Factory<V, ?> sparsefactory;
 
+  /**
+   * (Reused) set of values for the number vector.
+   */
+  TIntDoubleHashMap values = new TIntDoubleHashMap();
+
+  /**
+   * (Reused) label buffer.
+   */
+  ArrayList<String> labels = new ArrayList<>();
+
   /**
    * Constructor.
    * 
    * @param colSep Column separator
-   * @param quoteChar Quotation character
-   * @param labelIndices Label indexes
+   * @param quoteChars Quotation character
+   * @param comment Comment pattern
+   * @param labelIndices Indices to use as labels
    * @param factory Vector factory
    */
-  public SparseNumberVectorLabelParser(Pattern colSep, char quoteChar, BitSet labelIndices, SparseNumberVector.Factory<V, ?> factory) {
-    super(colSep, quoteChar, labelIndices, factory);
+  public SparseNumberVectorLabelParser(Pattern colSep, String quoteChars, Pattern comment, BitSet labelIndices, SparseNumberVector.Factory<V, ?> factory) {
+    super(colSep, quoteChars, comment, labelIndices, factory);
     this.sparsefactory = factory;
   }
 
   @Override
   protected void parseLineInternal(String line) {
-    List<String> entries = tokenize(line);
-    int cardinality = Integer.parseInt(entries.get(0));
+    tokenizer.initialize(line, 0, lengthWithoutLinefeed(line));
+    int cardinality = (int) tokenizer.getLongBase10();
 
-    TIntDoubleHashMap values = new TIntDoubleHashMap(cardinality, 1);
-    LabelList labels = null;
+    values.clear();
+    labels.clear();
+    int thismax = 0;
 
-    for (int i = 1; i < entries.size() - 1; i++) {
-      if (labelIndices == null || !labelIndices.get(i)) {
+    while(tokenizer.valid()) {
+      if(values.size() < cardinality) {
         try {
-          int index = Integer.parseInt(entries.get(i));
-          if (index >= maxdim) {
-            maxdim = index + 1;
+          int index = (int) tokenizer.getLongBase10();
+          tokenizer.advance();
+          // Respect labelIndices.
+          if(labelIndices == null || !labelIndices.get(index)) {
+            double attribute = tokenizer.getDouble();
+            thismax = Math.max(thismax, index + 1);
+            values.put(index, attribute);
+            tokenizer.advance();
+            continue;
           }
-          double attribute = Double.parseDouble(entries.get(i));
-          values.put(index, attribute);
-          i++;
-        } catch (NumberFormatException e) {
-          if (labels == null) {
-            labels = new LabelList(1);
-          }
-          labels.add(entries.get(i));
-          continue;
         }
-      } else {
-        if (labels == null) {
-          labels = new LabelList(1);
+        catch(NumberFormatException e) {
+          // continue with fallback below.
         }
-        labels.add(entries.get(i));
       }
+      // Fallback: treat as label
+      haslabels = true;
+      labels.add(tokenizer.getSubstring());
+      tokenizer.advance();
     }
-    if (values.size() > maxdim) {
-      throw new AbortException("Invalid sparse vector seen: " + line);
-    }
-    curvec = sparsefactory.newNumberVector(values, maxdim);
-    curlbl = labels;
+    curvec = sparsefactory.newNumberVector(values, thismax);
+    curlbl = LabelList.make(labels);
   }
 
   @Override
-  protected SimpleTypeInformation<V> getTypeInformation(int dimensionality) {
-    if (dimensionality > 0) {
-      return new VectorFieldTypeInformation<V>(factory, dimensionality);
+  protected SimpleTypeInformation<V> getTypeInformation(int mindim, int maxdim) {
+    if(mindim == maxdim) {
+      return new VectorFieldTypeInformation<>(factory, mindim);
     }
-    if (dimensionality == DIMENSIONALITY_VARIABLE) {
-      return new SimpleTypeInformation<V>(factory.getRestrictionClass(), factory.getDefaultSerializer());
+    else if(mindim < maxdim) {
+      return new VectorTypeInformation<>(factory.getRestrictionClass(), factory.getDefaultSerializer(), mindim, maxdim);
     }
     throw new AbortException("No vectors were read from the input file - cannot determine vector data type.");
   }
@@ -174,15 +177,15 @@ public class SparseNumberVectorLabelParser<V extends SparseNumberVector<?>> exte
   public static class Parameterizer<V extends SparseNumberVector<?>> extends NumberVectorLabelParser.Parameterizer<V> {
     @Override
     protected void getFactory(Parameterization config) {
-      ObjectParameter<SparseNumberVector.Factory<V, ?>> factoryP = new ObjectParameter<SparseNumberVector.Factory<V, ?>>(VECTOR_TYPE_ID, SparseNumberVector.Factory.class, SparseFloatVector.Factory.class);
-      if (config.grab(factoryP)) {
+      ObjectParameter<SparseNumberVector.Factory<V, ?>> factoryP = new ObjectParameter<>(VECTOR_TYPE_ID, SparseNumberVector.Factory.class, SparseFloatVector.Factory.class);
+      if(config.grab(factoryP)) {
         factory = factoryP.instantiateClass(config);
       }
     }
 
     @Override
     protected SparseNumberVectorLabelParser<V> makeInstance() {
-      return new SparseNumberVectorLabelParser<V>(colSep, quoteChar, labelIndices, (SparseNumberVector.Factory<V, ?>) factory);
+      return new SparseNumberVectorLabelParser<>(colSep, quoteChars, comment, labelIndices, (SparseNumberVector.Factory<V, ?>) factory);
     }
   }
 }
