@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.distance.distancefunction;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -37,6 +37,8 @@ import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
  * 
  * @author Arthur Zimek
  * 
+ * @apiviz.excludeSubtypes
+ * 
  * @param <D> the type of Distance used
  */
 public abstract class AbstractDBIDDistanceFunction<D extends Distance<D>> implements DBIDDistanceFunction<D> {
@@ -73,6 +75,6 @@ public abstract class AbstractDBIDDistanceFunction<D extends Distance<D>> implem
   @SuppressWarnings("unchecked")
   @Override
   final public <O extends DBID> DistanceQuery<O, D> instantiate(Relation<O> database) {
-    return (DistanceQuery<O, D>) new DBIDDistanceQuery<D>((Relation<DBID>) database, this);
+    return (DistanceQuery<O, D>) new DBIDDistanceQuery<>((Relation<DBID>) database, this);
   }
 }
\ No newline at end of file
