@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.distance.distancefunction;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -33,6 +33,8 @@ import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
  * 
  * @author Arthur Zimek
  * 
+ * @apiviz.excludeSubtypes
+ * 
  * @param <O> the type of objects to compute the distances in between
  * @param <D> the type of Distance used
  */
@@ -70,6 +72,6 @@ public abstract class AbstractPrimitiveDistanceFunction<O, D extends Distance<D>
    */
   @Override
   public <T extends O> DistanceQuery<T, D> instantiate(Relation<T> relation) {
-    return new PrimitiveDistanceQuery<T, D>(relation, this);
+    return new PrimitiveDistanceQuery<>(relation, this);
   }
 }
\ No newline at end of file
