@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.distance.distancefunction.colorhistogram;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -31,8 +31,7 @@ import de.lmu.ifi.dbs.elki.math.linearalgebra.Matrix;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.AbstractParameterizer;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.GreaterEqualConstraint;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.ListEachConstraint;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.CommonConstraints;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.ListSizeConstraint;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntListParameter;
@@ -89,8 +88,14 @@ public class HSBHistogramQuadraticDistanceFunction extends WeightedDistanceFunct
         final int sy = (y / quantb) % quants;
         final int by = y % quantb;
 
-        final double cos = Math.cos((hx + .5) / quanth * MathUtil.TWOPI) * (sx + .5) / quants - Math.cos((hy + .5) / quanth * MathUtil.TWOPI) * (sy + .5) / quants;
-        final double sin = Math.sin((hx + .5) / quanth * MathUtil.TWOPI) * (sx + .5) / quants - Math.sin((hy + .5) / quanth * MathUtil.TWOPI) * (sy + .5) / quants;
+        final double chx = Math.cos((hx + .5) / quanth * MathUtil.TWOPI);
+        final double chy = Math.cos((hy + .5) / quanth * MathUtil.TWOPI);
+        // final double shx = Math.sin((hx + .5) / quanth * MathUtil.TWOPI);
+        final double shx = MathUtil.cosToSin((hx + .5) / quanth * MathUtil.TWOPI, chx);
+        // final double shy = Math.sin((hy + .5) / quanth * MathUtil.TWOPI);
+        final double shy = MathUtil.cosToSin((hy + .5) / quanth * MathUtil.TWOPI, chy);
+        final double cos = chx * (sx + .5) / quants - chy * (sy + .5) / quants;
+        final double sin = shx * (sx + .5) / quants - shy * (sy + .5) / quants;
         final double db = (bx - by) / (double) quantb;
         final double val = 1. - Math.sqrt((db * db + sin * sin + cos * cos) / 5);
         m.set(x, y, val);
@@ -107,12 +112,12 @@ public class HSBHistogramQuadraticDistanceFunction extends WeightedDistanceFunct
     if(obj == null) {
       return false;
     }
-    if (!this.getClass().equals(obj.getClass())) {
+    if(!this.getClass().equals(obj.getClass())) {
       return false;
     }
-    return this.weightMatrix.equals(((HSBHistogramQuadraticDistanceFunction)obj).weightMatrix);
+    return this.weightMatrix.equals(((HSBHistogramQuadraticDistanceFunction) obj).weightMatrix);
   }
-  
+
   /**
    * Parameterization class.
    * 
@@ -132,7 +137,7 @@ public class HSBHistogramQuadraticDistanceFunction extends WeightedDistanceFunct
       super.makeOptions(config);
       IntListParameter param = new IntListParameter(BPP_ID);
       param.addConstraint(new ListSizeConstraint(3));
-      param.addConstraint(new ListEachConstraint<Integer>(new GreaterEqualConstraint(1)));
+      param.addConstraint(CommonConstraints.GREATER_EQUAL_ONE_INT_LIST);
       if(config.grab(param)) {
         List<Integer> quant = param.getValue();
         assert (quant.size() == 3);
