@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.distance.distancefunction.external;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -28,7 +28,6 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.util.HashMap;
-import java.util.List;
 import java.util.Map;
 import java.util.regex.Pattern;
 
@@ -40,6 +39,7 @@ import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.ids.ModifiableDBIDs;
 import de.lmu.ifi.dbs.elki.datasource.parser.AbstractParser;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
+import de.lmu.ifi.dbs.elki.distance.distancevalue.DoubleDistance;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.NumberDistance;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.logging.progress.IndefiniteProgress;
@@ -78,11 +78,12 @@ public class NumberDistanceParser<D extends NumberDistance<D, ?>> extends Abstra
    * Constructor.
    * 
    * @param colSep Column separator pattern
-   * @param quoteChar Quote character
+   * @param quoteChars Quote characters
+   * @param comment Comment pattern
    * @param distanceFactory Distance factory to use
    */
-  public NumberDistanceParser(Pattern colSep, char quoteChar, D distanceFactory) {
-    super(colSep, quoteChar);
+  public NumberDistanceParser(Pattern colSep, String quoteChars, Pattern comment, D distanceFactory) {
+    super(colSep, quoteChars, comment);
     this.distanceFactory = distanceFactory;
   }
 
@@ -93,61 +94,87 @@ public class NumberDistanceParser<D extends NumberDistance<D, ?>> extends Abstra
 
     IndefiniteProgress prog = LOG.isVerbose() ? new IndefiniteProgress("Parsing distance matrix", LOG) : null;
     ModifiableDBIDs ids = DBIDUtil.newHashSet();
-    Map<DBIDPair, D> distanceCache = new HashMap<DBIDPair, D>();
+    Map<DBIDPair, D> distanceCache = new HashMap<>();
     try {
-      for (String line; (line = reader.readLine()) != null; lineNumber++) {
-        if (prog != null) {
+      for(String line; (line = reader.readLine()) != null; lineNumber++) {
+        if(prog != null) {
           prog.incrementProcessed(LOG);
         }
-        if (!line.startsWith(COMMENT) && line.length() > 0) {
-          List<String> entries = tokenize(line);
-          if (entries.size() != 3) {
-            throw new IllegalArgumentException("Line " + lineNumber + " does not have the " + "required input format: id1 id2 distanceValue! " + line);
-          }
+        // Skip empty lines and comments
+        if(line.length() <= 0 || (comment != null && comment.matcher(line).matches())) {
+          continue;
+        }
+        tokenizer.initialize(line, 0, lengthWithoutLinefeed(line));
 
-          DBID id1, id2;
-          try {
-            id1 = DBIDUtil.importInteger(Integer.parseInt(entries.get(0)));
-          } catch (NumberFormatException e) {
-            throw new IllegalArgumentException("Error in line " + lineNumber + ": id1 is no integer!");
-          }
+        if(!tokenizer.valid()) {
+          throw new IllegalArgumentException("Less than three values in line " + lineNumber);
+        }
+        DBID id1, id2;
+        try {
+          id1 = DBIDUtil.importInteger((int) tokenizer.getLongBase10());
+          tokenizer.advance();
+        }
+        catch(NumberFormatException e) {
+          throw new IllegalArgumentException("Error in line " + lineNumber + ": id1 is no integer!");
+        }
+        if(!tokenizer.valid()) {
+          throw new IllegalArgumentException("Less than three values in line " + lineNumber);
+        }
 
-          try {
-            id2 = DBIDUtil.importInteger(Integer.parseInt(entries.get(1)));
-          } catch (NumberFormatException e) {
-            throw new IllegalArgumentException("Error in line " + lineNumber + ": id2 is no integer!");
-          }
+        try {
+          id2 = DBIDUtil.importInteger((int) tokenizer.getLongBase10());
+          tokenizer.advance();
+        }
+        catch(NumberFormatException e) {
+          throw new IllegalArgumentException("Error in line " + lineNumber + ": id2 is no integer!");
+        }
+        if(!tokenizer.valid()) {
+          throw new IllegalArgumentException("Less than three values in line " + lineNumber);
+        }
 
-          try {
-            D distance = distanceFactory.parseString(entries.get(2));
-            put(id1, id2, distance, distanceCache);
-            ids.add(id1);
-            ids.add(id2);
-          } catch (IllegalArgumentException e) {
-            throw new IllegalArgumentException("Error in line " + lineNumber + ":" + e.getMessage(), e);
+        try {
+          final D distance;
+          if(distanceFactory == DoubleDistance.FACTORY) {
+            @SuppressWarnings("unchecked")
+            D dist = (D) DoubleDistance.FACTORY.fromDouble(tokenizer.getDouble());
+            distance = dist;
           }
+          else {
+            distance = distanceFactory.parseString(tokenizer.getSubstring());
+          }
+          tokenizer.advance();
+          put(id1, id2, distance, distanceCache);
+          ids.add(id1);
+          ids.add(id2);
+        }
+        catch(IllegalArgumentException e) {
+          throw new IllegalArgumentException("Error in line " + lineNumber + ":" + e.getMessage(), e);
+        }
+        if(tokenizer.valid()) {
+          throw new IllegalArgumentException("More than three values in line " + lineNumber);
         }
       }
-    } catch (IOException e) {
+    }
+    catch(IOException e) {
       throw new IllegalArgumentException("Error while parsing line " + lineNumber + ".");
     }
 
-    if (prog != null) {
+    if(prog != null) {
       prog.setCompleted(LOG);
     }
 
     // check if all distance values are specified
-    for (DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
-      for (DBIDIter iter2 = ids.iter(); iter2.valid(); iter2.advance()) {
-        if (DBIDUtil.compare(iter2, iter) <= 0) {
+    for(DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
+      for(DBIDIter iter2 = ids.iter(); iter2.valid(); iter2.advance()) {
+        if(DBIDUtil.compare(iter2, iter) <= 0) {
           continue;
         }
-        if (!containsKey(iter, iter2, distanceCache)) {
+        if(!containsKey(iter, iter2, distanceCache)) {
           throw new IllegalArgumentException("Distance value for " + DBIDUtil.toString(iter) + " - " + DBIDUtil.toString(iter2) + " is missing!");
         }
       }
     }
-    return new DistanceParsingResult<D>(distanceCache);
+    return new DistanceParsingResult<>(distanceCache);
   }
 
   /**
@@ -160,14 +187,14 @@ public class NumberDistanceParser<D extends NumberDistance<D, ?>> extends Abstra
    */
   private void put(DBID id1, DBID id2, D distance, Map<DBIDPair, D> cache) {
     // the smaller id is the first key
-    if (DBIDUtil.compare(id1, id2) > 0) {
+    if(DBIDUtil.compare(id1, id2) > 0) {
       put(id2, id1, distance, cache);
       return;
     }
 
     D oldDistance = cache.put(DBIDUtil.newPair(id1, id2), distance);
 
-    if (oldDistance != null) {
+    if(oldDistance != null) {
       throw new IllegalArgumentException("Distance value for specified ids is already assigned!");
     }
   }
@@ -183,7 +210,7 @@ public class NumberDistanceParser<D extends NumberDistance<D, ?>> extends Abstra
    *         specified ids, false otherwise
    */
   public boolean containsKey(DBIDRef id1, DBIDRef id2, Map<DBIDPair, D> cache) {
-    if (DBIDUtil.compare(id1, id2) > 0) {
+    if(DBIDUtil.compare(id1, id2) > 0) {
       return containsKey(id2, id1, cache);
     }
 
@@ -211,15 +238,15 @@ public class NumberDistanceParser<D extends NumberDistance<D, ?>> extends Abstra
     @Override
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
-      ObjectParameter<D> distFuncP = new ObjectParameter<D>(DISTANCE_ID, Distance.class);
-      if (config.grab(distFuncP)) {
+      ObjectParameter<D> distFuncP = new ObjectParameter<>(DISTANCE_ID, Distance.class);
+      if(config.grab(distFuncP)) {
         distanceFactory = distFuncP.instantiateClass(config);
       }
     }
 
     @Override
     protected NumberDistanceParser<D> makeInstance() {
-      return new NumberDistanceParser<D>(colSep, quoteChar, distanceFactory);
+      return new NumberDistanceParser<>(colSep, quoteChars, comment, distanceFactory);
     }
   }
 }
