@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.distance.distancevalue;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -40,23 +40,6 @@ import java.io.Externalizable;
  * @param <D> the type of Distance used
  */
 public interface Distance<D extends Distance<D>> extends Comparable<D>, Externalizable {
-  /**
-   * Returns a new distance as sum of this distance and the given distance.
-   * 
-   * @param distance the distance to be added to this distance
-   * @return a new distance as sum of this distance and the given distance
-   */
-  D plus(D distance);
-
-  /**
-   * Returns a new Distance by subtracting the given distance from this
-   * distance.
-   * 
-   * @param distance the distance to be subtracted from this distance
-   * @return a new Distance by subtracting the given distance from this distance
-   */
-  D minus(D distance);
-
   /**
    * Any implementing class should implement a proper toString-method for
    * printing the result-values.
