@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.distance.distancevalue;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -29,6 +29,8 @@ import java.io.ObjectOutput;
 import java.util.BitSet;
 import java.util.regex.Pattern;
 
+import de.lmu.ifi.dbs.elki.utilities.FormatUtil;
+
 /**
  * A PreferenceVectorBasedCorrelationDistance holds additionally to the
  * CorrelationDistance the common preference vector of the two objects defining
@@ -104,42 +106,6 @@ public class PreferenceVectorBasedCorrelationDistance extends CorrelationDistanc
     return super.toString() + SEPARATOR + commonPreferenceVector.toString();
   }
 
-  /**
-   * @throws IllegalArgumentException if the dimensionality values and common
-   *         preference vectors of this distance and the specified distance are
-   *         not equal
-   */
-  @Override
-  public PreferenceVectorBasedCorrelationDistance plus(PreferenceVectorBasedCorrelationDistance distance) {
-    if(this.dimensionality != distance.dimensionality) {
-      throw new IllegalArgumentException("The dimensionality values of this distance " + "and the specified distance need to be equal.\n" + "this.dimensionality     " + this.dimensionality + "\n" + "distance.dimensionality " + distance.dimensionality + "\n");
-    }
-
-    if(!this.commonPreferenceVector.equals(distance.commonPreferenceVector)) {
-      throw new IllegalArgumentException("The common preference vectors of this distance " + "and the specified distance need to be equal.\n" + "this.commonPreferenceVector     " + this.commonPreferenceVector + "\n" + "distance.commonPreferenceVector " + distance.commonPreferenceVector + "\n");
-    }
-
-    return new PreferenceVectorBasedCorrelationDistance(dimensionality, getCorrelationValue() + distance.getCorrelationValue(), getEuclideanValue() + distance.getEuclideanValue(), (BitSet) commonPreferenceVector.clone());
-  }
-
-  /**
-   * @throws IllegalArgumentException if the dimensionality values and common
-   *         preference vectors of this distance and the specified distance are
-   *         not equal
-   */
-  @Override
-  public PreferenceVectorBasedCorrelationDistance minus(PreferenceVectorBasedCorrelationDistance distance) {
-    if(this.dimensionality != distance.dimensionality) {
-      throw new IllegalArgumentException("The dimensionality values of this distance " + "and the specified distance need to be equal.\n" + "this.dimensionality     " + this.dimensionality + "\n" + "distance.dimensionality " + distance.dimensionality + "\n");
-    }
-
-    if(!this.commonPreferenceVector.equals(distance.commonPreferenceVector)) {
-      throw new IllegalArgumentException("The common preference vectors of this distance " + "and the specified distance need to be equal.\n" + "this.commonPreferenceVector     " + this.commonPreferenceVector + "\n" + "distance.commonPreferenceVector " + distance.commonPreferenceVector + "\n");
-    }
-
-    return new PreferenceVectorBasedCorrelationDistance(dimensionality, getCorrelationValue() - distance.getCorrelationValue(), getEuclideanValue() - distance.getEuclideanValue(), (BitSet) commonPreferenceVector.clone());
-  }
-
   /**
    * Checks if the dimensionality values of this distance and the specified
    * distance are equal. If the check fails an IllegalArgumentException is
@@ -217,7 +183,7 @@ public class PreferenceVectorBasedCorrelationDistance extends CorrelationDistanc
     }
     if(testInputPattern(pattern)) {
       String[] values = SEPARATOR.split(pattern);
-      return new PreferenceVectorBasedCorrelationDistance(-1, Integer.parseInt(values[0]), Double.parseDouble(values[1]), new BitSet());
+      return new PreferenceVectorBasedCorrelationDistance(-1, Integer.parseInt(values[0]), FormatUtil.parseDouble(values[1]), new BitSet());
     }
     else {
       throw new IllegalArgumentException("Given pattern \"" + pattern + "\" does not match required pattern \"" + requiredInputPattern() + "\"");
