@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.distance.similarityfunction;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -34,6 +34,8 @@ import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
  * 
  * @author Arthur Zimek
  * 
+ * @apiviz.excludeSubtypes
+ * 
  * @param <O> object type
  * @param <D> distance type
  */
@@ -59,6 +61,6 @@ public abstract class AbstractPrimitiveSimilarityFunction<O, D extends Distance<
 
   @Override
   public <T extends O> SimilarityQuery<T, D> instantiate(Relation<T> relation) {
-    return new PrimitiveSimilarityQuery<T, D>(relation, this);
+    return new PrimitiveSimilarityQuery<>(relation, this);
   }
-}
\ No newline at end of file
+}
