@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.evaluation.outlier;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -40,6 +40,7 @@ import de.lmu.ifi.dbs.elki.result.Result;
 import de.lmu.ifi.dbs.elki.result.ResultUtil;
 import de.lmu.ifi.dbs.elki.result.outlier.OutlierResult;
 import de.lmu.ifi.dbs.elki.result.textwriter.TextWriterStream;
+import de.lmu.ifi.dbs.elki.utilities.Alias;
 import de.lmu.ifi.dbs.elki.utilities.DatabaseUtil;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.AbstractParameterizer;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
@@ -65,7 +66,7 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.PatternParameter;
  * @apiviz.uses ROC
  * @apiviz.has ROCResult oneway - - «create»
  */
-// TODO: maybe add a way to process clustering results as well?
+@Alias({ "ComputeROCCurve", "de.lmu.ifi.dbs.elki.evaluation.roc.ComputeROCCurve" })
 public class OutlierROCCurve implements Evaluator {
   /**
    * The label we use for marking ROCAUC values.
@@ -102,12 +103,12 @@ public class OutlierROCCurve implements Evaluator {
   }
 
   private ROCResult computeROCResult(int size, SetDBIDs positiveids, DBIDs order) {
-    if(order.size() != size) {
+    if (order.size() != size) {
       throw new IllegalStateException("Iterable result doesn't match database size - incomplete ordering?");
     }
-    XYCurve roccurve = ROC.materializeROC(size, positiveids, new ROC.SimpleAdapter(order.iter()));
+    XYCurve roccurve = ROC.materializeROC(new ROC.DBIDsTest(positiveids), new ROC.SimpleAdapter(order.iter()));
     double rocauc = XYCurve.areaUnderCurve(roccurve);
-    if(LOG.isVerbose()) {
+    if (LOG.isVerbose()) {
       LOG.verbose(ROCAUC_LABEL + ": " + rocauc);
     }
 
@@ -117,9 +118,9 @@ public class OutlierROCCurve implements Evaluator {
   }
 
   private ROCResult computeROCResult(int size, SetDBIDs positiveids, OutlierResult or) {
-    XYCurve roccurve = ROC.materializeROC(size, positiveids, new ROC.OutlierScoreAdapter(or));
+    XYCurve roccurve = ROC.materializeROC(new ROC.DBIDsTest(positiveids), new ROC.OutlierScoreAdapter(or));
     double rocauc = XYCurve.areaUnderCurve(roccurve);
-    if(LOG.isVerbose()) {
+    if (LOG.isVerbose()) {
       LOG.verbose(ROCAUC_LABEL + ": " + rocauc);
     }
 
@@ -134,7 +135,7 @@ public class OutlierROCCurve implements Evaluator {
     // Prepare
     SetDBIDs positiveids = DBIDUtil.ensureSet(DatabaseUtil.getObjectsByLabelMatch(db, positiveClassName));
 
-    if(positiveids.size() == 0) {
+    if (positiveids.size() == 0) {
       LOG.warning("Computing a ROC curve failed - no objects matched.");
       return;
     }
@@ -143,7 +144,7 @@ public class OutlierROCCurve implements Evaluator {
     List<OutlierResult> oresults = ResultUtil.getOutlierResults(result);
     List<OrderingResult> orderings = ResultUtil.getOrderingResults(result);
     // Outlier results are the main use case.
-    for(OutlierResult o : oresults) {
+    for (OutlierResult o : oresults) {
       db.getHierarchy().add(o, computeROCResult(o.getScores().size(), positiveids, o));
       // Process them only once.
       orderings.remove(o.getOrdering());
@@ -152,13 +153,13 @@ public class OutlierROCCurve implements Evaluator {
 
     // FIXME: find appropriate place to add the derived result
     // otherwise apply an ordering to the database IDs.
-    for(OrderingResult or : orderings) {
+    for (OrderingResult or : orderings) {
       DBIDs sorted = or.iter(or.getDBIDs());
       db.getHierarchy().add(or, computeROCResult(or.getDBIDs().size(), positiveids, sorted));
       nonefound = false;
     }
 
-    if(nonefound) {
+    if (nonefound) {
       return;
       // logger.warning("No results found to process with ROC curve analyzer. Got "+iterables.size()+" iterables, "+orderings.size()+" orderings.");
     }
@@ -228,7 +229,7 @@ public class OutlierROCCurve implements Evaluator {
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
       PatternParameter positiveClassNameP = new PatternParameter(POSITIVE_CLASS_NAME_ID);
-      if(config.grab(positiveClassNameP)) {
+      if (config.grab(positiveClassNameP)) {
         positiveClassName = positiveClassNameP.getValue();
       }
     }
@@ -238,4 +239,4 @@ public class OutlierROCCurve implements Evaluator {
       return new OutlierROCCurve(positiveClassName);
     }
   }
-}
\ No newline at end of file
+}
