@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.evaluation.similaritymatrix;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -41,7 +41,7 @@ import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.DistanceFunction;
-import de.lmu.ifi.dbs.elki.distance.distancefunction.EuclideanDistanceFunction;
+import de.lmu.ifi.dbs.elki.distance.distancefunction.minkowski.EuclideanDistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.NumberDistance;
 import de.lmu.ifi.dbs.elki.evaluation.Evaluator;
 import de.lmu.ifi.dbs.elki.logging.Logging;
@@ -358,7 +358,7 @@ public class ComputeSimilarityMatrixImage<O> implements Evaluator {
         distanceFunction = distanceFunctionP.instantiateClass(config);
       }
 
-      ObjectParameter<ScalingFunction> scalingP = new ObjectParameter<ScalingFunction>(SCALING_ID, ScalingFunction.class, true);
+      ObjectParameter<ScalingFunction> scalingP = new ObjectParameter<>(SCALING_ID, ScalingFunction.class, true);
       if(config.grab(scalingP)) {
         scaling = scalingP.instantiateClass(config);
       }
@@ -371,7 +371,7 @@ public class ComputeSimilarityMatrixImage<O> implements Evaluator {
 
     @Override
     protected ComputeSimilarityMatrixImage<O> makeInstance() {
-      return new ComputeSimilarityMatrixImage<O>(distanceFunction, scaling, skipzero);
+      return new ComputeSimilarityMatrixImage<>(distanceFunction, scaling, skipzero);
     }
   }
 }
\ No newline at end of file
