@@ -1,9 +1,10 @@
 package de.lmu.ifi.dbs.elki.index;
+
 /*
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -22,20 +23,15 @@ package de.lmu.ifi.dbs.elki.index;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-import java.util.List;
-
-import de.lmu.ifi.dbs.elki.database.ids.ArrayDBIDs;
 import de.lmu.ifi.dbs.elki.database.ids.DBID;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
-import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.database.query.knn.AbstractDistanceKNNQuery;
 import de.lmu.ifi.dbs.elki.database.query.range.AbstractDistanceRangeQuery;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
-import de.lmu.ifi.dbs.elki.distance.distanceresultlist.DistanceDBIDResult;
-import de.lmu.ifi.dbs.elki.distance.distanceresultlist.KNNResult;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
-import de.lmu.ifi.dbs.elki.persistent.PageFileStatistics;
+import de.lmu.ifi.dbs.elki.logging.Logging;
+import de.lmu.ifi.dbs.elki.logging.statistics.Counter;
 
 /**
  * Abstract base class for Filter-refinement indexes.
@@ -44,16 +40,17 @@ import de.lmu.ifi.dbs.elki.persistent.PageFileStatistics;
  * 
  * @author Erich Schubert
  * 
+ * @apiviz.excludeSubtypes
  * @apiviz.has AbstractRangeQuery
  * @apiviz.has AbstractKNNQuery
  * 
  * @param <O> Object type
  */
-public abstract class AbstractRefiningIndex<O> extends AbstractIndex<O> implements PageFileStatistics {
+public abstract class AbstractRefiningIndex<O> extends AbstractIndex<O> {
   /**
    * Refinement counter.
    */
-  private int refinements;
+  private Counter refinements;
 
   /**
    * Constructor.
@@ -62,15 +59,34 @@ public abstract class AbstractRefiningIndex<O> extends AbstractIndex<O> implemen
    */
   public AbstractRefiningIndex(Relation<O> relation) {
     super(relation);
+    Logging log = getLogger();
+    refinements = log.isStatistics() ? log.newCounter(this.getClass().getName() + ".refinements") : null;
   }
 
   /**
-   * Initialize the index.
+   * Get the class logger.
    * 
-   * @param relation Relation to index
-   * @param ids database ids
+   * @return Logger
    */
-  protected abstract void initialize(Relation<O> relation, DBIDs ids);
+  abstract public Logging getLogger();
+
+  /**
+   * Increment the refinement counter, if in use.
+   * 
+   * @param i Increment.
+   */
+  protected void countRefinements(int i) {
+    if (refinements != null) {
+      refinements.increment(i);
+    }
+  }
+
+  @Override
+  public void logStatistics() {
+    if (refinements != null) {
+      getLogger().statistics(refinements);
+    }
+  }
 
   /**
    * Refine a given object (and count the refinement!).
@@ -79,51 +95,18 @@ public abstract class AbstractRefiningIndex<O> extends AbstractIndex<O> implemen
    * @return refined object
    */
   protected O refine(DBID id) {
-    refinements++;
+    countRefinements(1);
     return relation.get(id);
   }
 
-  @Override
-  public PageFileStatistics getPageFileStatistics() {
-    return this;
-  }
-
-  @Override
-  public long getReadOperations() {
-    return refinements;
-  }
-
-  @Override
-  public long getWriteOperations() {
-    return 0;
-  }
-
-  @Override
-  public void resetPageAccess() {
-    refinements = 0;
-  }
-
-  @Override
-  public PageFileStatistics getInnerStatistics() {
-    return null;
-  }
-
-  @Override
-  public void insertAll(DBIDs ids) {
-    initialize(relation, ids);
-  }
-
   /**
    * Range query for this index.
    * 
    * @author Erich Schubert
+   * 
+   * @apiviz.excludeSubtypes
    */
   public abstract class AbstractRangeQuery<D extends Distance<D>> extends AbstractDistanceRangeQuery<O, D> {
-    /**
-     * Hold the distance function to be used.
-     */
-    private DistanceQuery<O, D> distanceQuery;
-
     /**
      * Constructor.
      * 
@@ -131,12 +114,6 @@ public abstract class AbstractRefiningIndex<O> extends AbstractIndex<O> implemen
      */
     public AbstractRangeQuery(DistanceQuery<O, D> distanceQuery) {
       super(distanceQuery);
-      this.distanceQuery = distanceQuery;
-    }
-
-    @Override
-    public DistanceDBIDResult<D> getRangeForDBID(DBIDRef id, D range) {
-      return getRangeForObject(relation.get(id), range);
     }
 
     /**
@@ -147,7 +124,7 @@ public abstract class AbstractRefiningIndex<O> extends AbstractIndex<O> implemen
      * @return Distance
      */
     protected D refine(DBIDRef id, O q) {
-      AbstractRefiningIndex.this.refinements++;
+      AbstractRefiningIndex.this.countRefinements(1);
       return distanceQuery.distance(q, id);
     }
 
@@ -157,7 +134,7 @@ public abstract class AbstractRefiningIndex<O> extends AbstractIndex<O> implemen
      * @param c Refinements
      */
     protected void incRefinements(int c) {
-      AbstractRefiningIndex.this.refinements += c;
+      AbstractRefiningIndex.this.countRefinements(c);
     }
   }
 
@@ -165,6 +142,8 @@ public abstract class AbstractRefiningIndex<O> extends AbstractIndex<O> implemen
    * KNN query for this index.
    * 
    * @author Erich Schubert
+   * 
+   * @apiviz.excludeSubtypes
    */
   public abstract class AbstractKNNQuery<D extends Distance<D>> extends AbstractDistanceKNNQuery<O, D> {
     /**
@@ -176,16 +155,6 @@ public abstract class AbstractRefiningIndex<O> extends AbstractIndex<O> implemen
       super(distanceQuery);
     }
 
-    @Override
-    public List<KNNResult<D>> getKNNForBulkDBIDs(ArrayDBIDs ids, int k) {
-      throw new UnsupportedOperationException("Not yet implemented.");
-    }
-
-    @Override
-    public KNNResult<D> getKNNForDBID(DBIDRef id, int k) {
-      return getKNNForObject(relation.get(id), k);
-    }
-
     /**
      * Refinement distance computation.
      * 
@@ -194,7 +163,7 @@ public abstract class AbstractRefiningIndex<O> extends AbstractIndex<O> implemen
      * @return Distance
      */
     protected D refine(DBID id, O q) {
-      AbstractRefiningIndex.this.refinements++;
+      AbstractRefiningIndex.this.countRefinements(1);
       return distanceQuery.distance(q, id);
     }
 
@@ -204,7 +173,7 @@ public abstract class AbstractRefiningIndex<O> extends AbstractIndex<O> implemen
      * @param c Refinements
      */
     protected void incRefinements(int c) {
-      AbstractRefiningIndex.this.refinements += c;
+      AbstractRefiningIndex.this.countRefinements(c);
     }
   }
-}
\ No newline at end of file
+}
