@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.index.preprocessed.knn;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -29,19 +29,18 @@ import de.lmu.ifi.dbs.elki.database.ids.ArrayDBIDs;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
+import de.lmu.ifi.dbs.elki.database.ids.distance.KNNHeap;
+import de.lmu.ifi.dbs.elki.database.ids.distance.KNNList;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.DistanceFunction;
-import de.lmu.ifi.dbs.elki.distance.distanceresultlist.KNNHeap;
-import de.lmu.ifi.dbs.elki.distance.distanceresultlist.KNNResult;
-import de.lmu.ifi.dbs.elki.distance.distanceresultlist.KNNUtil;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.logging.progress.FiniteProgress;
 import de.lmu.ifi.dbs.elki.utilities.RandomFactory;
+import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.GreaterConstraint;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.LessConstraint;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.CommonConstraints;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.DoubleParameter;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.RandomParameter;
@@ -49,12 +48,22 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.RandomParameter;
 /**
  * Class that computed the kNN only on a random sample.
  * 
+ * Used in:
+ * <p>
+ * Subsampling for Efficient and Effective Unsupervised Outlier Detection
+ * Ensembles<br />
+ * A. Zimek and M. Gaudet and R. J. G. B. Campello and J. Sander<br />
+ * In: Proc. 19th ACM SIGKDD International Conference on Knowledge Discovery and
+ * Data Mining, KDD '13
+ * </p>
+ * 
  * @author Erich Schubert
  * 
  * @param <O> Object type
  * @param <D> Distance type
  */
-public class RandomSampleKNNPreprocessor<O, D extends Distance<D>> extends AbstractMaterializeKNNPreprocessor<O, D, KNNResult<D>> {
+@Reference(title = "Subsampling for Efficient and Effective Unsupervised Outlier Detection Ensembles", authors = "A. Zimek and M. Gaudet and R. J. G. B. Campello and J. Sander", booktitle = "Proc. 19th ACM SIGKDD International Conference on Knowledge Discovery and Data Mining, KDD '13")
+public class RandomSampleKNNPreprocessor<O, D extends Distance<D>> extends AbstractMaterializeKNNPreprocessor<O, D, KNNList<D>> {
   /**
    * Logger
    */
@@ -88,28 +97,28 @@ public class RandomSampleKNNPreprocessor<O, D extends Distance<D>> extends Abstr
   @Override
   protected void preprocess() {
     DistanceQuery<O, D> distanceQuery = relation.getDatabase().getDistanceQuery(relation, distanceFunction);
-    storage = DataStoreUtil.makeStorage(relation.getDBIDs(), DataStoreFactory.HINT_STATIC, KNNResult.class);
+    storage = DataStoreUtil.makeStorage(relation.getDBIDs(), DataStoreFactory.HINT_STATIC, KNNList.class);
     FiniteProgress progress = getLogger().isVerbose() ? new FiniteProgress("Materializing random-sample k nearest neighbors (k=" + k + ")", relation.size(), getLogger()) : null;
 
     final ArrayDBIDs ids = DBIDUtil.ensureArray(relation.getDBIDs());
     final int samplesize = (int) (ids.size() * share);
 
-    for (DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
-      KNNHeap<D> kNN = KNNUtil.newHeap(distanceFunction, k);
+    for(DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
+      KNNHeap<D> kNN = DBIDUtil.newHeap(distanceFunction.getDistanceFactory(), k);
 
       DBIDs rsamp = DBIDUtil.randomSample(ids, samplesize, rnd);
-      for (DBIDIter iter2 = rsamp.iter(); iter2.valid(); iter2.advance()) {
+      for(DBIDIter iter2 = rsamp.iter(); iter2.valid(); iter2.advance()) {
         D dist = distanceQuery.distance(iter, iter2);
-        kNN.add(dist, iter2);
+        kNN.insert(dist, iter2);
       }
 
       storage.put(iter, kNN.toKNNList());
-      if (progress != null) {
+      if(progress != null) {
         progress.incrementProcessed(getLogger());
       }
     }
 
-    if (progress != null) {
+    if(progress != null) {
       progress.ensureCompleted(getLogger());
     }
   }
@@ -129,6 +138,11 @@ public class RandomSampleKNNPreprocessor<O, D extends Distance<D>> extends Abstr
     return "random-sample-knn";
   }
 
+  @Override
+  public void logStatistics() {
+    // No statistics to log.
+  }
+
   /**
    * The parameterizable factory.
    * 
@@ -141,7 +155,7 @@ public class RandomSampleKNNPreprocessor<O, D extends Distance<D>> extends Abstr
    * @param <O> The object type
    * @param <D> The distance type
    */
-  public static class Factory<O, D extends Distance<D>> extends AbstractMaterializeKNNPreprocessor.Factory<O, D, KNNResult<D>> {
+  public static class Factory<O, D extends Distance<D>> extends AbstractMaterializeKNNPreprocessor.Factory<O, D, KNNList<D>> {
     /**
      * Relative share of objects to get
      */
@@ -168,7 +182,7 @@ public class RandomSampleKNNPreprocessor<O, D extends Distance<D>> extends Abstr
 
     @Override
     public RandomSampleKNNPreprocessor<O, D> instantiate(Relation<O> relation) {
-      return new RandomSampleKNNPreprocessor<O, D>(relation, distanceFunction, k, share, rnd);
+      return new RandomSampleKNNPreprocessor<>(relation, distanceFunction, k, share, rnd);
     }
 
     /**
@@ -215,20 +229,20 @@ public class RandomSampleKNNPreprocessor<O, D extends Distance<D>> extends Abstr
       protected void makeOptions(Parameterization config) {
         super.makeOptions(config);
         DoubleParameter shareP = new DoubleParameter(SHARE_ID);
-        shareP.addConstraint(new GreaterConstraint(0.0));
-        shareP.addConstraint(new LessConstraint(1.0));
-        if (config.grab(shareP)) {
+        shareP.addConstraint(CommonConstraints.GREATER_THAN_ZERO_DOUBLE);
+        shareP.addConstraint(CommonConstraints.LESS_THAN_ONE_DOUBLE);
+        if(config.grab(shareP)) {
           share = shareP.getValue();
         }
         RandomParameter rndP = new RandomParameter(SEED_ID);
-        if (config.grab(rndP)) {
+        if(config.grab(rndP)) {
           rnd = rndP.getValue();
         }
       }
 
       @Override
       protected RandomSampleKNNPreprocessor.Factory<O, D> makeInstance() {
-        return new RandomSampleKNNPreprocessor.Factory<O, D>(k, distanceFunction, share, rnd);
+        return new RandomSampleKNNPreprocessor.Factory<>(k, distanceFunction, share, rnd);
       }
     }
   }
