@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.index.tree;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -98,7 +98,7 @@ public abstract class AbstractNode<E extends Entry> extends AbstractExternalizab
       public IndexTreePath<E> nextElement() {
         synchronized(AbstractNode.this) {
           if(count < numEntries) {
-            return parentPath.pathByAddingChild(new TreeIndexPathComponent<E>(entries[count], count++));
+            return parentPath.pathByAddingChild(new TreeIndexPathComponent<>(entries[count], count++));
           }
         }
         throw new NoSuchElementException();
@@ -284,7 +284,7 @@ public abstract class AbstractNode<E extends Entry> extends AbstractExternalizab
    */
   @Deprecated
   public final List<E> getEntries() {
-    List<E> result = new ArrayList<E>(numEntries);
+    List<E> result = new ArrayList<>(numEntries);
     for(E entry : entries) {
       if(entry != null) {
         result.add(entry);
