@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.index.tree;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -74,7 +74,7 @@ public class BreadthFirstEnumeration<N extends Node<E>, E extends Entry> impleme
    */
   public BreadthFirstEnumeration(final IndexTree<N, E> index, final IndexTreePath<E> rootPath) {
     super();
-    this.queue = new LinkedList<Enumeration<IndexTreePath<E>>>();
+    this.queue = new LinkedList<>();
     this.index = index;
 
     Enumeration<IndexTreePath<E>> root_enum = new Enumeration<IndexTreePath<E>>() {
