@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.index.tree;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -250,7 +250,7 @@ public class TreeIndexHeader extends DefaultPageHeader {
   @SuppressWarnings("unchecked")
   public Stack<Integer> readEmptyPages(RandomAccessFile file) throws IOException, ClassNotFoundException {
     if(emptyPagesSize == 0) {
-      return new Stack<Integer>();
+      return new Stack<>();
     }
     byte[] bytes = new byte[emptyPagesSize];
     file.seek(file.length() - emptyPagesSize);
