@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.index.tree.metrical;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -25,7 +25,6 @@ package de.lmu.ifi.dbs.elki.index.tree.metrical;
 
 import java.util.List;
 
-import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.DistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
 import de.lmu.ifi.dbs.elki.index.tree.Entry;
@@ -60,13 +59,6 @@ public abstract class MetricalIndexTree<O, D extends Distance<D>, N extends Node
    */
   public abstract DistanceFunction<? super O, D> getDistanceFunction();
 
-  /**
-   * Returns the distance function of this metrical index.
-   * 
-   * @return the distance function of this metrical index
-   */
-  public abstract DistanceQuery<O, D> getDistanceQuery();
-
   /**
    * Returns a list of entries pointing to the leaf nodes of this spatial index.
    * 
