@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.index.tree.metrical.mtreevariants.mktrees.mkcop;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -23,8 +23,6 @@ package de.lmu.ifi.dbs.elki.index.tree.metrical.mtreevariants.mktrees.mkcop;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
-import de.lmu.ifi.dbs.elki.distance.distancevalue.NumberDistance;
 import de.lmu.ifi.dbs.elki.index.tree.metrical.mtreevariants.MTreeEntry;
 
 /**
@@ -36,17 +34,14 @@ import de.lmu.ifi.dbs.elki.index.tree.metrical.mtreevariants.MTreeEntry;
  * 
  * @apiviz.composedOf ApproximationLine
  */
-interface MkCoPEntry<D extends NumberDistance<D, ?>> extends MTreeEntry<D> {
-
+interface MkCoPEntry extends MTreeEntry {
   /**
    * Returns the conservative approximated knn distance of the entry.
    * 
-   * @param <O> Object type
    * @param k the parameter k of the knn distance
-   * @param distanceFunction the distance function
    * @return the conservative approximated knn distance of the entry
    */
-  public <O> D approximateConservativeKnnDistance(int k, DistanceQuery<O, D> distanceFunction);
+  public double approximateConservativeKnnDistance(int k);
 
   /**
    * Returns the conservative approximation line.
@@ -62,4 +57,4 @@ interface MkCoPEntry<D extends NumberDistance<D, ?>> extends MTreeEntry<D> {
    *        set
    */
   public void setConservativeKnnDistanceApproximation(ApproximationLine conservativeApproximation);
-}
\ No newline at end of file
+}
