@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.index.tree.metrical.mtreevariants.mktrees.mkcop;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -32,20 +32,20 @@ import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.ids.ModifiableDBIDs;
-import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
-import de.lmu.ifi.dbs.elki.distance.distancefunction.DistanceFunction;
-import de.lmu.ifi.dbs.elki.distance.distanceresultlist.DistanceDBIDResult;
-import de.lmu.ifi.dbs.elki.distance.distanceresultlist.DistanceDBIDResultIter;
-import de.lmu.ifi.dbs.elki.distance.distanceresultlist.GenericDistanceDBIDList;
-import de.lmu.ifi.dbs.elki.distance.distanceresultlist.KNNResult;
+import de.lmu.ifi.dbs.elki.database.ids.distance.DistanceDBIDList;
+import de.lmu.ifi.dbs.elki.database.ids.distance.DistanceDBIDListIter;
+import de.lmu.ifi.dbs.elki.database.ids.distance.KNNList;
+import de.lmu.ifi.dbs.elki.database.ids.generic.GenericDistanceDBIDList;
+import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.NumberDistance;
 import de.lmu.ifi.dbs.elki.index.tree.metrical.mtreevariants.mktrees.AbstractMkTree;
+import de.lmu.ifi.dbs.elki.index.tree.metrical.mtreevariants.mktrees.MkTreeSettings;
 import de.lmu.ifi.dbs.elki.index.tree.query.GenericMTreeDistanceSearchCandidate;
 import de.lmu.ifi.dbs.elki.logging.Logging;
+import de.lmu.ifi.dbs.elki.persistent.ByteArrayUtil;
 import de.lmu.ifi.dbs.elki.persistent.PageFile;
 import de.lmu.ifi.dbs.elki.utilities.FormatUtil;
-import de.lmu.ifi.dbs.elki.utilities.QueryStatistic;
-import de.lmu.ifi.dbs.elki.utilities.datastructures.heap.Heap;
+import de.lmu.ifi.dbs.elki.utilities.datastructures.heap.ComparableMinHeap;
 
 /**
  * MkCopTree is a metrical index structure based on the concepts of the M-Tree
@@ -55,46 +55,34 @@ import de.lmu.ifi.dbs.elki.utilities.datastructures.heap.Heap;
  * @author Elke Achtert
  * 
  * @apiviz.has MkCoPTreeNode oneway - - contains
- * @apiviz.uses ConvexHull
+ * @apiviz.has ConvexHull
  * 
  * @param <O> Object type
  * @param <D> Distance type
  */
-public class MkCoPTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree<O, D, MkCoPTreeNode<O, D>, MkCoPEntry<D>> {
+public class MkCoPTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree<O, D, MkCoPTreeNode<O, D>, MkCoPEntry, MkTreeSettings<O, D, MkCoPTreeNode<O, D>, MkCoPEntry>> {
   /**
    * The logger for this class.
    */
   private static final Logging LOG = Logging.getLogger(MkCoPTree.class);
 
-  /**
-   * Parameter k.
-   */
-  int k_max;
-
   /**
    * The values of log(1),..,log(k_max)
    */
   private double[] log_k;
 
-  /**
-   * Provides some statistics about performed reverse knn-queries.
-   */
-  private QueryStatistic rkNNStatistics = new QueryStatistic();
-
   /**
    * Constructor.
    * 
+   * @param relation Relation to index
    * @param pagefile Page file
-   * @param distanceQuery Distance query
-   * @param distanceFunction Distance function
-   * @param k_max Maximum value of k supported
+   * @param settings Tree settings
    */
-  public MkCoPTree(PageFile<MkCoPTreeNode<O, D>> pagefile, DistanceQuery<O, D> distanceQuery, DistanceFunction<O, D> distanceFunction, int k_max) {
-    super(pagefile, distanceQuery, distanceFunction);
-    this.k_max = k_max;
+  public MkCoPTree(Relation<O> relation, PageFile<MkCoPTreeNode<O, D>> pagefile, MkTreeSettings<O, D, MkCoPTreeNode<O, D>, MkCoPEntry> settings) {
+    super(relation, pagefile, settings);
     // init log k
-    log_k = new double[k_max];
-    for(int k = 1; k <= k_max; k++) {
+    log_k = new double[settings.k_max];
+    for (int k = 1; k <= settings.k_max; k++) {
       log_k[k - 1] = Math.log(k);
     }
   }
@@ -103,7 +91,7 @@ public class MkCoPTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
    * @throws UnsupportedOperationException since this operation is not supported
    */
   @Override
-  protected void preInsert(MkCoPEntry<D> entry) {
+  protected void preInsert(MkCoPEntry entry) {
     throw new UnsupportedOperationException("Insertion of single objects is not supported!");
   }
 
@@ -111,40 +99,40 @@ public class MkCoPTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
    * @throws UnsupportedOperationException since this operation is not supported
    */
   @Override
-  public void insert(MkCoPEntry<D> entry, boolean withPreInsert) {
+  public void insert(MkCoPEntry entry, boolean withPreInsert) {
     throw new UnsupportedOperationException("Insertion of single objects is not supported!");
   }
 
   @Override
-  public void insertAll(List<MkCoPEntry<D>> entries) {
-    if(entries.isEmpty()) {
+  public void insertAll(List<MkCoPEntry> entries) {
+    if (entries.isEmpty()) {
       return;
     }
 
-    if(LOG.isDebugging()) {
+    if (LOG.isDebugging()) {
       LOG.debugFine("insert " + entries + "\n");
     }
 
-    if(!initialized) {
+    if (!initialized) {
       initialize(entries.get(0));
     }
 
     ModifiableDBIDs ids = DBIDUtil.newArray(entries.size());
 
     // insert
-    for(MkCoPEntry<D> entry : entries) {
+    for (MkCoPEntry entry : entries) {
       ids.add(entry.getRoutingObjectID());
       // insert the object
       super.insert(entry, false);
     }
 
     // perform nearest neighbor queries
-    Map<DBID, KNNResult<D>> knnLists = batchNN(getRoot(), ids, k_max);
+    Map<DBID, KNNList<D>> knnLists = batchNN(getRoot(), ids, settings.k_max);
 
     // adjust the knn distances
     adjustApproximatedKNNDistances(getRootEntry(), knnLists);
 
-    if(EXTRA_INTEGRITY_CHECKS) {
+    if (EXTRA_INTEGRITY_CHECKS) {
       getRoot().integrityCheck(this, getRootEntry());
     }
   }
@@ -158,29 +146,30 @@ public class MkCoPTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
    * @return a List of the query results
    */
   @Override
-  public DistanceDBIDResult<D> reverseKNNQuery(DBIDRef id, int k) {
-    if(k > this.k_max) {
+  public DistanceDBIDList<D> reverseKNNQuery(DBIDRef id, int k) {
+    if (k > settings.k_max) {
       throw new IllegalArgumentException("Parameter k has to be less or equal than " + "parameter kmax of the MCop-Tree!");
     }
 
-    GenericDistanceDBIDList<D> result = new GenericDistanceDBIDList<D>();
+    GenericDistanceDBIDList<D> result = new GenericDistanceDBIDList<>();
     ModifiableDBIDs candidates = DBIDUtil.newArray();
     doReverseKNNQuery(k, id, result, candidates);
 
     // refinement of candidates
-    Map<DBID, KNNResult<D>> knnLists = batchNN(getRoot(), candidates, k);
+    Map<DBID, KNNList<D>> knnLists = batchNN(getRoot(), candidates, k);
 
     result.sort();
     // Collections.sort(candidates);
 
-    rkNNStatistics.addCandidates(candidates.size());
-    rkNNStatistics.addTrueHits(result.size());
+    // FIXME: re-add statistics.
+    // rkNNStatistics.addCandidates(candidates.size());
+    // rkNNStatistics.addTrueHits(result.size());
 
-    for(DBIDIter iter = candidates.iter(); iter.valid(); iter.advance()) {
+    for (DBIDIter iter = candidates.iter(); iter.valid(); iter.advance()) {
       DBID cid = DBIDUtil.deref(iter);
-      KNNResult<D> cands = knnLists.get(cid);
-      for (DistanceDBIDResultIter<D> iter2 = cands.iter(); iter2.valid(); iter2.advance()) {
-        if(DBIDUtil.equal(id, iter2)) {
+      KNNList<D> cands = knnLists.get(cid);
+      for (DistanceDBIDListIter<D> iter2 = cands.iter(); iter2.valid(); iter2.advance()) {
+        if (DBIDUtil.equal(id, iter2)) {
           result.add(iter2.getDistance(), cid);
           break;
         }
@@ -188,45 +177,30 @@ public class MkCoPTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
     }
     result.sort();
 
-    rkNNStatistics.addResults(result.size());
+    // FIXME: re-add statistics.
+    // rkNNStatistics.addResults(result.size());
     return result;
   }
 
-  /**
-   * Returns the statistic for performed rknn queries.
-   * 
-   * @return the statistic for performed rknn queries
-   */
-  public QueryStatistic getRkNNStatistics() {
-    return rkNNStatistics;
-  }
-
-  /**
-   * Clears the values of the statistic for performed rknn queries
-   */
-  public void clearRkNNStatistics() {
-    rkNNStatistics.clear();
-  }
-
   /**
    * Returns the value of the k_max parameter.
    * 
    * @return the value of the k_max parameter
    */
   public int getK_max() {
-    return k_max;
+    return settings.k_max;
   }
 
   /**
    * Determines the maximum and minimum number of entries in a node.
    */
   @Override
-  protected void initializeCapacities(MkCoPEntry<D> exampleLeaf) {
-    int distanceSize = exampleLeaf.getParentDistance().externalizableSize();
+  protected void initializeCapacities(MkCoPEntry exampleLeaf) {
+    int distanceSize = ByteArrayUtil.SIZE_DOUBLE; // exampleLeaf.getParentDistance().externalizableSize();
 
     // overhead = index(4), numEntries(4), id(4), isLeaf(0.125)
     double overhead = 12.125;
-    if(getPageSize() - overhead < 0) {
+    if (getPageSize() - overhead < 0) {
       throw new RuntimeException("Node size of " + getPageSize() + " Bytes is chosen too small!");
     }
 
@@ -234,11 +208,11 @@ public class MkCoPTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
     // coveringRadius + parentDistance + consApprox) + 1
     dirCapacity = (int) (getPageSize() - overhead) / (4 + 4 + distanceSize + distanceSize + 10) + 1;
 
-    if(dirCapacity <= 1) {
+    if (dirCapacity <= 1) {
       throw new RuntimeException("Node size of " + getPageSize() + " Bytes is chosen too small!");
     }
 
-    if(dirCapacity < 10) {
+    if (dirCapacity < 10) {
       LOG.warning("Page size is choosen too small! Maximum number of entries " + "in a directory node = " + (dirCapacity - 1));
     }
 
@@ -247,17 +221,17 @@ public class MkCoPTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
     // consApprox + progrApprox) + 1
     leafCapacity = (int) (getPageSize() - overhead) / (4 + distanceSize + 2 * 10) + 1;
 
-    if(leafCapacity <= 1) {
+    if (leafCapacity <= 1) {
       throw new RuntimeException("Node size of " + getPageSize() + " Bytes is chosen too small!");
     }
 
-    if(leafCapacity < 10) {
+    if (leafCapacity < 10) {
       LOG.warning("Page size is choosen too small! Maximum number of entries " + "in a leaf node = " + (leafCapacity - 1));
     }
 
     initialized = true;
 
-    if(LOG.isVerbose()) {
+    if (LOG.isVerbose()) {
       LOG.verbose("Directory Capacity: " + (dirCapacity - 1) + "\nLeaf Capacity:    " + (leafCapacity - 1));
     }
   }
@@ -272,45 +246,44 @@ public class MkCoPTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
    *        refinement)
    */
   private void doReverseKNNQuery(int k, DBIDRef q, GenericDistanceDBIDList<D> result, ModifiableDBIDs candidates) {
-    final Heap<GenericMTreeDistanceSearchCandidate<D>> pq = new Heap<GenericMTreeDistanceSearchCandidate<D>>();
+    final ComparableMinHeap<GenericMTreeDistanceSearchCandidate> pq = new ComparableMinHeap<>();
 
     // push root
-    pq.add(new GenericMTreeDistanceSearchCandidate<D>(getDistanceQuery().nullDistance(), getRootID(), null, null));
+    pq.add(new GenericMTreeDistanceSearchCandidate(0., getRootID(), null));
 
     // search in tree
-    while(!pq.isEmpty()) {
-      GenericMTreeDistanceSearchCandidate<D> pqNode = pq.poll();
+    while (!pq.isEmpty()) {
+      GenericMTreeDistanceSearchCandidate pqNode = pq.poll();
       // FIXME: cache the distance to the routing object in the queue node!
 
       MkCoPTreeNode<O, D> node = getNode(pqNode.nodeID);
 
       // directory node
-      if(!node.isLeaf()) {
-        for(int i = 0; i < node.getNumEntries(); i++) {
-          MkCoPEntry<D> entry = node.getEntry(i);
-          D distance = getDistanceQuery().distance(entry.getRoutingObjectID(), q);
-          D minDist = entry.getCoveringRadius().compareTo(distance) > 0 ? getDistanceQuery().nullDistance() : distance.minus(entry.getCoveringRadius());
-          D approximatedKnnDist_cons = entry.approximateConservativeKnnDistance(k, getDistanceQuery());
+      if (!node.isLeaf()) {
+        for (int i = 0; i < node.getNumEntries(); i++) {
+          MkCoPEntry entry = node.getEntry(i);
+          double distance = distance(entry.getRoutingObjectID(), q).doubleValue();
+          double minDist = entry.getCoveringRadius() > distance ? 0. : distance - entry.getCoveringRadius();
+          double approximatedKnnDist_cons = entry.approximateConservativeKnnDistance(k);
 
-          if(minDist.compareTo(approximatedKnnDist_cons) <= 0) {
-            pq.add(new GenericMTreeDistanceSearchCandidate<D>(minDist, getPageID(entry), entry.getRoutingObjectID(), null));
+          if (minDist <= approximatedKnnDist_cons) {
+            pq.add(new GenericMTreeDistanceSearchCandidate(minDist, getPageID(entry), entry.getRoutingObjectID()));
           }
         }
       }
       // data node
       else {
-        for(int i = 0; i < node.getNumEntries(); i++) {
-          MkCoPLeafEntry<D> entry = (MkCoPLeafEntry<D>) node.getEntry(i);
-          D distance = getDistanceQuery().distance(entry.getRoutingObjectID(), q);
-          D approximatedKnnDist_prog = entry.approximateProgressiveKnnDistance(k, getDistanceQuery());
+        for (int i = 0; i < node.getNumEntries(); i++) {
+          MkCoPLeafEntry entry = (MkCoPLeafEntry) node.getEntry(i);
+          D distance = distance(entry.getRoutingObjectID(), q);
+          double approximatedKnnDist_prog = entry.approximateProgressiveKnnDistance(k);
 
-          if(distance.compareTo(approximatedKnnDist_prog) <= 0) {
+          if (distance.doubleValue() <= approximatedKnnDist_prog) {
             result.add(distance, entry.getRoutingObjectID());
-          }
-          else {
-            D approximatedKnnDist_cons = entry.approximateConservativeKnnDistance(k, getDistanceQuery());
-            double diff = distance.doubleValue() - approximatedKnnDist_cons.doubleValue();
-            if(diff <= 0.0000000001) {
+          } else {
+            double approximatedKnnDist_cons = entry.approximateConservativeKnnDistance(k);
+            double diff = distance.doubleValue() - approximatedKnnDist_cons;
+            if (diff <= 1E-10) {
               candidates.add(entry.getRoutingObjectID());
             }
           }
@@ -325,23 +298,22 @@ public class MkCoPTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
    * @param entry the root entry of the current subtree
    * @param knnLists a map of knn lists for each leaf entry
    */
-  private void adjustApproximatedKNNDistances(MkCoPEntry<D> entry, Map<DBID, KNNResult<D>> knnLists) {
+  private void adjustApproximatedKNNDistances(MkCoPEntry entry, Map<DBID, KNNList<D>> knnLists) {
     MkCoPTreeNode<O, D> node = getNode(entry);
 
-    if(node.isLeaf()) {
-      for(int i = 0; i < node.getNumEntries(); i++) {
-        MkCoPLeafEntry<D> leafEntry = (MkCoPLeafEntry<D>) node.getEntry(i);
+    if (node.isLeaf()) {
+      for (int i = 0; i < node.getNumEntries(); i++) {
+        MkCoPLeafEntry leafEntry = (MkCoPLeafEntry) node.getEntry(i);
         approximateKnnDistances(leafEntry, knnLists.get(leafEntry.getRoutingObjectID()));
       }
-    }
-    else {
-      for(int i = 0; i < node.getNumEntries(); i++) {
-        MkCoPEntry<D> dirEntry = node.getEntry(i);
+    } else {
+      for (int i = 0; i < node.getNumEntries(); i++) {
+        MkCoPEntry dirEntry = node.getEntry(i);
         adjustApproximatedKNNDistances(dirEntry, knnLists);
       }
     }
 
-    ApproximationLine approx = node.conservativeKnnDistanceApproximation(k_max);
+    ApproximationLine approx = node.conservativeKnnDistanceApproximation(settings.k_max);
     entry.setConservativeKnnDistanceApproximation(approx);
   }
 
@@ -351,7 +323,7 @@ public class MkCoPTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
   private double ssqerr(int k0, int kmax, double[] logk, double[] log_kDist, double m, double t) {
     int k = kmax - k0;
     double result = 0;
-    for(int i = 0; i < k; i++) {
+    for (int i = 0; i < k; i++) {
       // double h = log_kDist[i] - (m * (logk[i] - logk[0]) + t); ???
       double h = log_kDist[i] - m * logk[i] - t;
       result += h * h;
@@ -377,33 +349,32 @@ public class MkCoPTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
    * @param knnDistances TODO: Spezialbehandlung fuer identische Punkte in DB
    *        (insbes. Distanz 0)
    */
-  private void approximateKnnDistances(MkCoPLeafEntry<D> entry, KNNResult<D> knnDistances) {
+  private void approximateKnnDistances(MkCoPLeafEntry entry, KNNList<D> knnDistances) {
     StringBuilder msg = LOG.isDebugging() ? new StringBuilder() : null;
-    if(msg != null) {
+    if (msg != null) {
       msg.append("\nknnDistances ").append(knnDistances);
     }
 
     // count the zero distances
     int k_0 = 0;
-    for(int i = 0; i < k_max; i++) {
+    for (int i = 0; i < settings.k_max; i++) {
       double dist = knnDistances.get(i).getDistance().doubleValue();
-      if(dist == 0) {
+      if (dist == 0) {
         k_0++;
-      }
-      else {
+      } else {
         break;
       }
     }
 
     // init variables
-    double[] log_k = new double[k_max - k_0];
-    System.arraycopy(this.log_k, k_0, log_k, 0, k_max - k_0);
+    double[] log_k = new double[settings.k_max - k_0];
+    System.arraycopy(this.log_k, k_0, log_k, 0, settings.k_max - k_0);
 
     double sum_log_kDist = 0;
     double sum_log_k_kDist = 0;
-    double[] log_kDist = new double[k_max - k_0];
+    double[] log_kDist = new double[settings.k_max - k_0];
 
-    for(int i = 0; i < k_max - k_0; i++) {
+    for (int i = 0; i < settings.k_max - k_0; i++) {
       double dist = knnDistances.get(i + k_0).getDistance().doubleValue();
       log_kDist[i] = Math.log(dist);
       sum_log_kDist += log_kDist[i];
@@ -413,14 +384,14 @@ public class MkCoPTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
     double sum_log_k = 0;
     double sum_log_k2 = 0;
     // noinspection ForLoopReplaceableByForEach
-    for(int i = 0; i < log_k.length; i++) {
+    for (int i = 0; i < log_k.length; i++) {
       sum_log_k += log_k[i];
       sum_log_k2 += (log_k[i] * log_k[i]);
     }
 
-    if(msg != null) {
+    if (msg != null) {
       msg.append("\nk_0 ").append(k_0);
-      msg.append("\nk_max ").append(k_max);
+      msg.append("\nk_max ").append(settings.k_max);
       msg.append("\nlog_k(").append(log_k.length).append(") ").append(FormatUtil.format(log_k));
       msg.append("\nsum_log_k ").append(sum_log_k);
       msg.append("\nsum_log_k^2 ").append(sum_log_k2);
@@ -438,15 +409,15 @@ public class MkCoPTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
 
     ApproximationLine c2 = approximateUpperHull_PAPER(convexHull, log_k, sum_log_k, sum_log_k2, log_kDist, sum_log_kDist, sum_log_k_kDist);
 
-    double err1 = ssqerr(k_0, k_max, log_k, log_kDist, conservative.getM(), conservative.getT());
-    double err2 = ssqerr(k_0, k_max, log_k, log_kDist, c2.getM(), c2.getT());
+    double err1 = ssqerr(k_0, settings.k_max, log_k, log_kDist, conservative.getM(), conservative.getT());
+    double err2 = ssqerr(k_0, settings.k_max, log_k, log_kDist, c2.getM(), c2.getT());
 
-    if(msg != null) {
+    if (msg != null) {
       msg.append("err1 ").append(err1);
       msg.append("err2 ").append(err2);
     }
 
-    if(err1 > err2 && err1 - err2 > 0.000000001) {
+    if (err1 > err2 && err1 - err2 > 0.000000001) {
       // if (err1 > err2) {
 
       StringBuilder warning = new StringBuilder();
@@ -460,7 +431,7 @@ public class MkCoPTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
       warning.append("\nconservative1 ").append(conservative);
       warning.append("\nconservative2 ").append(c2);
 
-      for(int i = 0; i < u; i++) {
+      for (int i = 0; i < u; i++) {
         warning.append("\nlog_k[").append(upperHull[i]).append("] = ").append(log_k[upperHull[i]]);
         warning.append("\nlog_kDist[").append(upperHull[i]).append("] = ").append(log_kDist[upperHull[i]]);
       }
@@ -473,7 +444,7 @@ public class MkCoPTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
     entry.setConservativeKnnDistanceApproximation(conservative);
     entry.setProgressiveKnnDistanceApproximation(progressive);
 
-    if(msg != null) {
+    if (msg != null) {
       LOG.debugFine(msg.toString());
     }
   }
@@ -491,7 +462,7 @@ public class MkCoPTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
     StringBuilder msg = new StringBuilder();
     int[] lowerHull = convexHull.getLowerHull();
     int l = convexHull.getNumberOfPointsInLowerHull();
-    int k_0 = k_max - lowerHull.length + 1;
+    int k_0 = settings.k_max - lowerHull.length + 1;
 
     // linear search on all line segments on the lower convex hull
     msg.append("lower hull l = ").append(l).append("\n");
@@ -499,12 +470,12 @@ public class MkCoPTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
     double low_m = 0.0;
     double low_t = 0.0;
 
-    for(int i = 1; i < l; i++) {
+    for (int i = 1; i < l; i++) {
       double cur_m = (log_kDist[lowerHull[i]] - log_kDist[lowerHull[i - 1]]) / (log_k[lowerHull[i]] - log_k[lowerHull[i - 1]]);
       double cur_t = log_kDist[lowerHull[i]] - cur_m * log_k[lowerHull[i]];
-      double cur_error = ssqerr(k_0, k_max, log_k, log_kDist, cur_m, cur_t);
+      double cur_error = ssqerr(k_0, settings.k_max, log_k, log_kDist, cur_m, cur_t);
       msg.append("  Segment = ").append(i).append(" m = ").append(cur_m).append(" t = ").append(cur_t).append(" lowerror = ").append(cur_error).append("\n");
-      if(cur_error < low_error) {
+      if (cur_error < low_error) {
         low_error = cur_error;
         low_m = cur_m;
         low_t = cur_t;
@@ -513,13 +484,13 @@ public class MkCoPTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
 
     // linear search on all points of the lower convex hull
     boolean is_right = true; // NEEDED FOR PROOF CHECK
-    for(int i = 0; i < l; i++) {
-      double cur_m = optimize(k_0, k_max, sum_log_k, sum_log_k2, log_k[lowerHull[i]], log_kDist[lowerHull[i]], sum_log_k_kDist, sum_log_kDist);
+    for (int i = 0; i < l; i++) {
+      double cur_m = optimize(k_0, settings.k_max, sum_log_k, sum_log_k2, log_k[lowerHull[i]], log_kDist[lowerHull[i]], sum_log_k_kDist, sum_log_kDist);
       double cur_t = log_kDist[lowerHull[i]] - cur_m * log_k[lowerHull[i]];
       // only valid if both neighboring points are underneath y=mx+t
-      if((i == 0 || log_kDist[lowerHull[i - 1]] >= log_kDist[lowerHull[i]] - cur_m * (log_k[lowerHull[i]] - log_k[lowerHull[i - 1]])) && (i == l - 1 || log_kDist[lowerHull[i + 1]] >= log_kDist[lowerHull[i]] + cur_m * (log_k[lowerHull[i + 1]] - log_k[lowerHull[i]]))) {
-        double cur_error = ssqerr(k_0, k_max, log_k, log_kDist, cur_m, cur_t);
-        if(cur_error < low_error) {
+      if ((i == 0 || log_kDist[lowerHull[i - 1]] >= log_kDist[lowerHull[i]] - cur_m * (log_k[lowerHull[i]] - log_k[lowerHull[i - 1]])) && (i == l - 1 || log_kDist[lowerHull[i + 1]] >= log_kDist[lowerHull[i]] + cur_m * (log_k[lowerHull[i + 1]] - log_k[lowerHull[i]]))) {
+        double cur_error = ssqerr(k_0, settings.k_max, log_k, log_kDist, cur_m, cur_t);
+        if (cur_error < low_error) {
           low_error = cur_error;
           low_m = cur_m;
           low_t = cur_t;
@@ -527,9 +498,9 @@ public class MkCoPTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
       }
 
       // check proof of bisection search
-      if(!(i > 0 && log_kDist[lowerHull[i - 1]] < log_kDist[lowerHull[i]] - cur_m * (log_k[lowerHull[i]] - log_k[lowerHull[i - 1]])) && !is_right) {
+      if (!(i > 0 && log_kDist[lowerHull[i - 1]] < log_kDist[lowerHull[i]] - cur_m * (log_k[lowerHull[i]] - log_k[lowerHull[i - 1]])) && !is_right) {
         // warning("ERROR lower: The bisection search will not work properly !");
-        if(!(i < l - 1 && log_kDist[lowerHull[i + 1]] < log_kDist[lowerHull[i]] + cur_m * (log_k[lowerHull[i + 1]] - log_k[lowerHull[i]]))) {
+        if (!(i < l - 1 && log_kDist[lowerHull[i + 1]] < log_kDist[lowerHull[i]] + cur_m * (log_k[lowerHull[i + 1]] - log_k[lowerHull[i]]))) {
           is_right = false;
         }
       }
@@ -544,18 +515,18 @@ public class MkCoPTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
 
     int[] upperHull = convexHull.getUpperHull();
     int u = convexHull.getNumberOfPointsInUpperHull();
-    int k_0 = k_max - upperHull.length + 1;
+    int k_0 = settings.k_max - upperHull.length + 1;
 
     ApproximationLine approx = null;
     double error = Double.POSITIVE_INFINITY;
-    for(int i = 0; i < u - 1; i++) {
+    for (int i = 0; i < u - 1; i++) {
       int ii = upperHull[i];
       int jj = upperHull[i + 1];
       double current_m = (log_kDist[jj] - log_kDist[ii]) / (log_k[jj] - log_k[ii]);
       double current_t = log_kDist[ii] - current_m * log_k[ii];
       ApproximationLine current_approx = new ApproximationLine(k_0, current_m, current_t);
 
-      if(LOG.isDebugging()) {
+      if (LOG.isDebugging()) {
         msg.append("\nlog_kDist[").append(jj).append("] ").append(log_kDist[jj]);
         msg.append("\nlog_kDist[").append(ii).append("] ").append(log_kDist[ii]);
         msg.append("\nlog_k[").append(jj).append("] ").append(log_k[jj]);
@@ -566,22 +537,22 @@ public class MkCoPTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
 
       boolean ok = true;
       double currentError = 0;
-      for(int k = k_0; k <= k_max; k++) {
+      for (int k = k_0; k <= settings.k_max; k++) {
         double appDist = current_approx.getValueAt(k);
-        if(appDist < log_kDist[k - k_0] && log_kDist[k - k_0] - appDist > 0.000000001) {
+        if (appDist < log_kDist[k - k_0] && log_kDist[k - k_0] - appDist > 0.000000001) {
           ok = false;
           break;
         }
         currentError += (appDist - log_kDist[k - k_0]);
       }
 
-      if(ok && currentError < error) {
+      if (ok && currentError < error) {
         approx = current_approx;
         error = currentError;
       }
     }
 
-    if(LOG.isDebugging()) {
+    if (LOG.isDebugging()) {
       msg.append("\nupper Approx ").append(approx);
       LOG.debugFine(msg.toString());
     }
@@ -594,22 +565,22 @@ public class MkCoPTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
     int[] upperHull = convexHull.getUpperHull();
     int u = convexHull.getNumberOfPointsInUpperHull();
 
-    List<Integer> marked = new ArrayList<Integer>();
+    List<Integer> marked = new ArrayList<>();
 
-    int k_0 = k_max - upperHull.length + 1;
+    int k_0 = settings.k_max - upperHull.length + 1;
 
     int a = u / 2;
-    while(marked.size() != u) {
+    while (marked.size() != u) {
       marked.add(a);
       double x_a = log_k[upperHull[a]];
       double y_a = log_kDist[upperHull[a]];
 
-      double m_a = optimize(k_0, k_max, sum_log_k, sum_log_k2, x_a, y_a, sum_log_k_kDist, sum_log_kDist);
+      double m_a = optimize(k_0, settings.k_max, sum_log_k, sum_log_k2, x_a, y_a, sum_log_k_kDist, sum_log_kDist);
       double t_a = y_a - m_a * x_a;
 
-      if(msg != null) {
+      if (msg != null) {
         msg.append("\na=").append(a).append(" m_a=").append(m_a).append(", t_a=").append(t_a);
-        msg.append("\n err ").append(ssqerr(k_0, k_max, log_k, log_kDist, m_a, m_a));
+        msg.append("\n err ").append(ssqerr(k_0, settings.k_max, log_k, log_kDist, m_a, m_a));
       }
 
       double x_p = a == 0 ? Double.NaN : log_k[upperHull[a - 1]];
@@ -620,24 +591,23 @@ public class MkCoPTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
       boolean lessThanPre = a == 0 || y_p <= m_a * x_p + t_a;
       boolean lessThanSuc = a == u || y_s <= m_a * x_s + t_a;
 
-      if(lessThanPre && lessThanSuc) {
+      if (lessThanPre && lessThanSuc) {
         ApproximationLine appr = new ApproximationLine(k_0, m_a, t_a);
-        if(msg != null) {
+        if (msg != null) {
           msg.append("\n1 anchor = ").append(a);
           LOG.debugFine(msg.toString());
         }
         return appr;
-      }
-      else if(!lessThanPre) {
-        if(marked.contains(a - 1)) {
+      } else if (!lessThanPre) {
+        if (marked.contains(a - 1)) {
           m_a = (y_a - y_p) / (x_a - x_p);
-          if(y_a == y_p) {
+          if (y_a == y_p) {
             m_a = 0;
           }
           t_a = y_a - m_a * x_a;
 
           ApproximationLine appr = new ApproximationLine(k_0, m_a, t_a);
-          if(msg != null) {
+          if (msg != null) {
             msg.append("2 anchor = ").append(a);
             msg.append(" appr1 ").append(appr);
             msg.append(" x_a ").append(x_a).append(", y_a ").append(y_a);
@@ -647,28 +617,25 @@ public class MkCoPTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
             LOG.debugFine(msg.toString());
           }
           return appr;
-        }
-        else {
+        } else {
           a = a - 1;
         }
-      }
-      else {
-        if(marked.contains(a + 1)) {
+      } else {
+        if (marked.contains(a + 1)) {
           m_a = (y_a - y_s) / (x_a - x_s);
-          if(y_a == y_p) {
+          if (y_a == y_p) {
             m_a = 0;
           }
           t_a = y_a - m_a * x_a;
           ApproximationLine appr = new ApproximationLine(k_0, m_a, t_a);
 
-          if(msg != null) {
+          if (msg != null) {
             msg.append("3 anchor = ").append(a).append(" -- ").append((a + 1));
             msg.append(" appr2 ").append(appr);
             LOG.debugFine(msg.toString());
           }
           return appr;
-        }
-        else {
+        } else {
           a = a + 1;
         }
       }
@@ -683,18 +650,18 @@ public class MkCoPTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
     StringBuilder msg = new StringBuilder();
     int[] upperHull = convexHull.getUpperHull();
     int u = convexHull.getNumberOfPointsInUpperHull();
-    int k_0 = k_max - upperHull.length + 1;
+    int k_0 = settings.k_max - upperHull.length + 1;
 
     // linear search on all line segments on the upper convex hull
     msg.append("upper hull:").append(u);
     double upp_error = Double.MAX_VALUE;
     double upp_m = 0.0;
     double upp_t = 0.0;
-    for(int i = 1; i < u; i++) {
+    for (int i = 1; i < u; i++) {
       double cur_m = (log_kDist[upperHull[i]] - log_kDist[upperHull[i - 1]]) / (log_k[upperHull[i]] - log_k[upperHull[i - 1]]);
       double cur_t = log_kDist[upperHull[i]] - cur_m * log_k[upperHull[i]];
-      double cur_error = ssqerr(k_0, k_max, log_k, log_kDist, cur_m, cur_t);
-      if(cur_error < upp_error) {
+      double cur_error = ssqerr(k_0, settings.k_max, log_k, log_kDist, cur_m, cur_t);
+      if (cur_error < upp_error) {
         upp_error = cur_error;
         upp_m = cur_m;
         upp_t = cur_t;
@@ -702,13 +669,13 @@ public class MkCoPTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
     }
     // linear search on all points of the upper convex hull
     boolean is_left = true; // NEEDED FOR PROOF CHECK
-    for(int i = 0; i < u; i++) {
-      double cur_m = optimize(k_0, k_max, sum_log_k, sum_log_k2, log_k[upperHull[i]], log_kDist[upperHull[i]], sum_log_k_kDist, sum_log_kDist);
+    for (int i = 0; i < u; i++) {
+      double cur_m = optimize(k_0, settings.k_max, sum_log_k, sum_log_k2, log_k[upperHull[i]], log_kDist[upperHull[i]], sum_log_k_kDist, sum_log_kDist);
       double cur_t = log_kDist[upperHull[i]] - cur_m * log_k[upperHull[i]];
       // only valid if both neighboring points are underneath y=mx+t
-      if((i == 0 || log_kDist[upperHull[i - 1]] <= log_kDist[upperHull[i]] - cur_m * (log_k[upperHull[i]] - log_k[upperHull[i - 1]])) && (i == u - 1 || log_kDist[upperHull[i + 1]] <= log_kDist[upperHull[i]] + cur_m * (log_k[upperHull[i + 1]] - log_k[upperHull[i]]))) {
-        double cur_error = ssqerr(k_0, k_max, log_k, log_kDist, cur_m, cur_t);
-        if(cur_error < upp_error) {
+      if ((i == 0 || log_kDist[upperHull[i - 1]] <= log_kDist[upperHull[i]] - cur_m * (log_k[upperHull[i]] - log_k[upperHull[i - 1]])) && (i == u - 1 || log_kDist[upperHull[i + 1]] <= log_kDist[upperHull[i]] + cur_m * (log_k[upperHull[i + 1]] - log_k[upperHull[i]]))) {
+        double cur_error = ssqerr(k_0, settings.k_max, log_k, log_kDist, cur_m, cur_t);
+        if (cur_error < upp_error) {
           upp_error = cur_error;
           upp_m = cur_m;
           upp_t = cur_t;
@@ -716,12 +683,12 @@ public class MkCoPTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
       }
 
       // check proof of bisection search
-      if(!(i > 0 && log_kDist[upperHull[i - 1]] > log_kDist[upperHull[i]] - cur_m * (log_k[upperHull[i]] - log_k[upperHull[i - 1]])) && !is_left) {
+      if (!(i > 0 && log_kDist[upperHull[i - 1]] > log_kDist[upperHull[i]] - cur_m * (log_k[upperHull[i]] - log_k[upperHull[i - 1]])) && !is_left) {
         // warning("ERROR upper: The bisection search will not work properly !"
         // +
         // "\n" + Util.format(log_kDist));
       }
-      if(!(i < u - 1 && log_kDist[upperHull[i + 1]] > log_kDist[upperHull[i]] + cur_m * (log_k[upperHull[i + 1]] - log_k[upperHull[i]]))) {
+      if (!(i < u - 1 && log_kDist[upperHull[i + 1]] > log_kDist[upperHull[i]] + cur_m * (log_k[upperHull[i + 1]] - log_k[upperHull[i]]))) {
         is_left = false;
       }
     }
@@ -737,7 +704,7 @@ public class MkCoPTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
    */
   @Override
   protected MkCoPTreeNode<O, D> createNewLeafNode() {
-    return new MkCoPTreeNode<O, D>(leafCapacity, true);
+    return new MkCoPTreeNode<>(leafCapacity, true);
   }
 
   /**
@@ -747,7 +714,7 @@ public class MkCoPTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
    */
   @Override
   protected MkCoPTreeNode<O, D> createNewDirectoryNode() {
-    return new MkCoPTreeNode<O, D>(dirCapacity, false);
+    return new MkCoPTreeNode<>(dirCapacity, false);
   }
 
   /**
@@ -759,8 +726,8 @@ public class MkCoPTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
    *        the routing object of the parent node
    */
   @Override
-  protected MkCoPEntry<D> createNewDirectoryEntry(MkCoPTreeNode<O, D> node, DBID routingObjectID, D parentDistance) {
-    return new MkCoPDirectoryEntry<D>(routingObjectID, parentDistance, node.getPageID(), node.coveringRadius(routingObjectID, this), null);
+  protected MkCoPEntry createNewDirectoryEntry(MkCoPTreeNode<O, D> node, DBID routingObjectID, double parentDistance) {
+    return new MkCoPDirectoryEntry(routingObjectID, parentDistance, node.getPageID(), node.coveringRadius(routingObjectID, this), null);
     // node.conservativeKnnDistanceApproximation(k_max));
   }
 
@@ -770,12 +737,12 @@ public class MkCoPTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
    * @return an entry representing the root node
    */
   @Override
-  protected MkCoPEntry<D> createRootEntry() {
-    return new MkCoPDirectoryEntry<D>(null, null, 0, null, null);
+  protected MkCoPEntry createRootEntry() {
+    return new MkCoPDirectoryEntry(null, 0., 0, 0., null);
   }
 
   @Override
   protected Logging getLogger() {
     return LOG;
   }
-}
\ No newline at end of file
+}
