@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.index.tree.metrical.mtreevariants.mktrees.mkmax;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -23,30 +23,28 @@ package de.lmu.ifi.dbs.elki.index.tree.metrical.mtreevariants.mktrees.mkmax;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
 import de.lmu.ifi.dbs.elki.index.tree.metrical.mtreevariants.MTreeEntry;
 
 /**
  * Defines the requirements for an entry in an
- * {@link de.lmu.ifi.dbs.elki.index.tree.metrical.mtreevariants.mktrees.mkmax.MkMaxTreeNode}.
- * Additionally to an entry in an M-Tree an MkMaxEntry holds the k-nearest neighbor distance of the underlying
- * data object or MkMax-Tree node.
- *
+ * {@link de.lmu.ifi.dbs.elki.index.tree.metrical.mtreevariants.mktrees.mkmax.MkMaxTreeNode}
+ * . Additionally to an entry in an M-Tree an MkMaxEntry holds the k-nearest
+ * neighbor distance of the underlying data object or MkMax-Tree node.
+ * 
  * @author Elke Achtert
- * @param <D> the type of Distance used in the MkMaxTree
  */
-interface MkMaxEntry<D extends Distance<D>> extends MTreeEntry<D> {
+interface MkMaxEntry extends MTreeEntry {
   /**
    * Returns the knn distance of the entry.
-   *
+   * 
    * @return the knn distance of the entry
    */
-  public D getKnnDistance();
+  public double getKnnDistance();
 
   /**
    * Sets the knn distance of the entry.
-   *
+   * 
    * @param knnDistance the knn distance to be set
    */
-  public void setKnnDistance(D knnDistance);
+  public void setKnnDistance(double knnDistance);
 }
