@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.index.tree.metrical.mtreevariants.mktrees.mktab;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -23,46 +23,35 @@ package de.lmu.ifi.dbs.elki.index.tree.metrical.mtreevariants.mktrees.mktab;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-import java.util.List;
-
-import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
 import de.lmu.ifi.dbs.elki.index.tree.metrical.mtreevariants.MTreeEntry;
 
 /**
- * Defines the requirements for an entry in an MkCop-Tree node.
- * Additionally to an entry in an M-Tree an MkTabEntry holds a list of knn distances
- * for for parameters k <= k_max of the underlying data object or MkTab-Tree node.
- *
+ * Defines the requirements for an entry in an MkCop-Tree node. Additionally to
+ * an entry in an M-Tree an MkTabEntry holds a list of knn distances for for
+ * parameters k <= k_max of the underlying data object or MkTab-Tree node.
+ * 
  * @author Elke Achtert
  */
-interface MkTabEntry<D extends Distance<D>> extends MTreeEntry<D> {
+interface MkTabEntry extends MTreeEntry {
   /**
    * Returns the list of knn distances of the entry.
-   *
-   * @return the list of  knn distances of the entry
+   * 
+   * @return the list of knn distances of the entry
    */
-  public List<D> getKnnDistances();
+  public double[] getKnnDistances();
 
   /**
    * Sets the knn distances of the entry.
-   *
+   * 
    * @param knnDistances the knn distances to be set
    */
-  public void setKnnDistances(List<D> knnDistances);
+  public void setKnnDistances(double[] knnDistances);
 
   /**
    * Returns the knn distance of the entry for the specified parameter k.
-   *
+   * 
    * @param k the parameter k of the knn distance
    * @return the knn distance of the entry
    */
-  public D getKnnDistance(int k);
-
-  /**
-   * Returns the parameter k_max.
-   *
-   * @return the parameter k_max
-   */
-  public int getK_max();
-
+  public double getKnnDistance(int k);
 }
