@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.index.tree.query;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -41,7 +41,7 @@ public class GenericDistanceSearchCandidate<D extends Distance<D>> implements Co
   /**
    * Page id
    */
-  public Integer nodeID;
+  public int nodeID;
 
   /**
    * Constructor.
@@ -49,7 +49,7 @@ public class GenericDistanceSearchCandidate<D extends Distance<D>> implements Co
    * @param mindist The minimum distance to this candidate
    * @param pagenr The page number of this candidate
    */
-  public GenericDistanceSearchCandidate(final D mindist, final Integer pagenr) {
+  public GenericDistanceSearchCandidate(final D mindist, final int pagenr) {
     super();
     this.mindist = mindist;
     this.nodeID = pagenr;
