@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.index.tree.spatial.rstarvariants.query;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -62,17 +62,17 @@ public final class RStarTreeUtil {
    * @return Query object
    */
   @SuppressWarnings({ "cast", "unchecked" })
-  public static <O extends SpatialComparable, D extends Distance<D>> RangeQuery<O, D> getRangeQuery(AbstractRStarTree<?, ?> tree, SpatialDistanceQuery<O, D> distanceQuery, Object... hints) {
+  public static <O extends SpatialComparable, D extends Distance<D>> RangeQuery<O, D> getRangeQuery(AbstractRStarTree<?, ?, ?> tree, SpatialDistanceQuery<O, D> distanceQuery, Object... hints) {
     // Can we support this distance function - spatial distances only!
     SpatialPrimitiveDistanceFunction<? super O, D> df = distanceQuery.getDistanceFunction();
     // Can we use an optimized query?
     if(df instanceof SpatialPrimitiveDoubleDistanceFunction) {
       DistanceQuery<O, DoubleDistance> dqc = (DistanceQuery<O, DoubleDistance>) DistanceQuery.class.cast(distanceQuery);
       SpatialPrimitiveDoubleDistanceFunction<? super O> dfc = (SpatialPrimitiveDoubleDistanceFunction<? super O>) SpatialPrimitiveDoubleDistanceFunction.class.cast(df);
-      RangeQuery<O, ?> q = new DoubleDistanceRStarTreeRangeQuery<O>(tree, dqc, dfc);
+      RangeQuery<O, ?> q = new DoubleDistanceRStarTreeRangeQuery<>(tree, dqc, dfc);
       return (RangeQuery<O, D>) q;
     }
-    return new GenericRStarTreeRangeQuery<O, D>(tree, distanceQuery);
+    return new GenericRStarTreeRangeQuery<>(tree, distanceQuery);
   }
 
   /**
@@ -87,16 +87,16 @@ public final class RStarTreeUtil {
    * @return Query object
    */
   @SuppressWarnings({ "cast", "unchecked" })
-  public static <O extends SpatialComparable, D extends Distance<D>> KNNQuery<O, D> getKNNQuery(AbstractRStarTree<?, ?> tree, SpatialDistanceQuery<O, D> distanceQuery, Object... hints) {
+  public static <O extends SpatialComparable, D extends Distance<D>> KNNQuery<O, D> getKNNQuery(AbstractRStarTree<?, ?, ?> tree, SpatialDistanceQuery<O, D> distanceQuery, Object... hints) {
     // Can we support this distance function - spatial distances only!
     SpatialPrimitiveDistanceFunction<? super O, D> df = distanceQuery.getDistanceFunction();
     // Can we use an optimized query?
     if(df instanceof SpatialPrimitiveDoubleDistanceFunction) {
       DistanceQuery<O, DoubleDistance> dqc = (DistanceQuery<O, DoubleDistance>) DistanceQuery.class.cast(distanceQuery);
       SpatialPrimitiveDoubleDistanceFunction<? super O> dfc = (SpatialPrimitiveDoubleDistanceFunction<? super O>) SpatialPrimitiveDoubleDistanceFunction.class.cast(df);
-      KNNQuery<O, ?> q = new DoubleDistanceRStarTreeKNNQuery<O>(tree, dqc, dfc);
+      KNNQuery<O, ?> q = new DoubleDistanceRStarTreeKNNQuery<>(tree, dqc, dfc);
       return (KNNQuery<O, D>) q;
     }
-    return new GenericRStarTreeKNNQuery<O, D>(tree, distanceQuery);
+    return new GenericRStarTreeKNNQuery<>(tree, distanceQuery);
   }
 }
\ No newline at end of file
