@@ -38,7 +38,9 @@ import de.lmu.ifi.dbs.elki.data.type.TypeUtil;
 import de.lmu.ifi.dbs.elki.database.ids.DBID;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
-import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
+import de.lmu.ifi.dbs.elki.database.ids.distance.DoubleDistanceDBIDPairList;
+import de.lmu.ifi.dbs.elki.database.ids.distance.DoubleDistanceKNNHeap;
+import de.lmu.ifi.dbs.elki.database.ids.distance.DoubleDistanceKNNList;
 import de.lmu.ifi.dbs.elki.database.query.DatabaseQuery;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.database.query.knn.KNNQuery;
@@ -46,27 +48,25 @@ import de.lmu.ifi.dbs.elki.database.query.range.RangeQuery;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.database.relation.RelationUtil;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.DistanceFunction;
-import de.lmu.ifi.dbs.elki.distance.distancefunction.LPNormDistanceFunction;
+import de.lmu.ifi.dbs.elki.distance.distancefunction.minkowski.LPNormDistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.subspace.SubspaceLPNormDistanceFunction;
-import de.lmu.ifi.dbs.elki.distance.distanceresultlist.DoubleDistanceDBIDList;
-import de.lmu.ifi.dbs.elki.distance.distanceresultlist.DoubleDistanceKNNHeap;
-import de.lmu.ifi.dbs.elki.distance.distanceresultlist.DoubleDistanceKNNList;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.DoubleDistance;
 import de.lmu.ifi.dbs.elki.index.AbstractRefiningIndex;
 import de.lmu.ifi.dbs.elki.index.IndexFactory;
 import de.lmu.ifi.dbs.elki.index.KNNIndex;
 import de.lmu.ifi.dbs.elki.index.RangeIndex;
-import de.lmu.ifi.dbs.elki.index.tree.TreeIndexFactory;
 import de.lmu.ifi.dbs.elki.logging.Logging;
+import de.lmu.ifi.dbs.elki.logging.statistics.Counter;
 import de.lmu.ifi.dbs.elki.math.MathUtil;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.Vector;
+import de.lmu.ifi.dbs.elki.persistent.AbstractPageFileFactory;
 import de.lmu.ifi.dbs.elki.persistent.ByteArrayUtil;
 import de.lmu.ifi.dbs.elki.utilities.datastructures.heap.DoubleMaxHeap;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.AbstractParameterizer;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.GreaterConstraint;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.CommonConstraints;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntParameter;
 import de.lmu.ifi.dbs.elki.utilities.pairs.DoubleObjPair;
@@ -89,7 +89,7 @@ import de.lmu.ifi.dbs.elki.utilities.pairs.DoubleObjPair;
  * @apiviz.landmark
  * 
  * @apiviz.composedOf DAFile
- * @apiviz.uses PartialVACandidate
+ * @apiviz.has PartialVACandidate
  * @apiviz.has PartialVAFileRangeQuery
  * @apiviz.has PartialVAFileKNNQuery
  * 
@@ -143,11 +143,11 @@ public class PartialVAFile<V extends NumberVector<?>> extends AbstractRefiningIn
     super(relation);
     this.pageSize = pageSize;
     this.partitions = partitions;
-    this.stats = new Statistics();
+    this.stats = new Statistics(this.getClass().getName());
   }
 
   @Override
-  public void initialize(Relation<V> relation, DBIDs ids) throws IllegalStateException {
+  public void initialize() throws IllegalStateException {
     if(splitPartitions != null) {
       throw new IllegalStateException("Data already inserted.");
     }
@@ -159,15 +159,15 @@ public class PartialVAFile<V extends NumberVector<?>> extends AbstractRefiningIn
     final int dimensions = RelationUtil.dimensionality(relation);
 
     splitPartitions = new double[dimensions][];
-    daFiles = new ArrayList<DAFile>(dimensions);
+    daFiles = new ArrayList<>(dimensions);
     for(int d = 0; d < dimensions; d++) {
       final DAFile f = new DAFile(relation, d, partitions);
       splitPartitions[d] = f.getSplitPositions();
       daFiles.add(f);
     }
 
-    vectorApprox = new ArrayList<VectorApproximation>();
-    for (DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
+    vectorApprox = new ArrayList<>();
+    for(DBIDIter iter = relation.iterDBIDs(); iter.valid(); iter.advance()) {
       DBID id = DBIDUtil.deref(iter);
       V dv = relation.get(id);
       VectorApproximation va = calculateFullApproximation(id, dv);
@@ -185,6 +185,16 @@ public class PartialVAFile<V extends NumberVector<?>> extends AbstractRefiningIn
     return "partial va-file";
   }
 
+  @Override
+  public Logging getLogger() {
+    return LOG;
+  }
+
+  @Override
+  public void logStatistics() {
+    stats.logStatistics();
+  }
+
   /**
    * Fake subspace (full-dimensional).
    * 
@@ -298,21 +308,21 @@ public class PartialVAFile<V extends NumberVector<?>> extends AbstractRefiningIn
     final int dimensions = query.getDimensionality();
     double[] lowerVals = new double[dimensions];
     double[] upperVals = new double[dimensions];
-  
+
     VectorApproximation queryApprox = calculatePartialApproximation(null, query, daFiles);
-  
+
     for(int i = 0; i < dimensions; i++) {
       final double val = query.doubleValue(i);
       lowerVals[i] = val - epsilon;
       upperVals[i] = val + epsilon;
     }
-  
+
     Vector lowerEpsilon = new Vector(lowerVals);
     VectorApproximation lowerEpsilonPartitions = calculatePartialApproximation(null, lowerEpsilon, daFiles);
-  
+
     Vector upperEpsilon = new Vector(upperVals);
     VectorApproximation upperEpsilonPartitions = calculatePartialApproximation(null, upperEpsilon, daFiles);
-  
+
     for(int i = 0; i < daFiles.size(); i++) {
       int coeff = (queryApprox.getApproximation(i) - lowerEpsilonPartitions.getApproximation(i)) + (upperEpsilonPartitions.getApproximation(i) - queryApprox.getApproximation(i)) + 1;
       daFiles.get(i).first = coeff;
@@ -334,7 +344,7 @@ public class PartialVAFile<V extends NumberVector<?>> extends AbstractRefiningIn
       double[] borders = daFiles.get(i).second.getSplitPositions();
       assert borders != null : "borders are null";
       int lastBorderIndex = borders.length - 1;
-  
+
       // value is lower outlier
       if(val < borders[0]) {
         approximation[i] = 0;
@@ -361,13 +371,59 @@ public class PartialVAFile<V extends NumberVector<?>> extends AbstractRefiningIn
    * @apiviz.exclude
    */
   public static class Statistics {
-    public long scannedBytes = 0;
+    private Counter scannedBytes;
 
-    public long queryTime = 0;
+    private Counter queryTime;
 
-    public int issuedQueries = 0;
+    private Counter issuedQueries;
 
-    public int refinements = 0;
+    private Counter refinements;
+
+    protected Statistics(String parent) {
+      scannedBytes = LOG.isStatistics() ? LOG.newCounter(parent + ".scannedBytes") : null;
+      queryTime = LOG.isStatistics() ? LOG.newCounter(parent + ".queryTime") : null;
+      issuedQueries = LOG.isStatistics() ? LOG.newCounter(parent + ".issuedQueries") : null;
+      refinements = LOG.isStatistics() ? LOG.newCounter(parent + ".refinements") : null;
+    }
+
+    public void logStatistics() {
+      if(scannedBytes != null) {
+        LOG.statistics(scannedBytes);
+      }
+      if(queryTime != null) {
+        LOG.statistics(queryTime);
+      }
+      if(issuedQueries != null) {
+        LOG.statistics(issuedQueries);
+      }
+      if(refinements != null) {
+        LOG.statistics(refinements);
+      }
+    }
+
+    protected void incrementScannedBytes(long bytes) {
+      if(scannedBytes != null) {
+        scannedBytes.increment(bytes);
+      }
+    }
+
+    protected void incrementQueryTime(long time) {
+      if(queryTime != null) {
+        queryTime.increment(time);
+      }
+    }
+
+    protected void incrementIssuedQueries() {
+      if(issuedQueries != null) {
+        issuedQueries.increment();
+      }
+    }
+
+    protected void incrementRefinements() {
+      if(refinements != null) {
+        refinements.increment();
+      }
+    }
   }
 
   /**
@@ -453,8 +509,8 @@ public class PartialVAFile<V extends NumberVector<?>> extends AbstractRefiningIn
     }
 
     @Override
-    public DoubleDistanceDBIDList getRangeForObject(V query, DoubleDistance range) {
-      stats.issuedQueries++;
+    public DoubleDistanceDBIDPairList getRangeForObject(V query, DoubleDistance range) {
+      stats.incrementIssuedQueries();
       long t = System.nanoTime();
 
       final double epsilonP = Math.pow(range.doubleValue(), p);
@@ -468,10 +524,10 @@ public class PartialVAFile<V extends NumberVector<?>> extends AbstractRefiningIn
       // filter step
 
       // calculate selectivity coefficients
-      List<DoubleObjPair<DAFile>> subspaceDAFiles = new ArrayList<DoubleObjPair<DAFile>>(subspace.cardinality());
+      List<DoubleObjPair<DAFile>> subspaceDAFiles = new ArrayList<>(subspace.cardinality());
       for(int d = subspace.nextSetBit(0); d >= 0; d = subspace.nextSetBit(d + 1)) {
         DAFile daFile = daFiles.get(d);
-        subspaceDAFiles.add(new DoubleObjPair<DAFile>(-1, daFile));
+        subspaceDAFiles.add(new DoubleObjPair<>(-1, daFile));
       }
       calculateSelectivityCoeffs(subspaceDAFiles, query, range.doubleValue());
       // sort DA files by selectivity
@@ -481,7 +537,7 @@ public class PartialVAFile<V extends NumberVector<?>> extends AbstractRefiningIn
       // create candidate list (all objects) and prune candidates w.r.t.
       // mindist (i.e. remove them from the list)
       // important: this structure contains the maxDist values for refinement!
-      DoubleDistanceDBIDList result = new DoubleDistanceDBIDList();
+      DoubleDistanceDBIDPairList result = new DoubleDistanceDBIDPairList();
       int candidates = 0;
       for(VectorApproximation va : vectorApprox) {
         DBID id = va.getId();
@@ -508,7 +564,7 @@ public class PartialVAFile<V extends NumberVector<?>> extends AbstractRefiningIn
           }
           else { // refine candidate - true refinement
             DoubleDistance dis = refine(id, query);
-            stats.refinements += 1;
+            stats.incrementRefinements();
             if(dis.doubleValue() <= range.doubleValue()) {
               result.add(dis.doubleValue(), id);
             }
@@ -517,9 +573,9 @@ public class PartialVAFile<V extends NumberVector<?>> extends AbstractRefiningIn
       }
       result.sort();
 
-      stats.scannedBytes += relation.size() * VectorApproximation.byteOnDisk(subspace.cardinality(), partitions);
+      stats.incrementScannedBytes(relation.size() * VectorApproximation.byteOnDisk(subspace.cardinality(), partitions));
 
-      stats.queryTime += System.nanoTime() - t;
+      stats.incrementQueryTime(System.nanoTime() - t);
 
       if(LOG.isDebuggingFine()) {
         LOG.fine("query = " + query);
@@ -562,7 +618,7 @@ public class PartialVAFile<V extends NumberVector<?>> extends AbstractRefiningIn
 
     @Override
     public DoubleDistanceKNNList getKNNForObject(V query, int k) {
-      stats.issuedQueries++;
+      stats.incrementIssuedQueries();
       long t = System.nanoTime();
 
       // generate query approximation and lookup table
@@ -603,9 +659,9 @@ public class PartialVAFile<V extends NumberVector<?>> extends AbstractRefiningIn
           if(candidates2 != null) {
             candidates1 = candidates2;
           }
-          candidates2 = new LinkedList<PartialVACandidate>();
+          candidates2 = new LinkedList<>();
 
-          DoubleMaxHeap kMinMaxDists = new DoubleMaxHeap(k+1);
+          DoubleMaxHeap kMinMaxDists = new DoubleMaxHeap(k + 1);
           for(PartialVACandidate va : candidates1) {
             int dimension = daFiles.get(addition).getDimension();
             int objectCell = va.getApproximation(dimension);
@@ -628,21 +684,21 @@ public class PartialVAFile<V extends NumberVector<?>> extends AbstractRefiningIn
         }
       }
 
-      stats.scannedBytes += relation.size() * VectorApproximation.byteOnDisk(addition, partitions);
+      stats.incrementScannedBytes(relation.size() * VectorApproximation.byteOnDisk(addition, partitions));
 
       // refinement step
-      ArrayList<PartialVACandidate> sortedCandidates = new ArrayList<PartialVACandidate>(candidates2);
+      ArrayList<PartialVACandidate> sortedCandidates = new ArrayList<>(candidates2);
       // sort candidates by lower bound (minDist)
       Collections.sort(sortedCandidates);
       DoubleDistanceKNNList result = retrieveAccurateDistances(sortedCandidates, k, subspace, query);
 
-      stats.queryTime += System.nanoTime() - t;
+      stats.incrementQueryTime(System.nanoTime() - t);
       return result;
     }
 
     private LinkedList<PartialVACandidate> filter1(int k, int reducedDims, List<DAFile> daFiles, VectorApproximation queryApprox, int subspaceDims, VALPNormDistance dist) {
-      LinkedList<PartialVACandidate> candidates1 = new LinkedList<PartialVACandidate>();
-      DoubleMaxHeap minmaxdist = new DoubleMaxHeap(k+1);
+      LinkedList<PartialVACandidate> candidates1 = new LinkedList<>();
+      DoubleMaxHeap minmaxdist = new DoubleMaxHeap(k + 1);
 
       for(VectorApproximation va : vectorApprox) {
         PartialVACandidate pva = new PartialVACandidate(va);
@@ -704,7 +760,7 @@ public class PartialVAFile<V extends NumberVector<?>> extends AbstractRefiningIn
      */
     public List<DAFile> getWorstCaseDistOrder(VALPNormDistance dist, BitSet subspace) {
       int subspaceLength = subspace.cardinality();
-      List<DAFile> result = new ArrayList<DAFile>(subspaceLength);
+      List<DAFile> result = new ArrayList<>(subspaceLength);
       for(int i = subspace.nextSetBit(0); i >= 0; i = subspace.nextSetBit(i + 1)) {
         result.add(daFiles.get(i));
       }
@@ -713,15 +769,15 @@ public class PartialVAFile<V extends NumberVector<?>> extends AbstractRefiningIn
     }
 
     protected DoubleDistanceKNNList retrieveAccurateDistances(List<PartialVACandidate> sortedCandidates, int k, BitSet subspace, V query) {
-      DoubleDistanceKNNHeap result = new DoubleDistanceKNNHeap(k);
+      DoubleDistanceKNNHeap result = DBIDUtil.newDoubleDistanceHeap(k);
       for(PartialVACandidate va : sortedCandidates) {
         double stopdist = result.doubleKNNDistance();
         DBID currentID = va.getId();
         if(result.size() < k || va.minDistP < stopdist) {
           DoubleDistance dist = refine(currentID, query);
-          stats.refinements += 1;
+          stats.incrementRefinements();
           if(dist.doubleValue() < stopdist) {
-            result.add(dist.doubleValue(), currentID);
+            result.insert(dist.doubleValue(), currentID);
           }
         }
       }
@@ -791,7 +847,7 @@ public class PartialVAFile<V extends NumberVector<?>> extends AbstractRefiningIn
 
     @Override
     public PartialVAFile<V> instantiate(Relation<V> relation) {
-      return new PartialVAFile<V>(pagesize, relation, numpart);
+      return new PartialVAFile<>(pagesize, relation, numpart);
     }
 
     @Override
@@ -820,13 +876,13 @@ public class PartialVAFile<V extends NumberVector<?>> extends AbstractRefiningIn
       @Override
       protected void makeOptions(Parameterization config) {
         super.makeOptions(config);
-        IntParameter pagesizeP = new IntParameter(TreeIndexFactory.PAGE_SIZE_ID, 1024);
-        pagesizeP.addConstraint(new GreaterConstraint(0));
+        IntParameter pagesizeP = new IntParameter(AbstractPageFileFactory.Parameterizer.PAGE_SIZE_ID, 1024);
+        pagesizeP.addConstraint(CommonConstraints.GREATER_EQUAL_ONE_INT);
         if(config.grab(pagesizeP)) {
           pagesize = pagesizeP.getValue();
         }
         IntParameter partitionsP = new IntParameter(Factory.PARTITIONS_ID);
-        partitionsP.addConstraint(new GreaterConstraint(2));
+        partitionsP.addConstraint(CommonConstraints.GREATER_THAN_ONE_INT);
         if(config.grab(partitionsP)) {
           numpart = partitionsP.getValue();
         }
@@ -834,8 +890,8 @@ public class PartialVAFile<V extends NumberVector<?>> extends AbstractRefiningIn
 
       @Override
       protected Factory<?> makeInstance() {
-        return new Factory<NumberVector<?>>(pagesize, numpart);
+        return new Factory<>(pagesize, numpart);
       }
     }
   }
-}
\ No newline at end of file
+}
