@@ -34,7 +34,9 @@ import de.lmu.ifi.dbs.elki.data.type.TypeUtil;
 import de.lmu.ifi.dbs.elki.database.ids.DBID;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
-import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
+import de.lmu.ifi.dbs.elki.database.ids.distance.DoubleDistanceDBIDPairList;
+import de.lmu.ifi.dbs.elki.database.ids.distance.DoubleDistanceKNNHeap;
+import de.lmu.ifi.dbs.elki.database.ids.distance.DoubleDistanceKNNList;
 import de.lmu.ifi.dbs.elki.database.query.DatabaseQuery;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.database.query.knn.KNNQuery;
@@ -42,24 +44,21 @@ import de.lmu.ifi.dbs.elki.database.query.range.RangeQuery;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.database.relation.RelationUtil;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.DistanceFunction;
-import de.lmu.ifi.dbs.elki.distance.distancefunction.LPNormDistanceFunction;
-import de.lmu.ifi.dbs.elki.distance.distanceresultlist.DoubleDistanceDBIDList;
-import de.lmu.ifi.dbs.elki.distance.distanceresultlist.DoubleDistanceKNNHeap;
-import de.lmu.ifi.dbs.elki.distance.distanceresultlist.DoubleDistanceKNNList;
+import de.lmu.ifi.dbs.elki.distance.distancefunction.minkowski.LPNormDistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.DoubleDistance;
 import de.lmu.ifi.dbs.elki.index.AbstractRefiningIndex;
 import de.lmu.ifi.dbs.elki.index.IndexFactory;
 import de.lmu.ifi.dbs.elki.index.KNNIndex;
 import de.lmu.ifi.dbs.elki.index.RangeIndex;
-import de.lmu.ifi.dbs.elki.index.tree.TreeIndexFactory;
 import de.lmu.ifi.dbs.elki.logging.Logging;
+import de.lmu.ifi.dbs.elki.persistent.AbstractPageFileFactory;
 import de.lmu.ifi.dbs.elki.utilities.datastructures.heap.DoubleMaxHeap;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Title;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.AbstractParameterizer;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.GreaterConstraint;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.CommonConstraints;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntParameter;
 import de.lmu.ifi.dbs.elki.utilities.pairs.DoubleObjPair;
@@ -131,13 +130,13 @@ public class VAFile<V extends NumberVector<?>> extends AbstractRefiningIndex<V>
     this.partitions = partitions;
     this.pageSize = pageSize;
     this.scans = 0;
-    this.vectorApprox = new ArrayList<VectorApproximation>();
+    this.vectorApprox = new ArrayList<>();
   }
 
   @Override
-  protected void initialize(Relation<V> relation, DBIDs ids) {
+  public void initialize() {
     setPartitions(relation);
-    for (DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
+    for(DBIDIter iter = relation.iterDBIDs(); iter.valid(); iter.advance()) {
       DBID id = DBIDUtil.deref(iter);
       vectorApprox.add(calculateApproximation(id, relation.get(id)));
     }
@@ -212,24 +211,10 @@ public class VAFile<V extends NumberVector<?>> extends AbstractRefiningIndex<V>
     return new VectorApproximation(id, approximation);
   }
 
-  @Override
-  public long getReadOperations() {
-    return getRandomReadOnly() + getScannedPages();
-  }
-
-  /**
-   * Get the number of random read operations only.
-   * 
-   * @return Random read operations.
-   */
-  public long getRandomReadOnly() {
-    return super.getReadOperations();
-  }
-
   /**
    * Get the number of scanned bytes.
    * 
-   * @return Numebr of scanned bytes.
+   * @return Number of scanned bytes.
    */
   public long getScannedPages() {
     int vacapacity = pageSize / VectorApproximation.byteOnDisk(splitPositions.length, partitions);
@@ -238,15 +223,15 @@ public class VAFile<V extends NumberVector<?>> extends AbstractRefiningIndex<V>
   }
 
   @Override
-  public long getWriteOperations() {
-    return -1;
+  public Logging getLogger() {
+    return LOG;
   }
 
   @Override
-  public void resetPageAccess() {
-    super.resetPageAccess();
-    scans = 0;
-    // FIXME: writes
+  public void logStatistics() {
+    super.logStatistics();
+    // FIXME:
+    LOG.statistics("scanned pages:" + getScannedPages());
   }
 
   @Override
@@ -317,7 +302,7 @@ public class VAFile<V extends NumberVector<?>> extends AbstractRefiningIndex<V>
     }
 
     @Override
-    public DoubleDistanceDBIDList getRangeForObject(V query, DoubleDistance range) {
+    public DoubleDistanceDBIDPairList getRangeForObject(V query, DoubleDistance range) {
       final double eps = range.doubleValue();
       // generate query approximation and lookup table
       VectorApproximation queryApprox = calculateApproximation(null, query);
@@ -328,7 +313,7 @@ public class VAFile<V extends NumberVector<?>> extends AbstractRefiningIndex<V>
       // Count a VA file scan
       scans += 1;
 
-      DoubleDistanceDBIDList result = new DoubleDistanceDBIDList();
+      DoubleDistanceDBIDPairList result = new DoubleDistanceDBIDPairList();
       // Approximation step
       for(int i = 0; i < vectorApprox.size(); i++) {
         VectorApproximation va = vectorApprox.get(i);
@@ -383,10 +368,10 @@ public class VAFile<V extends NumberVector<?>> extends AbstractRefiningIndex<V>
       VALPNormDistance vadist = new VALPNormDistance(p, splitPositions, query, queryApprox);
 
       // Heap for the kth smallest maximum distance (yes, we need a max heap!)
-      DoubleMaxHeap minMaxHeap = new DoubleMaxHeap(k+1);
+      DoubleMaxHeap minMaxHeap = new DoubleMaxHeap(k + 1);
       double minMaxDist = Double.POSITIVE_INFINITY;
       // Candidates with minDist <= kth maxDist
-      ArrayList<DoubleObjPair<DBID>> candidates = new ArrayList<DoubleObjPair<DBID>>(vectorApprox.size());
+      ArrayList<DoubleObjPair<DBID>> candidates = new ArrayList<>(vectorApprox.size());
 
       // Count a VA file scan
       scans += 1;
@@ -401,7 +386,7 @@ public class VAFile<V extends NumberVector<?>> extends AbstractRefiningIndex<V>
         if(minDist > minMaxDist) {
           continue;
         }
-        candidates.add(new DoubleObjPair<DBID>(minDist, va.id));
+        candidates.add(new DoubleObjPair<>(minDist, va.id));
 
         // Update candidate pruning heap
         minMaxHeap.add(maxDist, k);
@@ -413,7 +398,7 @@ public class VAFile<V extends NumberVector<?>> extends AbstractRefiningIndex<V>
       Collections.sort(candidates);
 
       // refinement step
-      DoubleDistanceKNNHeap result = new DoubleDistanceKNNHeap(k);
+      DoubleDistanceKNNHeap result = DBIDUtil.newDoubleDistanceHeap(k);
 
       // log.fine("candidates size " + candidates.size());
       // retrieve accurate distances
@@ -428,7 +413,7 @@ public class VAFile<V extends NumberVector<?>> extends AbstractRefiningIndex<V>
 
         // refine the next element
         final double dist = refine(va.second, query).doubleValue();
-        result.add(dist, va.second);
+        result.insert(dist, va.second);
       }
       if(LOG.isDebuggingFinest()) {
         LOG.finest("query = (" + query + ")");
@@ -483,7 +468,7 @@ public class VAFile<V extends NumberVector<?>> extends AbstractRefiningIndex<V>
 
     @Override
     public VAFile<V> instantiate(Relation<V> relation) {
-      return new VAFile<V>(pagesize, relation, numpart);
+      return new VAFile<>(pagesize, relation, numpart);
     }
 
     @Override
@@ -512,13 +497,13 @@ public class VAFile<V extends NumberVector<?>> extends AbstractRefiningIndex<V>
       @Override
       protected void makeOptions(Parameterization config) {
         super.makeOptions(config);
-        IntParameter pagesizeP = new IntParameter(TreeIndexFactory.PAGE_SIZE_ID, 1024);
-        pagesizeP.addConstraint(new GreaterConstraint(0));
+        IntParameter pagesizeP = new IntParameter(AbstractPageFileFactory.Parameterizer.PAGE_SIZE_ID, 1024);
+        pagesizeP.addConstraint(CommonConstraints.GREATER_EQUAL_ONE_INT);
         if(config.grab(pagesizeP)) {
           pagesize = pagesizeP.getValue();
         }
         IntParameter partitionsP = new IntParameter(Factory.PARTITIONS_ID);
-        partitionsP.addConstraint(new GreaterConstraint(2));
+        partitionsP.addConstraint(CommonConstraints.GREATER_THAN_ONE_INT);
         if(config.grab(partitionsP)) {
           numpart = partitionsP.getValue();
         }
@@ -526,8 +511,8 @@ public class VAFile<V extends NumberVector<?>> extends AbstractRefiningIndex<V>
 
       @Override
       protected Factory<?> makeInstance() {
-        return new Factory<NumberVector<?>>(pagesize, numpart);
+        return new Factory<>(pagesize, numpart);
       }
     }
   }
-}
\ No newline at end of file
+}
