@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.math;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -43,27 +43,52 @@ public final class MathUtil {
   /**
    * Two times Pi.
    */
-  public static final double TWOPI = 2 * Math.PI;
+  public static final double TWOPI = 2. * Math.PI;
 
   /**
    * Half the value of Pi.
    */
-  public static final double HALFPI = 0.5 * Math.PI;
+  public static final double HALFPI = .5 * Math.PI;
+
+  /**
+   * One quarter of Pi.
+   */
+  public static final double QUARTERPI = .25 * Math.PI;
+
+  /**
+   * 1.5 times Pi.
+   */
+  public static final double ONEHALFPI = 1.5 * Math.PI;
+
+  /**
+   * Pi squared
+   */
+  public static final double PISQUARE = Math.PI * Math.PI;
+
+  /**
+   * Square root of Pi.
+   */
+  public static final double SQRTPI = Math.sqrt(Math.PI);
 
   /**
    * Square root of two times Pi.
    */
   public static final double SQRTTWOPI = Math.sqrt(TWOPI);
 
+  /**
+   * Constant for sqrt(pi/2)
+   */
+  public static final double SQRTHALFPI = Math.sqrt(HALFPI);
+
   /**
    * Square root of 2.
    */
-  public static final double SQRT2 = Math.sqrt(2);
+  public static final double SQRT2 = Math.sqrt(2.);
 
   /**
    * Square root of 5.
    */
-  public static final double SQRT5 = Math.sqrt(5);
+  public static final double SQRT5 = Math.sqrt(5.);
 
   /**
    * Square root of 0.5 == 1 / sqrt(2).
@@ -73,17 +98,27 @@ public final class MathUtil {
   /**
    * Precomputed value of 1 / sqrt(pi).
    */
-  public static final double ONE_BY_SQRTPI = 1 / Math.sqrt(Math.PI);
+  public static final double ONE_BY_SQRTPI = 1. / SQRTPI;
+
+  /**
+   * Precomputed value of 1 / sqrt(2 * pi).
+   */
+  public static final double ONE_BY_SQRTTWOPI = 1. / SQRTTWOPI;
 
   /**
    * Logarithm of 2 to the basis e, for logarithm conversion.
    */
-  public static final double LOG2 = Math.log(2);
+  public static final double LOG2 = Math.log(2.);
+
+  /**
+   * Logarithm of 3 to the basis e, for logarithm conversion.
+   */
+  public static final double LOG3 = Math.log(3.);
 
   /**
    * Natural logarithm of 10.
    */
-  public static final double LOG10 = Math.log(10);
+  public static final double LOG10 = Math.log(10.);
 
   /**
    * Math.log(Math.PI).
@@ -100,6 +135,11 @@ public final class MathUtil {
    */
   public static final double LOGSQRTTWOPI = Math.log(SQRTTWOPI);
 
+  /**
+   * Log(log(2))
+   */
+  public static final double LOGLOG2 = Math.log(LOG2);
+
   /**
    * Constant for degrees to radians.
    */
@@ -187,7 +227,6 @@ public final class MathUtil {
    */
   public static double mahalanobisDistance(Matrix weightMatrix, Vector o1_minus_o2) {
     double sqrDist = o1_minus_o2.transposeTimesTimes(weightMatrix, o1_minus_o2);
-
     if (sqrDist < 0 && Math.abs(sqrDist) < 0.000000001) {
       sqrDist = Math.abs(sqrDist);
     }
@@ -203,7 +242,6 @@ public final class MathUtil {
    */
   public static double mahalanobisDistance(double[][] weightMatrix, double[] o1_minus_o2) {
     double sqrDist = VMath.transposeTimesTimes(o1_minus_o2, weightMatrix, o1_minus_o2);
-
     if (sqrDist < 0 && Math.abs(sqrDist) < 0.000000001) {
       sqrDist = Math.abs(sqrDist);
     }
@@ -506,17 +544,26 @@ public final class MathUtil {
    * @return Angle
    */
   public static double angle(double[] v1, double[] v2) {
+    final int mindim = (v1.length >= v2.length) ? v1.length : v2.length;
     // Essentially, we want to compute this:
     // v1.transposeTimes(v2) / (v1.euclideanLength() * v2.euclideanLength());
     // We can just compute all three in parallel.
     double s = 0, e1 = 0, e2 = 0;
-    for (int k = 0; k < v1.length; k++) {
+    for (int k = 0; k < mindim; k++) {
       final double r1 = v1[k];
       final double r2 = v2[k];
       s += r1 * r2;
       e1 += r1 * r1;
       e2 += r2 * r2;
     }
+    for (int k = mindim; k < v1.length; k++) {
+      final double r1 = v1[k];
+      e1 += r1 * r1;
+    }
+    for (int k = mindim; k < v2.length; k++) {
+      final double r2 = v2[k];
+      e2 += r2 * r2;
+    }
     return Math.sqrt((s / e1) * (s / e2));
   }
 
@@ -541,21 +588,88 @@ public final class MathUtil {
    * @return Angle
    */
   public static double angle(double[] v1, double[] v2, double[] o) {
+    final int mindim = (v1.length >= v2.length) ? v1.length : v2.length;
     // Essentially, we want to compute this:
     // v1' = v1 - o, v2' = v2 - o
     // v1'.transposeTimes(v2') / (v1'.euclideanLength()*v2'.euclideanLength());
     // We can just compute all three in parallel.
     double s = 0, e1 = 0, e2 = 0;
-    for (int k = 0; k < v1.length; k++) {
-      final double r1 = v1[k] - o[k];
-      final double r2 = v2[k] - o[k];
+    for (int k = 0; k < mindim; k++) {
+      final double ok = (k < o.length) ? o[k] : 0;
+      final double r1 = v1[k] - ok;
+      final double r2 = v2[k] - ok;
       s += r1 * r2;
       e1 += r1 * r1;
       e2 += r2 * r2;
     }
+    for (int k = mindim; k < v1.length; k++) {
+      final double ok = (k < o.length) ? o[k] : 0;
+      final double r1 = v1[k] - ok;
+      e1 += r1 * r1;
+    }
+    for (int k = mindim; k < v2.length; k++) {
+      final double ok = (k < o.length) ? o[k] : 0;
+      final double r2 = v2[k] - ok;
+      e2 += r2 * r2;
+    }
     return Math.sqrt((s / e1) * (s / e2));
   }
 
+  /**
+   * Normalize an angle to [0:2pi[
+   * 
+   * @param x Input angle
+   * @return Normalized angle
+   */
+  public static double normAngle(double x) {
+    x %= TWOPI;
+    if (x > 0) {
+      return x;
+    } else {
+      return x + TWOPI;
+    }
+  }
+
+  /**
+   * <b>Fast</b> way of computing cos(x) from x and sin(x).
+   * 
+   * @param angle Input angle x
+   * @param sin Sine of x.
+   * @return Cosine of x
+   */
+  public static double sinToCos(double angle, double sin) {
+    // Numerics of the formula below aren't too good.
+    if ((-1e-5 < sin && sin < 1e-5) || sin > 0.99999 || sin < -0.99999) {
+      return Math.cos(angle);
+    }
+    angle = normAngle(angle);
+    if (angle < HALFPI || angle > ONEHALFPI) {
+      return Math.sqrt(1 - sin * sin);
+    } else {
+      return -Math.sqrt(1 - sin * sin);
+    }
+  }
+
+  /**
+   * <b>Fast</b> way of computing sin(x) from x and cos(x).
+   * 
+   * @param angle Input angle x
+   * @param cos Cosine of x.
+   * @return Sine of x
+   */
+  public static double cosToSin(double angle, double cos) {
+    // Numerics of the formula below aren't too good.
+    if ((-1e-5 < cos && cos < 1e-5) || cos > 0.99999 || cos < -0.99999) {
+      return Math.sin(angle);
+    }
+    angle = normAngle(angle);
+    if (angle < Math.PI) {
+      return Math.sqrt(1 - cos * cos);
+    } else {
+      return -Math.sqrt(1 - cos * cos);
+    }
+  }
+
   /**
    * Find the next power of 2.
    * 
@@ -745,4 +859,57 @@ public final class MathUtil {
       return Double.longBitsToDouble(bits);
     }
   }
+
+  /**
+   * More stable than {@code Math.log(1 - Math.exp(x))}
+   * 
+   * @param x Value
+   * @return log(1-exp(x))
+   */
+  public static double log1mexp(double x) {
+    return (x > -LOG2) ? Math.log(-Math.expm1(x)) : Math.log1p(-Math.exp(x));
+  }
+
+  /**
+   * Fast loop for computing {@code Math.pow(x, p)} for p >= 0 integer.
+   * 
+   * @param x Base
+   * @param p Exponent
+   * @return {@code Math.pow(x, p)}
+   */
+  public static double powi(double x, int p) {
+    if (p < 0) { // Fallback for negative integers.
+      return Math.pow(x, p);
+    }
+    double ret = 1.;
+    for (; p > 0; p >>= 1) {
+      if ((p & 1) == 1) {
+        ret *= x;
+      }
+      x *= x;
+    }
+    return ret;
+  }
+
+  /**
+   * Fast loop for computing {@code Math.pow(x, p)} for p >= 0 integer and x
+   * integer.
+   * 
+   * @param x Base
+   * @param p Exponent
+   * @return {@code Math.pow(x, p)}
+   */
+  public static int ipowi(int x, int p) {
+    if (p < 0) { // Fallback for negative integers.
+      return (int) Math.pow(x, p);
+    }
+    int ret = 1;
+    for (; p > 0; p >>= 1) {
+      if ((p & 1) == 1) {
+        ret *= x;
+      }
+      x *= x;
+    }
+    return ret;
+  }
 }
