@@ -12,7 +12,7 @@ import de.lmu.ifi.dbs.elki.math.linearalgebra.Vector;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -64,11 +64,11 @@ public class AlphaShape {
     // Compute delaunay triangulation:
     delaunay = (new SweepHullDelaunay2D(points)).getDelaunay();
 
-    List<Polygon> polys = new ArrayList<Polygon>();
+    List<Polygon> polys = new ArrayList<>();
 
     // Working data
     BitSet used = new BitSet(delaunay.size());
-    List<Vector> cur = new ArrayList<Vector>();
+    List<Vector> cur = new ArrayList<>();
 
     for(int i = 0 /* = used.nextClearBit(0) */; i < delaunay.size() && i >= 0; i = used.nextClearBit(i + 1)) {
       if(!used.get(i)) {
@@ -82,7 +82,7 @@ public class AlphaShape {
         }
         if(cur.size() > 0) {
           polys.add(new Polygon(cur));
-          cur = new ArrayList<Vector>();
+          cur = new ArrayList<>();
         }
       }
     }
