@@ -21,7 +21,7 @@ import de.lmu.ifi.dbs.elki.utilities.pairs.IntIntPair;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -113,7 +113,7 @@ public class SweepHullDelaunay2D {
       throw new UnsupportedOperationException("There is no delaunay triangulation for less than three objects!");
     }
     int len = points.size() - 1;
-    hull = new LinkedList<IntIntPair>();
+    hull = new LinkedList<>();
     tris = hullonly ? null : new ArrayList<Triangle>(len);
 
     final Vector seed;
@@ -614,7 +614,7 @@ public class SweepHullDelaunay2D {
     }
     DoubleMinMax minmaxX = new DoubleMinMax();
     DoubleMinMax minmaxY = new DoubleMinMax();
-    List<Vector> hullp = new ArrayList<Vector>(hull.size());
+    List<Vector> hullp = new ArrayList<>(hull.size());
     for(IntIntPair pair : hull) {
       Vector v = points.get(pair.first);
       hullp.add(v);
