@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.math.geometry;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -151,7 +151,7 @@ public class XYCurve implements Result, TextWriteable {
   public void addAndSimplify(double x, double y) {
     // simplify curve when possible:
     final int len = data.size();
-    if(len >= 4) {
+    if (len >= 4) {
       // Look at the previous 2 points
       final double l1x = data.get(len - 4);
       final double l1y = data.get(len - 3);
@@ -163,16 +163,16 @@ public class XYCurve implements Result, TextWriteable {
       final double cdx = x - l2x;
       final double cdy = y - l2y;
       // X simplification
-      if((ldx == 0) && (cdx == 0)) {
+      if ((ldx == 0) && (cdx == 0)) {
         data.remove(len - 2, 2);
       }
       // horizontal simplification
-      else if((ldy == 0) && (cdy == 0)) {
+      else if ((ldy == 0) && (cdy == 0)) {
         data.remove(len - 2, 2);
       }
       // diagonal simplification
-      else if(ldy > 0 && cdy > 0) {
-        if(Math.abs((ldx / ldy) - (cdx / cdy)) < THRESHOLD) {
+      else if (ldy > 0 && cdy > 0) {
+        if (Math.abs((ldx / ldy) - (cdx / cdy)) < THRESHOLD) {
           data.remove(len - 2, 2);
         }
       }
@@ -254,6 +254,21 @@ public class XYCurve implements Result, TextWriteable {
     return data.get((off << 1) + 1);
   }
 
+  /**
+   * Rescale the graph.
+   * 
+   * @param sx Scaling factor for X axis
+   * @param sy Scaling factor for Y axis
+   */
+  public void rescale(double sx, double sy) {
+    for (int i = 0; i < data.size(); i += 2) {
+      data.set(i, sx * data.get(i));
+      data.set(i + 1, sy * data.get(i + 1));
+    }
+    maxx *= sx;
+    maxy *= sy;
+  }
+
   /**
    * Size of curve.
    * 
@@ -294,7 +309,7 @@ public class XYCurve implements Result, TextWriteable {
     out.commentPrint(" ");
     out.commentPrint(labely);
     out.flush();
-    for(int pos = 0; pos < data.size(); pos+=2) {
+    for (int pos = 0; pos < data.size(); pos += 2) {
       out.inlinePrint(Double.toString(data.get(pos)));
       out.inlinePrint(Double.toString(data.get(pos + 1)));
       out.flush();
@@ -306,7 +321,7 @@ public class XYCurve implements Result, TextWriteable {
     StringBuilder buf = new StringBuilder();
     buf.append("XYCurve[");
     buf.append(labelx).append(',').append(labely).append(':');
-    for(int pos = 0; pos < data.size(); pos += 2) {
+    for (int pos = 0; pos < data.size(); pos += 2) {
       buf.append(' ').append(data.get(pos)).append(',').append(data.get(pos + 1));
     }
     buf.append(']');
@@ -334,21 +349,21 @@ public class XYCurve implements Result, TextWriteable {
   public static double areaUnderCurve(XYCurve curve) {
     TDoubleArrayList data = curve.data;
     double prevx = data.get(0), prevy = data.get(1);
-    if(prevx > curve.minx) {
+    if (prevx > curve.minx) {
       throw new UnsupportedOperationException("Curves must be monotone on X for areaUnderCurve to be valid.");
     }
     double area = 0.0;
-    for(int pos = 2; pos < data.size(); pos += 2) {
+    for (int pos = 2; pos < data.size(); pos += 2) {
       final double curx = data.get(pos), cury = data.get(pos + 1);
-      if(prevx > curx) {
+      if (prevx > curx) {
         throw new UnsupportedOperationException("Curves must be monotone on X for areaUnderCurve to be valid.");
       }
       area += (curx - prevx) * (prevy + cury) * .5; // .5 = mean Y
       prevx = curx;
       prevy = cury;
     }
-    if(prevx < curve.maxx) {
-      throw new UnsupportedOperationException("Curves must be monotone on X for areaUnderCurve to be valid.");
+    if (prevx < curve.maxx) {
+      throw new UnsupportedOperationException("Curves must be complete on X for areaUnderCurve to be valid.");
     }
     return area;
   }
@@ -415,4 +430,4 @@ public class XYCurve implements Result, TextWriteable {
       return pos < data.size();
     }
   }
-}
\ No newline at end of file
+}
