@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.math.linearalgebra;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -1448,10 +1448,10 @@ public class Matrix {
     final StreamTokenizer tokenizer = new StreamTokenizer(input);
 
     // Although StreamTokenizer will parse numbers, it doesn't recognize
-    // scientific notation (E or D); however, Double.valueOf does.
+    // scientific notation (E or D); however, FormatUtil.parseDouble does.
     // The strategy here is to disable StreamTokenizer's number parsing.
     // We'll only get whitespace delimited words, EOL's and EOF's.
-    // These words should all be numbers, for Double.valueOf to parse.
+    // These words should all be numbers, for FormatUtil.parseDouble to parse.
 
     tokenizer.resetSyntax();
     tokenizer.wordChars(0, 255);
@@ -1467,14 +1467,14 @@ public class Matrix {
       throw new java.io.IOException("Unexpected EOF on matrix read.");
     }
     do {
-      v.add(Double.parseDouble(tokenizer.sval)); // Read & store 1st
+      v.add(FormatUtil.parseDouble(tokenizer.sval)); // Read & store 1st
       // row.
     }
     while (tokenizer.nextToken() == StreamTokenizer.TT_WORD);
 
     int n = v.size(); // Now we've got the number of columns!
     double row[] = v.toArray();
-    ArrayList<double[]> rowV = new ArrayList<double[]>();
+    ArrayList<double[]> rowV = new ArrayList<>();
     rowV.add(row); // Start storing rows instead of columns.
     while (tokenizer.nextToken() == StreamTokenizer.TT_WORD) {
       // While non-empty lines
@@ -1484,7 +1484,7 @@ public class Matrix {
         if (j >= n) {
           throw new java.io.IOException("Row " + v.size() + " is too long.");
         }
-        row[j++] = Double.parseDouble(tokenizer.sval);
+        row[j++] = FormatUtil.parseDouble(tokenizer.sval);
       }
       while (tokenizer.nextToken() == StreamTokenizer.TT_WORD);
       if (j < n) {
