@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.math.scales;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -67,7 +67,11 @@ public final class Scales {
     for (DBIDIter iditer = db.iterDBIDs(); iditer.valid(); iditer.advance()) {
       O v = db.get(iditer);
       for (int d = 0; d < dim; d++) {
-        minmax[d].put(v.doubleValue(d));
+        final double val = v.doubleValue(d);
+        if(val != val) {
+          continue; // NaN
+        }
+        minmax[d].put(val);
       }
     }
 
