@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.math.spacefillingcurves;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -49,6 +49,7 @@ public interface SpatialSorter {
    * @param start First index to sort (e.g. 0)
    * @param end End of range (e.g. <code>site()</code>)
    * @param minmax Array with dim pairs of (min, max) of value ranges
+   * @param dims Dimensions to sort by, for indexing vectors <em>and</em> {@code minmax}.
    */
-  <T extends SpatialComparable> void sort(List<T> objs, int start, int end, double[] minmax);
+  <T extends SpatialComparable> void sort(List<T> objs, int start, int end, double[] minmax, int[] dims);
 }
\ No newline at end of file
