@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.math.spacefillingcurves;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -37,8 +37,8 @@ public class ZCurveSpatialSorter extends AbstractSpatialSorter {
   private static final double STOPVAL = 1E-10;
 
   @Override
-  public <T extends SpatialComparable> void sort(List<T> objs, int start, int end, double[] minmax) {
-    zSort(objs, start, end, minmax, 0);
+  public <T extends SpatialComparable> void sort(List<T> objs, int start, int end, double[] minmax, int[] dims) {
+    zSort(objs, start, end, minmax, dims, 0);
   }
 
   /**
@@ -48,44 +48,44 @@ public class ZCurveSpatialSorter extends AbstractSpatialSorter {
    * @param start Start
    * @param end End
    * @param mms Min-Max value ranges
-   * @param dim Current dimension
+   * @param dims Dimensions to process
+   * @param depth Current dimension
    */
-  protected <T extends SpatialComparable> void zSort(List<T> objs, int start, int end, double[] mms, int dim) {
+  protected <T extends SpatialComparable> void zSort(List<T> objs, int start, int end, double[] mms, int[] dims, int depth) {
+    final int numdim = (dims != null) ? dims.length : (mms.length >> 1);
+    final int edim = (dims != null) ? dims[depth] : depth;
     // Find the splitting points.
-    final double min = mms[2 * dim], max = mms[2 * dim + 1];
+    final double min = mms[2 * edim], max = mms[2 * edim + 1];
     double spos = (min + max) / 2.;
     // Safeguard against duplicate points:
-    if(max - spos < STOPVAL || spos - min < STOPVAL) {
+    if (max - spos < STOPVAL || spos - min < STOPVAL) {
       boolean ok = false;
-      for(int d = 0; d < mms.length; d += 2) {
-        if(mms[d + 1] - mms[d] >= STOPVAL) {
-          // LoggingUtil.warning("No: " + (mms[d + 1] - mms[d]));
+      for (int d = 0; d < numdim; d++) {
+        int d2 = ((dims != null) ? dims[d] : d) << 1;
+        if (mms[d2 + 1] - mms[d2] >= STOPVAL) {
           ok = true;
           break;
         }
       }
-      if(!ok) {
+      if (!ok) {
         return;
       }
     }
-    int split = pivotizeList1D(objs, start, end, dim, spos, false);
+    int split = pivotizeList1D(objs, start, end, edim, spos, false);
     assert (start <= split && split <= end);
-    int nextdim = (dim + 1) % objs.get(0).getDimensionality();
-    // LoggingUtil.warning("dim: " + dim + " min: " + min + " split: " + spos +
-    // " max:" + max + " " + start + " < " + split + " < " + end);
-    if(start < split - 1) {
-      mms[2 * dim] = min;
-      mms[2 * dim + 1] = spos;
-      zSort(objs, start, split, mms, nextdim);
+    int nextdim = (depth + 1) % numdim;
+    if (start < split - 1) {
+      mms[2 * edim] = min;
+      mms[2 * edim + 1] = spos;
+      zSort(objs, start, split, mms, dims, nextdim);
     }
-    if(split < end - 1) {
-      mms[2 * dim] = spos;
-      mms[2 * dim + 1] = max;
-      zSort(objs, split, end, mms, nextdim);
+    if (split < end - 1) {
+      mms[2 * edim] = spos;
+      mms[2 * edim + 1] = max;
+      zSort(objs, split, end, mms, dims, nextdim);
     }
     // Restore ranges
-    mms[2 * dim] = min;
-    mms[2 * dim + 1] = max;
-    // FIXME: implement completely and test.
+    mms[2 * edim] = min;
+    mms[2 * edim + 1] = max;
   }
-}
\ No newline at end of file
+}
