@@ -1,12 +1,10 @@
 package de.lmu.ifi.dbs.elki.math.statistics.distribution;
 
-import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
-
 /*
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -24,6 +22,13 @@ import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
  You should have received a copy of the GNU Affero General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
+import java.util.Random;
+
+import de.lmu.ifi.dbs.elki.utilities.RandomFactory;
+import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.DoubleParameter;
 
 /**
  * Chi-Squared distribution (a specialization of the Gamma distribution).
@@ -37,7 +42,27 @@ public class ChiSquaredDistribution extends GammaDistribution {
    * @param dof Degrees of freedom.
    */
   public ChiSquaredDistribution(double dof) {
-    super(.5 * dof, .5);
+    this(dof, (Random) null);
+  }
+
+  /**
+   * Constructor.
+   * 
+   * @param dof Degrees of freedom.
+   * @param random Random generator.
+   */
+  public ChiSquaredDistribution(double dof, Random random) {
+    super(.5 * dof, .5, random);
+  }
+
+  /**
+   * Constructor.
+   * 
+   * @param dof Degrees of freedom.
+   * @param random Random generator.
+   */
+  public ChiSquaredDistribution(double dof, RandomFactory random) {
+    super(.5 * dof, .5, random);
   }
 
   /**
@@ -59,14 +84,14 @@ public class ChiSquaredDistribution extends GammaDistribution {
    * @return probability density
    */
   public static double pdf(double x, double dof) {
-    if(x <= 0) {
+    if (x <= 0) {
       return 0.0;
     }
-    if(x == 0) {
+    if (x == 0) {
       return 0.0;
     }
     final double k = dof * .5;
-    if(Math.abs(k - 1.0) < Double.MIN_NORMAL) {
+    if (Math.abs(k - 1.0) < Double.MIN_NORMAL) {
       return Math.exp(-x * 2.0) * 2.0;
     }
     return Math.exp((k - 1.0) * Math.log(x * 2.0) - x * 2.0 - logGamma(k)) * 2.0;
@@ -90,4 +115,41 @@ public class ChiSquaredDistribution extends GammaDistribution {
   public static double quantile(double x, double dof) {
     return GammaDistribution.quantile(x, .5 * dof, .5);
   }
-}
\ No newline at end of file
+
+  @Override
+  public String toString() {
+    return "ChiSquaredDistribution(dof=" + (2 * getK()) + ")";
+  }
+
+  /**
+   * Parameterization class
+   * 
+   * @author Erich Schubert
+   * 
+   * @apiviz.exclude
+   */
+  public static class Parameterizer extends AbstractDistribution.Parameterizer {
+    /**
+     * Degrees of freedom parameter.
+     */
+    public static final OptionID DOF_ID = new OptionID("distribution.chi.dof", "Chi distribution degrees of freedom parameter.");
+
+    /** Parameters. */
+    double dof;
+
+    @Override
+    protected void makeOptions(Parameterization config) {
+      super.makeOptions(config);
+
+      DoubleParameter dofP = new DoubleParameter(DOF_ID);
+      if (config.grab(dofP)) {
+        dof = dofP.doubleValue();
+      }
+    }
+
+    @Override
+    protected ChiSquaredDistribution makeInstance() {
+      return new ChiSquaredDistribution(dof, rnd);
+    }
+  }
+}
