@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.math.statistics.distribution;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -23,14 +23,14 @@ package de.lmu.ifi.dbs.elki.math.statistics.distribution;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.Parameterizable;
+
 /**
- * Statistical distributions, with their common functions. See
- * {@link DistributionWithRandom} for distributions that also have a random
- * generator included.
+ * Statistical distributions, with their common functions.
  * 
  * @author Erich Schubert
  */
-public interface Distribution {
+public interface Distribution extends Parameterizable {
   /**
    * Return the density of an existing value
    * 
@@ -55,6 +55,13 @@ public interface Distribution {
    */
   double quantile(double val);
 
+  /**
+   * Generate a new random value
+   * 
+   * @return new random value
+   */
+  double nextRandom();
+
   /**
    * Describe the distribution
    * 
@@ -62,4 +69,4 @@ public interface Distribution {
    */
   @Override
   String toString();
-}
\ No newline at end of file
+}
