@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.math.statistics.distribution;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -26,13 +26,19 @@ package de.lmu.ifi.dbs.elki.math.statistics.distribution;
 import java.util.Random;
 
 import de.lmu.ifi.dbs.elki.math.MathUtil;
+import de.lmu.ifi.dbs.elki.utilities.Alias;
+import de.lmu.ifi.dbs.elki.utilities.RandomFactory;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.CommonConstraints;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.DoubleParameter;
 
 /**
  * Gaussian distribution aka normal distribution
  * 
  * @author Erich Schubert
  */
-public class NormalDistribution implements DistributionWithRandom {
+@Alias({ "GaussianDistribution", "normal", "gauss" })
+public class NormalDistribution extends AbstractDistribution {
   /**
    * Coefficients for erf approximation.
    * 
@@ -105,6 +111,11 @@ public class NormalDistribution implements DistributionWithRandom {
    */
   static final double ERFINV_D[] = { 7.784695709041462e-03, 3.224671290700398e-01, 2.445134137142996e+00, 3.754408661907416e+00 };
 
+  /**
+   * 1 / CDFINV(0.75)
+   */
+  public static final double ONEBYPHIINV075 = 1.48260221850560186054;
+
   /**
    * Mean value for the generator
    */
@@ -116,9 +127,17 @@ public class NormalDistribution implements DistributionWithRandom {
   private double stddev;
 
   /**
-   * The random generator.
+   * Constructor for Gaussian distribution
+   * 
+   * @param mean Mean
+   * @param stddev Standard Deviation
+   * @param random Random generator
    */
-  private Random random;
+  public NormalDistribution(double mean, double stddev, RandomFactory random) {
+    super(random);
+    this.mean = mean;
+    this.stddev = stddev;
+  }
 
   /**
    * Constructor for Gaussian distribution
@@ -128,10 +147,9 @@ public class NormalDistribution implements DistributionWithRandom {
    * @param random Random generator
    */
   public NormalDistribution(double mean, double stddev, Random random) {
-    super();
+    super(random);
     this.mean = mean;
     this.stddev = stddev;
-    this.random = random;
   }
 
   /**
@@ -141,7 +159,7 @@ public class NormalDistribution implements DistributionWithRandom {
    * @param stddev Standard Deviation
    */
   public NormalDistribution(double mean, double stddev) {
-    this(mean, stddev, new Random());
+    this(mean, stddev, (Random) null);
   }
 
   @Override
@@ -194,28 +212,28 @@ public class NormalDistribution implements DistributionWithRandom {
    * @return erfc(x)
    */
   public static double erfc(double x) {
-    if(Double.isNaN(x)) {
+    if (Double.isNaN(x)) {
       return Double.NaN;
     }
-    if(Double.isInfinite(x)) {
+    if (Double.isInfinite(x)) {
       return (x < 0.0) ? 2 : 0;
     }
 
     double result = Double.NaN;
     double absx = Math.abs(x);
     // First approximation interval
-    if(absx < 0.46875) {
+    if (absx < 0.46875) {
       double z = x * x;
       result = 1 - x * ((((ERFAPP_A[0] * z + ERFAPP_A[1]) * z + ERFAPP_A[2]) * z + ERFAPP_A[3]) * z + ERFAPP_A[4]) / ((((ERFAPP_B[0] * z + ERFAPP_B[1]) * z + ERFAPP_B[2]) * z + ERFAPP_B[3]) * z + ERFAPP_B[4]);
     }
     // Second approximation interval
-    else if(absx < 4.0) {
+    else if (absx < 4.0) {
       double z = absx;
       result = ((((((((ERFAPP_C[0] * z + ERFAPP_C[1]) * z + ERFAPP_C[2]) * z + ERFAPP_C[3]) * z + ERFAPP_C[4]) * z + ERFAPP_C[5]) * z + ERFAPP_C[6]) * z + ERFAPP_C[7]) * z + ERFAPP_C[8]) / ((((((((ERFAPP_D[0] * z + ERFAPP_D[1]) * z + ERFAPP_D[2]) * z + ERFAPP_D[3]) * z + ERFAPP_D[4]) * z + ERFAPP_D[5]) * z + ERFAPP_D[6]) * z + ERFAPP_D[7]) * z + ERFAPP_D[8]);
       double rounded = Math.round(result * 16.0) / 16.0;
       double del = (absx - rounded) * (absx + rounded);
       result = Math.exp(-rounded * rounded) * Math.exp(-del) * result;
-      if(x < 0.0) {
+      if (x < 0.0) {
         result = 2.0 - result;
       }
     }
@@ -227,7 +245,7 @@ public class NormalDistribution implements DistributionWithRandom {
       double rounded = Math.round(result * 16.0) / 16.0;
       double del = (absx - rounded) * (absx + rounded);
       result = Math.exp(-rounded * rounded) * Math.exp(-del) * result;
-      if(x < 0.0) {
+      if (x < 0.0) {
         result = 2.0 - result;
       }
     }
@@ -254,7 +272,74 @@ public class NormalDistribution implements DistributionWithRandom {
    * @return erfinv(x)
    */
   public static double erfinv(double x) {
-    return standardNormalQuantile(0.5 * (x + 1)) / MathUtil.SQRT2;
+    return standardNormalQuantile(0.5 * (x + 1)) * MathUtil.SQRTHALF;
+  }
+
+  /**
+   * Probability density function of the normal distribution.
+   * 
+   * <pre>
+   * 1/(SQRT(2*pi*sigma^2)) * e^(-(x-mu)^2/2sigma^2)
+   * </pre>
+   * 
+   * @param x The value.
+   * @param mu The mean.
+   * @param sigma The standard deviation.
+   * @return PDF of the given normal distribution at x.
+   */
+  public static double pdf(double x, double mu, double sigma) {
+    x = (x - mu) / sigma;
+    return MathUtil.ONE_BY_SQRTTWOPI / sigma * Math.exp(-.5 * x * x);
+  }
+
+  /**
+   * Probability density function of the normal distribution.
+   * 
+   * <pre>
+   * 1/(SQRT(2*pi*sigma^2)) * e^(-(x-mu)^2/2sigma^2)
+   * </pre>
+   * 
+   * @param x The value.
+   * @return PDF of the given normal distribution at x.
+   */
+  public static double standardNormalPDF(double x) {
+    return Math.exp(-.5 * x * x) * MathUtil.SQRTHALF;
+  }
+
+  /**
+   * Cumulative probability density function (CDF) of a normal distribution.
+   * 
+   * @param x value to evaluate CDF at
+   * @param mu Mean value
+   * @param sigma Standard deviation.
+   * @return The CDF of the given normal distribution at x.
+   */
+  public static double cdf(double x, double mu, double sigma) {
+    x = (x - mu) / sigma;
+    return .5 + .5 * erf(x * MathUtil.SQRTHALF);
+  }
+
+  /**
+   * Cumulative probability density function (CDF) of a normal distribution.
+   * 
+   * @param x value to evaluate CDF at
+   * @return The CDF of the given normal distribution at x.
+   */
+  public static double standardNormalCDF(double x) {
+    return .5 + .5 * erf(x * MathUtil.SQRTHALF);
+  }
+
+  /**
+   * Inverse cumulative probability density function (probit) of a normal
+   * distribution.
+   * 
+   * @param x value to evaluate probit function at
+   * @param mu Mean value
+   * @param sigma Standard deviation.
+   * @return The probit of the given normal distribution at x.
+   */
+  public static double quantile(double x, double mu, double sigma) {
+    return mu + sigma * standardNormalQuantile(x);
   }
 
   /**
@@ -273,26 +358,21 @@ public class NormalDistribution implements DistributionWithRandom {
    * @return Inverse erf.
    */
   public static double standardNormalQuantile(double d) {
-    if(d == 0) {
+    if (d == 0) {
       return Double.NEGATIVE_INFINITY;
-    }
-    else if(d == 1) {
+    } else if (d == 1) {
       return Double.POSITIVE_INFINITY;
-    }
-    else if(Double.isNaN(d) || d < 0 || d > 1) {
+    } else if (Double.isNaN(d) || d < 0 || d > 1) {
       return Double.NaN;
-    }
-    else if(d < P_LOW) {
+    } else if (d < P_LOW) {
       // Rational approximation for lower region:
       double q = Math.sqrt(-2 * Math.log(d));
       return (((((ERFINV_C[0] * q + ERFINV_C[1]) * q + ERFINV_C[2]) * q + ERFINV_C[3]) * q + ERFINV_C[4]) * q + ERFINV_C[5]) / ((((ERFINV_D[0] * q + ERFINV_D[1]) * q + ERFINV_D[2]) * q + ERFINV_D[3]) * q + 1);
-    }
-    else if(P_HIGH < d) {
+    } else if (P_HIGH < d) {
       // Rational approximation for upper region:
       double q = Math.sqrt(-2 * Math.log(1 - d));
       return -(((((ERFINV_C[0] * q + ERFINV_C[1]) * q + ERFINV_C[2]) * q + ERFINV_C[3]) * q + ERFINV_C[4]) * q + ERFINV_C[5]) / ((((ERFINV_D[0] * q + ERFINV_D[1]) * q + ERFINV_D[2]) * q + ERFINV_D[3]) * q + 1);
-    }
-    else {
+    } else {
       // Rational approximation for central region:
       double q = d - 0.5D;
       double r = q * q;
@@ -301,45 +381,35 @@ public class NormalDistribution implements DistributionWithRandom {
   }
 
   /**
-   * Probability density function of the normal distribution.
+   * Parameterization class
    * 
-   * <pre>
-   * 1/(SQRT(2*pi*sigma^2)) * e^(-(x-mu)^2/2sigma^2)
-   * </pre>
+   * @author Erich Schubert
    * 
-   * @param x The value.
-   * @param mu The mean.
-   * @param sigma The standard deviation.
-   * @return PDF of the given normal distribution at x.
+   * @apiviz.exclude
    */
-  public static double pdf(double x, double mu, double sigma) {
-    final double x_mu = x - mu;
-    final double sigmasq = sigma * sigma;
-    return 1 / (Math.sqrt(MathUtil.TWOPI * sigmasq)) * Math.exp(-.5 * x_mu * x_mu / sigmasq);
-  }
+  public static class Parameterizer extends AbstractDistribution.Parameterizer {
+    /** Parameters. */
+    double mu, sigma;
 
-  /**
-   * Cumulative probability density function (CDF) of a normal distribution.
-   * 
-   * @param x value to evaluate CDF at
-   * @param mu Mean value
-   * @param sigma Standard deviation.
-   * @return The CDF of the normal given distribution at x.
-   */
-  public static double cdf(double x, double mu, double sigma) {
-    return .5 * (1 + erf((x - mu) / (MathUtil.SQRT2 * sigma)));
-  }
+    @Override
+    protected void makeOptions(Parameterization config) {
+      super.makeOptions(config);
 
-  /**
-   * Inverse cumulative probability density function (probit) of a normal
-   * distribution.
-   * 
-   * @param x value to evaluate probit function at
-   * @param mu Mean value
-   * @param sigma Standard deviation.
-   * @return The probit of the normal given distribution at x.
-   */
-  public static double quantile(double x, double mu, double sigma) {
-    return mu + sigma * standardNormalQuantile(x);
+      DoubleParameter muP = new DoubleParameter(LOCATION_ID);
+      if (config.grab(muP)) {
+        mu = muP.doubleValue();
+      }
+
+      DoubleParameter sigmaP = new DoubleParameter(SCALE_ID);
+      sigmaP.addConstraint(CommonConstraints.GREATER_THAN_ZERO_DOUBLE);
+      if (config.grab(sigmaP)) {
+        sigma = sigmaP.doubleValue();
+      }
+    }
+
+    @Override
+    protected NormalDistribution makeInstance() {
+      return new NormalDistribution(mu, sigma, rnd);
+    }
   }
 }
