@@ -1,15 +1,10 @@
 package de.lmu.ifi.dbs.elki.math.statistics.distribution;
 
-import de.lmu.ifi.dbs.elki.math.MathUtil;
-import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
-import de.lmu.ifi.dbs.elki.utilities.exceptions.AbortException;
-import de.lmu.ifi.dbs.elki.utilities.exceptions.ExceptionMessages;
-
 /*
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -27,6 +22,18 @@ import de.lmu.ifi.dbs.elki.utilities.exceptions.ExceptionMessages;
  You should have received a copy of the GNU Affero General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
+import java.util.Random;
+
+import de.lmu.ifi.dbs.elki.math.MathUtil;
+import de.lmu.ifi.dbs.elki.utilities.RandomFactory;
+import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
+import de.lmu.ifi.dbs.elki.utilities.exceptions.ExceptionMessages;
+import de.lmu.ifi.dbs.elki.utilities.exceptions.NotImplementedException;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.CommonConstraints;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.DoubleParameter;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntParameter;
 
 /**
  * INCOMPLETE implementation of the poisson distribution.
@@ -41,7 +48,7 @@ import de.lmu.ifi.dbs.elki.utilities.exceptions.ExceptionMessages;
  * 
  * @author Erich Schubert
  */
-public class PoissonDistribution implements Distribution {
+public class PoissonDistribution extends AbstractDistribution {
   /**
    * Number of tries
    */
@@ -109,79 +116,80 @@ public class PoissonDistribution implements Distribution {
   /**
    * Constructor.
    * 
-   * Private: API not yet completely implemented!
-   * 
    * @param n Number of tries
    * @param p Success probability
    */
   public PoissonDistribution(int n, double p) {
-    super();
+    this(n, p, (Random) null);
+  }
+
+  /**
+   * Constructor.
+   * 
+   * @param n Number of tries
+   * @param p Success probability
+   * @param random Random generator
+   */
+  public PoissonDistribution(int n, double p, Random random) {
+    super(random);
     this.n = n;
     this.p = p;
   }
 
   /**
-   * Poisson PMF for integer values.
+   * Constructor.
+   * 
+   * @param n Number of tries
+   * @param p Success probability
+   * @param random Random generator
+   */
+  public PoissonDistribution(int n, double p, RandomFactory random) {
+    super(random);
+    this.n = n;
+    this.p = p;
+  }
+
+  /**
+   * Poisson probability mass function (PMF) for integer values.
+   * 
+   * @param x integer values
+   * @return Probability
+   */
+  public double pmf(int x) {
+    return pmf(x, n, p);
+  }
+
+  @Override
+  public double pdf(double x) {
+    // FIXME: return 0 for non-integer x?
+    return pmf(x, n, p);
+  }
+
+  /**
+   * Poisson probability mass function (PMF) for integer values.
    * 
    * @param x integer values
    * @return Probability
    */
   @Reference(title = "Fast and accurate computation of binomial probabilities", authors = "C. Loader", booktitle = "", url = "http://projects.scipy.org/scipy/raw-attachment/ticket/620/loader2000Fast.pdf")
-  public double pmf(int x) {
+  public static double pmf(double x, int n, double p) {
     // Invalid values
     if(x < 0 || x > n) {
-      return 0.0;
+      return 0.;
     }
     // Extreme probabilities
-    if(p <= 0d) {
-      return x == 0 ? 1.0 : 0.0;
+    if(p <= 0.) {
+      return x == 0 ? 1. : 0.;
     }
-    if(p >= 1d) {
-      return x == n ? 1.0 : 0.0;
-    }
-    // Extreme values of x
-    if(x == 0) {
-      if(p < 0.1) {
-        return Math.exp(-devianceTerm(n, n * (1.0 - p)) - n * p);
-      }
-      else {
-        return Math.exp(n * Math.log(1.0 - p));
-      }
-    }
-    if(x == n) {
-      if(p > 0.9) {
-        return Math.exp(-devianceTerm(n, n * p) - n * (1 - p));
-      }
-      else {
-        return Math.exp(n * Math.log(p));
-      }
-    }
-
-    final double lc = stirlingError(n) - stirlingError(x) - stirlingError(n - x) - devianceTerm(x, n * p) - devianceTerm(n - x, n * (1.0 - p));
-    final double f = (MathUtil.TWOPI * x * (n - x)) / n;
-    return Math.exp(lc) / Math.sqrt(f);
-  }
-
-  @Override
-  @Reference(title = "Fast and accurate computation of binomial probabilities", authors = "C. Loader", booktitle = "", url = "http://projects.scipy.org/scipy/raw-attachment/ticket/620/loader2000Fast.pdf")
-  public double pdf(double x) {
-    // Invalid values
-    if(x < 0 || x > n) {
-      return 0.0;
-    }
-    // Extreme probabilities
-    if(p <= 0d) {
-      return x == 0 ? 1.0 : 0.0;
-    }
-    if(p >= 1d) {
-      return x == n ? 1.0 : 0.0;
+    if(p >= 1.) {
+      return x == n ? 1. : 0.;
     }
     final double q = 1 - p;
     // FIXME: check for x to be integer, return 0 otherwise?
 
     // Extreme values of x
     if(x == 0) {
-      if(p < 0.1) {
+      if(p < .1) {
         return Math.exp(-devianceTerm(n, n * q) - n * p);
       }
       else {
@@ -189,7 +197,7 @@ public class PoissonDistribution implements Distribution {
       }
     }
     if(x == n) {
-      if(p > 0.9) {
+      if(p > .9) {
         return Math.exp(-devianceTerm(n, n * p) - n * q);
       }
       else {
@@ -204,13 +212,19 @@ public class PoissonDistribution implements Distribution {
   // FIXME: implement!
   @Override
   public double cdf(double val) {
-    throw new AbortException(ExceptionMessages.UNSUPPORTED_NOT_YET);
+    throw new NotImplementedException(ExceptionMessages.UNSUPPORTED_NOT_YET);
   }
 
   // FIXME: implement!
   @Override
   public double quantile(double val) {
-    throw new AbortException(ExceptionMessages.UNSUPPORTED_NOT_YET);
+    throw new NotImplementedException(ExceptionMessages.UNSUPPORTED_NOT_YET);
+  }
+
+  // FIXME: implement!
+  @Override
+  public double nextRandom() {
+    throw new NotImplementedException(ExceptionMessages.UNSUPPORTED_NOT_YET);
   }
 
   /**
@@ -408,4 +422,61 @@ public class PoissonDistribution implements Distribution {
     final double y = -stirlingError(x) - devianceTerm(x, lambda);
     return -0.5 * Math.log(f) + y;
   }
-}
\ No newline at end of file
+
+  @Override
+  public String toString() {
+    return "PoissonDistribution(n=" + n + ", p=" + p + ")";
+  }
+
+  /**
+   * Parameterization class
+   * 
+   * @author Erich Schubert
+   * 
+   * @apiviz.exclude
+   */
+  public static class Parameterizer extends AbstractDistribution.Parameterizer {
+    /**
+     * Number of trials.
+     */
+    public static final OptionID N_ID = new OptionID("distribution.poisson.n", "Number of trials.");
+
+    /**
+     * Success probability.
+     */
+    public static final OptionID PROB_ID = new OptionID("distribution.poisson.probability", "Success probability.");
+
+    /**
+     * Number of trials.
+     */
+    int n;
+
+    /**
+     * Success probability.
+     */
+    double p;
+
+    @Override
+    protected void makeOptions(Parameterization config) {
+      super.makeOptions(config);
+
+      IntParameter nP = new IntParameter(N_ID);
+      nP.addConstraint(CommonConstraints.GREATER_EQUAL_ONE_INT);
+      if(config.grab(nP)) {
+        n = nP.intValue();
+      }
+
+      DoubleParameter probP = new DoubleParameter(PROB_ID);
+      probP.addConstraint(CommonConstraints.GREATER_EQUAL_ZERO_DOUBLE);
+      probP.addConstraint(CommonConstraints.LESS_EQUAL_ONE_DOUBLE);
+      if(config.grab(probP)) {
+        p = probP.doubleValue();
+      }
+    }
+
+    @Override
+    protected PoissonDistribution makeInstance() {
+      return new PoissonDistribution(n, p, rnd);
+    }
+  }
+}
