@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.persistent;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -25,15 +25,14 @@ package de.lmu.ifi.dbs.elki.persistent;
 
 import java.util.Stack;
 
+import de.lmu.ifi.dbs.elki.logging.statistics.LongStatistic;
+
 /**
  * Abstract class implementing general methods of a PageFile. A PageFile stores
  * objects that implement the <code>Page</code> interface.
  * 
  * @author Elke Achtert
  * 
- * @apiviz.has Page
- * @apiviz.has PageFileStatistics
- * 
  * @param <P> Page type
  */
 public abstract class AbstractStoringPageFile<P extends Page> extends AbstractPageFile<P> {
@@ -56,7 +55,7 @@ public abstract class AbstractStoringPageFile<P extends Page> extends AbstractPa
    * Creates a new PageFile.
    */
   protected AbstractStoringPageFile(int pageSize) {
-    this.emptyPages = new Stack<Integer>();
+    this.emptyPages = new Stack<>();
     this.nextPageID = 0;
     this.pageSize = pageSize;
   }
@@ -148,8 +147,10 @@ public abstract class AbstractStoringPageFile<P extends Page> extends AbstractPa
   }
 
   @Override
-  public PageFileStatistics getInnerStatistics() {
-    // Default: no nested page file.
-    return null;
+  public void logStatistics() {
+    super.logStatistics();
+    if (getLogger().isStatistics()) {
+      getLogger().statistics(new LongStatistic(this.getClass().getName() + ".numpages", nextPageID - emptyPages.size()));
+    }
   }
 }
\ No newline at end of file
