@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.persistent;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -25,6 +25,7 @@ package de.lmu.ifi.dbs.elki.persistent;
 
 import gnu.trove.map.TIntObjectMap;
 import gnu.trove.map.hash.TIntObjectHashMap;
+import de.lmu.ifi.dbs.elki.logging.Logging;
 
 /**
  * A memory based implementation of a PageFile that simulates I/O-access.<br>
@@ -35,6 +36,11 @@ import gnu.trove.map.hash.TIntObjectHashMap;
  * @param <P> Page type
  */
 public class MemoryPageFile<P extends Page> extends AbstractStoringPageFile<P> {
+  /**
+   * Class logger.
+   */
+  private static final Logging LOG = Logging.getLogger(MemoryPageFile.class);
+
   /**
    * Holds the pages.
    */
@@ -48,18 +54,18 @@ public class MemoryPageFile<P extends Page> extends AbstractStoringPageFile<P> {
    */
   public MemoryPageFile(int pageSize) {
     super(pageSize);
-    this.file = new TIntObjectHashMap<P>();
+    this.file = new TIntObjectHashMap<>();
   }
 
   @Override
   public synchronized P readPage(int pageID) {
-    readAccess++;
+    countRead();
     return file.get(pageID);
   }
-  
+
   @Override
   protected void writePage(int pageID, P page) {
-    writeAccess++;
+    countWrite();
     file.put(pageID, page);
     page.setDirty(false);
   }
@@ -71,7 +77,7 @@ public class MemoryPageFile<P extends Page> extends AbstractStoringPageFile<P> {
     super.deletePage(pageID);
 
     // delete from file
-    writeAccess++;
+    countWrite();
     file.remove(pageID);
   }
 
@@ -79,4 +85,9 @@ public class MemoryPageFile<P extends Page> extends AbstractStoringPageFile<P> {
   public void clear() {
     file.clear();
   }
-}
\ No newline at end of file
+
+  @Override
+  protected Logging getLogger() {
+    return LOG;
+  }
+}
