@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.persistent;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -23,15 +23,16 @@ package de.lmu.ifi.dbs.elki.persistent;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-
 /**
  * Page file interface.
  * 
  * @author Erich Schubert
  * 
+ * @apiviz.has Page
+ * 
  * @param <P> Page file
  */
-public interface PageFile<P extends Page> extends PageFileStatistics {
+public interface PageFile<P extends Page> {
   /**
    * Sets the id of the given page.
    * 
@@ -103,4 +104,9 @@ public interface PageFile<P extends Page> extends PageFileStatistics {
    * @return true when the file already existed.
    */
   public boolean initialize(PageHeader header);
-}
\ No newline at end of file
+
+  /**
+   * Log some statistics to the appropriate logger.
+   */
+  public void logStatistics();
+}
