@@ -1,12 +1,10 @@
 package de.lmu.ifi.dbs.elki.result;
 
-import de.lmu.ifi.dbs.elki.utilities.InspectionUtilFrequentlyScanned;
-
 /*
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -25,8 +23,6 @@ import de.lmu.ifi.dbs.elki.utilities.InspectionUtilFrequentlyScanned;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-
-
 /**
  * Interface for any class that can handle results
  * 
@@ -34,7 +30,7 @@ import de.lmu.ifi.dbs.elki.utilities.InspectionUtilFrequentlyScanned;
  * 
  * @apiviz.uses Result oneway - - processes
  */
-public interface ResultProcessor extends InspectionUtilFrequentlyScanned {
+public interface ResultProcessor {
   /**
    * Process a result.
    * 
