@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.result.textwriter;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -29,22 +29,27 @@ import java.io.PrintStream;
 /**
  * Interface for output handling (single file, multiple files, ...)
  * 
+ * Note: these classes need to be rewritten. Contributions welcome!
+ * 
  * @author Erich Schubert
  */
 public interface StreamFactory {
   /**
-   * Retrieve a print stream for output using the given label.
-   * Note that multiple labels MAY result in the same PrintStream, so you
-   * should be printing to only one stream at a time to avoid mixing outputs.
+   * Retrieve a print stream for output using the given label. Note that
+   * multiple labels MAY result in the same PrintStream, so you should be
+   * printing to only one stream at a time to avoid mixing outputs.
    * 
    * @param label Output label.
    * @return stream object for the given label
    * @throws IOException on IO error
    */
   public PrintStream openStream(String label) throws IOException;
-  
+
   /**
-   * Close (and forget) all streams the factory has opened.
+   * Close the given output stream (Note: when writing to a single stream
+   * output, it will actually not be closed!)
+   * 
+   * @param stream Stream to close
    */
-  public void closeAllStreams();
-}
\ No newline at end of file
+  public void closeStream(PrintStream stream);
+}
