@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.utilities;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -325,7 +325,7 @@ public final class ClassGenericsUtil {
   public static <T> ArrayList<T>[] newArrayOfEmptyArrayList(int len) {
     ArrayList[] result = new ArrayList[len];
     for (int i = 0; i < len; i++) {
-      result[i] = new ArrayList<T>();
+      result[i] = new ArrayList<>();
     }
     return result;
   }
@@ -344,7 +344,7 @@ public final class ClassGenericsUtil {
   public static <T> HashSet<T>[] newArrayOfEmptyHashSet(int len) {
     HashSet[] result = new HashSet[len];
     for (int i = 0; i < len; i++) {
-      result[i] = new HashSet<T>();
+      result[i] = new HashSet<>();
     }
     return result;
   }
