@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.utilities;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -72,7 +72,7 @@ public final class FormatUtil {
 
   static {
     NF.setMinimumFractionDigits(0);
-    NF.setMaximumFractionDigits(10);
+    NF.setMaximumFractionDigits(8);
     NF.setGroupingUsed(false);
     NF0.setMinimumFractionDigits(0);
     NF0.setMaximumFractionDigits(0);
@@ -167,10 +167,11 @@ public final class FormatUtil {
    */
   public static String format(double[] d, String sep) {
     StringBuilder buffer = new StringBuilder();
-    for (int i = 0; i < d.length; i++) {
-      if (i > 0) {
+    for(int i = 0; i < d.length; i++) {
+      if(i > 0) {
         buffer.append(sep).append(d[i]);
-      } else {
+      }
+      else {
         buffer.append(d[i]);
       }
     }
@@ -189,10 +190,11 @@ public final class FormatUtil {
    */
   public static String format(double[] d, String sep, int digits) {
     StringBuilder buffer = new StringBuilder();
-    for (int i = 0; i < d.length; i++) {
-      if (i < d.length - 1) {
+    for(int i = 0; i < d.length; i++) {
+      if(i < d.length - 1) {
         buffer.append(format(d[i], digits)).append(sep);
-      } else {
+      }
+      else {
         buffer.append(format(d[i], digits));
       }
     }
@@ -221,10 +223,11 @@ public final class FormatUtil {
    */
   public static String format(double[] d, String sep, NumberFormat nf) {
     StringBuilder buffer = new StringBuilder();
-    for (int i = 0; i < d.length; i++) {
-      if (i < d.length - 1) {
+    for(int i = 0; i < d.length; i++) {
+      if(i < d.length - 1) {
         buffer.append(format(d[i], nf)).append(sep);
-      } else {
+      }
+      else {
         buffer.append(format(d[i], nf));
       }
     }
@@ -261,7 +264,7 @@ public final class FormatUtil {
    */
   public static String format(double[][] d) {
     StringBuilder buffer = new StringBuilder();
-    for (double[] array : d) {
+    for(double[] array : d) {
       buffer.append(format(array, ", ", 2)).append('\n');
     }
     return buffer.toString();
@@ -280,10 +283,11 @@ public final class FormatUtil {
   public static String format(double[][] d, String sep1, String sep2, int digits) {
     StringBuilder buffer = new StringBuilder();
 
-    for (int i = 0; i < d.length; i++) {
-      if (i < d.length - 1) {
+    for(int i = 0; i < d.length; i++) {
+      if(i < d.length - 1) {
         buffer.append(format(d[i], sep2, digits)).append(sep1);
-      } else {
+      }
+      else {
         buffer.append(format(d[i], sep2, digits));
       }
     }
@@ -303,10 +307,11 @@ public final class FormatUtil {
    */
   public static String format(Double[] f, String sep, int digits) {
     StringBuilder buffer = new StringBuilder();
-    for (int i = 0; i < f.length; i++) {
-      if (i < f.length - 1) {
+    for(int i = 0; i < f.length; i++) {
+      if(i < f.length - 1) {
         buffer.append(format(f[i].doubleValue(), digits)).append(sep);
-      } else {
+      }
+      else {
         buffer.append(format(f[i].doubleValue(), digits));
       }
     }
@@ -335,10 +340,11 @@ public final class FormatUtil {
    */
   public static String format(Double[] f, String sep, NumberFormat nf) {
     StringBuilder buffer = new StringBuilder();
-    for (int i = 0; i < f.length; i++) {
-      if (i < f.length - 1) {
+    for(int i = 0; i < f.length; i++) {
+      if(i < f.length - 1) {
         buffer.append(format(f[i].doubleValue(), nf)).append(sep);
-      } else {
+      }
+      else {
         buffer.append(format(f[i].doubleValue(), nf));
       }
     }
@@ -368,10 +374,11 @@ public final class FormatUtil {
    */
   public static String format(float[] f, String sep, int digits) {
     StringBuilder buffer = new StringBuilder();
-    for (int i = 0; i < f.length; i++) {
-      if (i < f.length - 1) {
+    for(int i = 0; i < f.length; i++) {
+      if(i < f.length - 1) {
         buffer.append(format(f[i], digits)).append(sep);
-      } else {
+      }
+      else {
         buffer.append(format(f[i], digits));
       }
     }
@@ -398,10 +405,11 @@ public final class FormatUtil {
    */
   public static String format(int[] a, String sep) {
     StringBuilder buffer = new StringBuilder();
-    for (int i = 0; i < a.length; i++) {
-      if (i < a.length - 1) {
+    for(int i = 0; i < a.length; i++) {
+      if(i < a.length - 1) {
         buffer.append(a[i]).append(sep);
-      } else {
+      }
+      else {
         buffer.append(a[i]);
       }
     }
@@ -428,10 +436,11 @@ public final class FormatUtil {
    */
   public static String format(Integer[] a, String sep) {
     StringBuilder buffer = new StringBuilder();
-    for (int i = 0; i < a.length; i++) {
-      if (i < a.length - 1) {
+    for(int i = 0; i < a.length; i++) {
+      if(i < a.length - 1) {
         buffer.append(a[i]).append(sep);
-      } else {
+      }
+      else {
         buffer.append(a[i]);
       }
     }
@@ -456,10 +465,11 @@ public final class FormatUtil {
    */
   public static String format(long[] a) {
     StringBuilder buffer = new StringBuilder();
-    for (int i = 0; i < a.length; i++) {
-      if (i < a.length - 1) {
+    for(int i = 0; i < a.length; i++) {
+      if(i < a.length - 1) {
         buffer.append(a[i]).append(", ");
-      } else {
+      }
+      else {
         buffer.append(a[i]);
       }
     }
@@ -474,10 +484,11 @@ public final class FormatUtil {
    */
   public static String format(byte[] a) {
     StringBuilder buffer = new StringBuilder();
-    for (int i = 0; i < a.length; i++) {
-      if (i < a.length - 1) {
+    for(int i = 0; i < a.length; i++) {
+      if(i < a.length - 1) {
         buffer.append(a[i]).append(", ");
-      } else {
+      }
+      else {
         buffer.append(a[i]);
       }
     }
@@ -494,10 +505,11 @@ public final class FormatUtil {
    */
   public static String format(boolean[] b, final String sep) {
     StringBuilder buffer = new StringBuilder();
-    for (int i = 0; i < b.length; i++) {
-      if (i < b.length - 1) {
+    for(int i = 0; i < b.length; i++) {
+      if(i < b.length - 1) {
         buffer.append(format(b[i])).append(sep);
-      } else {
+      }
+      else {
         buffer.append(format(b[i]));
       }
     }
@@ -511,7 +523,7 @@ public final class FormatUtil {
    * @return a String representing of the boolean b
    */
   public static String format(final boolean b) {
-    if (b) {
+    if(b) {
       return "1";
     }
     return "0";
@@ -528,13 +540,14 @@ public final class FormatUtil {
   public static String format(BitSet bitSet, int dim, String sep) {
     StringBuilder msg = new StringBuilder();
 
-    for (int d = 0; d < dim; d++) {
-      if (d > 0) {
+    for(int d = 0; d < dim; d++) {
+      if(d > 0) {
         msg.append(sep);
       }
-      if (bitSet.get(d)) {
+      if(bitSet.get(d)) {
         msg.append('1');
-      } else {
+      }
+      else {
         msg.append('0');
       }
     }
@@ -563,16 +576,16 @@ public final class FormatUtil {
    * @return a String representing the String Collection d
    */
   public static String format(Collection<String> d, String sep) {
-    if (d.size() == 0) {
+    if(d.size() == 0) {
       return "";
     }
-    if (d.size() == 1) {
+    if(d.size() == 1) {
       return d.iterator().next();
     }
     StringBuilder buffer = new StringBuilder();
     boolean first = true;
-    for (String str : d) {
-      if (!first) {
+    for(String str : d) {
+      if(!first) {
         buffer.append(sep);
       }
       buffer.append(str);
@@ -600,12 +613,12 @@ public final class FormatUtil {
     int width = w + 1;
     StringBuilder msg = new StringBuilder();
     msg.append('\n'); // start on new line.
-    for (int i = 0; i < m.getRowDimensionality(); i++) {
-      for (int j = 0; j < m.getColumnDimensionality(); j++) {
+    for(int i = 0; i < m.getRowDimensionality(); i++) {
+      for(int j = 0; j < m.getColumnDimensionality(); j++) {
         String s = format.format(m.get(i, j)); // format the number
         int padding = Math.max(1, width - s.length()); // At _least_ 1
         // space
-        for (int k = 0; k < padding; k++) {
+        for(int k = 0; k < padding; k++) {
           msg.append(' ');
         }
         msg.append(s);
@@ -636,11 +649,11 @@ public final class FormatUtil {
     int width = w + 1;
     StringBuilder msg = new StringBuilder();
     msg.append('\n'); // start on new line.
-    for (int i = 0; i < v.getDimensionality(); i++) {
+    for(int i = 0; i < v.getDimensionality(); i++) {
       String s = format.format(v.get(i)); // format the number
       int padding = Math.max(1, width - s.length()); // At _least_ 1
       // space
-      for (int k = 0; k < padding; k++) {
+      for(int k = 0; k < padding; k++) {
         msg.append(' ');
       }
       msg.append(s);
@@ -660,11 +673,11 @@ public final class FormatUtil {
   public static String format(Matrix m, String pre) {
     StringBuilder output = new StringBuilder();
     output.append(pre).append("[\n").append(pre);
-    for (int i = 0; i < m.getRowDimensionality(); i++) {
+    for(int i = 0; i < m.getRowDimensionality(); i++) {
       output.append(" [");
-      for (int j = 0; j < m.getColumnDimensionality(); j++) {
+      for(int j = 0; j < m.getColumnDimensionality(); j++) {
         output.append(' ').append(m.get(i, j));
-        if (j < m.getColumnDimensionality() - 1) {
+        if(j < m.getColumnDimensionality() - 1) {
           output.append(',');
         }
       }
@@ -685,26 +698,26 @@ public final class FormatUtil {
   public static String format(Matrix m, NumberFormat nf) {
     int[] colMax = new int[m.getColumnDimensionality()];
     String[][] entries = new String[m.getRowDimensionality()][m.getColumnDimensionality()];
-    for (int i = 0; i < m.getRowDimensionality(); i++) {
-      for (int j = 0; j < m.getColumnDimensionality(); j++) {
+    for(int i = 0; i < m.getRowDimensionality(); i++) {
+      for(int j = 0; j < m.getColumnDimensionality(); j++) {
         entries[i][j] = nf.format(m.get(i, j));
-        if (entries[i][j].length() > colMax[j]) {
+        if(entries[i][j].length() > colMax[j]) {
           colMax[j] = entries[i][j].length();
         }
       }
     }
     StringBuilder output = new StringBuilder();
     output.append("[\n");
-    for (int i = 0; i < m.getRowDimensionality(); i++) {
+    for(int i = 0; i < m.getRowDimensionality(); i++) {
       output.append(" [");
-      for (int j = 0; j < m.getColumnDimensionality(); j++) {
+      for(int j = 0; j < m.getColumnDimensionality(); j++) {
         output.append(' ');
         int space = colMax[j] - entries[i][j].length();
-        for (int s = 0; s < space; s++) {
+        for(int s = 0; s < space; s++) {
           output.append(' ');
         }
         output.append(entries[i][j]);
-        if (j < m.getColumnDimensionality() - 1) {
+        if(j < m.getColumnDimensionality() - 1) {
           output.append(',');
         }
       }
@@ -721,7 +734,7 @@ public final class FormatUtil {
    * @return String representation of this Matrix
    */
   public static String format(Matrix m) {
-    return format(m, FormatUtil.NF8);
+    return format(m, FormatUtil.NF);
   }
 
   /**
@@ -741,7 +754,7 @@ public final class FormatUtil {
    * @return String representation of this Vector
    */
   public static String format(Vector m) {
-    return format(m, FormatUtil.NF8);
+    return format(m, FormatUtil.NF);
   }
 
   /**
@@ -754,9 +767,9 @@ public final class FormatUtil {
   public static String format(Vector v, String pre) {
     StringBuilder output = new StringBuilder();
     output.append(pre).append("[\n").append(pre);
-    for (int j = 0; j < v.getDimensionality(); j++) {
+    for(int j = 0; j < v.getDimensionality(); j++) {
       output.append(' ').append(v.get(j));
-      if (j < v.getDimensionality() - 1) {
+      if(j < v.getDimensionality() - 1) {
         output.append(',');
       }
     }
@@ -774,32 +787,32 @@ public final class FormatUtil {
    * @return a string representation of this matrix
    */
   public static String format(Matrix m, String pre, NumberFormat nf) {
-    if (nf == null) {
+    if(nf == null) {
       return FormatUtil.format(m, pre);
     }
 
     int[] colMax = new int[m.getColumnDimensionality()];
     String[][] entries = new String[m.getRowDimensionality()][m.getColumnDimensionality()];
-    for (int i = 0; i < m.getRowDimensionality(); i++) {
-      for (int j = 0; j < m.getColumnDimensionality(); j++) {
+    for(int i = 0; i < m.getRowDimensionality(); i++) {
+      for(int j = 0; j < m.getColumnDimensionality(); j++) {
         entries[i][j] = nf.format(m.get(i, j));
-        if (entries[i][j].length() > colMax[j]) {
+        if(entries[i][j].length() > colMax[j]) {
           colMax[j] = entries[i][j].length();
         }
       }
     }
     StringBuilder output = new StringBuilder();
     output.append(pre).append("[\n").append(pre);
-    for (int i = 0; i < m.getRowDimensionality(); i++) {
+    for(int i = 0; i < m.getRowDimensionality(); i++) {
       output.append(" [");
-      for (int j = 0; j < m.getColumnDimensionality(); j++) {
+      for(int j = 0; j < m.getColumnDimensionality(); j++) {
         output.append(' ');
         int space = colMax[j] - entries[i][j].length();
-        for (int s = 0; s < space; s++) {
+        for(int s = 0; s < space; s++) {
           output.append(' ');
         }
         output.append(entries[i][j]);
-        if (j < m.getColumnDimensionality() - 1) {
+        if(j < m.getColumnDimensionality() - 1) {
           output.append(',');
         }
       }
@@ -821,22 +834,22 @@ public final class FormatUtil {
   public static int findSplitpoint(String s, int width) {
     // the newline (or EOS) is the fallback split position.
     int in = s.indexOf(NEWLINE);
-    if (in < 0) {
+    if(in < 0) {
       in = s.length();
     }
     // Good enough?
-    if (in < width) {
+    if(in < width) {
       return in;
     }
     // otherwise, search for whitespace
     int iw = s.lastIndexOf(' ', width);
     // good whitespace found?
-    if (iw >= 0 && iw < width) {
+    if(iw >= 0 && iw < width) {
       return iw;
     }
     // sub-optimal splitpoint - retry AFTER the given position
     int bp = nextPosition(s.indexOf(' ', width), s.indexOf(NEWLINE, width));
-    if (bp >= 0) {
+    if(bp >= 0) {
       return bp;
     }
     // even worse - can't split!
@@ -853,10 +866,10 @@ public final class FormatUtil {
    *         otherwise whichever is positive.
    */
   private static int nextPosition(int a, int b) {
-    if (a < 0) {
+    if(a < 0) {
       return b;
     }
-    if (b < 0) {
+    if(b < 0) {
       return a;
     }
     return Math.min(a, b);
@@ -871,22 +884,22 @@ public final class FormatUtil {
    * @return string fragments
    */
   public static List<String> splitAtLastBlank(String s, int width) {
-    List<String> chunks = new ArrayList<String>();
+    List<String> chunks = new ArrayList<>();
 
     String tmp = s;
-    while (tmp.length() > 0) {
+    while(tmp.length() > 0) {
       int index = findSplitpoint(tmp, width);
       // store first part
       chunks.add(tmp.substring(0, index));
       // skip whitespace at beginning of line
-      while (index < tmp.length() && tmp.charAt(index) == ' ') {
+      while(index < tmp.length() && tmp.charAt(index) == ' ') {
         index += 1;
       }
       // remove a newline
-      if (index < tmp.length() && tmp.regionMatches(index, NEWLINE, 0, NEWLINE.length())) {
+      if(index < tmp.length() && tmp.regionMatches(index, NEWLINE, 0, NEWLINE.length())) {
         index += NEWLINE.length();
       }
-      if (index >= tmp.length()) {
+      if(index >= tmp.length()) {
         break;
       }
       tmp = tmp.substring(index);
@@ -902,11 +915,11 @@ public final class FormatUtil {
    * @return a string with the specified number of blanks
    */
   public static String whitespace(int n) {
-    if (n < WHITESPACE_BUFFER.length()) {
+    if(n < WHITESPACE_BUFFER.length()) {
       return WHITESPACE_BUFFER.substring(0, n);
     }
     char[] buf = new char[n];
-    for (int i = 0; i < n; i++) {
+    for(int i = 0; i < n; i++) {
       buf[i] = WHITESPACE_BUFFER.charAt(0);
     }
     return new String(buf);
@@ -920,7 +933,7 @@ public final class FormatUtil {
    * @return padded string of at least length len (and o otherwise)
    */
   public static String pad(String o, int len) {
-    if (o.length() >= len) {
+    if(o.length() >= len) {
       return o;
     }
     return o + whitespace(len - o.length());
@@ -934,7 +947,7 @@ public final class FormatUtil {
    * @return padded string of at least length len (and o otherwise)
    */
   public static String padRightAligned(String o, int len) {
-    if (o.length() >= len) {
+    if(o.length() >= len) {
       return o;
     }
     return whitespace(len - o.length()) + o;
@@ -950,9 +963,11 @@ public final class FormatUtil {
     final int default_termwidth = 78;
     try {
       return Integer.parseInt(System.getenv("COLUMNS")) - 1;
-    } catch (SecurityException e) {
+    }
+    catch(SecurityException e) {
       return default_termwidth;
-    } catch (NumberFormatException e) {
+    }
+    catch(NumberFormatException e) {
       return default_termwidth;
     }
   }
@@ -967,18 +982,18 @@ public final class FormatUtil {
     final StringBuilder sb = new StringBuilder();
     final Formatter fmt = new Formatter(sb);
 
-    for (int i = TIME_UNIT_SIZES.length - 1; i >= 0; --i) {
+    for(int i = TIME_UNIT_SIZES.length - 1; i >= 0; --i) {
       // We do not include ms if we are in the order of minutes.
-      if (i == 0 && sb.length() > 4) {
+      if(i == 0 && sb.length() > 4) {
         continue;
       }
       // Separator
-      if (sb.length() > 0) {
+      if(sb.length() > 0) {
         sb.append(sep);
       }
       final long acValue = time / TIME_UNIT_SIZES[i];
       time = time % TIME_UNIT_SIZES[i];
-      if (!(acValue == 0 && sb.length() == 0)) {
+      if(!(acValue == 0 && sb.length() == 0)) {
         fmt.format("%0" + TIME_UNIT_DIGITS[i] + "d%s", Long.valueOf(acValue), TIME_UNIT_NAMES[i]);
       }
     }
@@ -996,13 +1011,341 @@ public final class FormatUtil {
    */
   public static String format(String[] d, String sep) {
     StringBuilder buffer = new StringBuilder();
-    for (int i = 0; i < d.length; i++) {
-      if (i > 0) {
+    for(int i = 0; i < d.length; i++) {
+      if(i > 0) {
         buffer.append(sep).append(d[i]);
-      } else {
+      }
+      else {
         buffer.append(d[i]);
       }
     }
     return buffer.toString();
   }
+
+  /**
+   * Preallocated exceptions.
+   */
+  private static final NumberFormatException EXPONENT_OVERFLOW = new NumberFormatException("Precision overflow for double exponent.") {
+    private static final long serialVersionUID = 1L;
+
+    @Override
+    public synchronized Throwable fillInStackTrace() {
+      return this;
+    }
+  };
+
+  /**
+   * Preallocated exceptions.
+   */
+  private static final NumberFormatException INVALID_EXPONENT = new NumberFormatException("Invalid exponent") {
+    private static final long serialVersionUID = 1L;
+
+    @Override
+    public synchronized Throwable fillInStackTrace() {
+      return this;
+    }
+  };
+
+  /**
+   * Preallocated exceptions.
+   */
+  private static final NumberFormatException TRAILING_CHARACTERS = new NumberFormatException("String sequence was not completely consumed.") {
+    private static final long serialVersionUID = 1L;
+
+    @Override
+    public synchronized Throwable fillInStackTrace() {
+      return this;
+    }
+  };
+
+  /**
+   * Preallocated exceptions.
+   */
+  private static final NumberFormatException PRECISION_OVERFLOW = new NumberFormatException("Precision overflow for long values.") {
+    private static final long serialVersionUID = 1L;
+
+    @Override
+    public synchronized Throwable fillInStackTrace() {
+      return this;
+    }
+  };
+
+  /**
+   * Preallocated exceptions.
+   */
+  private static final NumberFormatException NOT_A_NUMBER = new NumberFormatException("Number must start with a digit or dot.") {
+    private static final long serialVersionUID = 1L;
+
+    @Override
+    public synchronized Throwable fillInStackTrace() {
+      return this;
+    }
+  };
+
+  /**
+   * Parse a double from a character sequence.
+   * 
+   * In contrast to Javas {@link Double#parseDouble}, this will <em>not</em>
+   * create an object and thus is expected to put less load on the garbage
+   * collector. It will accept some more spellings of NaN and infinity, thus
+   * removing the need for checking for these independently.
+   * 
+   * @param str String
+   * @return Double value
+   */
+  public static double parseDouble(final CharSequence str) {
+    return parseDouble(str, 0, str.length());
+  }
+
+  /**
+   * Parse a double from a character sequence.
+   * 
+   * In contrast to Javas {@link Double#parseDouble}, this will <em>not</em>
+   * create an object and thus is expected to put less load on the garbage
+   * collector. It will accept some more spellings of NaN and infinity, thus
+   * removing the need for checking for these independently.
+   * 
+   * @param str String
+   * @param start Begin
+   * @param end End
+   * @return Double value
+   */
+  public static double parseDouble(final CharSequence str, final int start, final int end) {
+    // Current position and character.
+    int pos = start;
+    char cur = str.charAt(pos);
+
+    // Match for NaN spellings
+    if(matchNaN(str, cur, pos, end)) {
+      return Double.NaN;
+    }
+    // Match sign
+    boolean isNegative = (cur == '-');
+    // Carefully consume the - character, update c and i:
+    if((isNegative || (cur == '+')) && (++pos < end)) {
+      cur = str.charAt(pos);
+    }
+    if(matchInf(str, cur, pos, end)) {
+      return isNegative ? Double.NEGATIVE_INFINITY : Double.POSITIVE_INFINITY;
+    }
+
+    // Begin parsing real numbers!
+    if(((cur < '0') || (cur > '9')) && (cur != '.')) {
+      throw NOT_A_NUMBER;
+    }
+
+    // Parse digits into a long, remember offset of decimal point.
+    long decimal = 0;
+    int decimalPoint = -1;
+    while(true) {
+      final int digit = cur - '0';
+      if((digit >= 0) && (digit <= 9)) {
+        final long tmp = (decimal << 3) + (decimal << 1) + digit;
+        if((decimal > MAX_LONG_OVERFLOW) || (tmp < decimal)) {
+          throw PRECISION_OVERFLOW;
+        }
+        decimal = tmp;
+      }
+      else if((cur == '.') && (decimalPoint < 0)) {
+        decimalPoint = pos;
+      }
+      else { // No more digits, or a second dot.
+        break;
+      }
+      if(++pos < end) {
+        cur = str.charAt(pos);
+      }
+      else {
+        break;
+      }
+    }
+    // We need the offset from the back for adjusting the exponent:
+    // Note that we need the current value of i!
+    decimalPoint = (decimalPoint >= 0) ? pos - decimalPoint - 1 : 0;
+
+    // Reads exponent.
+    int exp = 0;
+    if((pos < end) && ((cur == 'E') || (cur == 'e'))) {
+      cur = str.charAt(++pos);
+      final boolean isNegativeExp = (cur == '-');
+      if((isNegativeExp || (cur == '+')) && (++pos < end)) {
+        cur = str.charAt(pos);
+      }
+      if((cur < '0') || (cur > '9')) { // At least one digit required.
+        throw INVALID_EXPONENT;
+      }
+      while(true) {
+        final int digit = cur - '0';
+        if((digit >= 0) && (digit < 10)) {
+          final int tmp = (exp << 3) + (exp << 1) + digit;
+          // Actually, double can only handle Double.MAX_EXPONENT? How about
+          // subnormal?
+          if((exp > MAX_INT_OVERFLOW) || (tmp < exp)) {
+            throw EXPONENT_OVERFLOW;
+          }
+          exp = tmp;
+        }
+        else {
+          break;
+        }
+        if(++pos < end) {
+          cur = str.charAt(pos);
+        }
+        else {
+          break;
+        }
+      }
+      if(isNegativeExp) {
+        exp = -exp;
+      }
+    }
+    // Adjust exponent by the offset of the dot in our long.
+    if(decimalPoint >= 0) {
+      exp = exp - decimalPoint;
+    }
+    if(pos != end) {
+      throw TRAILING_CHARACTERS;
+    }
+
+    return BitsUtil.lpow10(isNegative ? -decimal : decimal, exp);
+  }
+
+  /**
+   * Match "NaN" in a number of different capitalizations.
+   * 
+   * @param str String to match
+   * @param firstchar First character
+   * @param start Interval begin
+   * @param end Interval end
+   * @return {@code true} when NaN was recognized.
+   */
+  private static boolean matchNaN(CharSequence str, char firstchar, int start, int end) {
+    final int len = end - start;
+    if(len < 2 || len > 3) {
+      return false;
+    }
+    if(firstchar != 'N' && firstchar != 'n') {
+      return false;
+    }
+    final char c1 = str.charAt(start + 1);
+    if(c1 != 'a' && c1 != 'A') {
+      return false;
+    }
+    // Accept just "NA", too:
+    if(len == 2) {
+      return true;
+    }
+    final char c2 = str.charAt(start + 2);
+    if(c2 != 'N' && c2 != 'n') {
+      return false;
+    }
+    return true;
+  }
+
+  /**
+   * Maximum long that we can process without overflowing.
+   */
+  private static final long MAX_LONG_OVERFLOW = Long.MAX_VALUE / 10;
+
+  /**
+   * Maximum integer that we can process without overflowing.
+   */
+  private static final int MAX_INT_OVERFLOW = Integer.MAX_VALUE / 10;
+
+  /**
+   * Infinity pattern, with any capitalization
+   */
+  private static final char[] INFINITY_PATTERN = { //
+  'I', 'n', 'f', 'i', 'n', 'i', 't', 'y', //
+  'i', 'N', 'F', 'I', 'N', 'I', 'T', 'Y' };
+
+  /** Length of pattern */
+  private static final int INFINITY_LENGTH = INFINITY_PATTERN.length >> 1;
+
+  /**
+   * Match "inf", "infinity" in a number of different capitalizations.
+   * 
+   * @param str String to match
+   * @param firstchar First character
+   * @param start Interval begin
+   * @param end Interval end
+   * @return {@code true} when infinity was recognized.
+   */
+  private static boolean matchInf(CharSequence str, char firstchar, int start, int end) {
+    final int len = end - start;
+    // The wonders of unicode. This is more than one byte on UTF-8
+    if(len == 1 && firstchar == '∞') {
+      return true;
+    }
+    if(len != 3 && len != INFINITY_LENGTH) {
+      return false;
+    }
+    // Test beginning: "inf"
+    if(firstchar != 'I' && firstchar != 'i') {
+      return false;
+    }
+    for(int i = 1, j = INFINITY_LENGTH + 1; i < INFINITY_LENGTH; i++, j++) {
+      final char c = str.charAt(start + i);
+      if(c != INFINITY_PATTERN[i] && c != INFINITY_PATTERN[j]) {
+        return false;
+      }
+      if(i == 2 && len == 3) {
+        return true;
+      }
+    }
+    return true;
+  }
+
+  /**
+   * Parse a long integer from a character sequence.
+   * 
+   * @param str String
+   * @param start Begin
+   * @param end End
+   * @return Double value
+   */
+  public static long parseLongBase10(final CharSequence str, final int start, final int end) {
+    // Current position and character.
+    int pos = start;
+    char cur = str.charAt(pos);
+
+    // Match sign
+    boolean isNegative = (cur == '-');
+    // Carefully consume the - character, update c and i:
+    if((isNegative || (cur == '+')) && (++pos < end)) {
+      cur = str.charAt(pos);
+    }
+
+    // Begin parsing real numbers!
+    if((cur < '0') || (cur > '9')) {
+      throw NOT_A_NUMBER;
+    }
+
+    // Parse digits into a long, remember offset of decimal point.
+    long decimal = 0;
+    while(true) {
+      final int digit = cur - '0';
+      if((digit >= 0) && (digit <= 9)) {
+        final long tmp = (decimal << 3) + (decimal << 1) + digit;
+        if(tmp < decimal) {
+          throw PRECISION_OVERFLOW;
+        }
+        decimal = tmp;
+      }
+      else { // No more digits, or a second dot.
+        break;
+      }
+      if(++pos < end) {
+        cur = str.charAt(pos);
+      }
+      else {
+        break;
+      }
+    }
+    if(pos != end) {
+      throw TRAILING_CHARACTERS;
+    }
+
+    return isNegative ? -decimal : decimal;
+  }
 }
