@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.utilities;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -42,7 +42,7 @@ public final class HandlerList<H> {
    * List with registered Handlers. The list is kept in backwards order, that is
    * the later entrys take precedence.
    */
-  private ArrayList<Pair<Class<?>, H>> handlers = new ArrayList<Pair<Class<?>, H>>();
+  private ArrayList<Pair<Class<?>, H>> handlers = new ArrayList<>();
 
   /**
    * Insert a handler to the beginning of the stack.
