@@ -1,18 +1,10 @@
 package de.lmu.ifi.dbs.elki.utilities.datastructures;
 
-import java.util.Comparator;
-import java.util.List;
-
-import de.lmu.ifi.dbs.elki.database.ids.ArrayModifiableDBIDs;
-import de.lmu.ifi.dbs.elki.database.ids.DBID;
-import de.lmu.ifi.dbs.elki.database.ids.DBIDArrayIter;
-import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
-
 /*
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -31,6 +23,14 @@ import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+import java.util.Comparator;
+import java.util.List;
+
+import de.lmu.ifi.dbs.elki.database.ids.ArrayModifiableDBIDs;
+import de.lmu.ifi.dbs.elki.database.ids.DBID;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDArrayIter;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
+
 /**
  * QuickSelect computes ("selects") the element at a given rank and can be used
  * to compute Medians and arbitrary quantiles by computing the appropriate rank.
@@ -43,13 +43,297 @@ import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
  * 
  * @apiviz.uses ArrayModifiableDBIDs
  * @apiviz.uses List
- * @apiviz.uses Comparator
+ * @apiviz.uses Adapter
  */
 public class QuickSelect {
   /**
    * For small arrays, use a simpler method:
    */
-  private static final int SMALL = 10;
+  private static final int SMALL = 47;
+
+  /**
+   * Choose the best pivot for the given rank.
+   * 
+   * @param rank Rank
+   * @param m1 Pivot candidate
+   * @param m2 Pivot candidate
+   * @param m3 Pivot candidate
+   * @param m4 Pivot candidate
+   * @param m5 Pivot candidate
+   * @return Best pivot candidate
+   */
+  private static final int bestPivot(int rank, int m1, int m2, int m3, int m4, int m5) {
+    if (rank < m1) {
+      return m1;
+    }
+    if (rank > m5) {
+      return m5;
+    }
+    if (rank < m2) {
+      return m2;
+    }
+    if (rank > m4) {
+      return m4;
+    }
+    return m3;
+  }
+
+  /**
+   * QuickSelect is essentially quicksort, except that we only "sort" that half
+   * of the array that we are interested in.
+   * 
+   * @param data Data to process
+   * @param start Interval start
+   * @param end Interval end (exclusive)
+   * @param rank rank position we are interested in (starting at 0)
+   */
+  public static <T> void quickSelect(T data, Adapter<T> adapter, int start, int end, int rank) {
+    while (true) {
+      // Optimization for small arrays
+      // This also ensures a minimum size below
+      if (start + SMALL > end) {
+        insertionSort(data, adapter, start, end);
+        return;
+      }
+
+      // Best of 5 pivot picking:
+      // Choose pivots by looking at five candidates.
+      final int len = end - start;
+      final int seventh = (len >> 3) + (len >> 6) + 1;
+      final int m3 = (start + end) >> 1; // middle
+      final int m2 = m3 - seventh;
+      final int m1 = m2 - seventh;
+      final int m4 = m3 + seventh;
+      final int m5 = m4 + seventh;
+
+      // Explicit (and optimal) sorting network for 5 elements
+      // See Knuth for details.
+      if (adapter.compareGreater(data, m1, m2)) {
+        adapter.swap(data, m1, m2);
+      }
+      if (adapter.compareGreater(data, m1, m3)) {
+        adapter.swap(data, m1, m3);
+      }
+      if (adapter.compareGreater(data, m2, m3)) {
+        adapter.swap(data, m2, m3);
+      }
+      if (adapter.compareGreater(data, m4, m5)) {
+        adapter.swap(data, m4, m5);
+      }
+      if (adapter.compareGreater(data, m1, m4)) {
+        adapter.swap(data, m1, m4);
+      }
+      if (adapter.compareGreater(data, m3, m4)) {
+        adapter.swap(data, m3, m4);
+      }
+      if (adapter.compareGreater(data, m2, m5)) {
+        adapter.swap(data, m2, m5);
+      }
+      if (adapter.compareGreater(data, m2, m3)) {
+        adapter.swap(data, m2, m3);
+      }
+      if (adapter.compareGreater(data, m4, m5)) {
+        adapter.swap(data, m4, m5);
+      }
+
+      int best = bestPivot(rank, m1, m2, m3, m4, m5);
+      // final double pivot = data[best];
+      // Move middle element out of the way.
+      adapter.swap(data, best, end - 1);
+
+      // Begin partitioning
+      int i = start, j = end - 2;
+      // This is classic quicksort stuff
+      while (true) {
+        while (i <= j && adapter.compareGreater(data, end - 1, i)) {
+          i++;
+        }
+        while (j >= i && !adapter.compareGreater(data, end - 1, j)) {
+          j--;
+        }
+        if (i >= j) {
+          break;
+        }
+        adapter.swap(data, i, j);
+      }
+
+      // Move pivot (former middle element) back into the appropriate place
+      adapter.swap(data, i, end - 1);
+
+      // In contrast to quicksort, we only need to recurse into the half we are
+      // interested in. Instead of recursion we now use iteration.
+      if (rank < i) {
+        end = i;
+      } else if (rank > i) {
+        start = i + 1;
+      } else {
+        break;
+      }
+    } // Loop until rank==i
+  }
+
+  /**
+   * Sort a small array using repetitive insertion sort.
+   * 
+   * @param data Data to sort
+   * @param start Interval start
+   * @param end Interval end
+   */
+  private static <T> void insertionSort(T data, Adapter<T> adapter, int start, int end) {
+    for (int i = start + 1; i < end; i++) {
+      for (int j = i; j > start && adapter.compareGreater(data, j - 1, j); j--) {
+        adapter.swap(data, j, j - 1);
+      }
+    }
+  }
+
+  /**
+   * Adapter class to apply QuickSelect to arbitrary data structures.
+   * 
+   * @author Erich Schubert
+   * 
+   * @param <T> Data structure type
+   */
+  public static interface Adapter<T> {
+    /**
+     * Swap the two elements at positions i and j.
+     * 
+     * @param data Data structure
+     * @param i Position i
+     * @param j Position j
+     */
+    void swap(T data, int i, int j);
+
+    /**
+     * Compare two elements.
+     * 
+     * @param data Data structure
+     * @param i Position i
+     * @param j Position j
+     * @return {@code true} when the element at position i is greater than that
+     *         at position j.
+     */
+    boolean compareGreater(T data, int i, int j);
+  }
+
+  /**
+   * Adapter for double arrays.
+   */
+  public static Adapter<double[]> DOUBLE_ADAPTER = new Adapter<double[]>() {
+    @Override
+    public void swap(double[] data, int i, int j) {
+      double tmp = data[i];
+      data[i] = data[j];
+      data[j] = tmp;
+    }
+
+    @Override
+    public boolean compareGreater(double[] data, int i, int j) {
+      return data[i] > data[j];
+    }
+  };
+
+  /**
+   * Adapter for integer arrays.
+   */
+  public static Adapter<int[]> INTEGER_ADAPTER = new Adapter<int[]>() {
+    @Override
+    public void swap(int[] data, int i, int j) {
+      int tmp = data[i];
+      data[i] = data[j];
+      data[j] = tmp;
+    }
+
+    @Override
+    public boolean compareGreater(int[] data, int i, int j) {
+      return data[i] > data[j];
+    }
+  };
+
+  /**
+   * Adapter for float arrays.
+   */
+  public static Adapter<float[]> FLOAT_ADAPTER = new Adapter<float[]>() {
+    @Override
+    public void swap(float[] data, int i, int j) {
+      float tmp = data[i];
+      data[i] = data[j];
+      data[j] = tmp;
+    }
+
+    @Override
+    public boolean compareGreater(float[] data, int i, int j) {
+      return data[i] > data[j];
+    }
+  };
+
+  /**
+   * Adapter for short arrays.
+   */
+  public static Adapter<short[]> SHORT_ADAPTER = new Adapter<short[]>() {
+    @Override
+    public void swap(short[] data, int i, int j) {
+      short tmp = data[i];
+      data[i] = data[j];
+      data[j] = tmp;
+    }
+
+    @Override
+    public boolean compareGreater(short[] data, int i, int j) {
+      return data[i] > data[j];
+    }
+  };
+
+  /**
+   * Adapter for long arrays.
+   */
+  public static Adapter<long[]> LONG_ADAPTER = new Adapter<long[]>() {
+    @Override
+    public void swap(long[] data, int i, int j) {
+      long tmp = data[i];
+      data[i] = data[j];
+      data[j] = tmp;
+    }
+
+    @Override
+    public boolean compareGreater(long[] data, int i, int j) {
+      return data[i] > data[j];
+    }
+  };
+
+  /**
+   * Adapter for byte arrays.
+   */
+  public static Adapter<byte[]> BYTE_ADAPTER = new Adapter<byte[]>() {
+    @Override
+    public void swap(byte[] data, int i, int j) {
+      byte tmp = data[i];
+      data[i] = data[j];
+      data[j] = tmp;
+    }
+
+    @Override
+    public boolean compareGreater(byte[] data, int i, int j) {
+      return data[i] > data[j];
+    }
+  };
+
+  /**
+   * Adapter for char arrays.
+   */
+  public static Adapter<char[]> CHAR_ADAPTER = new Adapter<char[]>() {
+    @Override
+    public void swap(char[] data, int i, int j) {
+      char tmp = data[i];
+      data[i] = data[j];
+      data[j] = tmp;
+    }
+
+    @Override
+    public boolean compareGreater(char[] data, int i, int j) {
+      return data[i] > data[j];
+    }
+  };
 
   /**
    * QuickSelect is essentially quicksort, except that we only "sort" that half
@@ -94,11 +378,10 @@ public class QuickSelect {
     // Integer division is "floor" since we are non-negative.
     final int left = begin + ((length - 1) >> 1);
     quickSelect(data, begin, end, left);
-    if(length % 2 == 1) {
+    if (length % 2 == 1) {
       return data[left];
-    }
-    else {
-      quickSelect(data, begin, end, left + 1);
+    } else {
+      quickSelect(data, left + 1, end, left + 1);
       return data[left] + .5 * (data[left + 1] - data[left]);
     }
   }
@@ -136,11 +419,10 @@ public class QuickSelect {
     final double err = dleft - ileft;
 
     quickSelect(data, begin, end, ileft);
-    if(err <= Double.MIN_NORMAL) {
+    if (err <= Double.MIN_NORMAL) {
       return data[ileft];
-    }
-    else {
-      quickSelect(data, begin, end, ileft + 1);
+    } else {
+      quickSelect(data, ileft + 1, end, ileft + 1);
       // Mix:
       double mix = data[ileft] + (data[ileft + 1] - data[ileft]) * err;
       return mix;
@@ -155,66 +437,94 @@ public class QuickSelect {
    * @param start Interval start
    * @param end Interval end (exclusive)
    * @param rank rank position we are interested in (starting at 0)
+   * @return Element at the given rank (starting at 0).
    */
-  public static void quickSelect(double[] data, int start, int end, int rank) {
-    while(true) {
+  public static double quickSelect(double[] data, int start, int end, int rank) {
+    while (true) {
       // Optimization for small arrays
       // This also ensures a minimum size below
-      if(start + SMALL > end) {
+      if (start + SMALL > end) {
         insertionSort(data, start, end);
-        return;
+        return data[rank];
       }
 
-      // Pick pivot from three candidates: start, middle, end
-      // Since we compare them, we can also just "bubble sort" them.
-      final int middle = (start + end) >> 1;
-      if(data[start] > data[middle]) {
-        swap(data, start, middle);
-      }
-      if(data[start] > data[end - 1]) {
-        swap(data, start, end - 1);
-      }
-      if(data[middle] > data[end - 1]) {
-        swap(data, middle, end - 1);
-      }
-      // TODO: use more candidates for larger arrays?
+      // Best of 5 pivot picking:
+      // Choose pivots by looking at five candidates.
+      final int len = end - start;
+      final int seventh = (len >> 3) + (len >> 6) + 1;
+      final int m3 = (start + end) >> 1; // middle
+      final int m2 = m3 - seventh;
+      final int m1 = m2 - seventh;
+      final int m4 = m3 + seventh;
+      final int m5 = m4 + seventh;
 
-      final double pivot = data[middle];
-      // Move middle element out of the way, just before end
-      // (Since we already know that "end" is bigger)
-      swap(data, middle, end - 2);
+      // Explicit (and optimal) sorting network for 5 elements
+      // See Knuth for details.
+      if (data[m1] > data[m2]) {
+        swap(data, m1, m2);
+      }
+      if (data[m1] > data[m3]) {
+        swap(data, m1, m3);
+      }
+      if (data[m2] > data[m3]) {
+        swap(data, m2, m3);
+      }
+      if (data[m4] > data[m5]) {
+        swap(data, m4, m5);
+      }
+      if (data[m1] > data[m4]) {
+        swap(data, m1, m4);
+      }
+      if (data[m3] > data[m4]) {
+        swap(data, m3, m4);
+      }
+      if (data[m2] > data[m5]) {
+        swap(data, m2, m5);
+      }
+      if (data[m2] > data[m3]) {
+        swap(data, m2, m3);
+      }
+      if (data[m4] > data[m5]) {
+        swap(data, m4, m5);
+      }
+
+      int best = bestPivot(rank, m1, m2, m3, m4, m5);
+      final double pivot = data[best];
+      // Move middle element out of the way.
+      swap(data, best, end - 1);
 
       // Begin partitioning
-      int i = start + 1, j = end - 3;
+      int i = start, j = end - 2;
       // This is classic quicksort stuff
-      while(true) {
-        while(data[i] <= pivot && i <= j) {
+      while (true) {
+        while (i <= j && data[i] <= pivot) {
           i++;
         }
-        while(data[j] >= pivot && j >= i) {
+        while (j >= i && data[j] >= pivot) {
           j--;
         }
-        if(i >= j) {
+        if (i >= j) {
           break;
         }
         swap(data, i, j);
+        i++;
+        j--;
       }
 
       // Move pivot (former middle element) back into the appropriate place
-      swap(data, i, end - 2);
+      swap(data, i, end - 1);
 
       // In contrast to quicksort, we only need to recurse into the half we are
       // interested in. Instead of recursion we now use iteration.
-      if(rank < i) {
+      if (rank < i) {
         end = i;
-      }
-      else if(rank > i) {
+      } else if (rank > i) {
         start = i + 1;
-      }
-      else {
+      } else {
         break;
       }
     } // Loop until rank==i
+    return data[rank];
   }
 
   /**
@@ -225,8 +535,8 @@ public class QuickSelect {
    * @param end Interval end
    */
   private static void insertionSort(double[] data, int start, int end) {
-    for(int i = start + 1; i < end; i++) {
-      for(int j = i; j > start && data[j - 1] > data[j]; j--) {
+    for (int i = start + 1; i < end; i++) {
+      for (int j = i; j > start && data[j - 1] > data[j]; j--) {
         swap(data, j, j - 1);
       }
     }
@@ -345,61 +655,85 @@ public class QuickSelect {
    * @param rank rank position we are interested in (starting at 0)
    */
   public static <T extends Comparable<? super T>> void quickSelect(T[] data, int start, int end, int rank) {
-    while(true) {
+    while (true) {
       // Optimization for small arrays
       // This also ensures a minimum size below
-      if(start + SMALL > end) {
+      if (start + SMALL > end) {
         insertionSort(data, start, end);
         return;
       }
 
-      // Pick pivot from three candidates: start, middle, end
-      // Since we compare them, we can also just "bubble sort" them.
-      final int middle = (start + end) >> 1;
-      if(data[start].compareTo(data[middle]) > 0) {
-        swap(data, start, middle);
-      }
-      if(data[start].compareTo(data[end - 1]) > 0) {
-        swap(data, start, end - 1);
-      }
-      if(data[middle].compareTo(data[end - 1]) > 0) {
-        swap(data, middle, end - 1);
-      }
-      // TODO: use more candidates for larger arrays?
+      // Best of 5 pivot picking:
+      // Choose pivots by looking at five candidates.
+      final int len = end - start;
+      final int seventh = (len >> 3) + (len >> 6) + 1;
+      final int m3 = (start + end) >> 1; // middle
+      final int m2 = m3 - seventh;
+      final int m1 = m2 - seventh;
+      final int m4 = m3 + seventh;
+      final int m5 = m4 + seventh;
 
-      final T pivot = data[middle];
-      // Move middle element out of the way, just before end
-      // (Since we already know that "end" is bigger)
-      swap(data, middle, end - 2);
+      // Explicit (and optimal) sorting network for 5 elements
+      // See Knuth for details.
+      if (data[m1].compareTo(data[m2]) > 0) {
+        swap(data, m1, m2);
+      }
+      if (data[m1].compareTo(data[m3]) > 0) {
+        swap(data, m1, m3);
+      }
+      if (data[m2].compareTo(data[m3]) > 0) {
+        swap(data, m2, m3);
+      }
+      if (data[m4].compareTo(data[m5]) > 0) {
+        swap(data, m4, m5);
+      }
+      if (data[m1].compareTo(data[m4]) > 0) {
+        swap(data, m1, m4);
+      }
+      if (data[m3].compareTo(data[m4]) > 0) {
+        swap(data, m3, m4);
+      }
+      if (data[m2].compareTo(data[m5]) > 0) {
+        swap(data, m2, m5);
+      }
+      if (data[m2].compareTo(data[m3]) > 0) {
+        swap(data, m2, m3);
+      }
+      if (data[m4].compareTo(data[m5]) > 0) {
+        swap(data, m4, m5);
+      }
+
+      int best = bestPivot(rank, m1, m2, m3, m4, m5);
+      final T pivot = data[best];
+      // Move middle element out of the way.
+      swap(data, best, end - 1);
 
       // Begin partitioning
-      int i = start + 1, j = end - 3;
+      int i = start, j = end - 2;
       // This is classic quicksort stuff
-      while(true) {
-        while(data[i].compareTo(pivot) <= 0 && i <= j) {
+      while (true) {
+        while (i <= j && data[i].compareTo(pivot) <= 0) {
           i++;
         }
-        while(data[j].compareTo(pivot) >= 0 && j >= i) {
+        while (j >= i && data[j].compareTo(pivot) >= 0) {
           j--;
         }
-        if(i >= j) {
+        if (i >= j) {
           break;
         }
         swap(data, i, j);
       }
 
       // Move pivot (former middle element) back into the appropriate place
-      swap(data, i, end - 2);
+      swap(data, i, end - 1);
 
       // In contrast to quicksort, we only need to recurse into the half we are
       // interested in. Instead of recursion we now use iteration.
-      if(rank < i) {
+      if (rank < i) {
         end = i;
-      }
-      else if(rank > i) {
+      } else if (rank > i) {
         start = i + 1;
-      }
-      else {
+      } else {
         break;
       }
     } // Loop until rank==i
@@ -414,8 +748,8 @@ public class QuickSelect {
    * @param end Interval end
    */
   private static <T extends Comparable<? super T>> void insertionSort(T[] data, int start, int end) {
-    for(int i = start + 1; i < end; i++) {
-      for(int j = i; j > start && data[j - 1].compareTo(data[j]) > 0; j--) {
+    for (int i = start + 1; i < end; i++) {
+      for (int j = i; j > start && data[j - 1].compareTo(data[j]) > 0; j--) {
         swap(data, j, j - 1);
       }
     }
@@ -536,61 +870,86 @@ public class QuickSelect {
    * @param rank rank position we are interested in (starting at 0)
    */
   public static <T extends Comparable<? super T>> void quickSelect(List<? extends T> data, int start, int end, int rank) {
-    while(true) {
+    while (true) {
       // Optimization for small arrays
       // This also ensures a minimum size below
-      if(start + SMALL > end) {
+      if (start + SMALL > end) {
         insertionSort(data, start, end);
         return;
       }
 
-      // Pick pivot from three candidates: start, middle, end
-      // Since we compare them, we can also just "bubble sort" them.
-      final int middle = (start + end) >> 1;
-      if(data.get(start).compareTo(data.get(middle)) > 0) {
-        swap(data, start, middle);
-      }
-      if(data.get(start).compareTo(data.get(end - 1)) > 0) {
-        swap(data, start, end - 1);
-      }
-      if(data.get(middle).compareTo(data.get(end - 1)) > 0) {
-        swap(data, middle, end - 1);
-      }
-      // TODO: use more candidates for larger arrays?
+      // Best of 5 pivot picking:
+      // Choose pivots by looking at five candidates.
+      final int len = end - start;
+      final int seventh = (len >> 3) + (len >> 6) + 1;
+      final int m3 = (start + end) >> 1; // middle
+      final int m2 = m3 - seventh;
+      final int m1 = m2 - seventh;
+      final int m4 = m3 + seventh;
+      final int m5 = m4 + seventh;
 
-      final T pivot = data.get(middle);
+      // Explicit (and optimal) sorting network for 5 elements
+      // See Knuth for details.
+      if (data.get(m1).compareTo(data.get(m2)) > 0) {
+        swap(data, m1, m2);
+      }
+      if (data.get(m1).compareTo(data.get(m3)) > 0) {
+        swap(data, m1, m3);
+      }
+      if (data.get(m2).compareTo(data.get(m3)) > 0) {
+        swap(data, m2, m3);
+      }
+      if (data.get(m4).compareTo(data.get(m5)) > 0) {
+        swap(data, m4, m5);
+      }
+      if (data.get(m1).compareTo(data.get(m4)) > 0) {
+        swap(data, m1, m4);
+      }
+      if (data.get(m3).compareTo(data.get(m4)) > 0) {
+        swap(data, m3, m4);
+      }
+      if (data.get(m2).compareTo(data.get(m5)) > 0) {
+        swap(data, m2, m5);
+      }
+      if (data.get(m2).compareTo(data.get(m3)) > 0) {
+        swap(data, m2, m3);
+      }
+      if (data.get(m4).compareTo(data.get(m5)) > 0) {
+        swap(data, m4, m5);
+      }
+
+      int best = bestPivot(rank, m1, m2, m3, m4, m5);
+      final T pivot = data.get(best);
       // Move middle element out of the way, just before end
       // (Since we already know that "end" is bigger)
-      swap(data, middle, end - 2);
+      swap(data, best, end - 1);
 
       // Begin partitioning
-      int i = start + 1, j = end - 3;
+      int i = start, j = end - 2;
       // This is classic quicksort stuff
-      while(true) {
-        while(data.get(i).compareTo(pivot) <= 0 && i <= j) {
+      while (true) {
+        while (i <= j && data.get(i).compareTo(pivot) <= 0) {
           i++;
         }
-        while(data.get(j).compareTo(pivot) >= 0 && j >= i) {
+        while (j >= i && data.get(j).compareTo(pivot) >= 0) {
           j--;
         }
-        if(i >= j) {
+        if (i >= j) {
           break;
         }
         swap(data, i, j);
       }
 
       // Move pivot (former middle element) back into the appropriate place
-      swap(data, i, end - 2);
+      swap(data, i, end - 1);
 
       // In contrast to quicksort, we only need to recurse into the half we are
       // interested in. Instead of recursion we now use iteration.
-      if(rank < i) {
+      if (rank < i) {
         end = i;
-      }
-      else if(rank > i) {
+      } else if (rank > i) {
         start = i + 1;
-      }
-      else {
+      } else {
         break;
       }
     } // Loop until rank==i
@@ -605,8 +964,8 @@ public class QuickSelect {
    * @param end Interval end
    */
   private static <T extends Comparable<? super T>> void insertionSort(List<T> data, int start, int end) {
-    for(int i = start + 1; i < end; i++) {
-      for(int j = i; j > start && data.get(j - 1).compareTo(data.get(j)) > 0; j--) {
+    for (int i = start + 1; i < end; i++) {
+      for (int j = i; j > start && data.get(j - 1).compareTo(data.get(j)) > 0; j--) {
         swap(data, j, j - 1);
       }
     }
@@ -731,61 +1090,86 @@ public class QuickSelect {
    * @param rank rank position we are interested in (starting at 0)
    */
   public static <T> void quickSelect(List<? extends T> data, Comparator<? super T> comparator, int start, int end, int rank) {
-    while(true) {
+    while (true) {
       // Optimization for small arrays
       // This also ensures a minimum size below
-      if(start + SMALL > end) {
+      if (start + SMALL > end) {
         insertionSort(data, comparator, start, end);
         return;
       }
 
-      // Pick pivot from three candidates: start, middle, end
-      // Since we compare them, we can also just "bubble sort" them.
-      final int middle = (start + end) >> 1;
-      if(comparator.compare(data.get(start), data.get(middle)) > 0) {
-        swap(data, start, middle);
-      }
-      if(comparator.compare(data.get(start), data.get(end - 1)) > 0) {
-        swap(data, start, end - 1);
-      }
-      if(comparator.compare(data.get(middle), data.get(end - 1)) > 0) {
-        swap(data, middle, end - 1);
-      }
-      // TODO: use more candidates for larger arrays?
+      // Best of 5 pivot picking:
+      // Choose pivots by looking at five candidates.
+      final int len = end - start;
+      final int seventh = (len >> 3) + (len >> 6) + 1;
+      final int m3 = (start + end) >> 1; // middle
+      final int m2 = m3 - seventh;
+      final int m1 = m2 - seventh;
+      final int m4 = m3 + seventh;
+      final int m5 = m4 + seventh;
 
-      final T pivot = data.get(middle);
+      // Explicit (and optimal) sorting network for 5 elements
+      // See Knuth for details.
+      if (comparator.compare(data.get(m1), data.get(m2)) > 0) {
+        swap(data, m1, m2);
+      }
+      if (comparator.compare(data.get(m1), data.get(m3)) > 0) {
+        swap(data, m1, m3);
+      }
+      if (comparator.compare(data.get(m2), data.get(m3)) > 0) {
+        swap(data, m2, m3);
+      }
+      if (comparator.compare(data.get(m4), data.get(m5)) > 0) {
+        swap(data, m4, m5);
+      }
+      if (comparator.compare(data.get(m1), data.get(m4)) > 0) {
+        swap(data, m1, m4);
+      }
+      if (comparator.compare(data.get(m3), data.get(m4)) > 0) {
+        swap(data, m3, m4);
+      }
+      if (comparator.compare(data.get(m2), data.get(m5)) > 0) {
+        swap(data, m2, m5);
+      }
+      if (comparator.compare(data.get(m2), data.get(m3)) > 0) {
+        swap(data, m2, m3);
+      }
+      if (comparator.compare(data.get(m4), data.get(m5)) > 0) {
+        swap(data, m4, m5);
+      }
+
+      int best = bestPivot(rank, m1, m2, m3, m4, m5);
+      final T pivot = data.get(best);
       // Move middle element out of the way, just before end
       // (Since we already know that "end" is bigger)
-      swap(data, middle, end - 2);
+      swap(data, best, end - 1);
 
       // Begin partitioning
-      int i = start + 1, j = end - 3;
+      int i = start, j = end - 2;
       // This is classic quicksort stuff
-      while(true) {
-        while(comparator.compare(data.get(i), pivot) <= 0 && i <= j) {
+      while (true) {
+        while (i <= j && comparator.compare(data.get(i), pivot) <= 0) {
           i++;
         }
-        while(comparator.compare(data.get(j), pivot) >= 0 && j >= i) {
+        while (j >= i && comparator.compare(data.get(j), pivot) >= 0) {
           j--;
         }
-        if(i >= j) {
+        if (i >= j) {
           break;
         }
         swap(data, i, j);
       }
 
       // Move pivot (former middle element) back into the appropriate place
-      swap(data, i, end - 2);
+      swap(data, i, end - 1);
 
       // In contrast to quicksort, we only need to recurse into the half we are
       // interested in. Instead of recursion we now use iteration.
-      if(rank < i) {
+      if (rank < i) {
         end = i;
-      }
-      else if(rank > i) {
+      } else if (rank > i) {
         start = i + 1;
-      }
-      else {
+      } else {
         break;
       }
     } // Loop until rank==i
@@ -800,8 +1184,8 @@ public class QuickSelect {
    * @param end Interval end
    */
   private static <T> void insertionSort(List<T> data, Comparator<? super T> comparator, int start, int end) {
-    for(int i = start + 1; i < end; i++) {
-      for(int j = i; j > start && comparator.compare(data.get(j - 1), data.get(j)) > 0; j--) {
+    for (int i = start + 1; i < end; i++) {
+      for (int j = i; j > start && comparator.compare(data.get(j - 1), data.get(j)) > 0; j--) {
         swap(data, j, j - 1);
       }
     }
@@ -882,7 +1266,7 @@ public class QuickSelect {
    * @param data Data to process
    * @param comparator Comparator to use
    * @param begin Begin of valid values
-   * @param end End of valid values (inclusive!)
+   * @param end End of valid values (exclusive)
    * @param quant Quantile to compute
    * @return Value at quantile
    */
@@ -904,75 +1288,115 @@ public class QuickSelect {
    * @param data Data to process
    * @param comparator Comparator to use
    * @param start Interval start
-   * @param end Interval end (inclusive)
+   * @param end Interval end (exclusive)
    * @param rank rank position we are interested in (starting at 0)
    */
   public static void quickSelect(ArrayModifiableDBIDs data, Comparator<? super DBIDRef> comparator, int start, int end, int rank) {
-    while(true) {
+    DBIDArrayIter refi = data.iter(), refj = data.iter(), pivot = data.iter();
+    while (true) {
       // Optimization for small arrays
       // This also ensures a minimum size below
-      if(start + SMALL > end) {
-        insertionSort(data, comparator, start, end);
+      if (start + SMALL > end) {
+        insertionSort(data, comparator, start, end, refi, refj);
         return;
       }
 
-      // Pick pivot from three candidates: start, middle, end
-      // Since we compare them, we can also just "bubble sort" them.
-      final int middle = (start + end) >> 1;
-      if(comparator.compare(data.get(start), data.get(middle)) > 0) {
-        data.swap(start, middle);
-      }
-      if(comparator.compare(data.get(start), data.get(end - 1)) > 0) {
-        data.swap(start, end - 1);
-      }
-      if(comparator.compare(data.get(middle), data.get(end - 1)) > 0) {
-        data.swap(middle, end - 1);
-      }
-      // TODO: use more candidates for larger arrays?
+      // Best of 5 pivot picking:
+      // Choose pivots by looking at five candidates.
+      final int len = end - start;
+      final int seventh = (len >> 3) + (len >> 6) + 1;
+      final int m3 = (start + end) >> 1; // middle
+      final int m2 = m3 - seventh;
+      final int m1 = m2 - seventh;
+      final int m4 = m3 + seventh;
+      final int m5 = m4 + seventh;
 
-      final DBID pivot = data.get(middle);
-      // Move middle element out of the way, just before end
-      // (Since we already know that "end" is bigger)
-      data.swap(middle, end - 2);
+      // Explicit (and optimal) sorting network for 5 elements
+      // See Knuth for details.
+      if (compare(refi, m1, refj, m2, comparator) > 0) {
+        data.swap(m1, m2);
+      }
+      if (compare(refi, m1, refj, m3, comparator) > 0) {
+        data.swap(m1, m3);
+      }
+      if (compare(refi, m2, refj, m3, comparator) > 0) {
+        data.swap(m2, m3);
+      }
+      if (compare(refi, m4, refj, m5, comparator) > 0) {
+        data.swap(m4, m5);
+      }
+      if (compare(refi, m1, refj, m4, comparator) > 0) {
+        data.swap(m1, m4);
+      }
+      if (compare(refi, m3, refj, m4, comparator) > 0) {
+        data.swap(m3, m4);
+      }
+      if (compare(refi, m2, refj, m5, comparator) > 0) {
+        data.swap(m2, m5);
+      }
+      if (compare(refi, m2, refj, m3, comparator) > 0) {
+        data.swap(m2, m3);
+      }
+      if (compare(refi, m4, refj, m5, comparator) > 0) {
+        data.swap(m4, m5);
+      }
+
+      int best = bestPivot(rank, m1, m2, m3, m4, m5);
+      // Move middle element out of the way.
+      data.swap(best, end - 1);
+      pivot.seek(end - 1);
 
       // Begin partitioning
-      int i = start + 1, j = end - 3;
-      DBIDArrayIter refi = data.iter(), refj = data.iter();
+      int i = start, j = end - 2;
       refi.seek(i);
       refj.seek(j);
       // This is classic quicksort stuff
-      while(true) {
-        while(comparator.compare(refi, pivot) <= 0 && i <= j) {
+      while (true) {
+        while (i <= j && comparator.compare(refi, pivot) <= 0) {
           i++;
           refi.advance();
         }
-        while(comparator.compare(refj, pivot) >= 0 && j >= i) {
+        while (j >= i && comparator.compare(refj, pivot) >= 0) {
           j--;
           refj.retract();
         }
-        if(i >= j) {
+        if (i >= j) {
           break;
         }
         data.swap(i, j);
       }
 
       // Move pivot (former middle element) back into the appropriate place
-      data.swap(i, end - 2);
+      data.swap(i, end - 1);
 
       // In contrast to quicksort, we only need to recurse into the half we are
       // interested in. Instead of recursion we now use iteration.
-      if(rank < i) {
+      if (rank < i) {
         end = i;
-      }
-      else if(rank > i) {
+      } else if (rank > i) {
         start = i + 1;
-      }
-      else {
+      } else {
         break;
       }
     } // Loop until rank==i
   }
 
+  /**
+   * Compare two elements.
+   * 
+   * @param i1 First scratch variable
+   * @param p1 Value for first
+   * @param i2 Second scratch variable
+   * @param p2 Value for second
+   * @param comp Comparator
+   * @return Comparison result
+   */
+  private static int compare(DBIDArrayIter i1, int p1, DBIDArrayIter i2, int p2, Comparator<? super DBIDRef> comp) {
+    i1.seek(p1);
+    i2.seek(p2);
+    return comp.compare(i1, i2);
+  }
+
   /**
    * Sort a small array using repetitive insertion sort.
    * 
@@ -980,17 +1404,16 @@ public class QuickSelect {
    * @param start Interval start
    * @param end Interval end
    */
-  private static void insertionSort(ArrayModifiableDBIDs data, Comparator<? super DBIDRef> comparator, int start, int end) {
-    DBIDArrayIter iter1 = data.iter(), iter2 = data.iter();
-    for(int i = start + 1; i < end; i++) {
-      iter1.seek(i - 1);
-      iter2.seek(i);
-      for(int j = i; j > start; j--, iter1.retract(), iter2.retract()) {
-        if(comparator.compare(iter1, iter2) > 0) {
+  private static void insertionSort(ArrayModifiableDBIDs data, Comparator<? super DBIDRef> comparator, int start, int end, DBIDArrayIter iter1, DBIDArrayIter iter2) {
+    for (int i = start + 1; i < end; i++) {
+      for (int j = i; j > start; j--) {
+        iter1.seek(j - 1);
+        iter2.seek(j);
+        if (comparator.compare(iter1, iter2) <= 0) {
           break;
         }
         data.swap(j, j - 1);
       }
     }
   }
-}
\ No newline at end of file
+}
