@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.utilities.datastructures.arraylike;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -27,6 +27,7 @@ import java.util.List;
 
 import de.lmu.ifi.dbs.elki.data.FeatureVector;
 import de.lmu.ifi.dbs.elki.data.NumberVector;
+import de.lmu.ifi.dbs.elki.math.linearalgebra.Vector;
 
 /**
  * Utility class that allows plug-in use of various "array-like" types such as
@@ -42,17 +43,17 @@ public final class ArrayLikeUtil {
   /**
    * Static instance for lists.
    */
-  private static final ListArrayAdapter<Object> LISTADAPTER = new ListArrayAdapter<Object>();
+  private static final ListArrayAdapter<Object> LISTADAPTER = new ListArrayAdapter<>();
 
   /**
    * Static instance for lists of numbers.
    */
-  private static final NumberListArrayAdapter<Number> NUMBERLISTADAPTER = new NumberListArrayAdapter<Number>();
+  private static final NumberListArrayAdapter<Number> NUMBERLISTADAPTER = new NumberListArrayAdapter<>();
 
   /**
    * Static instance.
    */
-  private static final IdentityArrayAdapter<?> IDENTITYADAPTER = new IdentityArrayAdapter<Object>();
+  private static final IdentityArrayAdapter<?> IDENTITYADAPTER = new IdentityArrayAdapter<>();
 
   /**
    * Static instance.
@@ -64,6 +65,11 @@ public final class ArrayLikeUtil {
    */
   public static final NumberVectorAdapter<?> NUMBERVECTORADAPTER = new NumberVectorAdapter<Double>();
 
+  /**
+   * Adapter for matrixes, reinterpreted as flat arrays.
+   */
+  public static final FlatMatrixAdapter FLATMATRIXADAPTER = new FlatMatrixAdapter();
+
   /**
    * Use a double array in the array API.
    */
@@ -83,7 +89,12 @@ public final class ArrayLikeUtil {
    * Use ArrayDBIDs as array.
    */
   public static final ArrayDBIDsAdapter ARRAYDBIDADAPTER = new ArrayDBIDsAdapter();
-  
+
+  /**
+   * Adapter for vectors.
+   */
+  public static final NumberArrayAdapter<Double, Vector> VECTORADAPTER = new VectorAdapter();
+
   /**
    * Fake constructor. Do not instantiate!
    */
@@ -169,9 +180,9 @@ public final class ArrayLikeUtil {
     final int size = adapter.size(array);
     int index = 0;
     double max = adapter.getDouble(array, 0);
-    for(int i = 1; i < size; i++) {
+    for (int i = 1; i < size; i++) {
       double val = adapter.getDouble(array, i);
-      if(val > max) {
+      if (val > max) {
         max = val;
         index = i;
       }
@@ -199,8 +210,11 @@ public final class ArrayLikeUtil {
    * @return primitive double array
    */
   public static <A> double[] toPrimitiveDoubleArray(A array, NumberArrayAdapter<?, ? super A> adapter) {
+    if (adapter == DOUBLEARRAYADAPTER) {
+      return ((double[]) array).clone();
+    }
     double[] ret = new double[adapter.size(array)];
-    for(int i = 0; i < ret.length; i++) {
+    for (int i = 0; i < ret.length; i++) {
       ret[i] = adapter.getDouble(array, i);
     }
     return ret;
@@ -234,8 +248,11 @@ public final class ArrayLikeUtil {
    * @return primitive float array
    */
   public static <A> float[] toPrimitiveFloatArray(A array, NumberArrayAdapter<?, ? super A> adapter) {
+    if (adapter == FLOATARRAYADAPTER) {
+      return ((float[]) array).clone();
+    }
     float[] ret = new float[adapter.size(array)];
-    for(int i = 0; i < ret.length; i++) {
+    for (int i = 0; i < ret.length; i++) {
       ret[i] = adapter.getFloat(array, i);
     }
     return ret;
@@ -260,4 +277,39 @@ public final class ArrayLikeUtil {
   public static <N extends Number> float[] toPrimitiveFloatArray(NumberVector<N> obj) {
     return toPrimitiveFloatArray(obj, numberVectorAdapter(obj));
   }
-}
\ No newline at end of file
+
+  /**
+   * Convert a numeric array-like to a <code>int[]</code>.
+   * 
+   * @param array Array-like
+   * @param adapter Adapter
+   * @return primitive double array
+   */
+  public static <A> int[] toPrimitiveIntegerArray(A array, NumberArrayAdapter<?, ? super A> adapter) {
+    int[] ret = new int[adapter.size(array)];
+    for (int i = 0; i < ret.length; i++) {
+      ret[i] = adapter.getInteger(array, i);
+    }
+    return ret;
+  }
+
+  /**
+   * Convert a list of numbers to <code>int[]</code>.
+   * 
+   * @param array List of numbers
+   * @return double array
+   */
+  public static int[] toPrimitiveIntegerArray(List<? extends Number> array) {
+    return toPrimitiveIntegerArray(array, NUMBERLISTADAPTER);
+  }
+
+  /**
+   * Convert a number vector to <code>int[]</code>.
+   * 
+   * @param obj Object to convert
+   * @return primitive double array
+   */
+  public static <N extends Number> int[] toPrimitiveIntegerArray(NumberVector<N> obj) {
+    return toPrimitiveIntegerArray(obj, numberVectorAdapter(obj));
+  }
+}
