@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.utilities.datastructures.arraylike;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -91,6 +91,6 @@ public class ExtendedArray<T> implements ArrayAdapter<T, ExtendedArray<T>> {
    */
   @SuppressWarnings("unchecked")
   public static <T, A> ExtendedArray<T> extend(A array, ArrayAdapter<T, A> getter, T extra) {
-    return new ExtendedArray<T>(array, (ArrayAdapter<T, Object>) getter, extra);
+    return new ExtendedArray<>(array, (ArrayAdapter<T, Object>) getter, extra);
   }
 }
\ No newline at end of file
