@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.utilities.ensemble;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -39,4 +39,14 @@ public interface EnsembleVoting extends Parameterizable {
    * @return combined score.
    */
   public double combine(double[] scores);
+
+  /**
+   * Combine scores function. Note: it is assumed that the scores are
+   * comparable.
+   * 
+   * @param scores Scores to combine
+   * @param count Number of entries to use.
+   * @return combined score.
+   */
+  public double combine(double[] scores, int count);
 }
