@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.utilities.ensemble;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -39,10 +39,15 @@ public class EnsembleVotingMean implements EnsembleVoting {
 
   @Override
   public double combine(double[] scores) {
-    double sum = 0.0;
-    for (double score : scores) {
-      sum += score;
+    return combine(scores, scores.length);
+  }
+
+  @Override
+  public double combine(double[] scores, int count) {
+    double sum = 0.;
+    for (int i = 0; i < count; i++) {
+      sum += scores[i];
     }
-    return sum / scores.length;
+    return sum / count;
   }
 }
