@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.utilities.ensemble;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -40,9 +40,16 @@ public class EnsembleVotingMin implements EnsembleVoting {
 
   @Override
   public double combine(double[] scores) {
-    double min = Double.POSITIVE_INFINITY;
-    for (double val : scores) {
-      min = Math.min(min, val);
+    return combine(scores, scores.length);
+  }
+
+  @Override
+  public double combine(double[] scores, int count) {
+    double min = scores[0];
+    for (int i = 1; i < count; i++) {
+      if (scores[i] < min) {
+        min = scores[i];
+      }
     }
     return min;
   }
