@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -39,7 +39,7 @@ public class GlobalListSizeConstraint implements GlobalParameterConstraint {
   /**
    * List parameter to be tested.
    */
-  private ListParameter<?> list;
+  private ListParameter<?, ?> list;
 
   /**
    * Integer parameter specifying the constraint list size.
@@ -55,7 +55,7 @@ public class GlobalListSizeConstraint implements GlobalParameterConstraint {
    * @param v the list parameter to be tested.
    * @param i integer parameter specifying the constraint list size.
    */
-  public GlobalListSizeConstraint(ListParameter<?> v, IntParameter i) {
+  public GlobalListSizeConstraint(ListParameter<?, ?> v, IntParameter i) {
     this.list = v;
     this.length = i;
   }
