@@ -62,7 +62,7 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.WrongParameterValueException
  * 
  * @param <E> Enum type
  */
-public class EnumParameter<E extends Enum<E>> extends AbstractParameter<E> {
+public class EnumParameter<E extends Enum<E>> extends AbstractParameter<EnumParameter<E>, E> {
 
   /**
    * Reference to the actual enum type, for T.valueOf().
@@ -111,14 +111,16 @@ public class EnumParameter<E extends Enum<E>> extends AbstractParameter<E> {
 
   @Override
   protected E parseValue(Object obj) throws ParameterException {
-    if(obj == null) {
+    if (obj == null) {
       throw new UnspecifiedParameterException(this);
     }
-    if(obj instanceof String) {
+    if (enumClass.isInstance(obj)) {
+      return enumClass.cast(obj);
+    }
+    if (obj instanceof String) {
       try {
         return Enum.valueOf(enumClass, (String) obj);
-      }
-      catch(IllegalArgumentException ex) {
+      } catch (IllegalArgumentException ex) {
         throw new WrongParameterValueException("Enum parameter " + getName() + " is invalid (must be one of [" + joinEnumNames(", ") + "].");
       }
     }
@@ -138,8 +140,8 @@ public class EnumParameter<E extends Enum<E>> extends AbstractParameter<E> {
   public Collection<String> getPossibleValues() {
     // Convert to string array
     final E[] enums = enumClass.getEnumConstants();
-    ArrayList<String> values = new ArrayList<String>(enums.length);
-    for(E t : enums) {
+    ArrayList<String> values = new ArrayList<>(enums.length);
+    for (E t : enums) {
       values.add(t.name());
     }
     return values;
@@ -155,8 +157,8 @@ public class EnumParameter<E extends Enum<E>> extends AbstractParameter<E> {
   private String joinEnumNames(String separator) {
     E[] enumTypes = enumClass.getEnumConstants();
     StringBuilder sb = new StringBuilder();
-    for(int i = 0; i < enumTypes.length; ++i) {
-      if(i > 0) {
+    for (int i = 0; i < enumTypes.length; ++i) {
+      if (i > 0) {
         sb.append(separator);
       }
       sb.append(enumTypes[i].name());
