@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.utilities.scaling.outlier;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -27,6 +27,7 @@ import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.math.DoubleMinMax;
 import de.lmu.ifi.dbs.elki.result.outlier.OutlierResult;
+import de.lmu.ifi.dbs.elki.utilities.datastructures.arraylike.NumberArrayAdapter;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
 
 /**
@@ -79,13 +80,27 @@ public class OutlierMinusLogScaling implements OutlierScalingFunction {
   public void prepare(OutlierResult or) {
     DoubleMinMax mm = new DoubleMinMax();
     Relation<Double> scores = or.getScores();
-    for(DBIDIter id = scores.iterDBIDs(); id.valid(); id.advance()) {
+    for (DBIDIter id = scores.iterDBIDs(); id.valid(); id.advance()) {
       double val = scores.get(id);
-      if(!Double.isNaN(val) && !Double.isInfinite(val)) {
+      if (!Double.isNaN(val) && !Double.isInfinite(val)) {
         mm.put(val);
       }
     }
     max = mm.getMax();
     mlogmax = -Math.log(mm.getMin() / max);
   }
-}
\ No newline at end of file
+
+  @Override
+  public <A> void prepare(A array, NumberArrayAdapter<?, A> adapter) {
+    DoubleMinMax mm = new DoubleMinMax();
+    final int size = adapter.size(array);
+    for (int i = 0; i < size; i++) {
+      double val = adapter.getDouble(array, i);
+      if (!Double.isNaN(val) && !Double.isInfinite(val)) {
+        mm.put(val);
+      }
+    }
+    max = mm.getMax();
+    mlogmax = -Math.log(mm.getMin() / max);
+  }
+}
