@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.utilities.scaling.outlier;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -30,6 +30,7 @@ import de.lmu.ifi.dbs.elki.math.Mean;
 import de.lmu.ifi.dbs.elki.math.MeanVariance;
 import de.lmu.ifi.dbs.elki.math.statistics.distribution.NormalDistribution;
 import de.lmu.ifi.dbs.elki.result.outlier.OutlierResult;
+import de.lmu.ifi.dbs.elki.utilities.datastructures.arraylike.NumberArrayAdapter;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.AbstractParameterizer;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
@@ -45,26 +46,17 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.DoubleParameter;
  * Where mean can be fixed to a given value, and stddev is then computed against
  * this mean.
  * 
+ * Reference:
+ * <p>
+ * H.-P. Kriegel, P. Kröger, E. Schubert, A. Zimek<br />
+ * Interpreting and Unifying Outlier Scores<br />
+ * Proc. 11th SIAM International Conference on Data Mining (SDM), Mesa, AZ, 2011
+ * </p>
+ * 
  * @author Erich Schubert
  */
-@Reference(authors="H.-P. Kriegel, P. Kröger, E. Schubert, A. Zimek", title="Interpreting and Unifying Outlier Scores", booktitle="Proc. 11th SIAM International Conference on Data Mining (SDM), Mesa, AZ, 2011", url="http://siam.omnibooksonline.com/2011datamining/data/papers/018.pdf")
+@Reference(authors = "H.-P. Kriegel, P. Kröger, E. Schubert, A. Zimek", title = "Interpreting and Unifying Outlier Scores", booktitle = "Proc. 11th SIAM International Conference on Data Mining (SDM), Mesa, AZ, 2011", url = "http://siam.omnibooksonline.com/2011datamining/data/papers/018.pdf")
 public class StandardDeviationScaling implements OutlierScalingFunction {
-  /**
-   * Parameter to specify a fixed mean to use.
-   * <p>
-   * Key: {@code -stddevscale.mean}
-   * </p>
-   */
-  public static final OptionID MEAN_ID = new OptionID("stddevscale.mean", "Fixed mean to use in standard deviation scaling.");
-
-  /**
-   * Parameter to specify the lambda value
-   * <p>
-   * Key: {@code -stddevscale.lambda}
-   * </p>
-   */
-  public static final OptionID LAMBDA_ID = new OptionID("stddevscale.lambda", "Significance level to use for error function.");
-
   /**
    * Field storing the fixed mean to use
    */
@@ -73,7 +65,7 @@ public class StandardDeviationScaling implements OutlierScalingFunction {
   /**
    * Field storing the lambda value
    */
-  protected Double lambda = null;
+  protected double lambda;
 
   /**
    * Mean to use
@@ -88,10 +80,10 @@ public class StandardDeviationScaling implements OutlierScalingFunction {
   /**
    * Constructor.
    * 
-   * @param fixedmean
-   * @param lambda
+   * @param fixedmean Fixed mean
+   * @param lambda Scaling factor lambda
    */
-  public StandardDeviationScaling(Double fixedmean, Double lambda) {
+  public StandardDeviationScaling(Double fixedmean, double lambda) {
     super();
     this.fixedmean = fixedmean;
     this.lambda = lambda;
@@ -107,7 +99,7 @@ public class StandardDeviationScaling implements OutlierScalingFunction {
   @Override
   public double getScaled(double value) {
     assert (factor != 0) : "prepare() was not run prior to using the scaling function.";
-    if(value <= mean) {
+    if (value <= mean) {
       return 0;
     }
     return Math.max(0, NormalDistribution.erf((value - mean) / factor));
@@ -115,12 +107,12 @@ public class StandardDeviationScaling implements OutlierScalingFunction {
 
   @Override
   public void prepare(OutlierResult or) {
-    if(fixedmean == null) {
+    if (fixedmean == null) {
       MeanVariance mv = new MeanVariance();
       Relation<Double> scores = or.getScores();
-      for(DBIDIter id = scores.iterDBIDs(); id.valid(); id.advance()) {
+      for (DBIDIter id = scores.iterDBIDs(); id.valid(); id.advance()) {
         double val = scores.get(id);
-        if(!Double.isNaN(val) && !Double.isInfinite(val)) {
+        if (!Double.isNaN(val) && !Double.isInfinite(val)) {
           mv.put(val);
         }
       }
@@ -129,14 +121,46 @@ public class StandardDeviationScaling implements OutlierScalingFunction {
       if (factor == 0.0) {
         factor = Double.MIN_NORMAL;
       }
-    }
-    else {
+    } else {
       mean = fixedmean;
       Mean sqsum = new Mean();
       Relation<Double> scores = or.getScores();
-      for(DBIDIter id = scores.iterDBIDs(); id.valid(); id.advance()) {
+      for (DBIDIter id = scores.iterDBIDs(); id.valid(); id.advance()) {
         double val = scores.get(id);
-        if(!Double.isNaN(val) && !Double.isInfinite(val)) {
+        if (!Double.isNaN(val) && !Double.isInfinite(val)) {
+          sqsum.put((val - mean) * (val - mean));
+        }
+      }
+      factor = lambda * Math.sqrt(sqsum.getMean()) * MathUtil.SQRT2;
+      if (factor == 0.0) {
+        factor = Double.MIN_NORMAL;
+      }
+    }
+  }
+
+  @Override
+  public <A> void prepare(A array, NumberArrayAdapter<?, A> adapter) {
+    if (fixedmean == null) {
+      MeanVariance mv = new MeanVariance();
+      final int size = adapter.size(array);
+      for (int i = 0; i < size; i++) {
+        double val = adapter.getDouble(array, i);
+        if (!Double.isInfinite(val)) {
+          mv.put(val);
+        }
+      }
+      mean = mv.getMean();
+      factor = lambda * mv.getSampleStddev() * MathUtil.SQRT2;
+      if (factor == 0.0) {
+        factor = Double.MIN_NORMAL;
+      }
+    } else {
+      mean = fixedmean;
+      Mean sqsum = new Mean();
+      final int size = adapter.size(array);
+      for (int i = 0; i < size; i++) {
+        double val = adapter.getDouble(array, i);
+        if (!Double.isInfinite(val)) {
           sqsum.put((val - mean) * (val - mean));
         }
       }
@@ -165,20 +189,36 @@ public class StandardDeviationScaling implements OutlierScalingFunction {
    * @apiviz.exclude
    */
   public static class Parameterizer extends AbstractParameterizer {
+    /**
+     * Parameter to specify a fixed mean to use.
+     * <p>
+     * Key: {@code -stddevscale.mean}
+     * </p>
+     */
+    public static final OptionID MEAN_ID = new OptionID("stddevscale.mean", "Fixed mean to use in standard deviation scaling.");
+
+    /**
+     * Parameter to specify the lambda value
+     * <p>
+     * Key: {@code -stddevscale.lambda}
+     * </p>
+     */
+    public static final OptionID LAMBDA_ID = new OptionID("stddevscale.lambda", "Significance level to use for error function.");
+
     protected Double fixedmean = null;
 
-    protected Double lambda = null;
+    protected double lambda;
 
     @Override
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
       DoubleParameter meanP = new DoubleParameter(MEAN_ID);
       meanP.setOptional(true);
-      if(config.grab(meanP)) {
+      if (config.grab(meanP)) {
         fixedmean = meanP.getValue();
       }
       DoubleParameter lambdaP = new DoubleParameter(LAMBDA_ID, 3.0);
-      if(config.grab(lambdaP)) {
+      if (config.grab(lambdaP)) {
         lambda = lambdaP.getValue();
       }
     }
@@ -188,4 +228,4 @@ public class StandardDeviationScaling implements OutlierScalingFunction {
       return new StandardDeviationScaling(fixedmean, lambda);
     }
   }
-}
\ No newline at end of file
+}
