@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.visualization.css;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -43,7 +43,7 @@ public class CSSClassManager {
   /**
    * Store the contained CSS classes.
    */
-  private HashMap<String, CSSClass> store = new HashMap<String, CSSClass>();
+  private HashMap<String, CSSClass> store = new HashMap<>();
   
   /**
    * Add a single class to the map.
