@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.visualization.gui.overview;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -43,6 +43,7 @@ import de.lmu.ifi.dbs.elki.result.Result;
 import de.lmu.ifi.dbs.elki.result.ResultHierarchy;
 import de.lmu.ifi.dbs.elki.result.ResultListener;
 import de.lmu.ifi.dbs.elki.result.ResultUtil;
+import de.lmu.ifi.dbs.elki.utilities.datastructures.hierarchy.Hierarchy;
 import de.lmu.ifi.dbs.elki.utilities.pairs.Pair;
 import de.lmu.ifi.dbs.elki.visualization.VisualizationTask;
 import de.lmu.ifi.dbs.elki.visualization.VisualizerContext;
@@ -90,7 +91,7 @@ public class OverviewPlot extends SVGPlot implements ResultListener {
   /**
    * Action listeners for this plot.
    */
-  private ArrayList<ActionListener> actionListeners = new ArrayList<ActionListener>();
+  private ArrayList<ActionListener> actionListeners = new ArrayList<>();
 
   /**
    * Single view mode
@@ -191,7 +192,7 @@ public class OverviewPlot extends SVGPlot implements ResultListener {
    * @return Arrangement
    */
   private RectangleArranger<PlotItem> arrangeVisualizations(double width, double height) {
-    RectangleArranger<PlotItem> plotmap = new RectangleArranger<PlotItem>(width, height);
+    RectangleArranger<PlotItem> plotmap = new RectangleArranger<>(width, height);
 
     ArrayList<Projector> projectors = ResultUtil.filterResults(context.getResult(), Projector.class);
     // Rectangle layout
@@ -209,8 +210,11 @@ public class OverviewPlot extends SVGPlot implements ResultListener {
     ResultHierarchy hier = context.getHierarchy();
     ArrayList<VisualizationTask> tasks = ResultUtil.filterResults(context.getResult(), VisualizationTask.class);
     nextTask: for (VisualizationTask task : tasks) {
-      for (Result parent : hier.getParents(task)) {
-        if (parent instanceof Projector) {
+      if (!task.visible) {
+        continue;
+      }
+      for (Hierarchy.Iter<Result> iter = hier.iterParents(task); iter.valid(); iter.advance()) {
+        if (iter.get() instanceof Projector) {
           continue nextTask;
         }
       }
@@ -276,7 +280,7 @@ public class OverviewPlot extends SVGPlot implements ResultListener {
           hasDetails |= !task.nodetail;
           Pair<Element, Visualization> pair = oldlayers.remove(it, task);
           if (pair == null) {
-            pair = new Pair<Element, Visualization>(null, null);
+            pair = new Pair<>(null, null);
             pair.first = svgElement(SVGConstants.SVG_G_TAG);
           }
           if (pair.second == null) {
@@ -368,7 +372,7 @@ public class OverviewPlot extends SVGPlot implements ResultListener {
             // New task?
             if (pair == null) {
               if (visibleInOverview(task)) {
-                pair = new Pair<Element, Visualization>(null, null);
+                pair = new Pair<>(null, null);
                 pair.first = svgElement(SVGConstants.SVG_G_TAG);
                 pair.second = embedOrThumbnail(thumbsize, it, task, pair.first);
                 vistoelem.get(it, null).first.appendChild(pair.first);
@@ -381,13 +385,6 @@ public class OverviewPlot extends SVGPlot implements ResultListener {
                 if (pair.first.hasAttribute(SVGConstants.CSS_VISIBILITY_PROPERTY)) {
                   pair.first.removeAttribute(SVGConstants.CSS_VISIBILITY_PROPERTY);
                 }
-                // if not yet rendered, add a thumbnail
-                if (!pair.first.hasChildNodes()) {
-                  LOG.warning("This codepath should no longer be needed.");
-                  Visualization visualization = embedOrThumbnail(thumbsize, it, task, pair.first);
-                  vistoelem.put(it, task, pair.first, visualization);
-                  refreshcss = true;
-                }
               } else {
                 // hide if there is anything to hide.
                 if (pair.first != null && pair.first.hasChildNodes()) {
@@ -562,6 +559,18 @@ public class OverviewPlot extends SVGPlot implements ResultListener {
   @Override
   public void resultChanged(Result current) {
     LOG.debug("result changed: " + current);
+    if (current instanceof VisualizationTask) {
+      boolean relayout = true;
+      for (Hierarchy.Iter<Result> iter = context.getHierarchy().iterParents(current); iter.valid(); iter.advance()) {
+        if (iter.get() instanceof Projector) {
+          relayout = false;
+          break;
+        }
+      }
+      if (relayout) {
+        reinitOnRefresh = true;
+      }
+    }
     lazyRefresh();
   }
 
