@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.visualization.opticsplot;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -50,4 +50,9 @@ public class OPTICSCorrelationDimensionalityDistance<D extends CorrelationDistan
     }
     return reachability.getCorrelationValue();
   }
+
+  @Override
+  public boolean isInfinite(ClusterOrderEntry<D> coe) {
+    return coe.getReachability().isInfiniteDistance();
+  }
 }
\ No newline at end of file
