@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.visualization.opticsplot;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -58,7 +58,7 @@ public class OPTICSCut {
   public static <D extends Distance<D>> Clustering<Model> makeOPTICSCut(ClusterOrderResult<D> co, OPTICSDistanceAdapter<D> adapter, double epsilon) {
     List<ClusterOrderEntry<D>> order = co.getClusterOrder();
     // Clustering model we are building
-    Clustering<Model> clustering = new Clustering<Model>("OPTICS Cut Clustering", "optics-cut");
+    Clustering<Model> clustering = new Clustering<>("OPTICS Cut Clustering", "optics-cut");
     // Collects noise elements
     ModifiableDBIDs noise = DBIDUtil.newHashSet();
 
@@ -88,7 +88,7 @@ public class OPTICSCut {
         if(!current.isEmpty()) {
           // TODO: do we want a minpts restriction?
           // But we get have only core points guaranteed anyway.
-          clustering.addCluster(new Cluster<Model>(current, ClusterModel.CLUSTER));
+          clustering.addToplevelCluster(new Cluster<Model>(current, ClusterModel.CLUSTER));
           current = DBIDUtil.newHashSet();
         }
         // Add to noise
@@ -97,10 +97,10 @@ public class OPTICSCut {
     }
     // Any unfinished cluster will also be added
     if(!current.isEmpty()) {
-      clustering.addCluster(new Cluster<Model>(current, ClusterModel.CLUSTER));
+      clustering.addToplevelCluster(new Cluster<Model>(current, ClusterModel.CLUSTER));
     }
     // Add noise
-    clustering.addCluster(new Cluster<Model>(noise, true, ClusterModel.CLUSTER));
+    clustering.addToplevelCluster(new Cluster<Model>(noise, true, ClusterModel.CLUSTER));
     return clustering;
   }
 }
\ No newline at end of file
