@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.visualization.opticsplot;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -40,5 +40,13 @@ public interface OPTICSDistanceAdapter<D extends Distance<D>> {
    * @param coe Cluster Order Entry
    * @return Double value (height)
    */
-  public double getDoubleForEntry(ClusterOrderEntry<D> coe);
+  double getDoubleForEntry(ClusterOrderEntry<D> coe);
+
+  /**
+   * Test whether the reachability is infinite.
+   * 
+   * @param coe Cluster order entry
+   * @return true, when infinite
+   */
+  boolean isInfinite(ClusterOrderEntry<D> coe);
 }
