@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.visualization.opticsplot;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -31,7 +31,7 @@ import de.lmu.ifi.dbs.elki.result.optics.ClusterOrderEntry;
  * 
  * @author Erich Schubert
  */
-public class OPTICSNumberDistance<D extends NumberDistance<D,?>> implements OPTICSDistanceAdapter<D> {
+public class OPTICSNumberDistance<D extends NumberDistance<D, ?>> implements OPTICSDistanceAdapter<D> {
   /**
    * Default constructor.
    */
@@ -46,4 +46,9 @@ public class OPTICSNumberDistance<D extends NumberDistance<D,?>> implements OPTI
     }
     return coe.getReachability().doubleValue();
   }
+
+  @Override
+  public boolean isInfinite(ClusterOrderEntry<D> coe) {
+    return coe.getReachability().isInfiniteDistance();
+  }
 }
