@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.visualization.projector;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -75,10 +75,10 @@ public class OPTICSProjector<D extends Distance<D>> extends AbstractHierarchical
 
   @Override
   public Collection<PlotItem> arrange() {
-    List<PlotItem> col = new ArrayList<PlotItem>(1);
+    List<PlotItem> col = new ArrayList<>(1);
     List<VisualizationTask> tasks = ResultUtil.filterResults(this, VisualizationTask.class);
     if (tasks.size() > 0) {
-      final PlotItem it = new PlotItem(4., 1., new OPTICSProjection<D>(this));
+      final PlotItem it = new PlotItem(4., 1., new OPTICSProjection<>(this));
       it.tasks = tasks;
       col.add(it);
     }
