@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.visualization.projector;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -53,7 +53,7 @@ public class OPTICSProjectorFactory implements ProjectorFactory {
     for(ClusterOrderResult<?> co : cos) {
       if(OPTICSPlot.canPlot(co)) {
         @SuppressWarnings("unchecked")
-        OPTICSProjector<?> proj = new OPTICSProjector<DoubleDistance>((ClusterOrderResult<DoubleDistance>) co);
+        OPTICSProjector<?> proj = new OPTICSProjector<>((ClusterOrderResult<DoubleDistance>) co);
         baseResult.getHierarchy().add(co, proj);
       }
     }
