@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.visualization.svg;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -119,7 +119,7 @@ public class SVGPlot {
   /**
    * Manage objects with an id.
    */
-  private HashMap<String, WeakReference<Element>> objWithId = new HashMap<String, WeakReference<Element>>();
+  private HashMap<String, WeakReference<Element>> objWithId = new HashMap<>();
 
   /**
    * Registers changes of this SVGPlot.
@@ -581,7 +581,7 @@ public class SVGPlot {
    * @param obj Element
    */
   public void putIdElement(String id, Element obj) {
-    objWithId.put(id, new WeakReference<Element>(obj));
+    objWithId.put(id, new WeakReference<>(obj));
   }
 
   /**
