@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.visualization.svg;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -43,6 +43,7 @@ import org.w3c.dom.svg.SVGMatrix;
 import org.w3c.dom.svg.SVGPoint;
 
 import de.lmu.ifi.dbs.elki.logging.LoggingUtil;
+import de.lmu.ifi.dbs.elki.math.MathUtil;
 
 /**
  * Utility class for SVG processing.
@@ -87,7 +88,7 @@ public final class SVGUtil {
 
   static {
     // Build a reasonably sized hashmap. Use 0
-    SVG_COLOR_NAMES = new TObjectIntHashMap<String>(90, .8f, NO_VALUE);
+    SVG_COLOR_NAMES = new TObjectIntHashMap<>(90, .8f, NO_VALUE);
     // List taken from SVG specification:
     // http://www.w3.org/TR/SVG/types.html#ColorKeywords
     SVG_COLOR_NAMES.put("aliceblue", 0xFFF0F8FF);
@@ -512,18 +513,20 @@ public final class SVGUtil {
   }
 
   /**
-   * Convert a color name from an AWT color object to CSS syntax
+   * Convert a color name from an integer RGB color to CSS syntax
    * 
-   * Note: currently only RGB (from ARGB order) are supported.
+   * Note: currently only RGB (from ARGB order) are supported. The alpha channel
+   * will be ignored.
    * 
    * @param col Color value
    * @return Color string
    */
   public static String colorToString(int col) {
-    char[] buf = new char[] { '#', '0', '0', '0', '0', '0', '0' };
-    for (int i = 7; i > 0; i--) {
-      buf[i] += (col & 0xF);
-      col >>= 4;
+    final char[] buf = new char[] { '#', 'X', 'X', 'X', 'X', 'X', 'X' };
+    for (int i = 6; i > 0; i--) {
+      final int v = (col & 0xF);
+      buf[i] = (char) ((v < 10) ? ('0' + v) : ('a' + v - 10));
+      col >>>= 4;
     }
     return new String(buf);
   }
@@ -642,10 +645,10 @@ public final class SVGUtil {
    */
   public static Element svgCircleSegment(SVGPlot svgp, double centerx, double centery, double angleStart, double angleDelta, double innerRadius, double outerRadius) {
     double sin1st = Math.sin(angleStart);
-    double cos1st = Math.cos(angleStart);
+    double cos1st = MathUtil.sinToCos(angleStart, sin1st);
 
     double sin2nd = Math.sin(angleStart + angleDelta);
-    double cos2nd = Math.cos(angleStart + angleDelta);
+    double cos2nd = MathUtil.sinToCos(angleStart + angleDelta, sin2nd);
 
     double inner1stx = centerx + (innerRadius * sin1st);
     double inner1sty = centery - (innerRadius * cos1st);
