@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.visualization.svg;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -46,7 +46,7 @@ public class UpdateRunner {
   /**
    * The queue of pending updates
    */
-  final private Queue<Runnable> queue = new ConcurrentLinkedQueue<Runnable>();
+  final private Queue<Runnable> queue = new ConcurrentLinkedQueue<>();
 
   /**
    * Synchronizer that can block events from being executed right away.
