@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.visualization.visualizers.histogram;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -45,7 +45,7 @@ import de.lmu.ifi.dbs.elki.utilities.datastructures.histogram.DoubleArrayStaticH
 import de.lmu.ifi.dbs.elki.utilities.exceptions.ObjectNotFoundException;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.AbstractParameterizer;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.GreaterEqualConstraint;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.CommonConstraints;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.Flag;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntParameter;
@@ -113,9 +113,9 @@ public class ColoredHistogramVisualizer extends AbstractVisFactory {
   public void processNewResult(HierarchicalResult baseResult, Result result) {
     // Find a style result to visualize:
     Collection<StyleResult> styleres = ResultUtil.filterResults(result, StyleResult.class);
-    for (StyleResult c : styleres) {
+    for(StyleResult c : styleres) {
       Collection<HistogramProjector<?>> ps = ResultUtil.filterResults(baseResult, HistogramProjector.class);
-      for (HistogramProjector<?> p : ps) {
+      for(HistogramProjector<?> p : ps) {
         // register self
         final VisualizationTask task = new VisualizationTask(CNAME, c, p.getRelation(), this);
         task.level = VisualizationTask.LEVEL_DATA;
@@ -196,9 +196,10 @@ public class ColoredHistogramVisualizer extends AbstractVisFactory {
       // Styling policy
       final StylingPolicy spol = style.getStylingPolicy();
       final ClassStylingPolicy cspol;
-      if (spol instanceof ClassStylingPolicy) {
+      if(spol instanceof ClassStylingPolicy) {
         cspol = (ClassStylingPolicy) spol;
-      } else {
+      }
+      else {
         cspol = null;
       }
       // TODO also use min style?
@@ -212,35 +213,37 @@ public class ColoredHistogramVisualizer extends AbstractVisFactory {
       final int cols = numc + 1;
       DoubleArrayStaticHistogram histogram = new DoubleArrayStaticHistogram(settings.bins, -.5, .5, cols);
 
-      if (cspol != null) {
-        for (int snum = 0; snum < numc; snum++) {
+      if(cspol != null) {
+        for(int snum = 0; snum < numc; snum++) {
           double[] inc = new double[cols];
           inc[0] = frac;
           inc[snum + 1] = frac;
-          for (DBIDIter iter = cspol.iterateClass(snum + off); iter.valid(); iter.advance()) {
-            if (!sample.getSample().contains(iter)) {
+          for(DBIDIter iter = cspol.iterateClass(snum + off); iter.valid(); iter.advance()) {
+            if(!sample.getSample().contains(iter)) {
               continue; // TODO: can we test more efficiently than this?
             }
             try {
               double pos = proj.fastProjectDataToRenderSpace(relation.get(iter)) / Projection.SCALE;
               histogram.increment(pos, inc);
-            } catch (ObjectNotFoundException e) {
+            }
+            catch(ObjectNotFoundException e) {
               // Ignore. The object was probably deleted from the database
             }
           }
         }
-      } else {
+      }
+      else {
         // Actual data distribution.
         double[] inc = new double[cols];
         inc[0] = frac;
-        for (DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
+        for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
           double pos = proj.fastProjectDataToRenderSpace(relation.get(iditer)) / Projection.SCALE;
           histogram.increment(pos, inc);
         }
       }
       // for scaling, get the maximum occurring value in the bins:
-      for (DoubleArrayStaticHistogram.Iter iter = histogram.iter(); iter.valid(); iter.advance()) {
-        for (double val : iter.getValue()) {
+      for(DoubleArrayStaticHistogram.Iter iter = histogram.iter(); iter.valid(); iter.advance()) {
+        for(double val : iter.getValue()) {
           minmax.put(val);
         }
       }
@@ -255,29 +258,30 @@ public class ColoredHistogramVisualizer extends AbstractVisFactory {
         // draw axes that are non-trivial
         final int dimensionality = RelationUtil.dimensionality(relation);
         double orig = proj.fastProjectScaledToRender(new Vector(dimensionality));
-        for (int d = 0; d < dimensionality; d++) {
+        for(int d = 0; d < dimensionality; d++) {
           Vector v = new Vector(dimensionality);
           v.set(d, 1);
           // projected endpoint of axis
           double ax = proj.fastProjectScaledToRender(v);
-          if (ax < orig || ax > orig) {
+          if(ax < orig || ax > orig) {
             final double left = (orig / Projection.SCALE + 0.5) * xsize;
             final double right = (ax / Projection.SCALE + 0.5) * xsize;
             SVGSimpleLinearAxis.drawAxis(svgp, layer, proj.getScale(d), left, ysize, right, ysize, SVGSimpleLinearAxis.LabelStyle.RIGHTHAND, style.getStyleLibrary());
           }
         }
-      } catch (CSSNamingConflict e) {
+      }
+      catch(CSSNamingConflict e) {
         LoggingUtil.exception("CSS class exception in axis class.", e);
       }
 
       // Visualizing
-      if (!settings.curves) {
-        for (DoubleArrayStaticHistogram.Iter iter = histogram.iter(); iter.valid(); iter.advance()) {
+      if(!settings.curves) {
+        for(DoubleArrayStaticHistogram.Iter iter = histogram.iter(); iter.valid(); iter.advance()) {
           double lpos = xscale.getScaled(iter.getLeft());
           double rpos = xscale.getScaled(iter.getRight());
           double stack = 0.0;
           final int start = numc > 0 ? 1 : 0;
-          for (int key = start; key < cols; key++) {
+          for(int key = start; key < cols; key++) {
             double val = yscale.getScaled(iter.getValue()[key]);
             Element row = SVGUtil.svgRect(svgp.getDocument(), xsize * lpos, ysize * (1 - (val + stack)), xsize * (rpos - lpos), ysize * val);
             stack = stack + val;
@@ -285,24 +289,25 @@ public class ColoredHistogramVisualizer extends AbstractVisFactory {
             layer.appendChild(row);
           }
         }
-      } else {
+      }
+      else {
         double left = xscale.getScaled(histogram.getCoverMinimum());
         double right = left;
 
         SVGPath[] paths = new SVGPath[cols];
         double[] lasty = new double[cols];
-        for (int i = 0; i < cols; i++) {
+        for(int i = 0; i < cols; i++) {
           paths[i] = new SVGPath(xsize * left, ysize * 1);
           lasty[i] = 0;
         }
 
         // draw histogram lines
-        for (DoubleArrayStaticHistogram.Iter iter = histogram.iter(); iter.valid(); iter.advance()) {
+        for(DoubleArrayStaticHistogram.Iter iter = histogram.iter(); iter.valid(); iter.advance()) {
           left = xscale.getScaled(iter.getLeft());
           right = xscale.getScaled(iter.getRight());
-          for (int i = 0; i < cols; i++) {
+          for(int i = 0; i < cols; i++) {
             double val = yscale.getScaled(iter.getValue()[i]);
-            if (lasty[i] > val || lasty[i] < val) {
+            if(lasty[i] > val || lasty[i] < val) {
               paths[i].lineTo(xsize * left, ysize * (1 - lasty[i]));
               paths[i].lineTo(xsize * left, ysize * (1 - val));
               paths[i].lineTo(xsize * right, ysize * (1 - val));
@@ -311,8 +316,8 @@ public class ColoredHistogramVisualizer extends AbstractVisFactory {
           }
         }
         // close and insert all lines.
-        for (int i = 0; i < cols; i++) {
-          if (lasty[i] != 0) {
+        for(int i = 0; i < cols; i++) {
+          if(lasty[i] != 0) {
             paths[i].lineTo(xsize * right, ysize * (1 - lasty[i]));
           }
           paths[i].lineTo(xsize * right, ysize * 1);
@@ -333,22 +338,24 @@ public class ColoredHistogramVisualizer extends AbstractVisFactory {
       ColorLibrary colors = style.getStyleLibrary().getColorSet(StyleLibrary.PLOT);
 
       CSSClass allInOne = new CSSClass(svgp, BIN + -1);
-      if (!settings.curves) {
+      if(!settings.curves) {
         allInOne.setStatement(SVGConstants.CSS_FILL_PROPERTY, SVGConstants.CSS_BLACK_VALUE);
         allInOne.setStatement(SVGConstants.CSS_FILL_OPACITY_PROPERTY, 1.0);
-      } else {
+      }
+      else {
         allInOne.setStatement(SVGConstants.CSS_STROKE_PROPERTY, SVGConstants.CSS_BLACK_VALUE);
         allInOne.setStatement(SVGConstants.CSS_STROKE_WIDTH_PROPERTY, style.getStyleLibrary().getLineWidth(StyleLibrary.PLOT));
         allInOne.setStatement(SVGConstants.CSS_FILL_PROPERTY, SVGConstants.CSS_NONE_VALUE);
       }
       svgp.addCSSClassOrLogError(allInOne);
 
-      for (int clusterID = 0; clusterID < numc; clusterID++) {
+      for(int clusterID = 0; clusterID < numc; clusterID++) {
         CSSClass bin = new CSSClass(svgp, BIN + clusterID);
 
-        if (!settings.curves) {
+        if(!settings.curves) {
           bin.setStatement(SVGConstants.CSS_FILL_PROPERTY, colors.getColor(clusterID));
-        } else {
+        }
+        else {
           bin.setStatement(SVGConstants.CSS_STROKE_PROPERTY, colors.getColor(clusterID));
           bin.setStatement(SVGConstants.CSS_STROKE_WIDTH_PROPERTY, style.getStyleLibrary().getLineWidth(StyleLibrary.PLOT));
           bin.setStatement(SVGConstants.CSS_FILL_PROPERTY, SVGConstants.CSS_NONE_VALUE);
@@ -399,12 +406,12 @@ public class ColoredHistogramVisualizer extends AbstractVisFactory {
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
       Flag curvesF = new Flag(STYLE_CURVES_ID);
-      if (config.grab(curvesF)) {
+      if(config.grab(curvesF)) {
         curves = curvesF.isTrue();
       }
       IntParameter binsP = new IntParameter(HISTOGRAM_BINS_ID, DEFAULT_BINS);
-      binsP.addConstraint(new GreaterEqualConstraint(2));
-      if (config.grab(binsP)) {
+      binsP.addConstraint(CommonConstraints.GREATER_THAN_ONE_INT);
+      if(config.grab(binsP)) {
         bins = binsP.intValue();
       }
     }
