@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.visualization.visualizers.pairsegments;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -48,6 +48,7 @@ import de.lmu.ifi.dbs.elki.result.HierarchicalResult;
 import de.lmu.ifi.dbs.elki.result.Result;
 import de.lmu.ifi.dbs.elki.result.ResultListener;
 import de.lmu.ifi.dbs.elki.result.ResultUtil;
+import de.lmu.ifi.dbs.elki.utilities.FormatUtil;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
 import de.lmu.ifi.dbs.elki.utilities.exceptions.AbortException;
 import de.lmu.ifi.dbs.elki.visualization.VisualizationTask;
@@ -227,7 +228,7 @@ public class CircleSegmentsVisualizer extends AbstractVisFactory {
     /**
      * Map to connect segments to their visual elements
      */
-    public Map<Segment, List<Element>> segmentToElements = new HashMap<Segment, List<Element>>();
+    public Map<Segment, List<Element>> segmentToElements = new HashMap<>();
 
     /**
      * Show unclustered Pairs in CircleSegments
@@ -319,7 +320,7 @@ public class CircleSegmentsVisualizer extends AbstractVisFactory {
 
       // Add ring:clustering info
       Element clrInfo = drawClusteringInfo();
-      Element c = checkbox.renderCheckBox(svgp, 1, 5 + Double.parseDouble(clrInfo.getAttribute(SVGConstants.SVG_HEIGHT_ATTRIBUTE)), 11);
+      Element c = checkbox.renderCheckBox(svgp, 1., 5. + FormatUtil.parseDouble(clrInfo.getAttribute(SVGConstants.SVG_HEIGHT_ATTRIBUTE)), 11);
       ctrlLayer.appendChild(clrInfo);
       ctrlLayer.appendChild(c);
 
@@ -412,7 +413,7 @@ public class CircleSegmentsVisualizer extends AbstractVisFactory {
 
         // ITERATE OVER ALL SEGMENT-CLUSTERS
 
-        ArrayList<Element> elems = new ArrayList<Element>(clusterings);
+        ArrayList<Element> elems = new ArrayList<>(clusterings);
         segmentToElements.put(segment, elems);
         // draw segment for every clustering
 
