@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.visualization.visualizers.scatterplot;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -69,7 +69,7 @@ public class MarkerVisualization extends AbstractVisFactory {
    */
   public MarkerVisualization() {
     super();
-    thumbmask |= ThumbnailVisualization.ON_DATA | ThumbnailVisualization.ON_STYLE;
+    this.thumbmask |= ThumbnailVisualization.ON_DATA | ThumbnailVisualization.ON_STYLE;
   }
 
   @Override
@@ -147,6 +147,9 @@ public class MarkerVisualization extends AbstractVisFactory {
             try {
               final NumberVector<?> vec = rel.get(iter);
               double[] v = proj.fastProjectDataToRenderSpace(vec);
+              if(v[0] != v[0] || v[1] != v[1]) {
+                continue; // NaN!
+              }
               ml.useMarker(svgp, layer, v[0], v[1], cnum, marker_size);
             }
             catch(ObjectNotFoundException e) {
