@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.visualization.visualizers.scatterplot.cluster;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -136,10 +136,13 @@ public class ClusterOrderVisualization extends AbstractVisFactory {
         double[] thisVec = proj.fastProjectDataToRenderSpace(rel.get(thisId));
         double[] prevVec = proj.fastProjectDataToRenderSpace(rel.get(prevId));
 
-        // FIXME: DO NOT COMMIT
-        thisVec[0] = thisVec[0] * 0.95 + prevVec[0] * 0.05;
-        thisVec[1] = thisVec[1] * 0.95 + prevVec[1] * 0.05;
-
+        if(thisVec[0] != thisVec[0] || thisVec[1] != thisVec[1]) {
+          continue; // NaN!
+        }
+        if(prevVec[0] != prevVec[0] || prevVec[1] != prevVec[1]) {
+          continue; // NaN!
+        }
+        // FIXME: add arrow decorations!
         Element arrow = svgp.svgLine(prevVec[0], prevVec[1], thisVec[0], thisVec[1]);
         SVGUtil.setCSSClass(arrow, cls.getName());
 
