@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.visualization.visualizers.scatterplot.cluster;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -93,7 +93,7 @@ public class EMClusterVisualization extends AbstractVisFactory {
 
   @Override
   public Instance<DoubleVector> makeVisualization(VisualizationTask task) {
-    return new Instance<DoubleVector>(task);
+    return new Instance<>(task);
   }
 
   @Override
@@ -346,7 +346,7 @@ public class EMClusterVisualization extends AbstractVisFactory {
       for(int i = 1; i <= times; i++) {
         SVGPath path = new SVGPath();
 
-        ArrayList<Vector> delta = new ArrayList<Vector>(chres.size());
+        ArrayList<Vector> delta = new ArrayList<>(chres.size());
         for(int p = 0; p < chres.size(); p++) {
           Vector prev = chres.get((p - 1 + chres.size()) % chres.size());
           Vector curr = chres.get(p);
