@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.visualization.visualizers.scatterplot.index;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -32,9 +32,9 @@ import de.lmu.ifi.dbs.elki.data.NumberVector;
 import de.lmu.ifi.dbs.elki.database.datastore.DataStoreListener;
 import de.lmu.ifi.dbs.elki.database.ids.DBID;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.DistanceFunction;
-import de.lmu.ifi.dbs.elki.distance.distancefunction.EuclideanDistanceFunction;
-import de.lmu.ifi.dbs.elki.distance.distancefunction.LPNormDistanceFunction;
-import de.lmu.ifi.dbs.elki.distance.distancefunction.ManhattanDistanceFunction;
+import de.lmu.ifi.dbs.elki.distance.distancefunction.minkowski.EuclideanDistanceFunction;
+import de.lmu.ifi.dbs.elki.distance.distancefunction.minkowski.LPNormDistanceFunction;
+import de.lmu.ifi.dbs.elki.distance.distancefunction.minkowski.ManhattanDistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.DoubleDistance;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.NumberDistance;
 import de.lmu.ifi.dbs.elki.index.tree.metrical.mtreevariants.AbstractMTree;
@@ -107,8 +107,8 @@ public class TreeSphereVisualization extends AbstractVisFactory {
   public void processNewResult(HierarchicalResult baseResult, Result result) {
     Collection<ScatterPlotProjector<?>> ps = ResultUtil.filterResults(baseResult, ScatterPlotProjector.class);
     for (ScatterPlotProjector<?> p : ps) {
-      Collection<AbstractMTree<?, DoubleDistance, ?, ?>> trees = ResultUtil.filterResults(result, AbstractMTree.class);
-      for (AbstractMTree<?, DoubleDistance, ?, ?> tree : trees) {
+      Collection<AbstractMTree<?, DoubleDistance, ?, ?, ?>> trees = ResultUtil.filterResults(result, AbstractMTree.class);
+      for (AbstractMTree<?, DoubleDistance, ?, ?, ?> tree : trees) {
         if (canVisualize(tree) && tree instanceof Result) {
           final VisualizationTask task = new VisualizationTask(NAME, (Result) tree, p.getRelation(), this);
           task.level = VisualizationTask.LEVEL_BACKGROUND + 1;
@@ -121,7 +121,7 @@ public class TreeSphereVisualization extends AbstractVisFactory {
 
   @Override
   public Visualization makeVisualization(VisualizationTask task) {
-    return new Instance<DoubleDistance, MTreeNode<Object, DoubleDistance>, MTreeEntry<DoubleDistance>>(task);
+    return new Instance<DoubleDistance, MTreeNode<Object, DoubleDistance>, MTreeEntry>(task);
   }
 
   /**
@@ -130,10 +130,10 @@ public class TreeSphereVisualization extends AbstractVisFactory {
    * @param tree Tree to visualize
    * @return p value
    */
-  public static double getLPNormP(AbstractMTree<?, ?, ?, ?> tree) {
+  public static double getLPNormP(AbstractMTree<?, ?, ?, ?, ?> tree) {
     // Note: we deliberately lose generics here, so the compilers complain
     // less on the next typecheck and cast!
-    DistanceFunction<?, ?> distanceFunction = tree.getDistanceQuery().getDistanceFunction();
+    DistanceFunction<?, ?> distanceFunction = tree.getDistanceFunction();
     if (LPNormDistanceFunction.class.isInstance(distanceFunction)) {
       return ((LPNormDistanceFunction) distanceFunction).getP();
     }
@@ -146,7 +146,7 @@ public class TreeSphereVisualization extends AbstractVisFactory {
    * @param tree Tree to visualize
    * @return whether the tree is visualizable
    */
-  public static boolean canVisualize(AbstractMTree<?, ?, ?, ?> tree) {
+  public static boolean canVisualize(AbstractMTree<?, ?, ?, ?, ?> tree) {
     return getLPNormP(tree) > 0;
   }
 
@@ -162,7 +162,7 @@ public class TreeSphereVisualization extends AbstractVisFactory {
    * @param <E> Tree entry type
    */
   // TODO: listen for tree changes!
-  public class Instance<D extends NumberDistance<D, ?>, N extends AbstractMTreeNode<?, D, N, E>, E extends MTreeEntry<D>> extends AbstractScatterplotVisualization implements DataStoreListener {
+  public class Instance<D extends NumberDistance<D, ?>, N extends AbstractMTreeNode<?, D, N, E>, E extends MTreeEntry> extends AbstractScatterplotVisualization implements DataStoreListener {
     protected double p;
 
     /**
@@ -173,7 +173,7 @@ public class TreeSphereVisualization extends AbstractVisFactory {
     /**
      * The tree we visualize
      */
-    protected AbstractMTree<?, D, N, E> tree;
+    protected AbstractMTree<?, D, N, E, ?> tree;
 
     /**
      * Constructor
@@ -197,9 +197,9 @@ public class TreeSphereVisualization extends AbstractVisFactory {
 
       p = getLPNormP(tree);
       if (tree != null) {
-        if (ManhattanDistanceFunction.class.isInstance(tree.getDistanceQuery())) {
+        if (ManhattanDistanceFunction.class.isInstance(tree.getDistanceFunction())) {
           dist = Modus.MANHATTAN;
-        } else if (EuclideanDistanceFunction.class.isInstance(tree.getDistanceQuery())) {
+        } else if (EuclideanDistanceFunction.class.isInstance(tree.getDistanceFunction())) {
           dist = Modus.EUCLIDEAN;
         } else {
           dist = Modus.LPCROSS;
@@ -240,11 +240,11 @@ public class TreeSphereVisualization extends AbstractVisFactory {
      * @param entry Current entry
      * @param depth Current depth
      */
-    private void visualizeMTreeEntry(SVGPlot svgp, Element layer, Projection2D proj, AbstractMTree<?, D, N, E> mtree, E entry, int depth) {
+    private void visualizeMTreeEntry(SVGPlot svgp, Element layer, Projection2D proj, AbstractMTree<?, D, N, E, ?> mtree, E entry, int depth) {
       DBID roid = entry.getRoutingObjectID();
       if (roid != null) {
         NumberVector<?> ro = rel.get(roid);
-        D rad = entry.getCoveringRadius();
+        double rad = entry.getCoveringRadius();
 
         final Element r;
         if (dist == Modus.MANHATTAN) {
