@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.visualization.visualizers.scatterplot.outlier;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -36,6 +36,7 @@ import de.lmu.ifi.dbs.elki.result.HierarchicalResult;
 import de.lmu.ifi.dbs.elki.result.Result;
 import de.lmu.ifi.dbs.elki.result.ResultUtil;
 import de.lmu.ifi.dbs.elki.result.outlier.OutlierResult;
+import de.lmu.ifi.dbs.elki.utilities.datastructures.hierarchy.Hierarchy;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.AbstractParameterizer;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
@@ -115,8 +116,8 @@ public class BubbleVisualization extends AbstractVisFactory {
       boolean vis = true;
       // Quick and dirty hack: hide if parent result is also an outlier result
       // Since that probably is already visible and we're redundant.
-      for(Result r : o.getHierarchy().getParents(o)) {
-        if(r instanceof OutlierResult) {
+      for(Hierarchy.Iter<Result> r = o.getHierarchy().iterParents(o); r.valid(); r.advance()) {
+        if(r.get() instanceof OutlierResult) {
           vis = false;
           break;
         }
@@ -182,6 +183,9 @@ public class BubbleVisualization extends AbstractVisFactory {
             final NumberVector<?> vec = rel.get(objId);
             if(vec != null) {
               double[] v = proj.fastProjectDataToRenderSpace(vec);
+              if(v[0] != v[0] || v[1] != v[1]) {
+                continue; // NaN!
+              }
               Element circle = svgp.svgCircle(v[0], v[1], radius * bubble_size);
               SVGUtil.addCSSClass(circle, BUBBLE + colors.getStyleForDBID(objId));
               layer.appendChild(circle);
@@ -197,6 +201,9 @@ public class BubbleVisualization extends AbstractVisFactory {
             final NumberVector<?> vec = rel.get(objId);
             if(vec != null) {
               double[] v = proj.fastProjectDataToRenderSpace(vec);
+              if(v[0] != v[0] || v[1] != v[1]) {
+                continue; // NaN!
+              }
               Element circle = svgp.svgCircle(v[0], v[1], radius * bubble_size);
               int color = stylepolicy.getColorForDBID(objId);
               final StringBuilder cssstyle = new StringBuilder();
@@ -320,7 +327,7 @@ public class BubbleVisualization extends AbstractVisFactory {
         fill = fillF.isTrue();
       }
 
-      ObjectParameter<ScalingFunction> scalingP = new ObjectParameter<ScalingFunction>(SCALING_ID, OutlierScalingFunction.class, true);
+      ObjectParameter<ScalingFunction> scalingP = new ObjectParameter<>(SCALING_ID, OutlierScalingFunction.class, true);
       if(config.grab(scalingP)) {
         scaling = scalingP.instantiateClass(config);
       }
@@ -331,4 +338,4 @@ public class BubbleVisualization extends AbstractVisFactory {
       return new BubbleVisualization(this);
     }
   }
-}
\ No newline at end of file
+}
