@@ -134,6 +134,9 @@ public class SelectionDotVisualization extends AbstractVisFactory {
         for(DBIDIter iter = selection.iter(); iter.valid(); iter.advance()) {
           try {
             double[] v = proj.fastProjectDataToRenderSpace(rel.get(iter));
+            if (v[0] != v[0] || v[1] != v[1]) {
+              continue; // NaN!
+            }
             Element dot = svgp.svgCircle(v[0], v[1], size);
             SVGUtil.addCSSClass(dot, MARKER);
             layer.appendChild(dot);
