@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.workflow;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2013
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -45,7 +45,7 @@ public class InputStep implements WorkflowStep {
 
   /**
    * Constructor.
-   *
+   * 
    * @param database Database to use
    */
   public InputStep(Database database) {
@@ -62,7 +62,7 @@ public class InputStep implements WorkflowStep {
     database.initialize();
     return database;
   }
-  
+
   /**
    * Parameterization class.
    * 
@@ -76,13 +76,23 @@ public class InputStep implements WorkflowStep {
      */
     protected Database database = null;
 
+    /**
+     * Option ID to specify the database type
+     * 
+     * Key:
+     * <p>
+     * {@code -db}
+     * </p>
+     */
+    public static final OptionID DATABASE_ID = new OptionID("db", "Database class.");
+
     @Override
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
-      final ObjectParameter<Database> dbP = new ObjectParameter<Database>(OptionID.DATABASE, Database.class, StaticArrayDatabase.class);
-      if(config.grab(dbP)) {
+      final ObjectParameter<Database> dbP = new ObjectParameter<>(DATABASE_ID, Database.class, StaticArrayDatabase.class);
+      if (config.grab(dbP)) {
         database = dbP.instantiateClass(config);
-      }      
+      }
     }
 
     @Override
@@ -90,4 +100,4 @@ public class InputStep implements WorkflowStep {
       return new InputStep(database);
     }
   }
-}
\ No newline at end of file
+}
