@@ -58,10 +58,8 @@ public class TestPreDeConResults extends AbstractSimpleAlgorithmTest implements
   public void testPreDeConResults() {
     // Additional input parameters
     ListParameterization inp = new ListParameterization();
-    inp.addParameter(ClassLabelFilter.CLASS_LABEL_INDEX_ID, 1);
+    inp.addParameter(ClassLabelFilter.Parameterizer.CLASS_LABEL_INDEX_ID, 1);
     Class<?>[] filters = new Class<?>[] { ClassLabelFilter.class };
-    // FIXME: makeSimpleDatabase currently does also add FILTERS, this doesn't
-    // work.
     Database db = makeSimpleDatabase(UNITTEST + "axis-parallel-subspace-clusters-6d.csv.gz", 2500, inp, filters);
 
     ListParameterization params = new ListParameterization();
@@ -108,4 +106,4 @@ public class TestPreDeConResults extends AbstractSimpleAlgorithmTest implements
     testFMeasure(db, result, 0.6470817);
     testClusterSizes(result, new int[] { 7, 10, 10, 13, 15, 16, 16, 18, 28, 131, 586 });
   }
-}
\ No newline at end of file
+}
