@@ -25,7 +25,6 @@ package de.lmu.ifi.dbs.elki.algorithm.outlier;
 
 import org.junit.Test;
 
-import de.lmu.ifi.dbs.elki.JUnit4Test;
 import de.lmu.ifi.dbs.elki.algorithm.AbstractSimpleAlgorithmTest;
 import de.lmu.ifi.dbs.elki.data.DoubleVector;
 import de.lmu.ifi.dbs.elki.database.Database;
@@ -36,16 +35,17 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.ListParamet
 /**
  * Tests the ABOD algorithm.
  * 
+ * Note: we don't implement JUnit4Test, as this test is slow.
+ * 
  * @author Lucia Cichella
  */
-public class TestABOD extends AbstractSimpleAlgorithmTest implements JUnit4Test {
+public class TestABOD extends AbstractSimpleAlgorithmTest {
   @Test
   public void testABOD() {
     Database db = makeSimpleDatabase(UNITTEST + "outlier-3d-3clusters.ascii", 960);
 
     // Parameterization
     ListParameterization params = new ListParameterization();
-    params.addParameter(ABOD.K_ID, 5);
 
     // setup Algorithm
     ABOD<DoubleVector> abod = ClassGenericsUtil.parameterizeOrAbort(ABOD.class, params);
@@ -54,7 +54,7 @@ public class TestABOD extends AbstractSimpleAlgorithmTest implements JUnit4Test
     // run ABOD on database
     OutlierResult result = abod.run(db);
 
-    testSingleScore(result, 945, 3.7108897864090475E-4);
-    testAUC(db, "Noise", result, 0.9638148148148148);
+    testAUC(db, "Noise", result, 0.94887037037037);
+    testSingleScore(result, 945, 1.88108120738508E-4);
   }
 }
\ No newline at end of file
