@@ -27,7 +27,7 @@ import org.junit.Test;
 
 import de.lmu.ifi.dbs.elki.JUnit4Test;
 import de.lmu.ifi.dbs.elki.algorithm.AbstractSimpleAlgorithmTest;
-import de.lmu.ifi.dbs.elki.algorithm.outlier.LOF;
+import de.lmu.ifi.dbs.elki.algorithm.outlier.lof.LOF;
 import de.lmu.ifi.dbs.elki.database.Database;
 import de.lmu.ifi.dbs.elki.result.outlier.OutlierResult;
 import de.lmu.ifi.dbs.elki.utilities.ClassGenericsUtil;
@@ -45,7 +45,7 @@ public class TestFeatureBagging extends AbstractSimpleAlgorithmTest implements J
 
     // Parameterization
     ListParameterization params = new ListParameterization();
-    params.addParameter(LOF.K_ID, 10);
+    params.addParameter(LOF.Parameterizer.K_ID, 10);
     params.addParameter(FeatureBagging.Parameterizer.NUM_ID, 10);
     params.addParameter(FeatureBagging.Parameterizer.SEED_ID, 1);
 
@@ -66,7 +66,7 @@ public class TestFeatureBagging extends AbstractSimpleAlgorithmTest implements J
 
     // Parameterization
     ListParameterization params = new ListParameterization();
-    params.addParameter(LOF.K_ID, 10);
+    params.addParameter(LOF.Parameterizer.K_ID, 10);
     params.addParameter(FeatureBagging.Parameterizer.NUM_ID, 10);
     params.addParameter(FeatureBagging.Parameterizer.SEED_ID, 5);
     params.addFlag(FeatureBagging.Parameterizer.BREADTH_ID);
