@@ -42,16 +42,18 @@ import org.junit.Test;
  * @author Erich Schubert
  */
 public class TestHeapPerformance {
-  final private int queueSize = 100000;
+  final private int queueSize = 200000;
 
-  final private int iterations = 20;
+  final private int preiterations = 20;
+  
+  final private int iterations = 200;
 
   final private long seed = 123456L;
 
   @Test
   public void testRuntime() throws Exception {
     // prepare the data set
-    final List<Integer> elements = new ArrayList<Integer>(queueSize);
+    final List<Integer> elements = new ArrayList<>(queueSize);
     {
       final Random random = new Random(seed);
       for(int i = 0; i < queueSize; i++) {
@@ -62,39 +64,40 @@ public class TestHeapPerformance {
 
     // Pretest, to trigger hotspot compiler, hopefully.
     {
-      for(int j = 0; j < iterations; j++) {
-        Heap<Integer> pq = new Heap<Integer>();
+      for(int j = 0; j < preiterations; j++) {
+        ComparableMinHeap<Integer> pq = new ComparableMinHeap<>();
         testHeap(elements, pq);
       }
-      for(int j = 0; j < iterations; j++) {
-        PriorityQueue<Integer> pq = new PriorityQueue<Integer>(); // 11,
+      for(int j = 0; j < preiterations; j++) {
+        PriorityQueue<Integer> pq = new PriorityQueue<>();
         testQueue(elements, pq);
       }
     }
 
+    long pqstart = System.nanoTime();
+    {
+      for(int j = 0; j < iterations; j++) {
+        PriorityQueue<Integer> pq = new PriorityQueue<>();
+        testQueue(elements, pq);
+      }
+    }
+    long pqtime = System.nanoTime() - pqstart;
+
     long hstart = System.nanoTime();
     {
       for(int j = 0; j < iterations; j++) {
-        Heap<Integer> pq = new Heap<Integer>();
+        ComparableMinHeap<Integer> pq = new ComparableMinHeap<>();
         testHeap(elements, pq);
       }
     }
     long htime = System.nanoTime() - hstart;
 
-    long pqstart = System.nanoTime();
-    {
-      for(int j = 0; j < iterations; j++) {
-        PriorityQueue<Integer> pq = new PriorityQueue<Integer>(); // 11
-        testQueue(elements, pq);
-      }
-    }
-    long pqtime = System.nanoTime() - pqstart;
     System.err.println("Heap performance test: us: " + htime*1E-9 + " java: " + pqtime*1E-9);
     assertTrue("Heap performance regression - run test individually, since the hotspot optimizations may make the difference! " + htime + " >>= " + pqtime, htime < 1.05 * pqtime);
     // 1.05 allows some difference in measuring
   }
 
-  private void testHeap(final List<Integer> elements, Heap<Integer> pq) {
+  private void testHeap(final List<Integer> elements, ComparableMinHeap<Integer> pq) {
     // Insert all
     for(int i = 0; i < elements.size(); i++) {
       pq.add(elements.get(i));
